/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      macinit.c                  KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*          Include All MAC & BDMAC Initialize Routine                   */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*      hbahn           09-15-1998      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002			*/
/*			*/

//#include <string.h>
#include "ks32c50.h"
#include "evm50100.h"
#include "mac.h"

/* Define MAC address source, IIC EEPROM or set value by manual */

#define MAC_ADDR_FROM_IIC_EEPROM 0

////////////////////////////////////////////////////////////////////////////////
// Rx/Tx Frame descriptors, Rx/Tx data /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

sFrameDes RxFrameDes[MAC_MAX_RX_DES];  	// Rx Frame Descriptor
sMACFrame RxFrameBuf[MAC_MAX_RX_DES];	// Rx Frame Buffer

sFrameDes TxFrameDes[MAC_MAX_TX_DES];   // Tx Frame Descriptor
sMACFrame TxFrameBuf[MAC_MAX_TX_DES];	// Tx Frame Buffer

////////////////////////////////////////////////////////////////////////////////
// Global variables  used for MAC driver ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// MACCON (MAC Control Register) ///////////////////////////////////////////////  
volatile U32 gMacCon    = 

 MACCON_FullDup;	// full-duplex
// MACCON_HaltReg	// stop tx & rxx after completion of ant current packets 
// MACCON_HaltImm	// Stop tx & rx immediately 
// MACCON_SwReset 	// reset all Ethernet controller state machines and FIFOs 
// MACCON_MACLoop	// MAC loopback
// MACCON_ConnM00	// Automatic-default 
// MACCON_ConnM01	// Force 10Mbits endec
// MACCON_ConnM10	// Force MII 
// MACCON_MIIOFF	// Force MII 
// MACCON_Loop10	// Loop 10Mbps 
// MACCON_MissRoll	// Missed error counter rolled over 
// MACCON_MDCOFF	// MII Station Management Clock Off
// MACCON_EnMissRoll	// Interrupt when missed error counter rolls over 
// MACCON_Link10	// Link status 10Mbps 


// MACTXCON (Transmit Control Register) ////////////////////////////////////////
volatile U32 gMacTxCon  = 

 MACTXCON_EnComp;	// + interrupt when the MAC transmits or defer one packet
// TxEn			// - transmit Enable 
// TxHalt		// - Transmit Halt Request 
// NoPad		// - suppress Padding 
// NoCRC		// - Suppress CRC 
// FBack		// - Fast Back-off
// NoDef		// - Disable the defer counter
// SdPause		// - Send Pause
// MII10En		// - MII 10Mbps mode enable
// EnUnder		// - Enable Underrun
// EnDefer		// - Enable Deferral
// EnNCarr		// - Enable No Carrier 
// EnExColl		// - interrupt if 16 collision occur in the same packet 
// EnLateColl		// - interrupt if collision occurs after 512 bit times(64 bytes times) 
// EnTxPar		// - interrupt if the MAC transmit FIFO has a parity error 

// MACRXCON (Receive Control Register) /////////////////////////////////////////

volatile U32 gMacRxCon  = 
 MACRXCON_RxEn 	  | 	// + Receive enable (RxEn)
 MACRXCON_StripCRC; 	// + Strip CRC
// RxHalt		// - Receive halt request 
// LongEn		// - Long enable
// ShortEn		// - Short enable
// PassCtl		// - Pass control packet
// IgnoreCRC		// - Ignore CRC value
// EnAlign		// - Enable alignment interrupt 
// EnCRCErr		// - Enable CRC error interrupt 
// EnOver		// - Enable overflow interrupt 
// EnLongErr		// - Enable long error interrupt 
// EnRxPar		// - Enable receive parity
// EnGood		// - Enable Good

////////////////////////////////////////////////////////////////////////////////
// BDMATXCON // Buffered DMA Trasmit Control Register //////////////////////////
////////////////////////////////////////////////////////////////////////////////

volatile U32 gBdmaTxCon = 
 
 BDMATXCON_TxBRST   | 	// + BDMA Tx Burst Size Mask = 16 
 BDMATXCON_TxSTSKO  |	// + BDMA Tx Stop (1)/Skip (0) Frame or Interrupt in case
			//   of not Owner the current frame 
// BDMATXCON_TxCPIE	// - BDMA Tx Complete to send control packet interrupt Enable
// BDMATXCON_TxNLIE     // - BDMA Tx Null list interrupt enable
// BDMATXCON_TxNOIE	// - BDMA Tx Buffer Not Owner
// BDMATXCON_TxEmpty	// - BDMA Tx Buffer Empty Interrupt 
 BDMATXCON_TxMSL110; 	// + wait to fill 6/8 of the BDMA
// BDMATXCON_TxEn	// - BDMA Tx Enable 
// BDMATXCON_TxRS	// - BDMA Rx Reset 


#ifdef LITTLE

volatile U32 gBdmaRxCon = 

 BDMARXCON_RxBRST   |	// + BDMA Rx Burst Size
 BDMARXCON_RxDIE    | 	// + BDMA Rx Every Received Frame Interrupt Enable
 BDMARXCON_RxEn     | 	// + BDMA Rx Enable
 BDMARXCON_RxLittle | 	// + BDMA Rx Big/Little Endian 
 BDMARXCON_RxMAINC  | 	// + BDMA Rx Memory Address Inc/Dec 
 BDMARXCON_RxNLIE   | 	// + BDMA Rx NULL List Interrupt Enable 
 BDMARXCON_RxNOIE   | 	// + BDMA Rx Not Owner Interrupt Enable
 BDMARXCON_RxSTSKO;	// + BDMA Rx Stop/Skip  Frame or Interrupt(=1) 

// BDMARXCON_RxMSOIE	// - BDMA Rx Maximum Size over Interrupr Enable 
// BDMARXCON_RxBig 	// - BDMA Rx Big/Little Endian
// BDMARXCON_RxWA01	// - BDMA Rx Word Alignment- one invalid byte 
// BDMARXCON_RxWA10	// - BDMA Rx Word Alignment- two invalid byte
// BDMARXCON_RxWA11	// - BDMA Rx Word Alignment- three invalid byte 
// BDMARXCON_RxRS	// - BDMA Rx Reset
// BDMARXCON_RxEmpty	// - BDMA Rx Buffer empty interrupt 
// BDMARXCON_RxEarly	// - BDMA Rx Early notify Interrupt

#else
volatile U32 gBdmaRxCon = BRxDIE | BRxEn | BRxBig | BRxMAINC | BRxBRST | \
                          BRxNLIE | BRxNOIE | BRxSTSKO ;
#endif

///////////////////////////////////////////////////////////////////////////////
// CAMCON // CAM control register /////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

volatile U32 gCamCon = 

 CAMCON_CompEn	| 	// + Compare Enable mode
 CAMCON_BroadAcc;	// + Accept any packet with a broadcast station station address  

// StationAcc		// - Accept any packet with a unicast station address 
// GroupAcc		// - Accept any packet with multicast-group  
// NegCAM		// - 


volatile U32 gCTxFDPtr;		/* Current Tx BDMA descriptor (moved by ISR)*/
volatile U32 gWTxFDPtr;		/* Write Tx Frame descriptor (moved by Send)*/
volatile U32 gCRxFDPtr;		/* Current Rx Frame descriptor (moved by ISR)*/
volatile U32 gRRxFDPtr;		/* Read Rx Frame descriptor (moved by Recv)*/
volatile U32 gRxFDCnt=0;	
volatile U32 gCam0_Addr0 = 0;	/* Content of CAM entry 0 */
volatile U32 gCam0_Addr1 = 0;	/* Content of CAM entry 1 */
volatile U32 gStaCon = 0;	/* MDC clock [15:13] STACON*/
volatile U8  MyMacSrcAddr[6];	/* MAC Stantion Adress */
volatile int gRxErrorPacketCnt=0;/* Total Received with error packets */


////////////////////////////////////////////////////////////////////////////////
// Global variable structure for store status //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// MAC Transmit status /////////////////////////////////////////////////////////

pMACTxStatus gsMacTxStatus = 
{
 0	,	// MacTxGood
 0	,	// ExCollErr	 (Transmit collission count) 
 0	,	// TxDefferedErr (Transmit deferred)
 0	,	// sPaused	 (Paused)
 0	,	// UnderErr	 (Underrun)
 0	,	// DeferErr	 (Deferral)
 0	,	// NCarrErr	 (No carrier)
 0	,	// sSQE		 (Signal quality error)
 0	,	// LateCollErr	 (Late collision)
 0	,	// TxParErr	 (Transmit parity error)
 0		// sTxHalted	 (Transmission halted)
};


// MAC Receive status //////////////////////////////////////////////////////////

pMACRxStatus gsMacRxStatus = 
{
 0	,	 // OvMaxSize
 0	,	 // sCtlRecd	 (Control frame received)
 0	,	 // sRx10Stat	 (Receive 10-Mb/s status)
 0	,	 // AllgnErr	 (Alignment error)
 0	,	 // sCRCErr	 (CRC error)
 0	,	 // OverflowErr	 (Overflow error)
 0	,	 // sLongErr	 (Long error)
 0	,	 // RxParErr	 (Receive parity error)
 0		 // sRxHalted	 (Reception halted)
};


// BDMA Transmit Status ////////////////////////////////////////////////////////


pBDMATxStatus gsBdmaTxStatus = 
{
 0	,	// BTxNLErr	(BDMA Tx null list)
 0	,	// BTxNOErr     (BDMA Tx not owner)
 0		// BTxEmptyErr	(BDMA Tx buffer empty)
};


// BDMA Receive Status /////////////////////////////////////////////////////////

pBDMARxStatus gsBdmaRxStatus = 
{
 0	,	// BdmaRxCnt
 0	,	// BdmaRxGood
 0	,	// BRxNLErr	(BDMA Rx null list)
 0	,	// BRxNOErr	(BDMA Rx not owner) 
 0		// BRxMSOErr	(BDMA Rx maximum size over)
};

// 
extern TIME tm0 ;

////////////////////////////////////////////////////////////////////////////////
// Initialize Tx frame descriptor area-buffers /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void TxFDInitialize(void)
{
 sFrameDes * pFrame;			// current frame descriptor pointer 
 sFrameDes * pStart;			// start frame descriptor pointer 
 sFrameDes * pLast = NULL;		// last frame desriptor pointer 
 U32 DataAddr;				// data buffer pointer 
 U32 i;

// Get Frame descriptor's base address.   
// +0x4000000 is for setting this area to non-cacheable area. 

 BDMATXPTR = (U32)TxFrameDes+0x4000000;	// BDMA Transmit Pointer  
 gWTxFDPtr = gCTxFDPtr = BDMATXPTR;	// 

// Get Transmit buffer base address ////////////////////////////////////////////
 DataAddr = (U32)TxFrameBuf + 0x4000000;

// Generate linked list ////////////////////////////////////////////////////////
 pFrame = (sFrameDes *) gCTxFDPtr;	// current frame descriptor 
 pStart = pFrame;			// start frame descriptor 

 for(i=0; i < MAC_MAX_TX_DES; i++)  
  {
   if(pLast != NULL)
    {
     pLast->NextDes = (U32) pFrame;	//  
    }

// 
   pFrame->DataPtr      = (U32)(DataAddr & fOwnership_CPU); // data pointer	
   pFrame->Reserved     = (U32)0x0;	// reserved field 
   pFrame->StatusLength = (U32)0x0;	// status & length field
   pFrame->NextDes      = NULL;		// next descriptor pointer 
// 
   pLast = pFrame;			// reset last frame descriptor pointer 
   pFrame++;				// move to next descriptor pointer 
   DataAddr += sizeof(sMACFrame);	// pointer to next data buffer
  }

// Make Frame descriptor to ring buffer type ///////////////////////////////////     

 pFrame--; 
 pFrame->NextDes = (U32)pStart;
}

////////////////////////////////////////////////////////////////////////////////
// Initialize Rx frame descriptor area-buffers /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void RxFDInitialize(void)
{
 sFrameDes * pFrame;			// current frame descriptor pointer 
 sFrameDes * pStart;			// pointer to start of list 
 sFrameDes * pLast = NULL;		// previous frame descriptor 
 U32 DataAddr;				// data buffer address
 U32 i;

// Get Frame descriptor's base address.  
// +0x4000000 is for setting this area to non-cacheable area. 
 BDMARXPTR = (U32)RxFrameDes + 0x4000000;// 
 gRRxFDPtr = gCRxFDPtr = BDMARXPTR;
 gRxFDCnt=0;	
	
// Get Transmit buffer base address.  
 DataAddr = (U32)RxFrameBuf + 0x4000000;

// Generate linked list. 
 pFrame = (sFrameDes *) gCRxFDPtr;
 pStart = pFrame;
	
 for(i=0; i < MAC_MAX_RX_DES; i++)  
  {
   if(pLast != NULL)
    {
     pLast->NextDes = (U32) pFrame;	// set NextDes field  
    }

// frame descriptor data pointer ///////////////////////////////////////////////
   pFrame->DataPtr =(U32)(DataAddr | fOwnership_BDMA | 0x4000000);
//
   pFrame->Reserved = (U32)0x0;		// reserved field 
   pFrame->StatusLength = (U32)0x0;	// status & length 
   pFrame->NextDes = NULL;		// next descriptor pointer 

// 		
   pLast = pFrame;			// reset last pointer 
   pFrame++;				// move to next pointer descriptor
   DataAddr += sizeof(sMACFrame); 	// pointer to next data buffer
  } 
      
// Make Frame descriptor to ring buffer type.     
 pFrame--; 
 pFrame->NextDes = (U32)pStart;
}

////////////////////////////////////////////////////////////////////////////////
// Initialize MAC and BDMA Controller //////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MacInitialize(void)
{
 int pos;
// Disable MAC and BDMA interrupts /////////////////////////////////////////////

 Disable_Int(nMAC_RX_INT );    	// Disable MAC Rx Interrupt
 Disable_Int(nMAC_TX_INT );	// Disable MAC Tx Interrupt
 Disable_Int(nBDMA_RX_INT);	// Disable BDMA Rx Interrupt
 Disable_Int(nBDMA_TX_INT);	// Disable BDMA Tx Interrupt

// BDMA and MAC interrupt vector setup /////////////////////////////////////////

 SysSetInterrupt(nMAC_RX_INT , MAC_Rx_isr );	// MAC  Rx ISR   
 SysSetInterrupt(nMAC_TX_INT , MAC_Tx_isr );	// MAC  Tx ISR  
 SysSetInterrupt(nBDMA_RX_INT, BDMA_Rx_isr); 	// BDMA Rx ISR  
 SysSetInterrupt(nBDMA_TX_INT, BDMA_Tx_isr);	// BDMA Tx ISR 

// reset PHY, (restart AN) /////////////////////////////////////////////////////
 ResetPhyChip();

// Set the initial condition of the BDMA, and MAC //////////////////////////////

 BDMARXCON = BDMARXCON_RxRS;    // Reset BDMAC Receiver
 BDMATXCON = BDMATXCON_TxRS;   	// Reset BDMAC Transceiver
 MACCON    = MACCON_SwReset;	// MAC Block Reset 
 BDMARXLSZ = MaxRxFrameSize;    // 
// Set the BDMA Tx/Rx Frame Descriptor /////////////////////////////////////////

 TxFDInitialize();		// Init Tx buffer descriptors list
 RxFDInitialize();		// Init Rx buffer descriptors list 

#if MAC_ADDR_FROM_IIC_EEPROM
// Get MAC Address From IIC EEPROM //////////////////////////////////////////////
 MacAddrReadIIC(MyMacSrcAddr);
#else	
// manually setting the MAC address
 MyMacSrcAddr[0] = 0x00;
 MyMacSrcAddr[1] = 0x00;
 MyMacSrcAddr[2] = 0xf0;
 MyMacSrcAddr[3] = 0x11;
 MyMacSrcAddr[4] = 0x00;
 MyMacSrcAddr[5] = 0x00;
#endif

/* Copy MAC Address to global variable */
 for(pos = 0; pos < 4; ++pos)
  {
   gCam0_Addr0 = (gCam0_Addr0 << 8) | MyMacSrcAddr[pos];
  }

 for(pos = 4; pos < 6; ++pos)
  {
   gCam0_Addr1 = (gCam0_Addr1 << 8) | MyMacSrcAddr[pos];
  }
 gCam0_Addr1 = (gCam0_Addr1 << 16) ;


/* Set CAM0 register : 0x9100~0x9103, 0x9104, 0x9105 */
 CAM_Reg(0) =  gCam0_Addr0;
 CAM_Reg(1) =  gCam0_Addr1;

// CAM Enable Register(CAMEN) //////////////////////////////////////////////////
 CAMEN  = 0x0001;		// CAM Address 0 = valid	
 CAMCON = gCamCon;		// CAM control register 

// Enable interrupt BDMA Rx and MAC Tx interrupt
 Enable_Int(nBDMA_RX_INT); 	// enable BDMA Receive interrupt 
 Enable_Int(nMAC_TX_INT );	// enable BDMA Transmit interrupt 

// Configure the BDMA and MAC control registers ////////////////////////////////

 MACCON    = gMacCon;		// MAC  Control Register 
 BDMATXCON = gBdmaTxCon;	// BDMA Transmit Control Register 		
 MACTXCON  = gMacTxCon;		// MAC  Transmit Control Register
 BDMARXCON = gBdmaRxCon;	// BDMA Receive  Control Register 
 MACRXCON  = gMacRxCon;		// MAC  Receive  Control Register
}


////////////////////////////////////////////////////////////////////////////////
// Reset The Phychip, Auto-Negotiation Enable //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void ResetPhyChip(void)
{
 int tm1en;				// timer 1 enable flag 
 TIME tm;				// TIME struct 
 U32 sec;				// start time, sec
 U32 val;				// PHY register value 

// 
 MiiStationWrite
  (
   PHY_CNTL_REG	, 			// control register 
   PHYHWADDR	, 			// PHY address 
   ENABLE_AN    | 			// enable autonegotiation
   RESTART_AN				// restart autonegotiation 
  ); 


 tm1en = TimerEnabled(1);		// read timer 1 enable flag 	
 if(tm1en == 0)
  {
// timer 1 disabled => enable timer 1 //////////////////////////////////////////
   TimerReset(1);				// reset timer 1 
   TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));	// init timer 1 
   TimerStart(1);				// start timer 1 
  }

 GetSysTime(1,& tm); 			// read start time, convert to ms
 sec = tm.tm_sec;			// start time sec

 do
  {
// wait AN complete, 3 sec 
   GetSysTime(1,& tm); 			// read start time, convert to ms
   val = MiiStationRead(PHY_STATUS_REG, PHYHWADDR); 
  }
 while((val & PHYSTAT_AN_COMPLETE) == 0 && tm.tm_sec < sec + 3);
 if(tm1en == 0)
  {					
   TimerStop(1);			// stop timer
  }

// read chip status register 
 val = MiiStationRead(PHY_CHIPSTAT_REG,PHYHWADDR); 
 if((val & PHYCHIPSTAT_LINKUP) == 0)
  {
//   Print("\nLink down");
  }

 if((val & PHYCHIPSTAT_FULLDUPLEX) != 0)
  {
// link up bit ignored 
   gMacCon |= MACCON_FullDup;
   MACCON  |= MACCON_FullDup;		// set full duplex bit
  }
 else
  {
   gMacCon &=~MACCON_FullDup;		// clear full duplex bit
   MACCON  &=~MACCON_FullDup;		// clear full duplex bit
  }
}

/*
 * Function    : MiiStationRead, MiiStationWrite  
 * Description : MII Interface Station Management Register Read or Write
 * Input       : PhyInAddr = PHY internal register address
 *               PhyAddr   = PHY unique address
 *               PhyWrData = When Write
 * Output      : PhyRdData = WhenRead
 */

void MiiStationWrite(U32 PhyInAddr, U32 PhyAddr, U32 PhyWrData)
{
 int i=1000 ;
 STADATA = PhyWrData;		// Station management data register
 STACON  = 			// Station control register 
  (
   PhyInAddr  | 		// PHY register address (Addr)
   PhyAddr    | 		// PHY address (PHY)
   STACON_Busy| 		// Busy bit (Busy)
   STACON_Wr  |			// Write bit (Wr)
   gStaCon			// MDC clock 
  );

 while((STACON & STACON_Busy));
 while(i--);			// delay physet
}

U32 MiiStationRead(U32 PhyInAddr, U32 PhyAddr) 
{
 U32	PhyRdData ;
 STACON = PhyInAddr | PhyAddr |  STACON_Busy | gStaCon;
 while( (STACON & STACON_Busy) )  ;
 PhyRdData = STADATA ;
 return(PhyRdData);
}


////////////////////////////////////////////////////////////////////////////////
// Interrupt Service Routine for MAC Tx ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MAC_Tx_isr(void)
{   
 sFrameDes * frame;			// Descriptor pointer 
 U32 status;				// status of transmitted descriptor
 U32 CTxPtr;				// current BDMA transmitted descriptor

// 
 CTxPtr = BDMATXPTR;

 while(gCTxFDPtr != CTxPtr)  
  { 
   frame = (sFrameDes *) gCTxFDPtr;	// 

// Check CPU ownership
// if Owner is BDMA then break
   if( (frame->DataPtr & fOwnership_BDMA) ) 
    { 
// Owner of descriptor is BDMA => descriptor is not transmitted ////////////////
     break;
    }	
   status = (frame->StatusLength >> 16) & 0xffff;

// divch remove from else
   if(status & TXFDST_Under)   {gsMacTxStatus.UnderErr++;} // Underrun (TXFDST_Under)
   if(status & TXFDST_ExColl)  {gsMacTxStatus.ExCollErr++;}// Excessive Collision (TXFDST_ExColl)
   if(status & TXFDST_TxDeffer){gsMacTxStatus.TxDefferedErr++;} // Transmit deffered (TXFDST_TxDeffer)
   if(status & TXFDST_Paused)  {gsMacTxStatus.sPaused++;}  // Paused (TXFDST_Paused)
   if(status & TXFDST_Defer)   {gsMacTxStatus.DeferErr++;} // TXFDST_Defer	
   if(status & TXFDST_NCarr)   {gsMacTxStatus.NCarrErr++;} // No Carrier sense

// fake collision signal didn't come from PHY for 1.6us. (TXFDST_SQErr) ////////
   if(status & TXFDST_SQErr)   {gsMacTxStatus.sSQE++;}
   if(status & TXFDST_LateColl){gsMacTxStatus.LateCollErr++;}// Late collision
   if(status & TXFDST_TxPar)   {gsMacTxStatus.TxParErr++;} // Transmit Parity Error
   if(status & TXFDST_TxHalted){gsMacTxStatus.sTxHalted++;}// Transmission halted 

   if(status & TXFDST_Comp) 
    { 
// MAC transmit or discards one packet (TXFDST_Comp) ///////////////////////////
     gsMacTxStatus.MacTxGood++ ;
    } 
   else
    {
// Save Error status
// Print MAC Debug status //////////////////////////////////////////////////////    		
     MacDebugStatus(); 
// Set MAC/BDMA Tx control register for next use.
     BDMATXCON = gBdmaTxCon;
     MACTXCON  = gMacTxCon;
    }

// Clear Framedata pointer already used.
   frame->StatusLength = (U32)0x0;
   gCTxFDPtr = (U32)frame->NextDes;
  } // end while loop
}

////////////////////////////////////////////////////////////////////////////////
// Interrupt Service Routine for MAC Rx ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MAC_Rx_isr(void) 
{ 
}


////////////////////////////////////////////////////////////////////////////////
// Interrupt Service Routine for BDMA Tx ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void BDMA_Tx_isr(void) 
{ 
}


int MACRecv(U8 * data, U32 * status, int * size)  
{
 sFrameDes * frame;			// pointer to Rx descriptor 
 U32 * dst;				// desination address
 U32 * src;				// source address
 int wlen;				// data length words
 int len;				// data lenght 
 if(data == 0 || status == 0 || size == 0)
  {
   return(0);
  }

 if(gRxFDCnt == 0)
  {
   return(0);
  } 

//
 frame = (sFrameDes *) gRRxFDPtr;
 if((frame->DataPtr & fOwnership_BDMA))
  {					// owner DMA
   return(0);				// => exit 
  }

 *status = (frame->StatusLength >> 16) & 0xFFFF; // frame status 
 len = frame->StatusLength & 0xFFFF;	// frame length
 if(len > *size)
  {					// out of input buffer
   len = *size;				// reset length to sizeof buffer
  }
 else
  {
   *size = len;				// readed bytes to buffer 
  }

 dst = (U32 *) data;			// destination address 
 src = (U32 *) frame->DataPtr;  	// source address
 wlen = len >> 2; 			// words count 
 while(wlen-- > 0)
  {
   *dst++ = *src++; 
  }

 if(len & 0x3)
  {
// copy remain bytes 
   data = (U8 *) dst;			// reset data pointer
   wlen = *src;				// read source word
   switch(len & 0x3)
    {
     case 3 : *(data + 2) = (wlen >> 16) & 0xFF;
     case 2 : *(data + 1) = (wlen >>  8) & 0xFF;
     case 1 : *(data + 0) =  wlen & 0xFF;
      break;
    } 
  }

//
 Disable_Int(nGLOBAL_INT);
 gRxFDCnt--;
 frame->StatusLength = 0x0;		// clear frame status 
 frame->DataPtr |= fOwnership_BDMA;	// owner DMA 
 gRRxFDPtr = (U32) frame->NextDes;	// move read pointer 
 Enable_Int(nGLOBAL_INT);

 if((BDMARXCON & BDMARXCON_RxEn) == 0) 
  {
   BDMARXCON |= BDMARXCON_RxEn;
  }
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// flush Rx BD /////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int MACRxFlush(void)
{
 sFrameDes * frame;			// current frame descriptor pointer 
 int cnt;				// Rx BD count   
 int fn;				// frame number

// 
 Disable_Int(nGLOBAL_INT);		// disable global int

// 
 cnt = gRxFDCnt;			// Rx BD count 
 frame = (sFrameDes *) gRRxFDPtr;	// current readed frame 
 for(fn = 0; fn < cnt; fn++)  
  {
   if(frame == 0)
    {					// null list	
     return(0);				// 
    } 

   if((frame->DataPtr & fOwnership_BDMA))
    {					// allready set BDMA owner bit 
     break;				// => break
    } 

   frame->DataPtr |= fOwnership_BDMA;	// set DMA owner bit 
   frame->Reserved = (U32)0x0;		// clear reserved field 
   frame->StatusLength = (U32)0x0;	// clear status & length 
   frame = (sFrameDes *)frame->NextDes;	// move to next descriptor 
  }

// 
 gRxFDCnt -= fn;			// 
 Enable_Int(nGLOBAL_INT);		// enable global int 
 return(1);
}


int MACRxBD_count(void)
{
 return(gRxFDCnt);
}

////////////////////////////////////////////////////////////////////////////////
// Interrupt Service Routine for BDMA Rx Ethenet Frame is received in //////////
// BDMA_Rx_isr /////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void BDMA_Rx_isr(void)
{
 sFrameDes * frame;			// pointer to Rx descriptor 
 U32 status; 				// RX frame status
 U32 CRxPtr;				// current Rx DBMA pointer (content of BDMARXPTR)

// Check Null List Interrupt ///////////////////////////////////////////////////
 if(BDMASTAT & BDMASTAT_BRxNL) 
  {
   BDMASTAT |= BDMASTAT_BRxNL;		// Clear BDMA Rx NULL List Interrupt 
   gsBdmaRxStatus.BRxNLErr++; 		// calculate BDMA NUll List Interrupt count
   MacInitialize();			// Reinitialize MAC  
   return;
  }

// Step 1. Get current frame descriptor
 CRxPtr    = BDMARXPTR;			// current BDMA Rx descriptor pointer

// Step 2. Clear BDMA status register bit by write 1
 BDMASTAT |= BDMASTAT_BRxRDF;		// BDMA Rx Done Every Received Frame 	

 do 
  {
// Step 3. Get Rx Frame Descriptor /////////////////////////////////////////////
   frame = (sFrameDes *) gCRxFDPtr;	// RX Descriptor pointer 	
   if((frame->DataPtr & fOwnership_BDMA) != 0)
    {					// owner BDMA => break  
     break;
    } 
//
   gsBdmaRxStatus.BdmaRxCnt++;		// Total received packets
   status =(frame->StatusLength >> 16) & 0xFFFF;// RX status 

// Step 4. If Rx frame is good, then process received frame ////////////////////
   if(status & RXFDST_RxGood) 
    {
     gsBdmaRxStatus.BdmaRxGood++;	// total received good packets
    } 
   else 
    {
////////////////////////////////////////////////////////////////////////////////
// Step 5. If Rx frame has error, then process error frame /////////////////////
////////////////////////////////////////////////////////////////////////////////
     gRxErrorPacketCnt++ ;

// Save Error status
// Check each status, because, error can duplicated

     if(status & RXFDST_OvMax)    gsMacRxStatus.OvMaxSize++; // Over Maximum Size
     if(status & RXFDST_CtlRecd)  gsMacRxStatus.sCtlRecd++;  // set if packet received is a MAC control frame.
     if(status & RXFDST_Rx10Stat) gsMacRxStatus.sRx10Stat++; // set if packet was received via the 10bits interface
     if(status & RXFDST_AlignErr) gsMacRxStatus.AllgnErr++;  // Alignment Error 
     if(status & RXFDST_CRCErr)   gsMacRxStatus.sCRCErr++;   // CRC error
     if(status & RXFDST_Overflow) gsMacRxStatus.OverflowErr++;// MAC receive FIFO was full 
     if(status & RXFDST_LongErr)  gsMacRxStatus.sLongErr++;  // received a frame longer than 1518 bytes 
     if(status & RXFDST_RxPar)    gsMacRxStatus.RxParErr++;  // MAC receive FIFO has detected a parity error
     if(status & RXFDST_RxHalted) gsMacRxStatus.sRxHalted++; // Transmission was halted by clearing RxEn
    }

// Step 6. Get Next Frame Descriptor pointer to process
   gCRxFDPtr = (U32)(frame->NextDes);
   gRxFDCnt++;	
  } 
 while (CRxPtr != gCRxFDPtr);

// Step 7. Check NotOwner status ///////////////////////////////////////////////
 if(BDMASTAT & BDMASTAT_BRxNO) 
  {
   BDMASTAT |= BDMASTAT_BRxNO;		// BDMA Rx Not Owner
   gsBdmaRxStatus.BRxNOErr++;		// count of BDMA Rx Ont owner errors
  }
}

////////////////////////////////////////////////////////////////////////////////
// Descripton : Send ethernet frame function ///////////////////////////////////
// Input      : frame data pointer, frame length ///////////////////////////////
// Output     : transmit ok(1) or error(0) /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int MACSend(U8 * data, int size)
{
 sFrameDes * frame;			// frame descriptor pointer 
 U32 * dst;
 U32 * src;
 int wlen;

// 1. Get Tx frame descriptor & data pointer ///////////////////////////////////
 frame = (sFrameDes *)gWTxFDPtr;	// pointer to frame
// 2. Check BDMA ownership /////////////////////////////////////////////////////
 if((frame->DataPtr & fOwnership_BDMA) ) 
  {					// descriptor is used by BDMA 
   return(0); 				// not enough frame descriptors
  }

// 3. Prepare Tx Frame data to Frame buffer ////////////////////////////////////

 size += sizeof(etheader);
 if(size > MaxRxFrameSize)
  {
   size = MaxRxFrameSize;
  } 
 wlen = (size + 3) >> 2;
 dst = (U32 *) frame->DataPtr;		// pointer to frame descriptor data
 src = (U32 *) data;		
 while(wlen-- > 0)
  {
   *dst++ = *src++; 
  } 


// 4. Set TX Frame flag & Length Field

#ifdef LITTLE
 frame->Reserved = 
  (
   TXFDCTRL_PaddingMode 	| 	// Padding mode 	
   TXFDCTRL_CRCMode 		| 	// CRC mode
   TXFDCTRL_SourceAddrIncrement | 	// Frame data pointer increment
   TXFDCTRL_LittleEndian 	| 	// Little-endian 
   TXFDCTRL_WidgetAlign00 	| 	// No Invalid bytes
   TXFDCTRL_MACTxIntEn		  	// transmit interrupt enable after transmission
  );
#else
 frame->Reserved = 
  (
   TXFDCTRL_PaddingMode 	| 	// Padding mode
   TXFDCTRL_CRCMode 		| 	// CRC Mode
   TXFDCTRL_SourceAddrIncrement | 	// Frame Data pointer Increment 
   TXFDCTRL_BigEndian 		| 	// Big Endian 
   TXFDCTRL_WidgetAlign00 	| 	// No invalid bytes
   TXFDCTRL_MACTxIntEn		  	// after frame fransfer interrupt 	
  );
#endif
 frame->StatusLength = size;

// 5. Cheange ownership to BDMA ////////////////////////////////////////////////

 frame->DataPtr |= fOwnership_BDMA;

// 6. Enable MAC and BDMA Tx control register //////////////////////////////////

 if((BDMATXCON & BDMATXCON_TxEn) == 0)  
  {
   BDMATXCON |= BDMATXCON_TxEn;
  }
 MACTXCON |= MACTXCON_TxEn;

// 7. Change the Tx frame descriptor for next use //////////////////////////////
 gWTxFDPtr = (U32)(frame->NextDes); 	// next descriptor to write 	

 if((INTMASK & GLOBAL_INT) != 0)
  {					// global interrupt disabed
   INTMASK &= ~GLOBAL_INT;		// enable global interrupt
  } 
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// Transfer Control Frame Data to another Host /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// To Send a Pause control packet, you write the CAM0 entry with the 
// destination address, the CAM1 entry with the source address,
// and CAM18 entry with length/type,opcode, and operand. You then 
// set the pause bit in the MAC transmit control register.   

void ControlFrameTransfer(void) 
{
 U8 it;

// 
 gBdmaTxCon |= BDMATXCON_TxCPIE;	// BDMA Tx Complete to send control 
 BDMATXCON   = gBdmaTxCon;		// set BDMA Transmit Register  
 MACTXCON    = gMacTxCon;		// set MAC Transmit Register 

 while(1) 
  {
   Print("\nSelect Transmit(T) or Quit(Q) ? ");
   it = get_upper();
   if(it == 'Q')
    {
     break;
    }
   else
   if(it != 'T')
    { 
     continue;
    } 

// Step 1. set destination address to CAM #0
// Step 2. set source address to CAM #1
// CAM #0 : 0000f0110000 (Destin Addr)
// CAM #1 : 11f000000000 (Source Addr)
   VPint(CAM_BaseAddr) = 0x0000f011 ;	
   VPint(CAM_BaseAddr + 0x4) = 0x000011f0;	
   VPint(CAM_BaseAddr + 0x8) = 0x00000000;	

// Step 3. set length or type field, opcode, and operand to CAM #18
// CAM #18 : Opcode & Operand
// CAM #18 : Pause Count Value
   VPint(CAM_BaseAddr + 0x6c) = 0x88080001;	

// Step 4. set to zero preceed CAM #18
// CAM #19 - #20 : Filled with Zero
   VPint(CAM_BaseAddr + 0x70) = 0x12340000;	
   VPint(CAM_BaseAddr + 0x74) = 0x00000000;	
   VPint(CAM_BaseAddr + 0x78) = 0x00000000;	
   VPint(CAM_BaseAddr + 0x7c) = 0x00000000;	

// Step 5. Enable CAM location
// CAM Enable
   CAMEN = 0x1c0003; 

// Step 6. Enable transmit control frame by 
//         set SendPause bit in MACTXCON
   MACTXCON |= MACTXCON_SdPause | MACTXCON_TxEn;

// Step 7. Wait control frame finished
   while((BDMASTAT & BDMASTAT_BTxCCP) == 0)
    {
/* [16] 	BDMA Tx complete to send control packet (BTxCCP) 	      */
/*              Bit [16] is set each time the MAC sends a complete control    */
/*              packet. 						      */
/* 									      */
/* 		0 = Normal operation. 					      */
/* 		1 = MAC send the control packet. 			      */
     ;
    }
  } 

 gBdmaTxCon &= ~BDMATXCON_TxCPIE;	// BDMA Tx Complete to send control 
}

////////////////////////////////////////////////////////////////////////////////
// View MAC status, and other //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MacDebugStatus(void) 
{
 sFrameDes * pRxFDptr;
 sFrameDes * pTxFDptr;
 U32 * pFDPtr ;

 pRxFDptr =  (sFrameDes *) gCRxFDPtr;	// current Rx Frame descriptor pointer 
 pFDPtr = (U32 *)pRxFDptr;		

////////////////////////////////////////////////////////////////////////////////
// Print Receive Statistics ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 Print("\n\n\nMAC Rx Frame : Good ( %d ), Error ( %d ) ",
 
  gsBdmaRxStatus.BdmaRxGood	, 
  gRxErrorPacketCnt
 );

// pointer to current Rx frame descriptor ////////////////////////////////////// 
 Print("\n Current Frame Descriptor Pointer : %08x", pFDPtr ); 

// 
 Print("\nFrame Data Ptr          : %08x", *pFDPtr++); // pointer to data buffer 
 Print("\nReserved Field          : %08x", *pFDPtr++); // reserved field  
 Print("\nStatus and Frame Length : %08x", *pFDPtr++); // status and length field 
 Print("\n Next Frame Descriptor   : %08x", *pFDPtr  ); // pointer to next Rx frame descriptor 

// 
 Print("\nBDMARXPTR : %08x", BDMARXPTR); // Receive frame descriptor start address	
 Print("  BDMASTAT  : %08x", BDMASTAT ); // BDMA Status Register 
 Print("\nBDMARXCON : %08x", BDMARXCON); // BDMA Receive Control Register  
 Print("  BDMARXLSZ : %08x", BDMARXLSZ); // Receive frame maximum size
 Print("  BDMARXBUF : %08x", BDMARXBUF); // BDMA receive (Rx) buffer (64 words)
 Print("\nCAM Base  : %08x", CAM_BASE ); // CAM Base Address
 Print("  CAMCON    : %08x", CAMCON   ); // CAM Control Register 
 Print("\nMACCON    : %08x", MACCON   ); // MAC Control Register 
 Print("\nMACRXCON  : %08x", MACRXCON ); // MAC Receive Control Register 
 Print("  MACRXSTAT : %08x", MACRXSTAT); // MAC Receive Statistics

////////////////////////////////////////////////////////////////////////////////
// Print Transmit statistics ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 pTxFDptr =  (sFrameDes *) gCTxFDPtr ;
 pFDPtr = (U32 *)pTxFDptr ;

 Print("\n\nMAC Tx Frame : Good ( %d )",gsMacTxStatus.MacTxGood);

// Current Transmit Frame Descriptor Pointer ///////////////////////////////////
 Print("\nCurrent Frame Descriptor Pointer : %08x", pFDPtr); 
// Fields of current transmit frame ////////////////////////////////////////////
 Print("\nFrame Data Ptr          : %08x", *pFDPtr++); // data buffer pointer 
 Print("\nReserved Field          : %08x", *pFDPtr++); // reserved field
 Print("\nStatus and Frame Length : %08x", *pFDPtr++); // status & length field 
 Print("\nNext Frame Descriptor   : %08x", *pFDPtr);   // next frame descriptor pointer 	

// 
 Print("\nBDMATXPTR : %08x", BDMATXPTR ); 
 Print("  BDMATXCON : %08x", BDMATXCON ); // BDMA Transmit Control Register 
 Print("\nBDMATXBUF : %08x", BDMATXBUF ); 
 Print("  MACTXCON  : %08x", MACTXCON  ); // MAC Transmit Control Register 	
 Print("\nMACTXSTAT : %08x", MACTXSTAT ); // MAC Transmit Status Register 
 Print("\nEMISSCNT  : %08x", EMISSCNT  ); 
 Print("  EPZCNT    : %08x\n\n", EPZCNT); 
}

