/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      snds.h                     KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*      Board Specific Routines for NetMCU-I series.                     */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*      hbahn           09-15-1998      Created initial version 1.0      */
/*      in4maker        05-10-1999      Added CAM_Reg(x)                 */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modifyed by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002, 2003		*/
/*			*/

#ifndef _SNDS_H
#define _SNDS_H

#ifdef  _S3C4530_	
#define _KS32C50100_
#else
#ifndef _KS32C50100_ 
#define _KS32C50100_ 
#endif
#endif 								// _S3C4530_


/*------------------------------------------------------------------------
 *	  ASIC Address Definition
 *----------------------------------------------------------------------*/

#define VPint    *(volatile unsigned int   *)		
#define VPshort  *(volatile unsigned short *)		
#define VPchar   *(volatile unsigned char  *)		

#define  	Base_Addr	0x3ff0000		
								
/************************************************************************/
/* System Configuration Register 					*/
/************************************************************************/

#define SYSCFG		(VPint(Base_Addr+0x0000))	

/************************************************************************/
/* System Configuration Register (SYSCFG). 				*/
/*									*/
/* 	 Offset      = 0x0000						*/
/*       Reset Value = 0x07FFFF91 KS32C50100				*/
/* 		       0x4FFFFF91 S3c4530A				*/
/* 									*/
/*[0] 	 Stal enable (SE). Must be set to zero 				*/
/*[1] 	 Cache enable (CE). Then set to "1", cache operation are enabled*/
/*[2]    Write buffer enable (WE). Then set to '1", write buffer 	*/
/* 	 operation  are enabled 					*/
/*[3]    Fixed Priority (FP)						*/
/* 	  0 = Round-robin 						*/
/* 	  1 = Fixed priority within Group 				*/
/* 	 NOTE: S3C4530A 						*/
/*[5:4]  Cache mode (CM). This 2-bit value determines how the internal 	*/
/* 	 memory is to be divided into cache and SRAM 			*/
/* 	 00 = 4-Kbyte SRAM, 4-Kbyte cache 				*/
/* 	 01 = 0-Kbyte SRAM, 8-Kbyte cache 				*/
/* 	 10 = 8-Kbyte SRAM, 0-Kbyte cache 				*/
/* 									*/
/* NOTE: When you write 10, to this field, the cache enable bit is 	*/
/* 	 is cleared automatically. 					*/
/* 									*/
/*[15:6] Internal SRAM base pointer. This 10-bit address becoms the 	*/
/* 	 the upper address of SRAM. A25 through A16, the remaining 	*/
/*       SRAM adresses, and A15 through A0, are filled with zeros. 	*/    
/* 									*/
/*[25:16]Special register bank base pointer. The resolution of this 	*/
/*       value is 64K.							*/
/* 									*/
/*[29:26]Product Identifier (PD_ID)  					*/
/*       0000=KS32C5000							*/
/*       0001=KS32C50100						*/
/* 									*/
/* 									*/
/*[30:26]Product Identifier (PD_ID) 					*/
/* 	 00001 = S3C4510X (KS32C50100) 					*/
/* 	 11001 = S3C4510B 						*/
/* 	 00011 = S3C4530X (KS32C50300) 					*/
/* 	 10011 = S3C4530A 						*/
/*[31]   Sync. DRAM mode 						*/	 
/* 	 0 = Noemal/EDO DRAM interface for 4 DRAM banks 		*/
/*       1 = Sync. DRAM interface for 4 DRAM banks 			*/
/************************************************************************/

#define SYSCFG_STALL         	0x00000001 	/* (SE) Stall enable bit */
#define SYSCFG_CACHE         	0x00000002	/* (CE) Cache enable bit */
#define SYSCFG_WRITE_BUFF    	0x00000004	/* (WE) Write enable bit */

/*[3] Fixed Priority (FP) */
#ifdef _S3C4530_
#define SYSCFG_FP         	0x00000008	/* (FP) Fixed Priority */
#endif							/* _S3C4530_ 	*/

#define SYSCFG_CACHE_MODE    	0x00000030	/* (CM) Cache mode */
#define SYSCFG_CACHE_MODE_00 	0x00000000	/* 4Kb SRAM, 4Kb cache */
#define SYSCFG_CACHE_MODE_01 	0x00000010	/* 0Kb SRAM, 8Kb cache */
#define SYSCFG_CACHE_MODE_10 	0x00000020	/* 8Kb SRAM, 0Kb cache */	
#define SYSCFG_CACHE_MODE_11 	0x00000030	/* unused */
#define SYSCFG_CACHE_MASK       0x0000003f	/* Cache configuration mask */

/* alternative names for cache mode */

#define SYSCFG_CACHE_MODE_4KB_CACHE	SYSCFG_CACHE_MODE_00
#define SYSCFG_CACHE_MODE_8KB_CACHE	SYSCFG_CACHE_MODE_01
#define SYSCFG_CACHE_MODE_8KB_SRAM	SYSCFG_CACHE_MODE_10

#ifdef _S3C4530_ 
/*[30:26]Product Identifier (PD_ID) */
#define SYSCFG_PD_KS32C50100 0x04000000 /* 00001 = S3C4510X (KS32C50100) */
#define SYSCFG_PD_S3C4510B   0x64000000 /* 11001 = S3C4510B */
#define SYSCFG_PD_S3C4530X   0x0C000000 /* 00011 = S3C4530X (KS32C50300) */
#define SYSCFG_PD_S3C4530A   0x4C000000 /* 10011 = S3C4530A */
#define SYSCFG_PD_ID         0x7C000000 /* 11111 = PD_ID mask */
#else
#ifdef _KS32C50100_
/*[29:26]Product Identifier (PD_ID) */
#define SYSCFG_PD_KS32C50000 0x00000000 /* 0000 = (KS32C5000) */
#define SYSCFG_PD_KS32C50100 0x04000000 /* 0001 = (KS32C50100) */
#define SYSCFG_PD_ID         0x3C000000 /* 1111 = PD_ID mask */
#endif
#endif							/* _S3C4530_ */

#define CLKCON          (VPint(Base_Addr+0x3000))	

/************************************************************************/
/* CLKCON Clock control register. 					*/
/* 									*/
/* 	 Offset 	= 0x3000					*/
/* 	 Reset Value 	= 0x00000000 					*/
/* 									*/
/*[15:0] Clock Divided Value 						*/
/* 	 If all bits are 0, a non-divided clock is used. Only one bit 	*/ 
/*	 can be set in CLKCON[15:0]. That is, the clock dividing 	*/
/* 	 values is defined as 1,2,4,8,16 ... 				*/ 
/* 	 Internal system clock, fMCLK = fICLK / (CLKCON + 1) 		*/ 
/* 									*/
/*[16]	 ROM bank 5 wait enable 					*/
/* 	 0 = Disable ROM bank 5 wait 					*/
/* 	 1 = Enable  ROM bank 5 wait 					*/
/* 									*/
/*[17]	 ROM bank 5 address/data bus Mux enable 			*/
/* 	 0 = Normal operation 						*/
/* 	 1 = Enable bus multiplexing 					*/
/* 									*/
/*[19:18]Mux bus Address Cycle (tAC) 					*/
/* 	 00 = 1 MCLK 							*/
/*       01 = 2 MCLK 							*/
/* 	 10 = 3 MCLK 							*/
/* 									*/
/*[20]   ROM bank 5 wait 1 cycle delay (S3C4530) 			*/
/*       Wait cycle will check the second cycle after a chip selection  */
/*       signal is activated. 						*/
/* 									*/
/*[21,23,25,27,29] ROM bank 4,3,2,1,0 wait enable (S3C4530) 		*/
/*       Wait cycle will check the next cycle after the each chip 	*/
/*       selection signal is activated. 				*/
/*        [21] = ROM bank 4						*/
/*        [23] = ROM bank 3						*/
/*        [25] = ROM bank 2 						*/
/*        [27] = ROM bank 1 						*/
/*        [29] = ROM bank 0 					        */
/* 									*/
/* [22,24,26,28,30] ROM bank 4,3,2,1,0 wait 1 cycle delay (S3C4530) 	*/
/*       Wait cycle will check the second cycle after the each chip     */
/*       selection signal is activated. 				*/
/*        [22] = ROM bank 4						*/
/*        [24] = ROM bank 3						*/
/*        [26] = ROM bank 2 						*/
/*        [28] = ROM bank 1						*/
/*        [30] = ROM bank 0 						*/
/* 									*/
/*[31]   Test bit 							*/
/* 	 This bit should be always 0.					*/
/* 									*/
/************************************************************************/

#define CLKCON_DIV_MASK        0x0000FFFF /* Clock Divided Value */
#define CLKCON_ROM5_WAIT_EN    0x00010000 /* ROM bank 5 wait enable */
#define CLKCON_ROM5_MUX_EN     0x00020000 /* ROM bank 5 address/data bus Mux enable */
// 
#define CLKCON_ROM5_MUX_TAC    0x000C0000 /* Mux bus Address Cycle (tAC) */
#define CLKCON_ROM5_MUX_TAC_1  0x00000000 /* 00 = 1 MCLK */	
#define CLKCON_ROM5_MUX_TAC_2  0x00040000 /* 01 = 2 MCLK */	
#define CLKCON_ROM5_MUX_TAC_3  0x00080000 /* 10 = 3 MCLK */	
#define CLKCON_ROM5_MUX_TAC_UN 0x000C0000 /* 11 = unused */

#ifdef _S3C4530_
#define CLKCON_ROM5_WAIT_DELAY 0x00100000 /*[20] ROM bank 5 wait 1 cycle delay (S3C4530)*/

#define CLKCON_ROM4_WAIT_EN    0x00200000 /*[21] ROM bank 4 wait enable*/
#define CLKCON_ROM4_WAIT_DELAY 0x00400000 /*[22] ROM bank 4 wait cycle delay */

#define CLKCON_ROM3_WAIT_EN    0x00800000 /*[23] ROM bank 3 wait enable*/
#define CLKCON_ROM3_WAIT_DELAY 0x01000000 /*[24] ROM bank 3 wait cycle delay */

#define CLKCON_ROM2_WAIT_EN    0x02000000 /*[25] ROM bank 2 wait enable*/
#define CLKCON_ROM2_WAIT_DELAY 0x04000000 /*[26] ROM bank 2 wait cycle delay */

#define CLKCON_ROM1_WAIT_EN    0x08000000 /*[27] ROM bank 1 wait enable*/
#define CLKCON_ROM1_WAIT_DELAY 0x10000000 /*[28] ROM bank 1 wait cycle delay */

#define CLKCON_ROM0_WAIT_EN    0x20000000 /*[29] ROM bank 0 wait enable*/
#define CLKCON_ROM0_WAIT_DELAY 0x40000000 /*[30] ROM bank 0 wait cycle delay */
#endif								/* _S3C4530_ */

// 
#define CLKCON_TEST_BIT        0x80000000 /* Test bit */


/* External I/O Access Control Registers */  

#define EXTACON0 	(VPint(Base_Addr+0x3008))	/* External I/O timing register 1 */
#define EXTACON1 	(VPint(Base_Addr+0x300c))	/* External I/O timing register 2 */

/************************************************************************/
/* Chip selection set-up time on nOE (tCOS0, tCOS1, tCOS2, tCOS3)	*/
/* 									*/
/* [0:2]   EXTACON0 tCOS0 						*/
/* [0:2]   EXTACON1 tCOS2 						*/
/* [18:16] EXTACON0 tCOS1 						*/
/* [18:16] EXTACON1 tCOS3 						*/
/* 									*/
/* Address set-up time before nECS (tACS0, tACS1, tACS2, tACS3)		*/ 
/* 									*/
/* [5:3]   EXTACON0 tACS0 						*/
/* [5:3]   EXTACON1 tACS2 						*/
/* [21:19] EXTACON0 tACS1 						*/
/* [21:19] EXTACON1 tACS3 						*/
/* 									*/
/* Chip selection hold time on nOE (tCOH0,tCOH1,tCOH2,tCOH3)		*/
/* 									*/
/* [8:6]   EXTACON0 tCOH0 						*/
/* [8:6]   EXTACON1 tCOH2 						*/
/* [24:22] EXTACON0 tCOH1 						*/
/* [24:22] EXTACON1 tCOH2 						*/
/* 									*/
/* Access cycles (nOE low time (tACC0,tACC1,tACC2,tACC3) 		*/
/* 									*/
/* [11:9]  EXTACON0 tACC0 						*/
/* [11:9]  EXTACON1 tACC2 						*/
/* [27:25] EXTACON0 tACC1 						*/
/* [27:25] EXTACON1 tACC2 						*/
/* 									*/
/* 	   000 = 0 cycle 						*/ 
/* 	   001 = 1 cycle 						*/ 
/* 	   010 = 2 cycle 						*/ 
/* 	   011 = 3 cycles 						*/ 
/* 	   100 = 4 cycles 						*/
/* 	   101 = 5 cycles 						*/
/* 	   110 = 6 cycles 						*/
/* 	   111 = 7 cycles 						*/
/************************************************************************/

/* Chip selection set-up time on nOE (tCOS0, tCOS1, tCOS2, tCOS3) */
#define EXTACON0_TCOS0_OFFSET  0 /* Bank 0 Chip selection set-up time on nOE */
#define EXTACON0_TCOS1_OFFSET 16 /* Bank 1 Chip selection set-up time on nOE */
#define EXTACON1_TCOS2_OFFSET  0 /* Bank 2 Chip selection set-up time on nOE */
#define EXTACON1_TCOS3_OFFSET 16 /* Bank 3 Chip selection set-up time on nOE */

/* Address set-up time before nECS (tACS0, tACS1, tACS2, tACS3)	*/ 
#define EXTACON0_TACS0_OFFSET  3 /* Bank 0 Address set-up time before nECS */
#define EXTACON0_TACS1_OFFSET 19 /* Bank 1 Address set-up time before nECS */
#define EXTACON1_TACS2_OFFSET  3 /* Bank 2 Address set-up time before nECS */
#define EXTACON1_TACS3_OFFSET 19 /* Bank 3 Address set-up time before nECS */

/* Chip selection hold time on nOE (tCOH0,tCOH1,tCOH2,tCOH3) */
#define EXTACON0_TCOH0_OFFSET  6 /* Bank 0 Chip selection hold time on nOE */
#define EXTACON0_TCOH1_OFFSET 22 /* Bank 1 Chip selection hold time on nOE */ 
#define EXTACON1_TCOH2_OFFSET  6 /* Bank 2 Chip selection hold time on nOE */
#define EXTACON1_TCOH3_OFFSET 22 /* Bank 3 Chip selection hold time on nOE */

/* Access cycles (nOE low time (tACC0,tACC1,tACC2,tACC3) */
#define EXTACON0_TACC0_OFFSET  9 /* Bank 0 Access cycles (nOE Low time) */
#define EXTACON0_TACC1_OFFSET 25 /* Bank 1 Access cycles (nOE Low time) */
#define EXTACON1_TACC2_OFFSET  9 /* Bank 2 Access cycles (nOE Low time) */
#define EXTACON1_TACC3_OFFSET 25 /* Bank 3 Access cycles (nOE Low time) */


#define EXTDBWTH	(VPint(Base_Addr+0x3010))

/************************************************************************/
/* EXTDBWTH Data bus width resgister. 					*/
/* 	  Offset      = 0x3010 						*/
/*        Reset value = 0x00000000      				*/ 
/* 									*/
/*[1 : 0] Data bus width for ROM/SRAM/FLASH bank0 (DSR0) 		*/
/* 	  DSR0 is read-only data at the B0SIZE[1:0] pins. DSR0 is 	*/
/* 	  read-only because ROM/SRAM/FLASH bank is used to boot the 	*/  
/* 	  while the data bus width for ROM/SRAM/FLASH bank0 is set 	*/
/* 	  used B0SIZE[1:0] 						*/  
/* 	  								*/
/* 	  00 = Not permitted 						*/
/* 	  01 = Byte (8 bits) 						*/
/*        10 = Half-word (16 bits)					*/ 
/*        11 = Word (32 bits)						*/ 
/* 									*/
/*[ 3: 2] Data bus width for ROM/SRAM/FLASH bank 1 (DSR1)		*/   
/*[ 5: 4] Data bus width for ROM/SRAM/FLASH bank 2 (DSR2)		*/   
/*[ 7: 6] Data bus width for ROM/SRAM/FLASH bank 3 (DSR3)		*/
/*[ 9: 8] Data bus width for ROM/SRAM/FLASH bank 4 (DSR4)		*/   
/*[11:10] Data bus width for ROM/SRAM/FLASH bank 5 (DSR5)		*/   
/* 									*/
/* 	  00 = Disabled 						*/
/* 	  01 = Byte (8 bits) 						*/
/*        10 = Half-word (16 bits)					*/ 
/*        11 = Word (32 bits)						*/ 
/* 									*/
/*[13:12] Data bus width for DRAM bank 0 (DSD0)				*/  
/*[15:14] Data bus width for DRAM bank 1 (DSD1)				*/  
/*[17:16] Data bus width for DRAM bank 2 (DSD2)				*/  
/*[19:18] Data bus width for DRAM bank 3 (DSD3)				*/  
/* 									*/
/* 	  00 = Disabled 						*/
/* 	  01 = Byte (8 bits) 						*/
/*        10 = Half-word (16 bits)					*/ 
/*        11 = Word (32 bits)						*/ 
/* 									*/
/*[21:20] Data bus width for external I/O bank 0 (DSX0)			*/
/*[23:22] Data bus width for external I/O bank 1 (DSX1)			*/
/*[25:24] Data bus width for external I/O bank 2 (DSX2)			*/
/*[27:26] Data bus width for external I/O bank 3 (DSX3)			*/  
/* 									*/
/* 	  00 = Disabled 						*/
/* 	  01 = Byte (8 bits) 						*/
/*        10 = Half-word (16 bits)					*/ 
/*        11 = Word (32 bits)						*/ 
/* 									*/
/************************************************************************/

#define EXTDBWTH_DSR0     0x00000003 /* Data bus width for ROM/SRAM/FLASH bank 0 */
#define EXTDBWTH_DSR1     0x0000000C /* Data bus width for ROM/SRAM/FLASH bank 1 */
#define EXTDBWTH_DSR2     0x00000030 /* Data bus width for ROM/SRAM/FLASH bank 2 */
#define EXTDBWTH_DSR3     0x000000C0 /* Data bus width for ROM/SRAM/FLASH bank 3 */
#define EXTDBWTH_DSR4     0x00000300 /* Data bus width for ROM/SRAM/FLASH bank 4 */
#define EXTDBWTH_DSR5     0x00000C00 /* Data bus width for ROM/SRAM/FLASH bank 5 */

#define EXTDBWTH_DSD0     0x00003000 /* Data bus width for DRAM bank 0 */
#define EXTDBWTH_DSD1     0x0000C000 /* Data bus width for DRAM bank 1 */
#define EXTDBWTH_DSD2     0x00030000 /* Data bus width for DRAM bank 2 */
#define EXTDBWTH_DSD3     0x000C0000 /* Data bus width for DRAM bank 3 */


#define EXTDBWTH_DSX0     0x00300000 /* Data bus width for external I/O bank 0 */
#define EXTDBWTH_DSX1     0x00C00000 /* Data bus width for external I/O bank 1 */
#define EXTDBWTH_DSX2     0x03000000 /* Data bus width for external I/O bank 2 */
#define EXTDBWTH_DSX3     0x0C000000 /* Data bus width for external I/O bank 3 */

#define EXTDBWTH_DSR0_OFF          0 /* */
#define EXTDBWTH_DSR1_OFF          2 /* */
#define EXTDBWTH_DSR2_OFF          4 /* */
#define EXTDBWTH_DSR3_OFF          6 /* */
#define EXTDBWTH_DSR4_OFF          8 /* */
#define EXTDBWTH_DSR5_OFF         10 /* */

#define EXTDBWTH_DSD0_OFF         12 /* */
#define EXTDBWTH_DSD1_OFF         14 /* */
#define EXTDBWTH_DSD2_OFF         16 /* */
#define EXTDBWTH_DSD3_OFF         18 /* */

#define EXTDBWTH_DSX0_OFF         20 /* */
#define EXTDBWTH_DSX1_OFF         22 /* */
#define EXTDBWTH_DSX2_OFF         24 /* */
#define EXTDBWTH_DSX3_OFF         26 /* */

//
/************************************************************************/
/* ROMCON0,ROMCON1,ROMCON2,ROMCON3,ROMCON4,ROMCON5			*/   
/* 									*/	
/* [1:0]   Page mode configuration (PMC) 				*/
/* 	    00 = Normal ROM 						*/
/*          01 = 4-word page 						*/
/*          10 = 8-word page 						*/
/*          11 = 16-word page 						*/
/* 									*/	
/* [3:2]   Page address access time (tPA) 			        */
/*          00 = 5 cycles 						*/
/*          01 = 2 cycles 						*/
/*          10 = 3 cycles 						*/
/*          11 = 4 cycles 						*/
/* 									*/	
/* [6:4]   Programmable access cycle (tACC)  				*/
/*          000 = Disable bank 						*/
/*          001 = 2 cycles 						*/
/*          010 = 3 cycles 						*/
/*          011 = 4 cycles 						*/
/*          100 = 5 cycles 						*/
/*          101 = 6 cycles 						*/
/*          110 = 7 cycles 						*/
/*          111 = Reserved 						*/
/* 									*/	
/* [19:10]  ROM/SRAM/Flash bank # base pointer				*/
/*	    This value is the start address of the ROM/SRAM/Flash bank. */
/*          The start address is calculated as ROM/SRAM/FLASH bank base */
/*          pointer << 16 						*/
/* 									*/	
/* [29:20]  ROM/SRAM/FLASH bank # next pointer 			        */
/*          This value is the current bank end address << 16 + 1 	*/
/* 									*/	
/************************************************************************/

#define ROMCON0		(VPint(Base_Addr+0x3014)) /* ROM/SRAM/Flash bank 0 control register */
#define ROMCON1		(VPint(Base_Addr+0x3018)) /* ROM/SRAM/Flash bank 1 control register */
#define ROMCON2		(VPint(Base_Addr+0x301c)) /* ROM/SRAM/Flash bank 2 control register */
#define ROMCON3		(VPint(Base_Addr+0x3020)) /* ROM/SRAM/Flash bank 3 control register */
#define ROMCON4		(VPint(Base_Addr+0x3024)) /* ROM/SRAM/Flash bank 4 control register */
#define ROMCON5		(VPint(Base_Addr+0x3028)) /* ROM/SRAM/Flash bank 5 control register */


/* Page mode configuration (PMC) */

#define ROMCON_PMC          0x00000003	/* Page mode mask */
#define ROMCON_PMC_NORMAL   0x00000000	/* 00 = Normal ROM */
#define ROMCON_PMC_4WPAGE   0x00000001	/* 01 = 4-word page */
#define ROMCON_PMC_8WPAGE   0x00000002	/* 10 = 8-word page */
#define ROMCON_PMC_16WPAGE  0x00000003	/* 11 = 16-word page */

/* [3:2] Page address access time (tPA) */

#define ROMCON_TPA          0x0000000C	/* page address access time mask */
#define ROMCON_TPA_5CLK     0x00000000	/* 00 = 5 cycles */
#define ROMCON_TPA_2CLK     0x00000004	/* 01 = 2 cycles */
#define ROMCON_TPA_3CLK     0x00000008	/* 10 = 3 cycles */
#define ROMCON_TPA_4CLK     0x0000000C	/* 11 = 4 cycles */

/* [6:4]   Programmable access cycle (tACC) */

#define ROMCON_TACC         0x00000070	/* access cycle mask */

#define ROMCON_TACC_DISABLE 0x00000000	/* 000 = Disable bank */
#define ROMCON_TACC_2CLK    0x00000010	/* 001 = 2 cycles */
#define ROMCON_TACC_3CLK    0x00000020	/* 010 = 3 cycles */
#define ROMCON_TACC_4CLK    0x00000030	/* 011 = 4 cycles */	
#define ROMCON_TACC_5CLK    0x00000040	/* 100 = 5 cycles */
#define ROMCON_TACC_6CLK    0x00000050	/* 101 = 6 cycles */
#define ROMCON_TACC_7CLK    0x00000060	/* 110 = 7 cycles */
#define ROMCON_TACC_UNUSED  0x00000070	/* 111 = Reserved */


#define ROMCON_BASE_MASK    0x000FFC00  /* 19:10  base pointer mask */
#define ROMCON_BASE_OFFSET          10  /* base pointer offset */

#define ROMCON_NEXT_MASK    0x3FF00000  /* 29:20  base pointer mask */
#define ROMCON_NEXT_OFFSET          20  /* base pointer offset */

//

/************************************************************************/
/* DRAMCON0 DRAM bank 0 control register 				*/
/*          Offset      = 0x302C 					*/
/*          Reset Value = 0x00000000 					*/
/* 									*/
/* DRAMCON1 DRAM bank 1 control register 	 			*/
/*          Offset      = 0x3030 					*/
/*          Reset Value = 0x00000000					*/
/* 									*/
/* DRAMCON2 DRAM bank 2 control register 				*/
/*          Offset      = 0x3034 					*/
/*          Reset Value = 0x00000000 					*/
/* 									*/
/* DRAMCON3 DRAM bank 3 control register				*/
/*          Reset Value = 0x00000000 					*/
/* 									*/
/* [0]      EDO mode(EDO) (note)					*/
/*          0 = Normal DRAM (Fast page mode DRAM) 			*/
/*          1 = EDO DRAM						*/
/* 									*/
/* [2:1]    CAS strobe time (tCS) 					*/
/*          00 = 1 cycle 						*/
/*          01 = 2 cycles 						*/
/*          10 = 3 cycles 						*/
/*          11 = 4 cycles 						*/
/* 									*/
/* [3:3]    CAS pre-charge time (tCP ) (note) 				*/
/*          0 = 1 cycle 						*/
/*          1 = 2 cycles 						*/
/* 									*/
/* NOTE     In SDRAM mode, this bit affect SDRAM cycle. 		*/
/*          tCS value [1] : 0 = 1 cycle, 1 = 2 cycle 			*/
/* 									*/
/* [6:4]    Reserved 							*/
/*          These bits default value is 000. But, you must set to 001.  */
/* 									*/
/* [7]      RAS to CAS delay(tRC or tRCD)  				*/
/*           0 = 1 cycle 						*/
/*           1 = 2 cycles 						*/
/* 									*/
/* [9:8]    RAS pre-charge time (tRP)  					*/
/*           00 = 1 cycle 						*/
/*           01 = 2 cycles 						*/
/*           10 = 3 cycles 						*/
/*           11 = 4 cycles						*/
/* 									*/
/* [19:10]  DRAM bank # base pointer 					*/
/*          This value indicates the start address of DRAM bank #. 	*/
/*          The start address is calculated as RAM bank # 		*/
/*          base pointer << 16  					*/
/* 									*/
/* [29:20]  DRAM bank # Next pointer 					*/
/*          This value isthe current bank end address << 16 + 1         */
/* 									*/
/* [31:30]  Number of column address bits in DRAM bank # (CAN)          */
/*           00 =  8 bits 						*/
/*           01 =  9 bits 					        */
/*           10 = 10 bits 						*/
/*           11 = 11 bits 						*/
/************************************************************************/

#define DRAMCON0 (VPint(Base_Addr+0x302c)) /* DRAM bank 0 control register */
#define DRAMCON1 (VPint(Base_Addr+0x3030)) /* DRAM bank 1 control register */
#define DRAMCON2 (VPint(Base_Addr+0x3034)) /* DRAM bank 2 control register */
#define DRAMCON3 (VPint(Base_Addr+0x3038)) /* DRAM bank 3 control register */


#define DRAMCON_NORMAL     0x00000000	/* Normal DRAM */
#define DRAMCON_EDO        0x00000001	/* 1 = EDO DRAM	*/

/* [2:1] CAS strobe time (tCS) */
#define DRAMCON_TCS        0x00000006	/* CAS strobe time mask */
#define DRAMCON_TCS_1CLK   0x00000000	/* 00 = 1 cycle */
#define DRAMCON_TCS_2CLK   0x00000002	/* 01 = 2 cycles */
#define DRAMCON_TCS_3CLK   0x00000004	/* 10 = 3 cycles */
#define DRAMCON_TCS_4CLK   0x00000006	/* 11 = 4 cycles */

/* [3:3] CAS precharge time (tCP) */
#define DRAMCON_TCP        0x00000008	/* CAS precharge time mask */
#define DRAMCON_TCP_1CLK   0x00000000	/* 0 = 1 cycle */
#define DRAMCON_TCP_2CLK   0x00000008	/* 1 = 2 cycle */	

/* [7] RAS to CAS delay(tRC or tRCD) */
#define DRAMCON_TRC        0x00000080	/* RAS to CAS delay mask */
#define DRAMCON_TRC_1CLK   0x00000000	/* 0 = 1 cycle */
#define DRAMCON_TRC_2CLK   0x00000080	/* 1 = 2 cycles */

/* [9:8] RAS pre-charge time (tRP) */
#define DRAMCON_TRP        0x00000300	/* RAS precharge mask */
#define DRAMCON_TRP_1CLK   0x00000000	/* 00 = 1 cycle */
#define DRAMCON_TRP_2CLK   0x00000100	/* 01 = 2 cycles */
#define DRAMCON_TRP_3CLK   0x00000200	/* 10 = 3 cycles */
#define DRAMCON_TRP_4CLK   0x00000300	/* 11 = 4 cycles */

/* [31:30]  Number of column address bits in DRAM bank # (CAN) */
#define DRAMCON_CAN        0xC0000000	/* CAN mask */
#define DRAMCON_CAN_8BITS  0x00000000	/* 00 =  8 bits */
#define DRAMCON_CAN_9BITS  0x40000000	/* 01 =  9 bits */
#define DRAMCON_CAN_10BITS 0x80000000	/* 10 = 10 bits */
#define DRAMCON_CAN_11BITS 0xC0000000	/* 11 = 11 bits */


/************************************************************************/
/* REFEXTCON Refresh and external I/O control register 			*/
/*           Offset      = 0x303C 					*/
/*           Reset Value = 0x00000000 					*/
/* 									*/
/* [9:0]     External I/O bank 0 base pointer (base address)		*/
/*           This value is the start address of external I/O bank 0. 	*/
/*           Start address is defined as External I/O bank 0 base 	*/
/*           pointer << 16. The end address of external I/O bank 0 is 	*/
/*           defined as External I/O bank 0 base pointer >> 16 + 16 K   */
/* 	     bytes-1.						        */
/* 									*/
/* NOTE:     All external I/O banks are located in the continuous 	*/
/* 	     address space which begins at the start address of         */
/*           external I/O bank 0. The size of each external I/O bank    */
/*           is fixed at 16 K bytes. The start and end addresses of the */
/*           other three external I/O banks can be derived from the     */
/*           external I/O bank 0 base pointer value.  			*/ 
/* 									*/
/* [15]      Validity of special register field(VSF) 			*/
/*            0 = Not accessible to memory bank 			*/
/*            1 = Accessible to memory bank 				*/
/* 									*/
/* [16]      Refresh enable (REN) 					*/
/*            0 = Disable DRAM refresh 					*/
/*            1 = Enable DRAM refresh					*/
/* 									*/
/* [19:17]   CAS hold time(tCHR) 					*/
/*           ROW Cycle Time (tRC) (note1)				*/
/*            000 = 1 cycle 						*/
/*            001 = 2 cycles 						*/
/*            010 = 3 cycles 						*/
/*            011 = 4 cycles 						*/
/*            100 = 5 cycles 						*/
/*            101 = Not used (6 cycles)					*/
/*            110 = Not used 						*/
/*            111 = Not used 						*/
/* 									*/
/* NOTE :    In EDO/normal DRAM mode, CAS hold time can be programmed   */ 
/*           upto 5 cycles. But in SDRAM mode, this bit fields function */
/*           are defined as ROW Cycle Time (tRC) and can be programmed  */
/*           upto 6 cycles.						*/
/* 									*/
/* [20]      CAS setup time(t CSR) (note2) 				*/
/*            0 = 1 cycle 					        */
/*            1 = 2 cycles 						*/
/* 									*/
/* NOTE :    In SDRAM mode, this bit field is reserved. 		*/ 
/* 									*/
/* [31:21]   Refresh count value (duration) 				*/
/*           The refresh period is calculated as 			*/
/*           (2**11 - Value + 1) / fMCLK   				*/ 
/************************************************************************/

#define REFEXTCON	(VPint(Base_Addr+0x303c)) 

#define REFEXTCON_VSF      0x00008000   /* [15] Validity of special register field(VSF) */
#define REFEXTCON_REN      0x00010000   /* [16] Refresh enable (REN) */

/* [19:17] CAS hold time(tCHR) */
#define REFEXTCON_TCHR     0x000E0000   /* CAS hold time */
#define REFEXTCON_TRC      0x000E0000   /* ROW Cycle Time */

#define REFEXTCON_TRC_1CLK 0x00000000  	/* 000 = 1 cycle */
#define REFEXTCON_TRC_2CLK 0x00020000   /* 001 = 2 cycles */
#define REFEXTCON_TRC_3CLK 0x00040000   /* 010 = 3 cycles */
#define REFEXTCON_TRC_4CLK 0x00060000   /* 011 = 4 cycles */
#define REFEXTCON_TRC_5CLK 0x00080000   /* 100 = 5 cycles */
#define REFEXTCON_TRC_6CLK 0x000A0000   /* 101 = Not used (6 cycles) */

/* [20] CAS setup time(t CSR) (note2) */
#define REFEXTCON_TCSR      0x00100000	/* */
#define REFEXTCON_TCSR_1CLK 0x00000000	/* 0 = 1 cycle */
#define REFEXTCON_TCSR_2CLK 0x00100000	/* 1 = 2 cycles */


/************************************************************************/
/* Ethernet BDMA Register  						*/
/************************************************************************/

#define BDMATXCON	(VPint(Base_Addr+0x9000)) /* Buffered DMA transmit control register */

/************************************************************************/
/* BDMATXCON 	Buffered DMA transmit control register 			*/
/* 		Offset Address = 0x9000, Reset Value = 0x00000000	*/
/* 									*/
/* [4:0] 	BDMA Tx burst size (BTxBRST)				*/
/*              (Word size + 1) of data bursts requested in BDMA mode.  */
/*              If BTxBRST is zero, the burst size is one word. If	*/
/*		BtXBRST is 31, the burst size is 32 words. 		*/
/* 									*/
/* [5] 		BDMA Tx stop/skip frame by owner bit (BTxSTSKO) 	*/
/* 		0 = Skips the current frame and goes to the next frame 	*/
/* 		    descriptor (if BDMA is not the owner of the frame) 	*/
/* 		1 = BDMA transmitter generates an interrupt (if enabled)*/
/* 									*/
/* [6]          Reserved Not applicable. 				*/
/* 									*/
/* [7] 		BDMA Tx complete to send control packet interrupt enable*/
/*              (BTxCCPIE). 						*/
/* 		0 = Disable complete to send control packet interrupt.	*/
/*              1 = Enable complete to send control packet interrupt. 	*/
/*              (The interrupt is generated when the MAC completes 	*/
/* 		sending the control packet.) 				*/
/* 									*/
/* [8] 		BDMA Tx Null list interrupt enable (BTxNLIE)		*/	
/* 		0 = Disable transmit Null list interrupt. 		*/
/* 		1 = Enable Null list interrupt to indicate that 	*/
/* 		BDMATxPTR in the BDMA Tx unit has a Null address 	*/
/* 		(0x00000000). 	      					*/
/* 									*/
/* [9] 		BDMA Tx not owner interrupt enable (BTxNOIE) 		*/
/* 		0 = Disable BDMA Tx not owner interrupt for the current */
/* 		    frame.						*/
/* 	        1 = Enable BDMA Tx not owner interrupt for the current 	*/
/* 		    frame (and if the BTxSTSKO bit is set). 		*/
/* 									*/
/* [10] 	BDMA Tx buffer empty interrupt enable (BTxEmpty) 	*/
/* 		0 = Disable TX buffer empty interrupt.			*/
/*              1 = Enable TX buffer empty interrupt. 			*/
/* 									*/
/* [13:11]      BDMA transmit to MAC Tx start level (BTxMSL) 		*/
/* 		These bits determine when the new frame data in BDMA Tx */
/* 		buffer can be moved to the MAC Tx FIFO when a new frame */
/* 		arrives. 						*/
/*       	"000" - no wait 					*/
/*              "001" - wait to fill 1/8 of the BDMA Tx buffer		*/
/*              "111" - which means wait to fill 7/8 of the BDMA Tx 	*/
/*                      buffer.   					*/
/* 									*/
/* [14] 	BDMA Tx enable (BTxEn) 					*/
/*		0 = Disable the BDMA transmitter.			*/
/*              1 = Enable the BDMA transmitter. 			*/
/* 									*/
/* [15] 	BDMA Tx reset (BTxRS) 					*/
/*     		0 = No effect. 						*/
/* 		1 = Reset the BDMA Tx block. 				*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// Buffered DMA	Trasmit	Control Register(BDMATXCON) ////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define	BDMATXCON_TxBRST	0x0001F	// BDMA Tx Burst Size Mask = 16 
#define	BDMATXCON_TxSTSKO	0x00020	// BDMA Tx Stop (1)/Skip (0) Frame or Interrupt in case
					// of not Owner the current frame 

#define	BDMATXCON_TxCPIE	0x00080	// BDMA Tx Complete to send control 
#define	BDMATXCON_TxCCPIE       0x00080	// packet interrupt Enable

#define BDMATXCON_TxNLIE        0x00100 // BDMA Tx Null list interrupt enable
#define	BDMATXCON_TxNOIE        0x00200	// BDMA Tx Buffer Not Owner
#define	BDMATXCON_TxEmpty	0x00400	// BDMA Tx Buffer Empty Interrupt 
// BDMA Tx buffer can be moved to the MAC Tx IO when the new frame comes in. 
// [13:11] BDMA transmit to MAC Tx start level (BTxMSL)
#define	BDMATXCON_TxMSL000	0x00000	// No wait to fill the BDMA 
#define	BDMATXCON_TxMSL001	0x00800	// wait to fill 1/8 of the BDMA 
#define	BDMATXCON_TxMSL010	0x01000	// wait to fill 2/8 of the BDMA
#define	BDMATXCON_TxMSL011	0x01800	// wait to fill 3/8 of the BDMA
#define	BDMATXCON_TxMSL100	0x02000	// wait to fill 4/8 of the BDMA
#define	BDMATXCON_TxMSL101	0x02800	// wait to fill 5/8 of the BDMA
#define	BDMATXCON_TxMSL110	0x03000	// wait to fill 6/8 of the BDMA
#define	BDMATXCON_TxMSL111	0x03800	// wait to fill 7/8 of the BDMA

#define	BDMATXCON_TxEn		0x04000	// BDMA Tx Enable 
#define	BDMATXCON_TxRS		0x08000	// BDMA Rx Reset 


#define BDMARXCON	(VPint(Base_Addr+0x9004)) /* Buffered DMA receive control register */

/************************************************************************/
/* BDMARXCON 	Buffered DMA receive control register 			*/
/* 		Offset Address = 0x9004					*/
/*              Reset value = 0x00000000 				*/
/* 									*/
/* [4:0] 	BDMA Rx burst size (BRxBRST) 				*/
/* 		Burst data size = (BRxBRST + 1) word 			*/
/* 									*/
/* [5] 		BDMA Rx stop/skip frame by owner bit (BRxSTSKO)		*/
/* 		This bit determines whether the BDMA Rx controller 	*/
/*              issues an interrupt, if enabled, or skips the current 	*/
/*              frame and goes to the next frame descriptor 		*/
/*              (assuming BDMA is not the owner).			*/
/* 		0 = Skips the current frame and goes to the next frame 	*/
/*                  descriptor. 					*/
/*              1 = BDMA receiver generates an interrupt (if enabled). 	*/
/* 									*/
/* [6] 		BDMA Rx memory address inc/dec (BRxMAINC) 		*/
/* 		0 = Decrement the frame memory address. 		*/
/* 		1 = Increment the frame memory address. 		*/
/* 									*/
/* [7] 		BDMA Rx every received frame interrupt enable (BRxDIE) 	*/
/*              This bit enables the BDMA Rx every received frame 	*/
/* 		interrupt which is generated by the BDMA controller 	*/
/*              each time is moves complete data frame into memory. 	*/
/* 									*/
/* 		0 = Disable frame receive done interrupt. 		*/
/*              1 = Enable frame receive done interrupt. 		*/
/* 									*/
/* [8] 		BDMA Rx Null list interrupt enable(BRxNLIE) 		*/
/* 		This bit enables the BDMA Rx Null list interrupt which 	*/
/*              indicates that the receive frame descriptor start 	*/
/*              address pointer, BDMARXPTR, in the BDMA Rx block has 	*/
/*  		a Null (0x00000000) address. 				*/
/* 									*/
/* 		0 = Disable Null address (0x00000000) receive interrupt.*/
/* 		1 = Enable Null address (0x00000000) receive interrupt. */
/* 									*/
/* [9] 		BDMA Rx not owner interrupt enable (BRxNOIE) 		*/
/* 		This bit enables the BDMA Rx not owner interrupt when 	*/
/* 		the ownership bit of the current frame does not belong 	*/
/*              to the BDMA controller, and if the BRxSTSKO bit is set.	*/
/* 									*/
/* 		0 = Disable interrupt for BDMA Rx not owner of the 	*/
/* 		    current frame. 					*/
/*              1 = Enable interrupt for BDMA Rx not owner of the 	*/
/*                  current frame. 					*/
/* 									*/
/* [10] 	BDMA Rx maximum size over interrupt enable (BRxMSOIE) 	*/
/* 		This bit enables the BDMA Rx maximum size over 		*/
/*              interrupt when the received frame size is larger than 	*/
/* 		the value in receive frame maximum size register. 	*/
/* 									*/
/* 		0 = Disable interrupt for received frame if larger than */
/*                  the maximum frame size. 				*/
/*              1 = Enable interrupt for received frame if larger than 	*/
/* 		    the maximum frame size. 				*/
/* 									*/
/* [11] 	BDMA Rx Big/Little Endian (BRxLittle) 			*/
/* 		This bit determines whether the data is stored in 	*/
/* 		Little- or Big-Endian format. If it is set to "1", 	*/
/* 		word swapping will take place between the receive 	*/
/* 		buffer and the system data bus.				*/
/* 									*/
/*   		0 = Big-Endian frame data format.			*/
/*              1 = Little-Endian. (Frame data in BDMA Rx buffer is 	*/
/* 		    word-swapped on the system bus.) 			*/
/* 									*/
/* [13:12] 	BDMA Rx word alignment (BRxWA) 				*/
/* 		The Rx word alignment bits determine how many bytes are */
/* 		invalid in the first word of each data frame. These 	*/
/* 		invalid bytes are inserted when the word is assembled 	*/
/* 		by the BDMA controller. 				*/
/*              "00" = No invalid bytes 				*/
/*              "01" = 1 invalid byte					*/
/* 		"10" = 2 invalid bytes					*/
/*              "11" = 3 invalid bytes 					*/
/* 									*/
/* [14] 	BDMA Rx enable (BRxEn) 					*/
/* 		When the Rx enable bit is set to "1", the BDMA Rx block */
/* 		is enabled. Even if this bit is disabled,the MAC will 	*/
/* 		receive Rx data until the MAC Rx FIFO overflows (as 	*/
/* 		long as the FIFO is not empty and the MAC Rx is 	*/
/* 		enabled). This bit is automatically disabled in the 	*/
/* 		following cases: 1) if the next frame pointer is Null,  */
/* 		or 2) if the owner bit is zero, and the			*/
/* 		BRxSTSKO bit is set. 					*/
/* 									*/
/* 		NOTE: The frame descriptor start address pointer must 	*/
/* 		be assigned before the BDMA Rx enable bit is set.	*/
/* 									*/
/*              0 = Disable the BDMA receiver. (If the MAC Rx FIFO is 	*/
/* 		    not empty, move data to the BDMA Rx buffer.)        */
/* 		1 = Enable the BDMA receiver. 				*/
/* 									*/
/* [15] 	BDMA Rx reset (BRxRS) Set this bit to "1" to reset the 	*/
/* 		BDMA Rx block. 						*/
/* 								        */
/* 		0 = No effect. 						*/
/*              1 = Reset the BDMA receiver. 				*/
/* 									*/
/* [16] 	BDMA Rx buffer empty interrupt  enable (BRxEmpty) 	*/
/*              Set this bit is "1" to enable the Rx buffer empty 	*/
/* 		interrupt.  						*/
/* 									*/
/* 		0 = Disable the Rx buffer empty interrupt. 		*/
/* 		1 = Enable the Rx buffer empty interrupt. 		*/
/* 									*/
/* [17] 	BDMA Rx early notify interrupt enable (BRxEarly) 	*/
/* 		Set this bit to "1" to enable the Rx early notify 	*/
/* 		interrupt.  						*/
/* 		The function of this interrupt is to note the length of */
/*		a data frame that is being received from its frame 	*/
/*              length field.						*/
/* 									*/
/* 		0 = Disable the Rx early notify interrupt 		*/
/*              1 = Enable the interrupt when BDMA captures the length  */
/*                  of the received frame type. 			*/
/* 									*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// BDMARXCON Buffered DMA Receiver Control Register  ///////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define	BDMARXCON_RxBRST	0x0001F	// BDMA Rx Burst Size
					// Burst data size = (BRxBRST + 1) word
#define	BDMARXCON_RxSTSKO	0x00020	// BDMA Rx Stop/Skip  Frame or Interrupt(=1) 
					// case of not OWNER the current Frame 
#define	BDMARXCON_RxMAINC	0x00040	// BDMA Rx Memory Address Inc/Dec 
#define	BDMARXCON_RxDIE		0x00080	// BDMA Rx Every Received Frame Interrupt Enable

#define	BDMARXCON_RxNLIE	0x00100	// BDMA Rx NULL List Interrupt Enable 
#define	BDMARXCON_RxNOIE	0x00200	// BDMA Rx Not Owner Interrupt Enable
#define	BDMARXCON_RxMSOIE	0x00400	// BDMA Rx Maximum Size over Interrupr Enable 
#define	BDMARXCON_RxLittle	0x00800	// BDMA Rx Big/Little Endian 
#define	BDMARXCON_RxBig 	0x00000	// BDMA Rx Big/Little Endian
#define	BDMARXCON_RxWA00        0x00000	// 00 	
#define BDMARXCON_RxWA01	0x01000	// BDMA Rx Word Alignment- one invalid byte 
#define BDMARXCON_RxWA10	0x02000	// BDMA Rx Word Alignment- two invalid byte
#define BDMARXCON_RxWA11	0x03000	// BDMA Rx Word Alignment- three invalid byte 
#define BDMARXCON_RxWA   	0x03000	// BDMA Rx Word Alignment mask
#define	BDMARXCON_RxEn		0x04000	// BDMA Rx Enable
#define	BDMARXCON_RxRS		0x08000	// BDMA Rx Reset
#define	BDMARXCON_RxEmpty	0x10000	// BDMA Rx Buffer empty interrupt 
#define	BDMARXCON_RxEarly	0x20000	// BDMA Rx Early notify Interrupt

#define BDMATXPTR (VPint(Base_Addr+0x9008)) /* Transmit frame descriptor start address */

/************************************************************************//*-*/
/* BDMATXPTR	Transmit Frame descriptor start address 		*/
/* 		Offset Adress = 0x9008, Reset Value = 0xFFFFFFFF  	*/
/* 									*/
/* [25:0] 	BDMA transmit frame descriptor start address		*/
/* 		The BDMA transmit frame descriptor start address 	*/
/* 		register contains the address of the frame descriptor 	*/
/* 		on the frame to be sent. During a BDMA operation, this  */
/* 		start address pointer is updated to the next frame 	*/
/* 		address. 		      				*/
/************************************************************************/

#define BDMARXPTR	(VPint(Base_Addr+0x900c)) /* Receive frame descriptor start address */ 

/************************************************************************//*-*/
/* BDMARXPTR 	Receive frame descriptor start adress 			*/
/* 		Offset Adress = 0x900C					*/
/* 		Reset Value   = 0xFFFFFFFF 				*/
/* 									*/
/* [25:0] 	BDMA receive frame descriptor start address 		*/
/* 		The BDMA receive frame descriptor start address 	*/
/* 		register contains the address of the frame descriptor 	*/
/* 		on the frame to be saved. During a BDMA operation, 	*/
/* 		this start address pointer is updated to the next 	*/
/* 		frame address. 		      				*/
/************************************************************************/

#define BDMARXLSZ	(VPint(Base_Addr+0x9010)) /* Receive frame maximum size */

/************************************************************************//*-*/
/* BDMARXLSZ 	Receive frame maximum size 				*/
/* 		Offset value =  0x9010, Reset Value = Undefined 	*/
/* 									*/
/* [15:0] 	BDMA receive frame maximum size (BRxLSZ) 		*/
/*              This register value controls how many bytes per frame 	*/
/* 		can be saved to memory. If the received frame size 	*/
/* 		exceeds the value stored in this location, an error 	*/
/* 		condition is reported.					*/
/* 									*/
/* [31:16] 	BDMA receive frame length (BRxFSZ), read-only 		*/
/* 		When an early notification (Early Notify) interrupt 	*/
/* 		occurs, the frame length/Ethernet type field contains 	*/
/* 		the Frame size of the frame that is currently being 	*/
/* 		received.						*/
/* 		To save space in the frame memory buffer, you can 	*/
/* 		determine the current frame length by 1) enabling the 	*/
/* 		early notification interrupt, and 2) reading the BRxFSZ */
/* 		field when the interrupt occurs. 			*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// BDMARXLSZ Receive frame maximum size register ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define BDMARxLSZ 0x0000FFFF	// BDMA receive frame maximum size
#define BDMARxFSZ 0xFFFF0000	// BDMA receive frame length (read-only)

#define BDMASTAT	(VPint(Base_Addr+0x9014)) /* Buffered DMA status */

/************************************************************************/
/* BDMASTAT	Buffered DMA status 					*/
/* 		Offset Address = 0x9014					*/
/* 		Reset Value    = 0x00000000 	      			*/
/* 									*/
/* [0] 		BDMA Rx done every received frame (BRxRDF) 		*/
/* 		This bit is set each time the BDMA receiver moves one   */
/*              received data frame to memory. This bit must be cleared */
/* 		for the receiving next frame interrupt generation.      */
/* 									*/
/* 		0 = Normal operation. 					*/
/*              1 = One frame is received. 				*/
/* 									*/
/* [1] 		BDMA Rx null list (BRxNL) 				*/
/*		If this bit is set, the BDMARXPTR has a null address. 	*/
/* 		Even if BDMA Rx is disabled, data is transferred from	*/
/* 		the MAC Rx FIFO to the BDMA Rx buffer until the BDMA Rx */
/* 		buffer overflows.					*/
/* 									*/
/* 		0 = Normal operation. 					*/
/*              1 = Current frame descriptor has null address 		*/
/* 		    (0x00000000).   					*/
/* 									*/
/* [2] 		BDMA Rx not owner (BRxNO) 				*/
/*              If this bit is set, BDMA is not the owner of the 	*/
/* 		current data frame. The BRxSTSKO bit is set and BDMA 	*/
/* 		Rx is stopped.        					*/
/* 									*/
/* 		0 = BDMA is owner of the current frame. 		*/
/* 		1 = The owner of the current frame is not BDMA. In this */
/* 		    case, BDMA Rx is stopped if the BSTSKO bit is set. 	*/
/* 									*/
/* [3] 		BDMA Rx maximum size over (BRxMSO) 			*/
/*              If this bit is set, the received frame size is larger 	*/
/* 		than the value in the Rx frame maximum size register, 	*/
/* 		BDMARXLSZ.       					*/
/* 									*/
/* 		0 = Normal operation. 					*/
/* 		0 = Received frame exceeds the user defined 		*/
/* 		BDMARXLSZ[15:0] value. 					*/
/* 									*/
/* [4] 		BDMA Rx buffer empty (BRxEmpty) 			*/
/* 		If this bit is set, the BDMA Rx buffer is empty.        */
/* 									*/
/* 		0 = Not empty. 						*/
/* 		1 = BDMA Rx buffer empty. 				*/
/* 									*/
/* [5] 		Early notification (BRxSEarly) 				*/
/* 		This bit is set when the BDMA receiver has received the */
/* 		length/Ether-type field of the current frame. 		*/
/* 									*/
/* 		0 = Normal operation. 					*/
/* 		1 = Length of current frame can be accessed by reading 	*/
/* 		    the BDMA receive maximum frame size register, 	*/
/* 		    BDMARXLSZ [31:16]. 					*/
/* 									*/
/* [6] 		Reserved Not applicable. 				*/
/* 									*/
/* [7] 		One more frame data in BDMA receive buffer (BRxFRF) 	*/
/* 		This bit is set whenever an additional data frame is 	*/
/* 		received in the BDMA receive buffer.			*/
/* 									*/
/* 		0 = Only one frame data reside in BDMA receive buffer. 	*/
/* 		1 = One more frame data reside in the BDMA receive 	*/
/* 		    buffer.    						*/
/* 									*/	
/* [15:8] 	Number of frames in BDMA receive buffer (BRxNFR) 	*/
/* 		This value indicates the total number of data frames    */
/* 		currently in the BDMA receive buffer. 			*/
/*  									*/
/* [16] 	BDMA Tx complete to send control packet (BTxCCP) 	*/
/*              Bit [16] is set each time the MAC sends a complete 	*/
/* 		control packet. 					*/
/* 									*/
/* 		0 = Normal operation. 					*/
/* 		1 = MAC send the control packet. 			*/
/* 									*/
/* [17] 	BDMA Tx null list (BTxNL)				*/
/*              If this bit is set, the BDMATXPTR value is a null 	*/
/* 		address. In this case, BDMA Tx is disabled but data 	*/
/* 		continues to be transferred from the BDMA Tx buffer to 	*/
/* 		the MAC Tx FIFO until the BDMA Tx buffer underflows. 	*/
/* 		This bit is read only. If you set BDMA Tx reset bit by  */
/* 		software, this bit is cleared automatically. To resume 	*/
/* 		data transfer, you must then set the new frame 		*/
/* 		descriptor pointer and enable BDMA Tx. 	      		*/
/* 									*/
/* [18] 	BDMA Tx not owner (BTxNO) 				*/
/*              If [18] is set, BDMA is not owner of the current frame. */
/*              In this case, the BSTSKO bit is set and BDMA Tx is 	*/
/* 		stopped.   						*/
/* 									*/
/* [19] 	Reserved Not applicable. 				*/
/* 									*/
/* [20] 	BDMA Tx buffer empty (BTxEmpty) 			*/
/* 		If this bit is set, the BDMA Tx buffer is empty.  	*/
/* 									*/
/* 		0 = Not empty 						*/
/* 		1 = BDMA Tx buffer empty. 				*/
/* 									*/
/* [31:21] 	Reserved Not applicable. 				*/
/* 									*/
/* NOTE: 	bit 0, 1, 2, 3, 4, 16, 17, 18, and 20 should be cleared */
/* 		for interrupt generation to the next frame. The method  */
/* 		is write a 1 to the corresponding bit location. 	*/
/* 									*/
/************************************************************************/ 

////////////////////////////////////////////////////////////////////////////////
// BDMASTAT BDMA Status Register ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define	BDMASTAT_BRxRDF	   0x00001 // 0 BDMA Rx Done Every Received Frame 
#define	BDMASTAT_BRxNL	   0x00002 // 1 BDMA Rx NULL List 
#define	BDMASTAT_BRxNO	   0x00004 // 2 BDMA Rx Not Owner 
#define	BDMASTAT_BRxMSO	   0x00008 // 3 BDMA Rx Maximum Size Over 
#define	BDMASTAT_BRxEmpty  0x00010 // 4 BDMA Rx Buffer Empty 
#define	BDMASTAT_BRxSEarly 0x00020 // 5 Early Notify 
#define	BDMASTAT_BRxFRF	   0x00080 // 7 One more frame data in BDMA receive buffer 
#define	BDMASTAT_BRxNFR    0x0FF00 // 15:8 Number of frames in BDMA receive buffer 
#define	BDMASTAT_BTxCCP	   0x10000 // 16 BDMA Tx Complete to send Control Packet 
#define	BDMASTAT_BTxNL	   0x20000 // 17 BDMA Tx Null List 
#define	BDMASTAT_BTxNO	   0x40000 // 18 BDMA Tx Not Owner
#define	BDMASTAT_BTxEmpty 0x100000 // 20 BDMA Tx Buffer Empty 

/************************************************************************/
/* CAM 		Content Address Memory (32 words)			*/
/* 		Offset Adress = 0x9100-0x917C				*/
/* 		Reset Value   = Undefined 				*/
/* 									*/
/* [31:0] 	CAM content (CAM) 					*/
/*   		The CPU uses the CAM content register as a data base 	*/
/* 		for destination addresss. To activate the CAM function, */
/* 		you must set the appropriate enable bits in CAM enable  */
/* 		register   						*/
/* 									*/
/* 	There are 21 CAM entries for the destination address and the 	*/
/* 	Pause control packet. For the destination address CAM value, 	*/
/* 	one destination address consists of 6 bytes. Using the 32-word 	*/
/* 	space (32  4 bytes), you can therefore maintain up to 21 	*/
/* 	separate destination addresses. 	      			*/
/* 									*/
/*      You use CAM entries 0, 1, and 18 to send Pause control packets. */
/* 	To send a Pause control packet, you write the CAM0 entry with 	*/
/* 	the  destination address, the CAM1 entry with the source 	*/
/* 	address, and the CAM 18 entry with length/type, opcode, and 	*/
/* 	operand. You then set the send pause bit in the MAC transmit 	*/
/* 	control register. 			      			*/
/************************************************************************/ 

#define CAM_BaseAddr	(Base_Addr + 0x9100)	  /* CAM content */	
#define CAM_BASE	(VPint(Base_Addr+0x9100)) 
#define CAM_Reg(x)      (VPint(Base_Addr+0x9100+(x*0x4)))

#define BDMATXBUF	(VPint(Base_Addr+0x9200)) /* BDMA transmit (Tx) buffer (64) words */
#define BDMARXBUF	(VPint(Base_Addr+0x9800)) /* BDMA receive (Rx) buffer (64 words) */

// CAM registers offset, BDMA Tx & Rx buffers offsets //////////////////////////  

#define CAM_BASE_OFFSET	 0x9100		/* CAM offset */
#define BDMATXBUF_OFFSET 0x9200 	/* BDMA transmit (Tx) buffer offset */
#define BDMARXBUF_OFFSET 0x9800 	/* BDMA receive (Rx) buffer offset */

/************************************************************************/
/* Ethernet MAC Register   						*/
/************************************************************************/

#define MACCON		(VPint(Base_Addr+0xa000)) /* MAC Control register */

/************************************************************************//*-*/
/* MACCON 	MAC control. 						*/
/* 		Offset Address = 0xA000					*/
/* 		Reset Value    = 0x00000000 				*/
/* 									*/
/* [0] 		Halt request (HaltReq) 					*/
/* 		Set this bit to stop data packet transmission and 	*/
/* 		reception as soon as Tx/Rx of any current packets has 	*/
/* 		been completed.   					*/
/* 									*/
/* [1] 		Halt immediate (HaltImm) 				*/
/*              Set this bit to immediately stop all transmission and 	*/
/* 		reception. 						*/
/* 									*/
/* [2] 		Software reset (Reset) 					*/
/* 		Set this bit to reset all MAC control and status 	*/
/* 		register and MAC state machines. 			*/
/* 									*/
/* [3] 		Full-duplex (FullDup) 					*/
/* 		Set this bit to start transmission while reception is 	*/
/* 		in progress. 						*/
/* 									*/
/* [4] 		MAC loopback (MACLoop) 					*/
/* 		Set this bit to cause transmission signals to be 	*/
/* 		presented as input to the receive circuit without 	*/
/*  		leaving the controller. 				*/
/* 									*/
/* [5] 		Reserved Not applicable 				*/
/*									*/
/* [6] 		MII-OFF 						*/
/* 		Use this bit to select the connection mode. 		*/
/* 		If this bit is set to one, 10 M bits/s interface will 	*/
/* 		select the 10 M bits/s endec. Otherwise, the MII will 	*/
/* 		be selected. 						*/
/* 									*/
/* [7] 		Loop 10 Mb/s (Loop10) If this bit is set, the Loop_10 	*/
/* 		external signal is asserted to the 10-Mb/s endec. 	*/
/* 									*/
/* [9:8] 	Reserved. Not applicable. 				*/
/* 									*/
/* [10] 	Missed roll (MissRoll) 					*/
/* 		This bit is automatically set when the missed error 	*/
/* 		counter rolls over. 					*/
/* 									*/
/* [11] 	Reserved Not applicable. 				*/
/* 									*/
/* [12]         MDC-OFF Clear this bit to enable the MDC clock 		*/
/* 		generation for power management. If it is set to one, 	*/
/* 		the MDC clock generation is disabled.			*/
/* 									*/
/* [13] 	Enable missed roll (EnMissRoll) 			*/
/* 		Set this bit to generate an interrupt whenever the 	*/
/* 		missed error counter rolls over. 			*/
/* 									*/
/* [14] 	Reserved Not applicable. 				*/
/* 									*/
/* [15] 	Link status 10 Mb/s (Link10) This bit value is read 	*/
/* 		as a buffered signal on the link 10 pin. 		*/
/* 									*/
/* [31:16] 	Reserved Not applicable. 				*/
/************************************************************************/

#define	MACCON_HaltReq	0x0001	// stop transmission and reception 
				// after completion of ant current packets 
#define	MACCON_HaltImm	0x0002	// Stop transmission and reception immediately 
#define	MACCON_SwReset  0x0004 	// reset all Ethernet controller state machines
				// and FIFOs 
#define	MACCON_FullDup	0x0008	// allow transmission to begin while reception
				// is occurring 
#define	MACCON_MACLoop	0x0010	// MAC loopback
#define	MACCON_ConnM00	0x0000	// Automatic-default 
#define	MACCON_ConnM01	0x0020	// Force 10Mbits endec
#define	MACCON_ConnM10	0x0040	// Force MII (rate determined by MII clock 
#define	MACCON_MIIOFF	0x0040	// Force MII Off

#define	MACCON_Loop10	0x0080	// Loop 10Mbps 
#define	MACCON_MissRoll	0x0400	// Missed error counter rolled over 
#define	MACCON_MDCOFF	0x1000	// MII Station Management Clock Off
#define	MACCON_EnMissRoll 0x2000// Interrupt when missed error counter rolls over 
#define	MACCON_Link10	0x8000	// Link status 10Mbps 

#define CAMCON		(VPint(Base_Addr+0xa004)) /* CAM Control register */

/************************************************************************/
/* CAMCON 	CAM Control Register 					*/ 
/* 		Offset Value = 0xA004					*/
/*		Reset Value = 0x00000000 	      			*/
/* 									*/
/* [0] 		Station accept (StationAcc) 				*/
/* 		Set this bit to accept any packet with a "unicast" 	*/
/* 		station address. (Station has even first byte. 		*/
/* 		For example 00-00-00-00-00-00) 				*/
/* 									*/
/* [1] 		Group accept (GroupAcc) 				*/
/* 		Accept any packet with a multicast-group address.       */
/* 		(Multicast-group packet has an odd first byte, but not 	*/
/* 		 FF-FF-FF-FF-FF-FF-FF-FF.For example,			*/
/* 		01-00-00-00-00-00-00-00) 				*/  
/* 									*/
/* [2] 		Broadcast accept (BroadAcc) 				*/
/* 		Accept any packet with a broadcast address.		*/
/* 		(FF-FF-FF-FF-FF-FF-FF-FF)				*/
/* 									*/
/* [3] 		Negative CAM (NegCAM) 					*/
/* 		When this bit is 0, packets the CAM recognizes are	*/
/* 		accepted and others are rejected. When "1", packets the */
/* 		CAM recognizes are rejected and others are accepted. 	*/
/* 									*/
/* [4] 		Compare enable (CompEn) Set this bit to enable compare 	*/
/* 		mode.  							*/
/* [31:5]	Reserved Not applicable. 				*/
/************************************************************************/

#define	CAMCON_StationAcc 0x0001 // Accept any packet with a unicast station address
#define	CAMCON_GroupAcc	  0x0002 // Accept any packet with multicast-group  station address  
#define	CAMCON_BroadAcc	  0x0004 // Accept any packet with a broadcast station address 
#define	CAMCON_NegCAM	  0x0008 // 0: Accept packets CAM recognizes, reject others
				 // 1: reject packets CAM recognizes, 
				 //    accept others 
#define	CAMCON_CompEn	  0x0010 // Compare Enable mode


#define MACTXCON	(VPint(Base_Addr+0xa008)) /* MAC Transmit control register */

/************************************************************************/
/* MACTXCON	MAC Transmit Control Register 				*/ 
/* 	        Offset Address = 0xA008					*/
/* 		Reset Value = 0x00000000 	      			*/
/* 									*/
/* [0] 		Transmit enable (TxEn) 					*/
/* 	        Set this bit to enable transmission. To stop 		*/
/* 		transmission immediately, clear the transmit enable	*/
/* 		bit to "0".            					*/
/* 									*/
/* [1]          Transmit halt request (TxHalt) 				*/
/* 		Set this bit to halt transmission after completing any  */
/* 		current packet. 					*/
/* 									*/
/* [2] 		Suppress padding (NoPad) 				*/
/* 		Set to not generate pad bytes for packets of less than  */
/* 		64 bytes. 						*/
/* 									*/
/* [3] 		Suppress CRC (NoCRC) 					*/
/* 		Set to suppress addition of a CRC at the end of a 	*/
/* 		packet.     						*/
/* 									*/
/* [4] 		Fast back-off (FBack) 					*/
/* 		Set this bit to use faster back-off times for testing.	*/
/* 									*/
/* [5] 		No defer (NoDef) 					*/
/* 		Set to disable the defer counter. (The defer counter 	*/
/* 		keeps counting until the carrier sense (CrS) bit is 	*/
/* 		turned off.)    					*/
/* 									*/
/* [6] 		Send Pause (SdPause) 					*/
/* 		Set this bit to send a Pause command or other MAC 	*/
/* 		control packet. The send Pause bit is automatically 	*/
/* 		cleared when a complete MAC control packet has been 	*/
/* 		transmitted. Writing a "0" to this register bit has no  */
/* 		effect. 		      				*/
/* 									*/
/* [7] 		MII 10-Mb/s SQE test mode enable (SQEn)			*/
/* 		Set this bit to enable MII 10-Mb/s SQE test mode. 	*/
/* 									*/
/* [8] 		Enable underrun (EnUnder) Set this bit to generate an   */
/* 		interrupt if the MAC transmit FIFO is empty during a 	*/
/* 		transmission. 						*/
/* 									*/
/* [9] 		Enable deferral (EnDefer) 				*/
/* 		Set this bit to generate an interrupt if the MAC defers */
/* 		for MAX_DEFERRAL time: "0" = 0.32768 ms at 100 Mb/s; 	*/
/* 		"1" = 3.2768 ms at 10-Mb/s. 				*/
/* 									*/ 
/* [10] 	Enable no carrier (EnNCarr) Set this bit to generate an */
/* 		interrupt if a carrier sense is not detected while an 	*/
/* 		entire 							*/
/* 		packet is transmitted. 					*/
/* 									*/
/* [11] 	Enable excessive collision (EnExColl) 			*/
/*		Set this bit to enable an interrupt if 16 collisions 	*/
/* 		occur in the same packet. 				*/
/* 									*/
/* [12] 	Enable late collision (EnLateColl) 			*/
/* 		Set this bit to generate an interrupt if a collision 	*/
/* 		occurs after 512 bit times (or 64 byte times). 		*/
/* 									*/
/* [13] 	Enable transmit parity (EnTxPar) 			*/
/*  	 	Set this bit to generate an interrupt if a parity error */
/* 		is detected in the MAC transmit FIFO. 			*/
/* 									*/
/* [14] 	Enable completion (EnComp) 				*/
/* 		Set this bit to generate an interrupt whenever the MAC  */
/* 		transmits or discards one packet. 			*/
/* 									*/ 
/* [31:15] 	Reserved Not applicable. 				*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// Transmit Control Register(MACTXCON) /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define	MACTXCON_TxEn		0x0001	// transmit Enable 
#define MACTXCON_TxHalt		0x0002	// Transmit Halt Request 
#define	MACTXCON_NoPad		0x0004	// suppress Padding 
#define	MACTXCON_NoCRC		0x0008	// Suppress CRC 
#define	MACTXCON_FBack		0x0010	// Fast Back-off
#define	MACTXCON_NoDef		0x0020	// Disable the defer counter
#define	MACTXCON_SdPause	0x0040	// Send Pause
#define	MACTXCON_MII10SQEn	0x0080	// MII 10Mbps SQE mode enable
#define	MACTXCON_EnUnder	0x0100	// Enable Underrun
#define	MACTXCON_EnDefer	0x0200	// Enable Deferral
#define	MACTXCON_EnNCarr	0x0400	// Enable No Carrier 
#define	MACTXCON_EnExColl	0x0800	// interrupt if 16 collision occur 
					// in the same packet 
#define MACTXCON_EnLateColl	0x1000	// interrupt if collision occurs after 
					// 512 bit times(64 bytes times) 
#define	MACTXCON_EnTxPar	0x2000	// interrupt if the MAC transmit FIFO 
					// has a parity error 
#define	MACTXCON_EnComp		0x4000	// interrupt when the MAC transmits or 
					// discards one packet 

#define MACTXSTAT	(VPint(Base_Addr+0xa00c)) /* MAC Transmit status register */

/************************************************************************/
/* MACTXSTAT 	Transmit status						*/
/* 		Offset Address = 0xA00C, Reset Value = 0x00000000       */
/* 									*/
/* [3:0] 	Transmit collision count (TxColl) 			*/
/*              This 4-bit value is the count of collisions that 	*/
/* 		occurred while successfully transmitting the packet.    */
/* 									*/
/* [4] 		Excessive collision (ExColl) 				*/
/*              This bit is set if 16 collisions occur while 		*/
/* 		transmitting the same packet. In this case, packet 	*/
/* 		transmission is aborted.    				*/
/* 									*/
/* [5] 		Transmit deferred (TxDeferred) 				*/
/* 		This bit is set if transmission of a packet was 	*/
/* 		deferred because of a delay during transmission.        */
/* 									*/
/* [6] 		Paused (Paused) 					*/
/* 		This bit is set if transmission of a packet was delayed */
/* 		due to a Pause being received. 				*/
/* 									*/
/* [7] 		Interrupt on transmit (IntTx) 				*/
/*              This bit is set if transmission of a packet causes an 	*/
/* 		interrupt condition. 					*/
/* 									*/
/* [8] 		Underrun (Under) 					*/
/* 		This bit is set if the MAC transmit FIFO becomes empty	*/
/* 		during a packet transmission. 				*/
/* 									*/
/* [9] 		Deferral (Defer) 					*/
/*              This bit is set if the MAC defers a transfer because of */
/*              MAX_DEFERRAL at 0.32768 ms for 100 Mb/s or 3.2768 	*/
/* 		ms for 10Mb/s. 						*/
/* 									*/
/* [10] 	No carrier (NCarr) 					*/
/* 		This bit is set if no carrier sense is detected during 	*/
/* 		the transmission a packet. 				*/
/* 									*/
/* [11] 	Signal quality error (SQE) According to the IEEE802.3 	*/
/* 		rule, the SQE signal reports the status of the PMA 	*/
/* 		(MAU or transceiver) operation to the MAC layer. 	*/
/* 		After transmission is complete and 1.6 ms has elapsed, 	*/
/* 		a collision detection signal is issued for 1.5 ms to 	*/
/* 		the MAC layer. This signal is called the SQE test 	*/
/* 		signal. The MAC sets the SQE bit in the MACTXSTAT 	*/
/* 		register if this signal is not reported within the 	*/
/* 		IFG time of 6.4ms. 		      			*/
/* 									*/
/* [12] 	Late collision (LateColl) 				*/
/*              This bit is set if a collision occurs after 512 bit 	*/
/* 		times (or 64 byte times). 				*/
/* 									*/
/* [13] 	Transmit parity error (TxPar) 				*/
/* 		This bit is set if a parity error is detected in the 	*/
/* 		MAC transmit FIFO. 					*/
/* 									*/
/* [14] 	Completion (Comp) 					*/
/* 		This bit is set when the MAC transmits, or discards, 	*/
/* 		one packet. 						*/
/* 									*/
/* [15] 	Transmission halted (TxHalted) 				*/
/* 		Transmission was halted by clearing the TxEn bit or 	*/
/* 		the halt immediate (HaltImm) bit. 			*/
/* 									*/
/* [31:16] 	Reserved Not applicable.				*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// Transmit Status Register(MACTXSTAT) /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define MACTXSTAT_TxColl        0x000F	// Transmit collision count
#define	MACTXSTAT_ExColl        0x0010	// Excessive collision set if 16 collisions occur for same packet
#define	MACTXSTAT_TxDeferred    0x0020	// transmission of a packet was deferred
					// because of a delay during transmission
#define	MACTXSTAT_Paused        0x0040	// packet waited because of pause during 
					// transmission 
#define	MACTXSTAT_IntTx	        0x0080	// set if transmission of packet causes an 
					// interrupt condiftion 
#define	MACTXSTAT_Under	        0x0100	// MAC transmit FIFO becomes empty during 
					// transmission 
#define MACTXSTAT_Defer		0x0200	// MAC defers for MAC deferral 

// This bit is set if the MAC defers a transfer  because of
// MAX_DEFERRAL at 0.32768 ms for 100 Mb/s or 3.2768 ms for 10Mb/s.

#define	MACTXSTAT_NCarr		0x0400	// No carrier sense detected during the 
					// transmission of a packet 

#define	MACTXSTAT_SQE		0x0800	// Signal Quality Error
#define	MACTXSTAT_LateColl	0x1000	// a collision occures after 512 bit times 
#define	MACTXSTAT_TxPar		0x2000	// MAC transmit FIFO has detected a parity error
#define	MACTXSTAT_Comp		0x4000	// MAC transmit or discards one packet 
#define	MACTXSTAT_TxHalted	0x8000	// Transmission was halted by clearing 
					// TxEn or Halt immedite 

#define MACRXCON	(VPint(Base_Addr+0xa010)) /* MAC Receive control register */

/************************************************************************/
/* MAXRXCON 	MAC Receive control 					*/
/* 		Offset Adress = 0xA010					*/
/* 		Reset Value = 0x00000000  	      			*/
/* 									*/
/* [0] 		Receive enable (RxEn) 					*/
/* 		Set this bit to "1" to enable MAC receive operation. 	*/
/* 		If "0", stop reception immediately. 			*/
/* 									*/
/* [1] 		Receive halt request (RxHalt) Set this bit to halt 	*/
/* 		reception after completing the reception of any current */
/* 		packet. 	      					*/
/* 									*/
/* [2] 		Long enable (LongEn) Set this bit to receive frames 	*/
/* 		with lengths greater than 1518 bytes. 			*/
/* 									*/
/* [3] 		Short enable (ShortEn) 					*/
/* 		Set this bit to receive frames with lengths less than 	*/
/* 		64 bytes. 						*/
/* 									*/
/* [4] 		Strip CRC value (StripCRC) 				*/
/* 		Set this bit to check the CRC, and then strip it from 	*/
/* 		the message. 						*/
/* 									*/
/* [5] 		Pass control packet (PassCtl) 				*/
/* 		Set this bit to enable the passing of control packets 	*/
/* 		to a MAC client. 					*/
/* 									*/
/* [6] 		Ignore CRC value (IgnoreCRC) 				*/
/* 		Set this bit to disable CRC value checking. 		*/
/* 									*/
/* [7] 		Reserved Not applicable. 				*/
/* 									*/
/* [8] 		Enable alignment (EnAlign) 				*/
/* 		Set this bit to enable the alignment interrupt. 	*/
/* 		An alignment interrupt occurs when a packet is 		*/
/* 		received whose length (in bits) is not a multiple of 	*/
/* 		eight, and whose CRC is invalid.   			*/
/* 									*/
/* [9] 		Enable CRC error (EnCRCErr) 				*/
/* 		Set this bit to enable the CRC interrupt. A CRC 	*/
/* 		interrupt occurs when a packet is received whose CRC 	*/
/* 		is invalid or if, during its reception, the PHY 	*/
/* 		asserts Rx_er.	      					*/
/* 									*/
/* [10] 	Enable overflow (EnOver) 				*/
/* 		Set this bit to enable the overflow interrupt. An 	*/
/* 		overflow interrupt is generated when a packet is 	*/
/* 		received and the MAC receive FIFO is full. 		*/
/* 									*/
/* [11] 	Enable long error (EnLongErr) 				*/
/* 		Set this bit to enable the long error interrupt. 	*/
/* 		A long error interrupt is generated when a frame 	*/
/* 		longer than 1518 bytes is received (unless the long 	*/
/* 		enable bit is set). 	      				*/
/* 									*/
/* [12] 	Reserved Not applicable.				*/
/* 									*/
/* [13] 	Enable receive parity (EnRxPar) 			*/
/* 		Set this bit to enable a receive parity interrupt 	*/
/* 		if the MAC receive FIFO detects a parity error.      	*/
/* 									*/
/* [14] 	Enable Good (EnGood) 					*/
/* 		Set this bit to enable the good (packet) interrupt upon */
/* 		error-free reception of a complete data packet.         */
/* 									*/
/* [31:15] 	Reserved Not applicable. 				*/
/* 									*/
/* NOTE: 	The frame lengths given above do not include preamble	*/
/* 		and start frame delimiter (SFD). 			*/
/************************************************************************/

#define	MACRXCON_RxEn		0x0001	// Receive enable (RxEn)
#define	MACRXCON_RxHalt		0x0002  // Receive halt request 
#define	MACRXCON_LongEn		0x0004  // Long enable
#define	MACRXCON_ShortEn	0x0008  // Short enable
#define	MACRXCON_StripCRC	0x0010  // Strip CRC value
#define	MACRXCON_PassCtl	0x0020  // Pass control packet
#define	MACRXCON_IgnoreCRC	0x0040	// Ignore CRC value
#define	MACRXCON_EnAlign	0x0100  // Enable alignment interrupt 
#define	MACRXCON_EnCRCErr	0x0200  // Enable CRC error interrupt 
#define	MACRXCON_EnOver		0x0400  // Enable overflow interrupt 
#define	MACRXCON_EnLongErr	0x0800  // Enable long error interrupt 
#define	MACRXCON_EnRxPar	0x2000  // Enable receive parity
#define	MACRXCON_EnGood		0x4000  // Enable Good

/************************************************************************/
/************************************************************************/
/************************************************************************/

#define MACRXSTAT	(VPint(Base_Addr+0xa014))  /* MAC Receive status register */

/************************************************************************/
/* MACRXSTAT 	MAC Receive status register 				*/
/* 		Offset Address = 0xA014, Reset Value = 0x00000000 	*/
/* 									*/
/* [4:0] 	Reserved Not applicable. 				*/
/* 									*/
/* [5] 		Control frame received (CtlRecd) 			*/
/* 		This bit is set if the packet received is a MAC control */
/* 		frame (type = 8808H), if the CAM recognizes the packet 	*/
/* 		address, and if the frame length is 64 bytes. 		*/
/* 									*/
/* [6] 		Interrupt on receive (IntRx) 				*/
/* 		This bit is set if the reception of a packet caused an  */
/* 		interrupt to be generated. This includes a good 	*/
/*		received interrupt, if the EnGood bit is set. 		*/
/* 									*/
/* [7] 		Receive 10-Mb/s status (Rx10Stat) 			*/
/* 		This bit is set to "1" if a packet was received over 	*/
/* 		the 10-Mb/s interface or to "0" if a packet was 	*/
/* 		received over the MII. 					*/
/* 									*/
/* [8] 		Alignment error (AlignErr) 				*/
/*              This bit is set if the frame length in bits was not a 	*/
/* 		multiple of eight and the CRC was invalid. 		*/
/* 									*/
/* [9] 		CRC error (CRCErr) 					*/
/*              This bit is set if the CRC at the end of a packet did 	*/
/* 		not match the computed value, or else the PHY asserted 	*/
/* 		Rx_er during packet reception. 				*/
/* 									*/
/* [10] 	Overflow error (overflow) 				*/
/* 		This bit is set if the MAC receive FIFO was full when 	*/
/* 		it needed to store a received byte. 			*/
/* 									*/
/* [11] 	Long error (LongErr)					*/
/* 		This bit is set if the MAC received a frame longer 	*/
/* 		than 1518 bytes. (It is not set if the long enable bit 	*/
/* 		in the receive control register, MACRXCON, is set.) 	*/
/* 									*/
/* [12] 	Reserved Not applicable. 				*/
/* 									*/
/* [13] 	Receive parity error (RxPar) 				*/
/* 		This bit is set if a parity error is detected in the    */
/* 		MAC receive FIFO. 					*/
/* 									*/
/* [14] 	Good received (Good) 					*/
/* 		This bit is set if a packet was successfully received 	*/
/* 		with no 						*/
/* 		errors. If EnGood = "1", an interrupt is also generated.*/
/* 									*/
/* [15] 	Reception halted (RxHalted) This bit is set if 		*/
/* 		reception was halted by clearing RxEn or by setting 	*/
/* 		the HaltImm bit in the MAC control register, MACON. 	*/
/* 									*/
/* [31:16] 	Reserved Not applicable. 				*/
/************************************************************************/

///////////////////////////////////////////////////////////////////////////////
// Receive Status Register(MACRXSTAT) /////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

#define MACRXSTAT_CtlRecd	0x0020	// Control frame received (CtlRecd)
#define	MACRXSTAT_IntRx		0x0040  // Interrupt on receive (IntRx)
#define	MACRXSTAT_Rx10Stat	0x0080  // Receive 10-Mb/s status (Rx10Stat)
#define	MACRXSTAT_AllignErr	0x0100  // Alignment error (AlignErr)
#define	MACRXSTAT_CRCErr	0x0200  // CRC error (CRCErr)
#define	MACRXSTAT_Overflow	0x0400  // Overflow error (overflow)
#define	MACRXSTAT_LongErr	0x0800  // Long error (LongErr)
#define	MACRXSTAT_RxPar		0x2000  // Receive parity error (RxPar)
#define	MACRXSTAT_RxGood	0x4000  // Good received (Good) 
#define	MACRXSTAT_RxHalted	0x8000  // Reception halted (RxHalted)

#define STADATA		(VPint(Base_Addr+0xa018)) /* Station management data */

/************************************************************************/
/* STADATA	MAC Station management data register 			*/
/* 		Offset Value = 0xA018					*/
/* 		Reset Value  = 0x00000000 				*/
/* 									*/
/* [15:0] 	Station management data. 				*/
/* 		This register contains a 16-bit data value for the 	*/
/* 		station management function. 				*/
/************************************************************************/

#define STACON		(VPint(Base_Addr+0xa01c)) /* Station management control and address */

/************************************************************************/
/* STACON	Station management control and address register 	*/ 
/* 		Offset Address = 0xA01C					*/
/* 		Reset Value    = 0x00008000 				*/
/* 								        */
/* [4:0] 	PHY register address (Addr) 				*/
/* 		A 5-bit address, contained in the PHY, of the register 	*/
/* 		to be read or written. 					*/
/* 									*/
/* [9:5] 	PHY address (PHY) 					*/
/* 		The 5-bit address of the PHY device to be read or 	*/
/* 		written.    						*/	
/* 									*/
/* [10] 	Write (Wr) 						*/
/* 		To initiate a write operation, set this bit to "1". 	*/
/* 		For a read operation, clear it to "0". 			*/
/* 									*/
/* [11] 	Busy bit (Busy) 					*/
/*              To start a read or write operation, set this bit to 	*/
/* 		"1". The MAC controller clears the Busy bit 		*/
/* 		automatically when the operation is completed. 		*/
/* 									*/
/* [12] 	Preamble suppress (PreSup) 				*/
/* 		If you set this bit, the preamble is not sent to the 	*/
/* 		PHY. If it is clear, the preamble is sent. 		*/
/* 									*/
/* [15:13]      MDC clock rating Control the MDC period. 		*/
/* 		MD_CA[15:13] MDC period 				*/
/* 		 000 16 x (1/fMCLK)					*/
/* 		 001 18 x (1/fMCLK)					*/
/* 		 010 20 x (1/fMCLK)					*/
/* 		  . .							*/
/* 		  . . 							*/
/* 		 111 30 x (1/fMCLK) 					*/
/* 									*/
/* 		MDC period = MD_CA[15:13] x 2 + 16			*/
/* 		Default MDC_CA[15:13] = 100 				*/
/* 									*/
/* [31:16] 	Reserved Not applicable. 				*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// STACON // Station management control and address register ///////////////////
////////////////////////////////////////////////////////////////////////////////

#define STACON_Addr     0x001F 	// PHY register address (Addr)
#define STACON_PHY      0x03E0  // PHY address (PHY)
#define STACON_PHY_Off       5  // PHY address offset
#define STACON_Wr       0x0400  // Write (Wr)
#define STACON_Busy     0x0800  // Busy bit (Busy)
 // To start a read or write operation, set this bit to "1". The
 // MAC controller clears the Busy bit automatically when the
 // operation is completed.
#define STACON_PreSup   0x1000  // Preamble suppress (PreSup)
#define STACON_MDC      0xE000  // MDC clock rating mask
#define STACON_MDC_Off      13  // MDC clock rating offset


#define CAMEN		(VPint(Base_Addr+0xa028)) /* CAM enable */

/************************************************************************/
/* CAMEN	CAM enable register 					*/
/* 		Offset Address = 0xA028, Reset value = 0x00000000 	*/ 
/* 									*/
/* [20:0] 	CAM enable (CAMEn) 					*/
/* 		Set the bits in this 21-bit value to selectively enable */
/* 		CAM locations 20 through 0. To disable a CAM location, 	*/
/* 		clear the appropriate bit. 				*/
/* 									*/
/* [31:21] 	Reserved Not applicable. 				*/
/************************************************************************/

#define EMISSCNT	(VPint(Base_Addr+0xa03c)) /* Missed error count */

/************************************************************************/
/* EMISSCNT 	Missed error count 				        */
/* 		Offset Address = 0xA03C, Reset Value = 0x00000000 	*/
/* 									*/
/* [15:0] 	Alignment error count (AlignErrCnt) 			*/
/*              The number of packets received with alignment errors. 	*/
/* 		This software counter increments at the end of a packet */
/* 		reception if the Rx_Stat value indicates an alignment 	*/
/* 		error.	      						*/	
/* 									*/
/* 		CRC error count (CRCErrCnt) 				*/
/* 		The number of packets received with a CRC error. 	*/
/* 		This software counter increments if the Rx_Stat value 	*/
/* 		indicates a CRC error. If the Rx_Stat value indicates 	*/
/* 		another type of error, such as an alignment error,this  */
/* 		counter is not incremented.				*/
/* 									*/
/* 		Missed error count (MissErrCnt) 			*/
/* 		The number of valid packets rejected by the MAC unit	*/
/* 		because of MAC receive FIFO overflows, parity errors, or*/
/* 		because the Rx_En bit was cleared. This count does not  */
/* 		include the number of packets rejected by the CAM.	*/
/* 									*/
/* [31:16] 	Reserved Not applicable.				*/
/************************************************************************/

#define EPZCNT		(VPint(Base_Addr+0xa040)) /* Pause count */

/************************************************************************/
/* EPZCNT	MAC Received Pause Count Register 			*/
/* 		Offset Address = 0xA040					*/
/*		Reset Value = 0x00000000	      			*/
/* 									*/
/* [15:0] 	Received Pause count (EPZCNT) 				*/
/* 		The count value indicates the number of time slots the	*/
/* 		transmitter was paused due to the receipt of 		*/
/* 		control Pause operation packets from the MAC.		*/
/************************************************************************/

#define ERMPZCNT	(VPint(Base_Addr+0xa044)) /* Remote pause count */

/************************************************************************/
/* ERMPZCNT	MAC Remote Pause Count Register				*/
/* 		Offset Address = 0xA044					*/
/*		Reset Value = 0x00000000 	      			*/
/* 									*/
/* [15:0] 	Remote Pause count (ERMPZCNT)				*/
/* 		The count value indicates the number of time slots 	*/
/* 		that a remote MAC was paused as a result of its sending */
/* 		control Pause operation packets. 			*/
/************************************************************************/

#define ETXSTAT		(VPint(Base_Addr+0x9040)) /* Transmit control frame status */ 

/************************************************************************/
/* ETXSTAT	Transmit control frame status register 			*/
/*              Offset Address = 0x9040, Reset Value = 0x00000000 	*/
/* 									*/
/* [15:0] 	Tx_Stat value 						*/
/* 		A 16-bit value indicating the status of a MAC control 	*/
/* 		packet as it is sent to a remote station. Read by the 	*/
/* 		DMA engine.    						*/
/************************************************************************/

#define MACRXDESTR	(VPint(Base_Addr+0xa064)) 
#define MACRXSTATEM	(VPint(Base_Addr+0xa090)) 
#define MACRXFIFO	(VPint(Base_Addr+0xa200)) 

#ifdef _KS32C50100_ 

/************************************************************************/
/* KS32C50100 : HDLC Channel A & B                     			*/
/************************************************************************/

#define HMODEA 	        (VPint(Base_Addr+0x7000)) /* HDLC Channel A Mode register */	
#define HMODEB 	        (VPint(Base_Addr+0x8000)) /* HDLC Channel B Mode register */
#define HMODE(channel) 	(VPint(Base_Addr+0x7000 + channel*0x1000))

/************************************************************************/
/* HMODE   HDLC control register. 					*/
/* 	   Offset=0x7000,0x8000. 					*/
/* 	   Reset Value = 0x00000000				        */
/* 									*/
/* [0] 	   Multi-Frame in HTxFIFO 					*/
/*         If this bit is set, more than one frame can be loaded into 	*/
/* 	   HTxFIFO. In this case, the frame size may be less than the 	*/
/* 	   FIFO size. in DMA operation (MFF) 				*/
/*          0 = Single frame in FIFO 					*/
/*  	    1 = Multiple frame in FIFO 					*/
/* 									*/
/* [3:1]   Reserved Not applicable. KS32C50100  			*/
/* [1] 	   Reserved Not applicable. S3C4530 				*/
/* [2]     Rx clock inversion(RXCINV) S3C4530 				*/
/*         If this bit is set to '0', the receive clock samples the     */
/*         data at the rising edge. If this bit is set to '1', the 	*/
/*         receive clock samples the data at the falling edge. 		*/
/* [3]     Tx clock inversion(TXCINV) S3C4530 				*/
/*         If this bit is set to '0', the transmit clock shifts the 	*/
/*         data at the falling edge. If this bit is set to '1', the 	*/
/* 	   transmit clock shifts the data at the rising edge. 		*/
/* 									*/
/* [4]     Rx Little-Endian mode (RxLittle)				*/
/*  	   This bit determines whether the data is in Little- or 	*/
/* 	   Big-endian format. HRXFIFO is in Little-endian. If this bit 	*/
/* 	   is set to '0', then the data on the system bus should be in  */
/* 	   Big-endian. Therefore the bytes will be swapped in 		*/
/* 	   Big-endian.		      					*/
/* 									*/
/* [5] 	   Tx Little-Endian mode (TxLittle) 				*/
/* 	   This bit determines whether Tx data is in Little or Big 	*/
/* 	   endian (TxLittle) format. HTxFIFO is in Little-endian. 	*/
/* 	   If this bit is set to '1', the data on the system bus is 	*/
/* 	   Little endian. If this bit is set to '0', the data on the 	*/
/* 	   system bus is in Big-endian. (that is,1 the data bytes are 	*/
/* 	   swapped to beLittle endian format.)  			*/
/* 									*/
/* [7:6]   Reserved Not applicable. KS32C50100 				*/
/* 									*/
/* [6] 	   Rx Transparent mode (RxTRANS) S3C4530 			*/
/*         If this bit set to one, HDLC Rx operates transparent mode.   */
/*         Otherwise, operates HDLC mode. 				*/
/* [7]     Tx Transparent mode (TxTRANS) S3C4530 			*/
/*         If this bit set to one, HDLC Tx operates transparent mode.   */
/*         Otherwise, operates HDLC mode. 				*/
/* 									*/
/* [10:8]  Tx preamble length (TxPL) 					*/
/* 	   These bits determine the length of preamble to be sent 	*/
/* 	   before the opening flag when the TxPRMB bit is set in the 	*/
/* 	   control register.   						*/ 
/*          000 = 1byte							*/
/*          001 = 2bytes						*/
/*          111 = 8bytes						*/
/* 									*/
/* [14:12] Data formats (DF) 						*/
/*          000 = NRZ							*/
/*          001 = NRZI (zero complement)				*/
/*          010 = FM0 							*/
/*          011 = FM1							*/
/*          100 = Manchester 						*/
/* 									*/
/* [15]    Reserved Not applicable. 					*/
/* 									*/
/* [18:16] DPLL clock select (DPLLCLK) 					*/
/* 	    000 = TXC							*/
/*          001 = RXC							*/
/*          010 = MCLK							*/
/*          011 = BRGOUT1						*/
/*          100 = BRGOUT2						*/
/* 									*/
/* [19]    BRG clock select (BRGCLK) 					*/
/*          1  = MCLK2 							*/
/*          0  = RXC 							*/
/* 									*/
/* [22:20] Tx clock select (TxCLK) 				        */
/*          000 = TXC							*/
/*          001 = RXC							*/
/*          010 = DPLLOUTT						*/
/*          011 = BRGOUT1						*/
/*          100 = BRGOUT2					        */
/* 									*/
/* [26:24] Rx clock select (RxCLK) 					*/
/*          000 = TXC							*/
/*          001 = RXC							*/
/*          010 = DPLLOUTR						*/
/*          011 = BRGOUT1						*/
/*          100 = BRGOUT2						*/
/* 									*/
/* [30:28] TXC output pin select (TXCOPS) 				*/
/*          000 = TxCLK							*/
/*          001 = RxCLK							*/
/*          010 = BRGOUT1						*/
/*          011 = BRGOUT2						*/
/*          100 = DPLLOUTT						*/
/*          ?   = DPLLOUTR						*/
/* 									*/
/* [31]    Reserved Not applicable 					*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// HDLC Mode Register  (HMODE) /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// [0] 	   Multi-Frame in HTxFIFO //////////////////////////////////////////////

#define HMODE_MFinFIFO	        0x00000001   	// 1 = Multiple frame in FIFO 

#ifdef _S3C4530_
#define HMODE_RXCINV            0x00000004	// 2 Rx clock inversion(RXCINV)
#define HMODE_TXCINV            0x00000008	// 3 Tx clock inversion(TXCINV)
#endif 								/* _S3C4530_ */

#define HMODE_RxLittle	        0x00000010      // Rx Little Endian

// [5] 	   Tx Little-Endian mode (TxLittle) ////////////////////////////////////
#define HMODE_TxLittle	        0x00000020      // Tx Little Endian

#ifdef _S3C4530_ 
#define HMODE_RxTRANS           0x00000040      // 6 Rx Transparent mode (RxTRANS)
#define HMODE_TxTRANS           0x00000080      // 7 Tx Transparent mode (TxTRANS)
#endif 								/* _S3C4530_ */

// [10:8]  Tx preamble length (TxPL) ///////////////////////////////////////////

#define	HMODE_TxPL1		0x00000000	// 000 = 1 byte
#define	HMODE_TxPL2		0x00000100	// 001 = 2 bytes	
#define	HMODE_TxPL3		0x00000200	
#define	HMODE_TxPL4		0x00000300	
#define	HMODE_TxPL5		0x00000400	
#define	HMODE_TxPL6		0x00000500	
#define	HMODE_TxPL7		0x00000600	
#define	HMODE_TxPL8		0x00000700	// 111 = 8 bytes	

// [14:12] Data formats (DF) ///////////////////////////////////////////////////

#define	HMODE_DF_NRZ		0x00000000	// 000 = NRZ
#define	HMODE_DF_NRZI		0x00001000	// 001 = NRZI (zero complement)
#define	HMODE_DF_FM0		0x00002000	// 010 = FM0
#define	HMODE_DF_FM1		0x00003000	// 011 = FM1
#define	HMODE_DF_Manchester	0x00004000	// 100 = Manchester

#define	HMODE_DF   		0x00007000	// 111 data format mask 

// [18:16] DPLL clock select (DPLLCLK) /////////////////////////////////////////

#define	HMODE_DPLLCLK_TXC	0x00000000	// 000 = TXC
#define	HMODE_DPLLCLK_RXC	0x00010000	// 001 = RXC
#define	HMODE_DPLLCLK_MCLK	0x00020000	// 010 = MCLK
#define	HMODE_DPLLCLK_BRGOUT1	0x00030000	// 011 = BRGOUT1
#define	HMODE_DPLLCLK_BRGOUT2	0x00040000	// 100 = BRGOUT2

#define	HMODE_DPLLCLK    	0x00070000	// DPLL mode mask 


// [19]    BRG clock select (BRGCLK) ///////////////////////////////////////////

#define	HMODE_BRGCLK_RXC 	0x00000000	// 0  = RXC
#define	HMODE_BRGCLK_MCLK2 	0x00080000	// 1  = MCLK2

// [22:20] Tx clock select (TxCLK) /////////////////////////////////////////////

#define	HMODE_TxCLK_TXC    	0x00000000      // 000 = TXC
#define	HMODE_TxCLK_RXC     	0x00100000      // 001 = RXC
#define	HMODE_TxCLK_DPLLOUTT    0x00200000	// 010 = DPLLOUTT
#define	HMODE_TxCLK_BRGOUT1     0x00300000	// 011 = BRGOUT1 
#define	HMODE_TxCLK_BRGOUT2 	0x00400000	// 100 = BRGOUT2

#define	HMODE_TxCLK       	0x00700000	// Tx clock select mask 

// [26:24] Rx clock select (RxCLK) /////////////////////////////////////////////

#define	HMODE_RxCLK_TXC	        0x00000000	// 000 = TXC	
#define	HMODE_RxCLK_RXC	        0x01000000	// 001 = RXC
#define	HMODE_RxCLK_DPLLOUTR	0x02000000	// 010 = DPLLOUTR
#define	HMODE_RxCLK_BRGOUT1	0x03000000	// 011 = BRGOUT1
#define	HMODE_RxCLK_BRGOUT2	0x04000000	// 100 = BRGOUT2

#define	HMODE_RxCLK      	0x07000000	// Rx clock select mask 

// [30:28] TXC output pin select (TXCOPS) //////////////////////////////////////

#define	HMODE_TXCOPS_TxCLK 	0x00000000	// 000 = TxCLK
#define	HMODE_TXCOPS_RxCLK  	0x10000000	// 001 = RxCLK	
#define	HMODE_TXCOPS_BRGOUT1    0x20000000	// 010 = BRGOUT1
#define	HMODE_TXCOPS_BRGOUT2    0x30000000	// 011 = BRGOUT2
#define	HMODE_TXCOPS_DPLLOUTT   0x40000000	// 100 = DPLLOUTT
#define	HMODE_TXCOPS_DPLLOUTR   0x50000000	// DPLLOUTR

#define	HMODE_TXCOPS  		0x70000000	// mask 

#define HCONA 	   (VPint(Base_Addr+0x7004))	/* HDLC Channel A Control register */
#define HCONB 	   (VPint(Base_Addr+0x8004))	/* HDLC Channel B Control register */

/* HDLC Chennel 0(A) & 1(B) Control register */
#define HCON(ch)   (VPint(Base_Addr+0x7004 + ch*0x1000))

/************************************************************************/
/* HCONA   HDLC channel A control register. 			        */
/* 	   Offset      = 0x7004 					*/
/* 	   Reset Value = 0x00000000 				        */
/* HCONB   HDLC channel B control register 			        */
/*         Offset      = 0x8004					        */
/*         Reset Value = 0x00000000 				      	*/
/* 									*/
/* [0]     Tx reset (TxRS) 						*/
/*          0 = Normal operation 					*/
/*          1 = TxFIFO and Tx block are reset 				*/
/* 									*/
/* [1] 	   Rx reset (RxRS) 						*/
/*          0 = Normal operation 					*/
/*          1 = RxFIFO and Rx block are reset 				*/
/* 									*/
/* [2]     DMA Tx reset (DTxRS)						*/
/*          0 = Normal operation 					*/
/*          1 = DMA Tx block is reset 					*/
/* 									*/
/* [3]     DMA Rx reset (DRxRS)						*/
/*          0 = Normal operation 					*/
/*          1 = DMA Rx block is reset 					*/
/* 									*/
/* [4]     Tx enable (TxEN) 						*/
/*          0 = Tx disabled						*/
/*          1 = Tx enabled 						*/
/* 									*/
/*         When the TxEN bit is '0', the transmitter enters a disabled	*/
/* 	   state and the line becomes high state. In this case, the 	*/
/* 	   transmitter block is cleared except for the HTxFIFO and the	*/
/* 	   status bits associated with transmit operation are cleared. 	*/
/* 	   Data cannot be loaded into the HTxFIFO.			*/
/* 									*/
/* [5]     Rx enable (RxEN) 						*/
/*          0 = Rx disabled 					        */
/*          1 = Rx enabled 						*/
/* 									*/
/*         When the RxEN bit is '0', the receiver enters a disabled 	*/
/* 	   state and can not detect the flag pattern, if any. In this 	*/
/* 	   case, receiver block is cleared except for the HRXFIFO and	*/
/* 	   the status bits associated with receiver operation are 	*/
/* 	   cleared. Data cannot be received.				*/
/* 									*/
/*         If this bit is set to '1', the flag pattern is detected.	*/
/* 	   In this case, the data received can be loaded into the 	*/
/* 	   HRXFIFO, and moved to system memory.				*/
/* 									*/
/* [6]     DMA Tx enable (DTxEN)					*/
/*          0 = DMA Tx disabled 					*/
/*          1 = DMA Tx enabled  					*/
/* 									*/
/*         The DTxEN bit lets the HDLC Tx operate on a bus system in 	*/
/* 	   DMA mode. When DMA Tx is enabled, an interrupt request 	*/
/* 	   caused by TxFA status is inhibited and the HDLC does not 	*/
/* 	   use the interrupt request to request a data transfer. 	*/
/* 	   DMA Tx monitors the HTxFIFO and fills the HTxFIFO. 		*/
/* 									*/
/*         This bit is auto disabled when Tx underrun occurs, or CTS    */
/*         lost, or next buffer descriptor pointer reach null, or the 	*/
/* 	   owner bit is not DMA mode when DTxSTSK bit is set.           */
/* 									*/
/*         If Tx underrun occurs, DTxABT(in HSTAT) bit set, and abort 	*/
/* 	   signal sended. 						*/
/* 									*/
/*         If CTS lost occurs, DTxABT bit set and TxD output goes high	*/
/* 	   state as long as  TS remains high level. 			*/
/* 									*/
/* [7]     DMA Rx enable (DRxEN)					*/
/*          0 = DMA Rx disabled 					*/
/*          1 = DMA Rx enabled 						*/
/* 									*/
/*         The DRxEN bit lets the HDLC Rx operate on a bus system in 	*/
/* 	   DMA mode. When DMA Rx is enabled, an interrupt request 	*/
/*         caused by the RxFA status is inhibited, and the HDLC does 	*/
/* 	   not use the interrupt request to request a data transfer. 	*/
/*         DMA Rx monitors the HRXFIFO and moves the data from the 	*/
/* 	   HRXFIFO to memory. 		      				*/
/* 									*/
/*         This bit is automatically disabled when the next buffer 	*/
/* 	   descriptor pointer becomes null, or the owner bit is not 	*/
/*	   in DMA mode when the DTxSTSK bit is set. 			*/
/* 									*/
/* [8]     DPLL enable (DPLLEN) 					*/
/*          0 = Disable  						*/
/*          1 = Enable; DPLL enters search mode for a locking edge 	*/
/*              in the incoming data stream. 				*/
/* 									*/
/* 	   Setting this bit enables the DPLL, causing the DPLL to enter */
/*         search mode. In Search mode, the DPLL searches for a locking */
/* 	   edge in the incoming data stream. 				*/
/* 									*/
/*         After DPLL is enabled (in NRZI mode for example), the DPLL 	*/
/* 	   starts sampling immediately after the first edge is detected.*/
/*         (In FM mode, the DPLL examines the clock edge of every 	*/
/*         other bit to decide what correction must be made to remain 	*/
/* 	   in sync.) 			      				*/
/* 									*/
/*         If the DPLL does not detect an edge during the expected 	*/
/* 	   window, it sets the one clock missing bit.			*/
/* 									*/
/*         If the DPLL does not detect an edge after two successive 	*/
/*         attempts, it sets the two clock missing bit and the DPLL 	*/
/*         automatically enters the Search mode. 			*/
/* 									*/
/*         To reset both clocks missing latches, you can disable and 	*/
/* 	   re-enable the DPLL using the reset Rx status. 		*/
/* 									*/
/* [9]     BRG enable (BRGEN) 						*/
/*          0 = BRG counter is inhibited 				*/
/*          1 = BRG counter is enabled 					*/
/* 									*/
/*         This bit controls the operation of the baud rate generator 	*/
/* 	   (BRG). To enable the BRG counter, set the BRGEN bit to '1'.  */
/* 	   To inhibit counting, clear the bit to '0'. 			*/
/* 									*/
/* [10]    Tx 4 word mode (Tx4WD) 					*/
/*          0 = 1-word mode selected 					*/
/*          1 = 4-word mode selected 					*/
/* 									*/
/*         When this bit is '0', and TxFA bit in status register 	*/
/* 	   is '1', it is indicated that Tx FIFO is empty for 1 word. 	*/
/* 	   It means that 1-word data can be loaded to Tx FIFO. 		*/
/* 									*/
/*         Similarly, when this bit is '1', the same status register 	*/
/*	   bit indicate that 4 words of data can be loaded to Tx FIFO 	*/
/* 	   without reading the status bit for a second time. 		*/
/* 									*/
/* [11]    Rx 4 word mode (Rx4WD) 					*/
/*          0 = 1-word mode selected 					*/
/*          1 = 4-word mode selected 					*/
/* 									*/
/*         When this bit is '0', and the RxFA bit in the status 	*/
/*         register is '1', it is indicated that Rx FIFO has 1-word 	*/
/*         data. It means that 1 word data can be moved to memory. 	*/
/* 									*/
/*         Similarly, when this bit is '1', the same status register 	*/
/*	   bit indicates that 4 words of data can be moved in the 	*/
/*	   memory without reading the status bit  for a second time. 	*/
/* 									*/
/* [13:12] Rx widget alignment (RxWA) 					*/
/*         These bits determine how many bytes are invalid in the 	*/
/*	   first memory	word of the frame to be received. The invalid 	*/
/*	   bytes are inserted when the word is assembled in the HRXFIFO.*/
/*          00 = No Invalid bytes  					*/
/*          01 = 1 invalid byte						*/
/*          10 = 2 invalid bytes 					*/
/*          11 = 3 invalid bytes 					*/
/* 									*/
/* [14]    DMA Tx stop or skip (DTxSTSK) 				*/
/*          0 = DMA Tx skips when DMA not owner bit is set 		*/
/*          1 = DMA Tx stops when DMA not owner bit is set 		*/
/* 									*/
/*         This bit determines a DMA Tx stop or skip when DMA has 	*/
/*	   not the ownership associated with the Tx buffer descriptor. 	*/
/*	   DMA Tx is disabled in this condition when this bit is set.   */
/* 									*/
/* [15]    DMA Rx stop or skip (DRxSTSK) 				*/
/*          0 = DMA Rx skips when DMA not owner bit is set 		*/
/*          1 = DMA Rx stops when DMA not owner bit is set 		*/
/* 									*/
/*         This bit determines a DMA Rx stop or skip when DMA has not 	*/
/* 	   the ownership associated with the Rx buffer descriptor. If 	*/
/*	   this bit is set, DMA Rx is disabled.				*/
/* 									*/
/* [16]    DMA Rx memory address decrement (DRxMADEC) 			*/
/*          0 = Address is incremented 					*/
/*          1 = Address is decremented 					*/
/* 									*/
/*         This bit determines whether the address is incremented or    */
/*         decremented. If this bit is set to '1', then the address 	*/
/*	   will be decremented. 					*/
/* 									*/
/* [17]    Tx flag idle (TxFLAG) 					*/
/*          0 = Enter mark idle mode (a bit pattern of consecutive ones)*/
/*          1 = Enter time fill mode (a bit pattern of consecutive 	*/
/*	        opening (closing) flag, as in string 			*/
/*	        01111110 01111110 ...            			*/
/* 									*/
/*         This bit selects the flag 'time fill' mode (active idle) 	*/
/*	   or the mark idle mode (inactive idle) for the transmitter. 	*/
/*	   The selected active or inactive idle state continues until 	*/
/*	   data is sent (after nRESET has return to High level). The 	*/
/*	   flag idle pattern is 7EH; the mark idle pattern is FFH. 	*/
/* 									*/
/* [18]    Tx single flag (TxSFLAG) 					*/
/*          0 = Double flag mode (a closing & opening flags are used to */
/*              separate frames)					*/
/*          1 = Single flag mode (only one flags are used to separate	*/
/*	        frames) 						*/
/*									*/
/* [19]    Tx loop-back mode (TxLOOP) 					*/
/*          0 = Normal operation 					*/
/*          1 = The transmit data output is internally connected to the */
/*              receiver data input for self testing 			*/
/* 									*/
/* [20]    Rx echo mode (RxECHO) 					*/
/*         Setting this bit to '1' selects the auto-echo mode of 	*/
/*	   operation. In this mode, the TXD pin is connected to RXD 	*/
/*	   as in local loop-back mode, but the receiver still monitors	*/
/*	   the RXD input. 	      					*/
/* 									*/
/* [21]    Tx abort extension (TxABTEXT) 				*/
/*          0 = At least consecutive eight 1s are transfered 		*/
/*          1 = At least 16 consecutive 1s are transferred 		*/
/* 									*/
/* [22]    Tx abort (TxABT) 						*/
/*          0 = Normal 							*/
/*          1 = Enable (at least eight consecutive 1s are transmitted)  */
/* 									*/
/* [23]    Tx preamble (TxPRMB) 					*/
/*          0 = Transmit a mark idle or time fill bit pattern 		*/
/*          1 = Transmit the content of HPRMB 				*/
/* 									*/
/*  	   When this bit is set to '1', the content of the HPRMB 	*/
/* 	   register is transmitted as many TxPL bit values in interrupt */
/*	   mode instead of mark idle or time fill mode. This is useful 	*/
/*	   for sending the data needed by the DPLL to lock the phase. 	*/
/* 	   In DMA mode, this bit is meaningless. 			*/	
/* 									*/
/* [24]    Tx data terminal ready (TxDTR)				*/
/*          0 = nDTR goes high level 					*/
/*          1 = nDTR goes low level 					*/ 
/* 								        */
/*         The TxDTR bit directly controls the nDTR output state.       */
/*         Setting this bit forces the nDTR pin to Low level. 	        */
/*         When you clear the TxDTR bit, nDTR goes High.                */
/* 									*/
/* [25]    Rx frame discontinue (RxDISCON) 				*/
/*          0 = Normal 							*/
/*          1 = Ignore the currently received frame 			*/ 
/* 									*/
/*         When this bit is set, the frame currently received is 	*/
/*	   ignored and the data in this frame is discarded. Only the 	*/
/*	   last frame received is affected. There is no effect on 	*/
/*	   subsequent frames, even if the next frame enters the 	*/
/*	   receiver when the discontinue bit is set.    		*/
/*         This bit is automatically cleared after a cycle. 		*/
/* 									*/
/* [26]    Tx no CRC (TxNOCRC) 						*/
/*          0 = Disable 						*/
/*          1 = CRC is not appended by hardware 			*/
/* 									*/
/* [27]    Rx no CRC (RxNOCRC) 						*/
/*          0 = Disable 						*/
/*          1 = Receiver does not check CRC by hardware 		*/ 
/*              (CRC is treated as data in any case) 			*/
/* 									*/
/* [28]    Auto enable (AutoEN) 					*/
/*          0 = Normal operation. The nCTS and nDCD become high, the    */
/*              transmitter sends mark idle and receiver receives data. */
/*          1 = The nDCD and nCTS become high, RxFIFO, Rx block, TxFIFO,*/
/*	        and Tx block are cleared. The transmitter sends mark 	*/
/*	        idle, and the receiver does not operate. 		*/
/* 									*/
/*         This bit programs the function of both nDCD and nCTS. 	*/
/*         However, TxEN and RxEN must be set before the nCTS and nDCD 	*/
/*         pins can be used. 						*/
/* 							                */
/*         When this bit is '0', if the nCTS becomes high, the 		*/
/*	   transmitter sends mark idle pattern. However, though the 	*/
/*	   nDCD becomes high, the receiver can receive the data. 	*/
/* 								        */
/*         When this bit is '1', if the nCTS becomes high, the 		*/
/*	   transmitter send mark idle but clears the HTxFIFO and the 	*/
/*	   Tx block. If nDCD becomes high, the receiver can't operate,  */
/*         and the HRXFIFO and Rx blocks are cleared. 			*/
/* 									*/
/* [31:29] Reserved Not applicable. KS32C50100 				*/
/* [29]    Transparent Rx stop (TRxSTOP) S3C4530 			*/
/*         This bit reset value is zero. If this bit set to one, the    */
/*         receive operation is ended in transparent mode. And then the */
/*         receiver start to search Sync. If this bit set to one in 	*/
/*         HDLC mode, Rx FIFO cleared except for receiver. So, the data */
/*         in receiver can be moved to Rx FIFO at this time. 		*/
/* [30]    Transmit reverse (TxREV) S3C4530 				*/
/*         If this bit set to one, the data will be sent MSB first. If  */
/*         this bit set to zero, LSB first. 				*/
/* [31]    Receive reverse (RxREV) S3C4530 				*/
/*         If this bit set to one, the received data will be MSB first. */
/* 	   If this bit set to zero, LSB first.				*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////	
// HDLC control register (HCON) ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////	

#define	HCON_TxRS	     0x00000001	// [0] Tx reset (TxRS)
#define	HCON_RxRS	     0x00000002	// [1] Rx reset (RxRS)
#define	HCON_DTxRS	     0x00000004	// [2] DMA Tx reset (DTxRS)
#define	HCON_DRxRS	     0x00000008	// [3] DMA Rx reset (DRxRS)
#define	HCON_TxEN	     0x00000010	// [4] Tx enable (TxEN)
#define	HCON_RxEN	     0x00000020	// [5] Rx enable (RxEN)
#define	HCON_DTxEN	     0x00000040	// [6] DMA Tx enable (DTxEN)
#define	HCON_DRxEN	     0x00000080	// [7] DMA Rx enable (DRxEN)
#define HCON_DPLLEN          0x00000100 // [8] DPLL enable (DPLLEN)
#define HCON_BRGEN           0x00000200 // [9] BRG enable (BRGEN)

// [10]    Tx 4 word mode (Tx4WD) //////////////////////////////////////////////
#define HCON_Tx1WD           0x00000000	//  0 = 1-word mode selected
#define HCON_Tx4WD           0x00000400	//  1 = 4-word mode selected

// [11]    Rx 4 word mode (Rx4WD) //////////////////////////////////////////////
#define HCON_Rx1WD           0x00000000	//  0 = 1-word mode selected
#define HCON_Rx4WD           0x00000800	//  1 = 4-word mode selected

// [13:12] Rx widget alignment (RxWA) //////////////////////////////////////////
#define HCON_WIDGETMASK      0x00003000	
#define HCON_RxWA0           0x00000000	// 00 = No Invalid bytes
#define HCON_RxWA1           0x00001000	// 01 = 1 invalid byte
#define HCON_RxWA2           0x00002000	// 10 = 2 invalid bytes
#define HCON_RxWA3           0x00003000	// 11 = 3 invalid bytes
					
#define HCON_DTxSTSK         0x00004000	// [14] DMA Tx stop or skip (DTxSTSK)
#define HCON_DRxSTSK         0x00008000	// [15] DMA Rx stop or skip (DRxSTSK)
#define HCON_DRxMADEC        0x00010000	// [16] DMA Rx memory address decrement (DRxMADEC)
#define HCON_TxFLAG          0x00020000	// [17] Tx flag idle (TxFLAG)
#define HCON_TxSFLAG         0x00040000	// [18] Tx single flag (TxSFLAG)
#define HCON_TxLOOP          0x00080000	// [19] Tx loop-back mode (TxLOOP)
#define HCON_RxECHO          0x00100000	// [20] Rx echo mode (RxECHO)
#define HCON_TxABTEXT        0x00200000	// [21] Tx abort extension (TxABTEXT)
#define HCON_TxABT           0x00400000	// [22] Tx abort (TxABT)
#define HCON_TxPRMB          0x00800000	// [23] Tx preamble (TxPRMB)
#define HCON_TxDTR           0x01000000	// [24] Tx data terminal ready (TxDTR)
#define HCON_RxDISCON        0x02000000	// [25] Rx frame discontinue (RxDISCON)
#define HCON_TxNOCRC         0x04000000	// [26] Tx no CRC (TxNOCRC)
#define HCON_RxNoCRC         0x08000000	// [27] Rx no CRC (RxNOCRC)
#define HCON_AutoEN          0x10000000	// [28] Auto enable (AutoEN)

#ifdef _S3C4530_
#define HCON_TRxSTOP         0x20000000	// [29] Transparent Rx stop () S3C4530
#define HCON_TxREV           0x40000000	// [30] Transmit reverse (TxREV) S3C4530
#define HCON_RxREV           0x80000000	// [31] Receive reverse (RxREV) S3C4530
#endif							/* _S3C4530_ */


#define HSTATA     (VPint(Base_Addr+0x7008))	/* HDLC Channel A Status Register */
#define HSTATB     (VPint(Base_Addr+0x8008))	/* HDLC Channel B Status Register */

#define HSTAT(ch)  (VPint(Base_Addr+0x7008 + ch*0x1000))

/************************************************************************/
/* HSTATA  HDLC Channel A Status Register  				*/
/*         Offset      = 0x7008 					*/
/*         Reset value = 0x00000000 					*/
/* HSTATB  HDLC Channel B Status Register 				*/
/*         Offset      = 0x8008						*/
/*         Reset Value = 0x00000000 					*/
/* 									*/
/* [3:0]   Rx remaining bytes (RxRB) 					*/
/*         (RxRB + 1) indicates how many data bytes are valid in a 	*/
/*	   1-word or 4-word boundary when the receiver has received a 	*/
/*	   complete frame. In 1-word transfer mode, the RxRB value 	*/
/*	   is either 0, 1, 2, or 3. In 4-word mode, it is 0, 1, ..., 	*/
/* 	   14, or 15. 			      				*/
/* 									*/
/* [4]     Tx frame complete (TxFC) 					*/
/*         This status bit is automatically set to '1' when the two 	*/
/*         conditions are met: 						*/
/*          1) there is no data in the Tx FIFO, and 			*/
/*          2) either an abort or a closing flag is transmitted.        */
/* 									*/
/*         You can clear this bit by writing '1' to this bit. 		*/
/* 									*/
/* [5]     Tx FIFO available  (TxFA) 					*/
/*          0 = Tx FIFO is not available. 				*/
/*          1 = Tx FIFO is available. (that is, the data to be 		*/
/*	        transmitted can now be loaded into the Tx FIFO.) 	*/
/* 									*/
/*         If this bit is '1', the data to be sent can be loaded into	*/
/*	   the HTxFIFO register. In 1-word transfer mode, the TxFA 	*/
/*	   status bit is set to '1' when the first register of the 	*/
/*	   HTxFIFO is empty.        					*/
/* 									*/
/*         In 4-word transfer mode, TxFA = '1' when the first four 	*/
/*	   32-bit registers of the HTxFIFO are empty. The TxFA status	*/
/*	   condition is automatically cleared when HTxFIFO is no longer */
/*         available. During DMA Tx operation, this bit is always 0,so 	*/
/*	   not generating interrupt.					*/
/* 									*/
/* [6]     Tx clear-to-send (TxCTS) 					*/
/*          0 = Level at the nCTS input pin is High.                    */
/*          1 = Level at the nCTS input pin is Low.                     */
/*         The nCTS input is projected to this status bit. 		*/
/* 									*/
/*         If the level at the nCTS input pin is Low, this status bit 	*/
/*	   is'1'. If nCTS input pin is High level, TxCTS is '0'. 	*/
/*         This bit does not generate an interrupt. 			*/
/* 									*/
/* [7]     Tx stored clear-to-send (TxSCTS) 				*/
/*          0 = Normal operation 					*/
/*          1 = A transition occurred at the nCTS input. (This 		*/
/* 		transitioncan be used to trigger an interrupt) 		*/
/* 									*/
/*         This bit is set to '1' each time a transition in nCTS input  */
/*         occurs. You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [8]      Tx underrun (TxU) 					        */
/*          When the transmitter runs out of data during a frame 	*/
/*	    transmission, an underrun occurs and the frame is 		*/
/*	    automatically terminated by transmitting an abort sequence. */
/*          The underrun condition is indicated when TxU is '1'. 	*/
/*          You can clear this bit by writing a '1' to this bit. 	*/
/* 									*/
/* [9]      Rx FIFO available (RxFA) 					*/
/*           0 = Normal operation 					*/
/*           1 = Data is available in the RxFIFO. 			*/
/* 									*/
/*          This status bit indicates when the data received can be 	*/
/*	    read from the Rx FIFO. When RxFA is '1', it indicates that 	*/
/*	    data (other than an address or a final data word) is 	*/
/*	    available in the HRXFIFO. In 1-word transfer mode, RxFA 	*/
/*	    bit set to 1 when received data is available in the last 	*/
/*	    FIFO register. In 4-word transfer mode, it is set to 1 	*/
/*	    when the data received is available in the last four 32-bit */
/*    	    FIFO registers. Even if the data reside in FIFO for only 	*/
/*	    two words, when the Last bit is set, Rx FIFO is regarded 	*/
/*          as valid. (The received data available condition is cleared */
/*          automatically when the data received is no longer 		*/
/*	    available.)     						*/
/* 								        */
/*          During DMA Rx operation, this bit is always 0, so does  	*/
/*          not generate an interrupt. 					*/
/* 									*/
/* [10]     Reserved Not applicable. KS32C50100 			*/
/* [10]     Tx Frame Good (TxFG) S3C4530 				*/
/*          This bit set to one when one frame sent well. 		*/
/* 									*/
/* [11]     Rx flag detected (RxFD) 					*/
/*          This bit is set to 1 when the last bit of the flag 		*/
/*	    sequence is received. This bit generates an interrupt if 	*/
/*	    enabled.             					*/
/* 									*/
/*          You can clear this bit by writing a '1' to this bit. 	*/
/* 									*/
/* [12]     Rx data carrier detected (RxDCD) 				*/
/*           0 = nDCD input pin is High. 				*/
/*           1 = nDCD input pin is Low. 				*/
/*          The DCD status bit mirrors the state of the nDCD input pin. */
/*	    If nDCD input pin is low, this status bit is '1'. If nDCD 	*/
/*	    input pin is High, it is '0'. This bit does not generate 	*/
/*	    an interrupt.     						*/
/* 									*/
/* [13]     Rx stored data carrier detected (RxSDCD) 			*/
/*           0 = Normal operation. 					*/
/*           1 = When a transition of the nDCD input occurs, this bit 	*/
/*	         is set. 						*/
/* 									*/
/*          This bit is set to '1' when a transition in nDCD input 	*/
/*	    occurs, and can generate interrupt, if enabled. You can 	*/
/*	    clear this bit by writing a '1' to this bit.  		*/
/* 									*/
/* [14]     Rx frame valid (RxFV) 					*/
/*           0 = Normal operation. 					*/
/*           1 = The last data byte of a frame is transferred into the	*/
/*	         last   						*/
/*               location of RxFIFO. 					*/
/* 									*/
/*          This bit signals frame's ending boundary to the CPU and also*/
/*          indicates that no frame error occurred. It is set when the	*/
/*	    last data byte of a frame is transferred into the last 	*/
/*	    location of the Rx FIFO and is available to be read. 	*/
/* 									*/
/* [15]     Rx idle (RxIDLE) 						*/
/*           0 = Normal operation. 					*/
/*           1 = A minimum 15 consecutive 1s have been received. 	*/
/* 									*/
/*          The RxIDLE status bit indicates that a minimum of 15 	*/
/*	    consecutive 1s have been received. The event is stored 	*/
/*	    in the status register and can be used to trigger a 	*/
/*	    receiver interrupt. The RxIDLE bit continues to reflect 	*/
/*	    the inactive idle condition until a '0' is received. 	*/
/* 									*/
/*          You can clear this bit by writeing a '1' to this bit.       */
/* 									*/
/* [16]     Rx abort (RxABT)						*/
/*           0 = Normal operation. 					*/
/*           1 = Seven or more consecutive 1s have been received, 	*/
/*	         in-frame condition.					*/
/* 									*/
/*          The RxABT status bit is set to '1' when seven    		*/
/*          or more consecutive 1s (abort sequence) have been received. */
/*          When an abort is received in an 'in-frame' condition, the 	*/
/*	    event is stored in the status register triggering an 	*/
/*	    interrupt request. You can clear this bit by writeing a 	*/
/*	    '1' to this bit.             				*/
/* 									*/
/* [17]     Rx CRC error (RxCRCE) 					*/
/*           0 = Normal operation. 					*/
/*           1 = A frame Rx operation is completed with a CRC error.    */
/* 									*/
/*          The RxCRCE status bit is set a frame is completed with a 	*/
/*	    CRC error							*/
/* 									*/
/* [18]     Rx non-octet align (RxNO) 					*/
/*           0 = Received frame is octet.	 			*/
/*           1 = Received frame is not octet. 				*/
/* 									*/
/*          The RxNO bit is set to '1', if received data is non-octet 	*/
/*	    aligned frame. 						*/
/* 									*/
/* [19]     Rx overrun (RxOV) 						*/
/*          The RxOV status bit is set to '1', if the data received is  */
/*          transferred into the HRXFIFO when it is full, resulting in 	*/
/*	    a loss of data. Continued overruns destroy data in the 	*/
/*	    first FIFO register. 					*/
/* 									*/
/* [20]     DMA Rx memory overflow (RxMOV) 				*/
/*           0 = Normal operation. 					*/
/*           1 = Indicates memory overflow when Rx buffer descriptor 	*/
/*		 next pointer has null address. 			*/
/* 									*/
/*          This bit is set when there is no more buffer during 	*/
/*	    receiving data. If this bit is set, DRxEN bit is cleared. 	*/
/* 									*/
/*          You can clear This bit by writeing '1' to this bit. 	*/
/* 									*/
/* [21]     Reserved. Not applicable. 					*/
/* 									*/
/* [22]     DMA Tx abort (DTxABT) 					*/
/*           0 = Normal operation. 					*/
/*           1 = Abort signal is sended and DMA Tx enable bit is 	*/
/*	         cleared.     						*/
/* 									*/
/*          This bit is set to 1 when abort signal is sended due to 	*/
/*	    the Tx underrun or CTS lost occurred. If this bit is set, 	*/
/*	    DTxEN(in HCON) bit cleared. 				*/
/* 									*/
/*          You can clear this bit by writeing '1' to this bit. 	*/
/* 									*/
/* [23]     Rx internal error (RxIERR) 					*/
/*           0 = Normal operation. 					*/
/*           1 = Received frame is not stable due to receive clock is 	*/
/*		 unstable						*/
/* 									*/
/*          This bit is set to '1' when received frame will be detected */
/*	    error possibility due to the receive clock is unstable. 	*/
/* 									*/
/* [24]     DMA Rx frame done every received frame (DRxFD)              */
/*           0 = Normal operation. 					*/
/*           1 = DMA Rx operation has successfully transferred a frame  */
/*	         from RxFIFO to buffer memory. 				*/
/* 									*/
/*          This bit is set when a DMA Rx operation has successfully 	*/
/*	    operated a frame to memory from HRXFIFO, and when the last  */
/*	    byte of a frame has been written to memory. 		*/
/*          This bit generate interrupt when set to '1' to know a 	*/
/*	    frame is received. 						*/
/*  	    You can clear this bit by writing '1' to this bit. 		*/
/* 									*/
/* [25]     DMA Rx null list (DRxNL)  					*/
/*           0 = Normal operation. 					*/
/*           1 = DMA Rx buffer descriptor pointer has a null address.   */
/* 									*/
/*          If this bit is set, the DMA Rx buffer descriptor pointer 	*/
/*	    has a null address. In this case, DMA Rx is disabled and 	*/
/*	    the data transfer from the Rx FIFO to buffer memory is 	*/
/*	    discontinued. So the HRXFIFO is cleared. 			*/
/*          You can clear this bit by writing '1' to this bit. 		*/
/* 									*/
/* [26]     DMA Rx not owner (DRxNO) 					*/
/*           0 = DMA has the ownership. 				*/
/*           1 = CPU has the ownership. 				*/
/* 									*/
/*          This bit is set, when DMA is not owner of the current 	*/
/*	    buffer descriptor, and DRxSTSK bit was set. In this case, 	*/
/*	    DMA Rx is disabled and can generate interrupt, if enabled.  */
/*	    If DRxSTSK bit is zero, this bit is always zero. 		*/
/* 									*/
/*          You can clear this bit by writing '1' to this bit.          */
/* 									*/
/* [27]     DMA Tx frame done (DTxFD) 					*/
/*           0 = Normal operation. 					*/
/*           1 = DMA Tx operation has successfully transferred a frame  */
/*	         from memory to Tx FIFO. 				*/
/* 									*/
/*          This bit is set to '1' when DMA Tx operation has 		*/
/*	    successfully transferred a frame from memory to Tx FIFO. 	*/
/*          You can clear this bit by writing '1' to this bit.          */
/* 									*/
/* [28]     DMA Tx null list (DTxNL) 					*/
/*           0 = Normal operation 					*/
/*           1 = DMA Tx buffer descriptor pointer has a null address.   */
/* 									*/
/*          If this bit is set '1', the DMA Tx buffer descriptor 	*/
/*	    pointer has a null address. In this case, DMA Tx is 	*/
/*	    disabled and the data to be transferred discontinued from 	*/
/*	    the buffer memory to Tx FIFO. You can clear this bit by 	*/
/*	    writing '1' to this bit. 		     			*/
/* 									*/
/* [29]     DMA Tx not owner (DTxNO) 					*/
/*           0 = DMA has the ownership. 				*/
/*           1 = CPU has the ownership. 				*/
/* 									*/
/*          This bit is set, when DMA is not owner of the current 	*/
/*	    buffer descriptor, and DTxSTSK bit was set. In this case, 	*/
/*	    DMA Tx disabled and can generate interrupt, if enabled. 	*/
/*	    If DTxSTSK bit is zero, this bit is always zero. 		*/
/*          You can  clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [30]     DPLL one clock missing (DPLLOM) 				*/
/*           0 = Normal operation 					*/
/*           1 = Set in FM/Manchester mode when DPLL does not detect 	*/
/*	         an edge on the first entry. 				*/
/* 									*/
/*          When operating in FM/Manchester mode, the DPLL sets this 	*/
/*	    bit to '1' if it does not detect an edge in its first 	*/
/*	    attempt. You can clear this bit by writing a '1' to 	*/
/*	    this bit. 		      					*/
/* 									*/
/* [31]     DPLL two clock missing (DPLLTM) 				*/
/*           0 = Normal operation 					*/
/*           1 = DPLL was not detected on two consecutive edges and 	*/
/*	         search mode was entered. 				*/
/* 									*/
/*          When it is operating in the FM/Manchester mode, the DPLL 	*/
/*	    sets this bit to '1' if it does not detect an edge in two 	*/
/*	    successive attempts. At the same time the DPLL enters 	*/
/*	    Search mode. In NRZ/NRZI mode, and while the DPLL is 	*/
/*	    disabled, this bit is always '0'. You can clear this bit 	*/
/*	    by writing a '1' to this bit.  				*/
/* 									*/
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// HDLC Status Register (HSTAT) ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// [3:0]   Rx remaining bytes (RxRB) ///////////////////////////////////////////

#define HSTAT_RxRB1   	     0x00000000	// 0 bytes valid
#define HSTAT_RxRB2   	     0x00000001	// 1 bytes valid  
#define HSTAT_RxRB3   	     0x00000002	// 2 bytes valid	
#define HSTAT_RxRB4   	     0x00000003	// 3 bytes valid	

#define HSTAT_RxRB   	     0x0000000F	// remain bytes valid

// (RxRB + 1) indicates how many data bytes are valid in a 1-word or 
// 4-word boundary when the receiver has received a complete frame.
// In 1-word transfer mode, the RxRB value is either 0, 1, 2, or 3.
// In 4-word mode, it is 0, 1, ..., 14, or 15.

#define HSTAT_TxFC    	     0x00000010 //  [4] Tx frame complete (TxFC)
#define HSTAT_TxFA           0x00000020	//  [5] Tx FIFO available  (TxFA)
#define HSTAT_TxCTS          0x00000040	//  [6] Tx clear-to-send (TxCTS)
#define HSTAT_TxSCTS         0x00000080	//  [7] Tx stored clear-to-send (TxSCTS)
#define HSTAT_TxU            0x00000100	//  [8] Tx underrun (TxU)
#define HSTAT_RxFA           0x00000200	//  [9] Rx FIFO available (RxFA)

#ifdef _S3C4530_
#define HSTAT_TxFG           0x00000400	//  [10] Tx Frame Good (TxFG) S3C4530 
#endif 								/* _S3C4530_ */

#define HSTAT_RxFD           0x00000800	// [11] Rx flag detected (RxFD)
#define HSTAT_RxDCD          0x00001000	// [12] Rx data carrier detected (RxDCD)
#define HSTAT_RxSDCD         0x00002000	// [13] Rx stored data carrier detected (RxSDCD)
#define HSTAT_RxFV           0x00004000	// [14] Rx frame valid (RxFV)
#define HSTAT_RxIDLE         0x00008000	// [15] Rx idle (RxIDLE)
#define HSTAT_RxABT          0x00010000	// [16] Rx abort (RxABT)
#define HSTAT_RxCRCE         0x00020000	// [17] Rx CRC error (RxCRCE)
#define HSTAT_RxNO	     0x00040000	// [18] Rx non-octet align (RxNO)
#define HSTAT_RxOV           0x00080000	// [19] Rx overrun (RxOV)
#define HSTAT_RxMOV          0x00100000	// [20] DMA Rx memory overflow (RxMOV)
#define HSTAT_DTxABT         0x00400000	// [22] DMA Tx abort (DTxABT)
#define HSTAT_RxIERR         0x00800000	// [23] Rx internal error (RxIERR)
#define HSTAT_DRxFD          0x01000000	// [24] DMA Rx frame done every received frame (DRxFD)
#define HSTAT_DRxNL          0x02000000	// [25] DMA Rx null list (DRxNL)
#define HSTAT_DRxNO          0x04000000	// [26] DMA Rx not owner (DRxNO)
#define HSTAT_DTxFD          0x08000000	// [27] DMA Tx frame done (DTxFD)
#define HSTAT_DTxNL          0x10000000	// [28] DMA Tx null list (DTxNL)
#define HSTAT_DTxNO          0x20000000	// [29] DMA Tx not owner (DTxNO)
#define HSTAT_DPLLOM         0x40000000	// [30] DPLL one clock missing (DPLLOM)
#define HSTAT_DPLLTM         0x80000000	// [31] DPLL two clock missing (DPLLTM)


#define HINTENA    (VPint(Base_Addr+0x700c))	/* HDLC Channel A Interrupt Enable Register */
#define HINTENB    (VPint(Base_Addr+0x800c))	/* HDLC Channel B Interrupt Enable Register */

#define HINTEN(ch) (VPint(Base_Addr+0x700c + ch*0x1000)) 


/************************************************************************/
/* HINTENA  HDLC Channel A Interrupt Enable Register                    */
/*          Offset      = 0x700c 					*/
/*          Reset Value = 0X00000000 					*/
/* HINTENB  HDLC Channel B Interrupt Enable Register                    */
/*          Offset      = 0x800c 					*/
/*          Reset Value = 0x00000000 					*/
/* 									*/
/* [3:0]    Reserved 							*/
/* [4]      TxFCIE Tx frame complete interrupt enable 			*/
/* [5]      TxFAIE Tx FIFO available to write interrupt enable 	        */
/* [6]      Reserved  							*/
/* [7]      TxSCTSIE CTS transition has occurred interrupt enable 	*/
/* [8]      TxUIE Tx underrun has occurred interrupt enable 		*/
/* [9]      RxFAIE Rx FIFO available to read interrupt enable 		*/
/* [10]     Reserved Ks32C50100 					*/
/* [10]     TxFGIE Tx Frame Good Int Enable 				*/ 
/* [11]     RxFDIE Rx flag detected interrupt enable 			*/
/* [12]     Reserved 							*/
/* [13]     RxSDCDIE DCD transition interrupt enable 			*/
/* [14]     RxFVIE Rx frame valid interrupt enable 			*/
/* [15]     RxIDLEIE Idle detected interrupt enable 			*/
/* [16]     RxABTIE Abort detected interrupt enable 			*/
/* [17]     RxCRCEIE CRC error frame interrupt enable 			*/
/* [18]     RxNOIE Non-octet aligned frame interrupt enable 		*/
/* [19]     RxOVIE Rx overrun interrupt enable  			*/
/* [20]     RxMOVIE Rx memory overflow interrupt enable 		*/
/* [21]     Reserved 							*/
/* [22]     DTxABTIE DMA Tx abort interrupt enable 			*/
/* [23]     RxIERRIE Rx internal error interrupt enable 		*/
/* [24]     DRxFDIE DMA Rx frame done interrupt enable 			*/
/* [25]     DRxNLIE DMA Rx null list interrupt enable 			*/
/* [26]     DRxNOIE DMA Rx not owner interrupt enable 			*/
/* [27]     DTxFDIE DMA Tx frame done every transmitted frame interrupt */
/*          enable							*/
/* [28]     DTxNLIE DMA Tx null list interrupt enable 			*/
/* [29]     DTxNOIE DMA Tx not owner interrupt enable 			*/
/* [30]     DPLLOMIE DPLL one clock missing interrupt enable 	        */
/* [31]     DPLLTMIE DPLL two clocks missing interrupt enable           */
/************************************************************************/

////////////////////////////////////////////////////////////////////////////////
// HDLC Interrupt Enable Register (HINTEN) /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define HINTEN_TxFCIE        0x00000010	// [4] Tx frame complete interrupt enable
#define HINTEN_TxFAIE        0x00000020	// [5] Tx FIFO available to write interrupt enable
#define HINTEN_TxSCTSIE      0x00000080	// [7] CTS transition has occurred interrupt enable
#define HINTEN_TxUIE         0x00000100	// [8] Tx underrun has occurred interrupt enable
#define HINTEN_RxFAIE        0x00000200	// [9] Rx FIFO available to read interrupt enable
#ifdef _S3C4530_
#define HINTEN_TxFGIE        0x00000400	// [10] Tx Frame Good (TxFG) S3C4530 
#endif 								/* _S3C4530_ */
#define HINTEN_RxFDIE        0x00000800	// [11]Rx flag detected interrupt enable
#define HINTEN_RxSDCDIE      0x00002000	// [13]DCD transition interrupt enable
#define HINTEN_RxFVIE        0x00004000	// [14]Rx frame valid interrupt enable
#define HINTEN_RxIDLEIE      0x00008000	// [15]Idle detected interrupt enable
#define HINTEN_RxABTIE       0x00010000	// [16]Abort detected interrupt enable
#define HINTEN_RxCRCEIE      0x00020000	// [17]CRC error frame interrupt enable
#define HINTEN_RxNOIE        0x00040000	// [18]Non-octet aligned frame interrupt enable
#define HINTEN_RxOVIE        0x00080000	// [19]Rx overrun interrupt enable
#define HINTEN_RxMOVIE       0x00100000	// [20]Rx memory overflow interrupt enable
#define HINTEN_DTxABTIE      0x00400000	// [22]DMA Tx abort interrupt enable
#define HINTEN_RxIERRIE      0x00800000	// [23]Rx internal error interrupt enable
#define HINTEN_DRxFDIE       0x01000000	// [24]DMA Rx frame done interrupt enable
#define HINTEN_DRxNLIE       0x02000000	// [25]DMA Rx null list interrupt enable
#define HINTEN_DRxNOIE       0x04000000	// [26]DMA Rx not owner interrupt enable
#define HINTEN_DTxFDIE       0x08000000	// [27]DMA Tx frame done every transmitted frame interrupt enable
#define HINTEN_DTxNLIE       0x10000000	// [28]DMA Tx null list interrupt enable
#define HINTEN_DTxNOIE       0x20000000	// [29]DMA Tx not owner interrupt enable
#define HINTEN_DPLLOMIE      0x40000000	// [30]DPLL one clock missing interrupt enable
#define HINTEN_DPLLTMIE      0x80000000	// [31]DPLL two clocks missing interrupt enable


#define HTXFIFOCA    (VPint(Base_Addr+0x7010))	/* HDLC Channel A FIFO Continue Register */
#define HTXFIFOCB    (VPint(Base_Addr+0x8010))  /* HDLC Chennel B FIFO Continue Register */

#define HTXFIFOC(ch) (VPint(Base_Addr+0x7010 + ch*0x1000)) 

/******************************************************************************/ 

#define HTXFIFOTA    (VPint(Base_Addr+0x7014))  /* HDLC Channel A FIFO Terminate Register */
#define HTXFIFOTB    (VPint(Base_Addr+0x8014))  /* HDLC Channel B FIFO Terminate Register */

#define HTXFIFOT(ch) (VPint(Base_Addr+0x7014 + ch*0x1000)) 

/******************************************************************************/ 

#define HRXFIFOA     (VPint(Base_Addr+0x7018))	
#define HRXFIFOB     (VPint(Base_Addr+0x8018))	

#define HRXFIFO(ch)  (VPint(Base_Addr+0x7018 + ch*0x1000))

/******************************************************************************/ 

#define HBRGTCA	     (VPint(Base_Addr+0x701c))	
#define HBRGTCB	     (VPint(Base_Addr+0x801c))	

#define HBRGTC(ch)   (VPint(Base_Addr+0x701c + ch * 0x1000))

/************************************************************************/
/* HBRGTCA  HDLC Channel A BRG Time Constant Register 			*/
/*          Offset      : 0x701c 					*/
/*          Reset Value : 0x00000000 					*/
/* HBRGTCB  HDLC Channel B BRG Time Constant Register 			*/
/*          Offset      : 0x801c   					*/
/*          Reset value : 0x00000000 					*/
/* 								        */
/* [1:0]    Time constant value for CNT2 				*/
/*           00 = divide by 1 						*/
/*           01 = divide by 16 						*/
/*           10 = divide by 32 						*/
/* [3:2]    Time constant value for CNT1 				*/
/*           00 = divide by 1 						*/
/*           01 = divide by 16 						*/
/* [15:4]   Time constant value for CNT0 				*/
/* 								        */
/* BRGOUT1 = (MCLK2 or RXC) / (CNT0 + 1) / (16 ** CNT1 ) 	        */
/* BRGOUT2 = BRGOUT1 / (1 or 16 or 32 according to CNT2 value of the 	*/
/*           HBRGTC)  							*/
/************************************************************************/ 

#define HPRMBA	   (VPint(Base_Addr+0x7020))	/* HDLC Channel A Preamble Register */
#define HPRMBB	   (VPint(Base_Addr+0x8020))    /* HDLC Channel B Preamble Register */

#define HPRMB(ch) (VPint(Base_Addr+0x7020 + ch*0x1000))

/************************************************************************/
/* HPRMBA    HDLC Channel A Preamble Constant Register 			*/
/*           Offset      : 0x7020 					*/
/*           Reset Value : 0x00000000 					*/
/* HPRMBB    HDLC Channel B Preamble Constant Register                  */
/*           Offset      : 0x8020 					*/
/*           Reset Value : 0x00000000 					*/
/* 									*/
/* [7:0]     Preamble Pattern 						*/
/************************************************************************/

#define HSAR0A 	   (VPint(Base_Addr+0x7024))	/* HDLC A station address 0 */
#define HSAR1A	   (VPint(Base_Addr+0x7028))	/* HDLC A station address 1 */
#define HSAR2A	   (VPint(Base_Addr+0x702c))	/* HDLC A station address 2 */
#define HSAR3A	   (VPint(Base_Addr+0x7030))	/* HDLC A station address 3 */
	
#define HSAR0B 	   (VPint(Base_Addr+0x8024))	/* HDLC B station address 0 */
#define HSAR1B	   (VPint(Base_Addr+0x8028))	/* HDLC B station address 1 */
#define HSAR2B	   (VPint(Base_Addr+0x802c))	/* HDLC B station address 2 */
#define HSAR3B	   (VPint(Base_Addr+0x8030))	/* HDLC A station address 3 */

#define HMASKA 	   (VPint(Base_Addr+0x7034))	/* HDLC A address mask */
#define HMASKB 	   (VPint(Base_Addr+0x8034))	/* HDLC B address mask */

#define HSAR0(ch)  (VPint(Base_Addr+0x7024 + ch*0x1000))
#define HSAR1(ch)  (VPint(Base_Addr+0x7028 + ch*0x1000))
#define HSAR2(ch)  (VPint(Base_Addr+0x702c + ch*0x1000))
#define HSAR3(ch)  (VPint(Base_Addr+0x7030 + ch*0x1000))
#define HMASK(ch)  (VPint(Base_Addr+0x7034 + ch*0x1000))
	

/************************************************************************/
/* Register * Offset * Reset      * Description 			*/
/* Name	    *	     * Value      *					*/
/************************************************************************/ 
/* HSADR0A  * 0x7024 * 0X00000000 * HDLC station address 0 		*/
/* HSADR1A  * 0x7028 * 0X00000000 * HDLC station address 1 		*/
/* HSADR2A  * 0x702c * 0X00000000 * HDLC station address 2 		*/
/* HSADR3A  * 0x7030 * 0X00000000 * HDLC station address 3 		*/
/* HMASKA   * 0x7034 * 0X00000000 * HDLC address mask register 		*/
/* HSADR0B  * 0x8024 * 0X00000000 * HDLC station address 0 		*/
/* HSADR1B  * 0x8028 * 0X00000000 * HDLC station address 1 		*/
/* HSADR2B  * 0x802c * 0X00000000 * HDLC station address 2 		*/
/* HSADR3B  * 0x8030 * 0X00000000 * HDLC station address 3 		*/
/* HMASKB   * 0x8034 * X000000000 * HDLC address mask register		*/
/* 									*/
/* Station address byte register and MASK register 			*/
/*  [31:24] First address byte 						*/
/*  [23:16] Second address byte 					*/
/*  [15:8]  Third address byte 						*/
/*  [7:0]   Fourth address byte 					*/
/************************************************************************/ 


#ifdef _S3C4530_
/************************************************************************/ 
/* HSYNCA 	HDLC A Sync Register 					*/
/*              Offset=0x7048						*/
/*              Reset Value=0x7E					*/
/* HSYNCB 	HDLC B Sync Register 					*/
/*              Offset=0x8048						*/
/* 		Reset Value=0x7E 					*/
/************************************************************************/ 

#define HSYNCA        (VPint(Base_Addr+0x7048))	/* HDLC A Sync Register */
#define HSYNCB        (VPint(Base_Addr+0x8048))	/* HDLC B Sync Register */
#endif							/* _S3C4530_ */ 

#ifdef _S3C4530_ 
/************************************************************************/ 
/* TCONA 	Transparent Control Register 				*/
/* 		Offset=0x704C, Reset Value=0xXXXX0000			*/
/* TCONB 	Transparent Control Register 				*/
/* 		Offset=0x804C, Reset Value=0xXXXX0000 			*/
/* 									*/
/* [1:0] 	Data sampling (DS) 					*/
/* 	 	These bit values determine which data bits are 		*/
/* 		regarded as valid after the nDCD state active. 		*/
/* 		  00 = the first valid bit is D4			*/
/* 		  01 = the first valid bit is D3			*/
/* 		  10 = D2						*/ 
/* 		  11 = D1						*/
/* 									*/
/*               +--+  +--+  +--+  +--+	+--+  +-- 			*/
/*           RXC |  |  |  |  |  |  |  |  |  |  |			*/
/*                  +--+  +--+  +--|  +--+  +--+ 			*/
/*                                 |  					*/
/*                   ____  ____  ____  ____  ____  ____			*/
/*                  / D1 \/ D2 \/ D3 \/ D4 \/ D5 \/    \		*/
/*                  \____/\____/\____/\____/\____/\____/  		*/
/*									*/
/*           nDCD----------------+					*/
/*                                \______________			*/
/* [3:2] 	Reserved Not applicable. 				*/
/* [4] 		RTS control(RTS) It this bit set to one, the nRTS pin   */
/*              goes Low. 						*/
/* [31:5] 	Reserved Not applicable. 				*/
/************************************************************************/ 
#define TCONA  (VPint(Base_Addr+0x704C)) /* Transparent Control Register */
#define TCONB  (VPint(Base_Addr+0x804C)) /* Transparent Control Register */

#define TCON_DS        0x00000003	/* Data Sampling (DS) */
#define TCON_DS_D4     0x00000000
#define TCON_DS_D3     0x00000001
#define TCON_DS_D2     0x00000002
#define TCON_DS_D1     0x00000003

#define TCON_RTS       0x00000010	/* RTS Pin Low */
#endif 							/* _S3C4530_ */

#ifdef _S3C4530_
/************************************************************************/ 
/* TSACFG 	TSA Configuration Register 				*/
/* 		Offset=0x7800, Reset Value=0x00000000 			*/
/* 									*/
/* [0] 	TSA Enable (TSAEN) 			`			*/
/* 	You can enable the Time Slot Assignor by setting this bit as 1. */
/* [1]  A mode select (AMODE) 						*/
/*      HDLC channel A can be used as a DCE mode or PCM highway. 	*/
/*      If you set this bit, A-channel will be assigned as PCM highway. */
/* [2]  B mode select(BMODE) 						*/
/*      HDLC channel B can be used as a DCE mode or PCM highway. 	*/
/*      If you set this bit, A-channel will be assigned as PCM highway. */
/* [3]  Clock divide (CLKDIV) 						*/
/*      Using this, the data clock (DCL) can be divided by 2. If you set*/
/*      as 1, the data clock will be a rate divided by 2 of the RxC pin.*/
/* [31:4] Reserved Not applicable. 					*/
/************************************************************************/ 
#define TSACFG    (VPint(Base_Addr+0x7800)) /* TSA Configuration register */

#define TSACFG_TSAEN  0x00000001 	/* TSA Enable */
#define TSACFG_AMODE  0x00000002 	/* AMODE */
#define TSACFG_BMODE  0x00000004 	/* BMODE */
#define TSACFG_CLKDIV 0x00000008 	/* */
#endif 							/* _S3C4530_ */

#ifdef _S3C4530_
/************************************************************************/ 
/* TSAACFG 	TSA A Time Slot Register 				*/
/* 		Offset=0x7804, Reset Value=0x00000000 			*/
/* TSABCFG 	TSA B Time Slot Register 				*/
/*              Offset=0x7808, Reset Value=0x00000000 			*/
/* [11:0] 	START 							*/
/* 		The location of start bit at which the HDLCx time slot  */
/* 		assigned 						*/
/* [15:12] 	Reserved Not applicable. 				*/
/* [27:16] 	STOP 							*/
/* 		The location of (stop+1) bit at which the HDLCx time 	*/
/* 		slot assigned For instance, if you want to stop the TSA */
/* 		at 7-th bit, then you should set it as the value 8. 	*/
/* [31:28] 	Reserved Not applicable. 				*/
/************************************************************************/ 
#define TSAACFG    (VPint(Base_Addr+0x7804)) /* TSA A Time Slot Configuration Register */
#define TSABCFG    (VPint(Base_Addr+0x7808)) /* TSA B Time Slot Configuration Reg */

#define TSACFG_START_OFF  	   0		/* start field offset */
#define TSACFG_START_MASK 0x00000FFF		/* start field mask */

#define TSACFG_STOP_OFF      	  16		/* stop field offset */
#define TSACFG_STOP_MASK  0x0FFF0000		/* stop field mask */ 
#endif 							/* _S3C4530_ */

#define HDMATXPTRA    (VPint(Base_Addr+0x7038))	/* HDLC A DMA Tx Buffer Descriptor Pointer */
#define HDMATXPTRB    (VPint(Base_Addr+0x8038))	/* HDLC B DMA Tx Buffer Descriptor Pointer */

#define HDMATXPTR(ch) (VPint(Base_Addr+0x7038 + ch*0x1000))

/************************************************************************/
/* HDMATXPTRA DMA Channel A Tx Buffer Descriptor Pointer                */
/*            Offset      : 0x7038 					*/
/*            Reset Value : 0xFFFFFFFF 					*/
/* HDMATXPTRB DMA Channel A Tx Buffer Descriptor Pointer 		*/
/*            Offset      : 0x8038 					*/
/*            Reset Value : 0xFFFFFFFF 					*/
/* 									*/
/* [25:0]     DMA Tx Buffer Descriptor Pointer 				*/
/************************************************************************/

/************************************************************************/
/* HDLC Transmit Descriptor  						*/
/* +---+--------------------------------------------------------+       */
/* | O |                 Buffer Data Pointer                    |	*/
/* +---+--------------------------------+---+---+---+---+---+---+    	*/
/* |           Reserved                 | WA| D | L | E | N | P |       */
/* +----------+---+-----------+---------+---+---+---+---+---+---+       */
/* | Reserved | T |           |        Buffer Length            |       */
/* +----------+---+-----------+---------------------------------+       */
/* |             Next Buffer Desriptor Pointer                  |       */
/* +------------------------------------------------------------+       */
/* 									*/
/* [30:0]   Buffer Data Pointer 					*/
/* [31]     Ownership (O) 						*/
/*           0 = CPU 							*/
/*           1 = DMA 							*/
/* 									*/
/* Tx Control Bits							*/
/* [0]      Preamble (P)     0 = No preamble 1 = Preamble 		*/
/* [1]      TxNoCRC Mode (N) 0 = CRC mode 1 = No CRC mode		*/
/* [2]      Little-Endian Mode (E) 0 = Big-Endian 1 = Little-Endian     */
/* [3]      Last (L)          						*/
/*           0 = This is not the last buffer in the frame. 		*/
/*           1 = This is the last buffer in the frame. 			*/
/* [4]      Buffer Data Pointer Decrement (D)				*/
/*           0 = Increment 						*/
/*           1 = Decrement						*/
/* [6:5]    Widget Aligment Control (WA) 				*/
/*           00 = No invalid bytes 					*/
/*           01 = 1 Invalid bytes 					*/
/*           10 = 2 invalid bytes 					*/
/*           11 = 3 invalid bytes 					*/
/* [23:0]   Buffer Length 						*/
/* 									*/
/* Tx Status Bit 							*/
/* These bit may be regarded as valid when the L bit (in Tx control bit)*/
/* is set. 								*/
/* [26]     Transmission Completion (T) 				*/
/*           0 = Normal 						*/
/*           1 = One frame completed. 					*/
/* [31:0]   Next Buffer Descriptor Pointer 				*/
/************************************************************************/ 

////////////////////////////////////////////////////////////////////////////////
// HDLC Tx Buffer Descriptor (HTxBD) ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define HDLCTxBD_CPU        0x7fffffff 
#define HDLCTxBD_DMA        0x80000000 // DMA owner bit
#define HDLCTxBD_Preamble   0x00000001 // [0]  Preamble (P) 0 = No preamble 1 = Preamble
#define HDLCTxBD_TxNoCRCM   0x00000002 // [1]  TxNoCRC Mode (N) 0 = CRC mode 1 = No CRC mode
#define HDLCTxBD_Little     0x00000004 // [2]  Little-Endian Mode (E) 0 = Big-Endian 1 = Little-Endian
#define HDLCTxBD_Last	    0x00000008 // [3]  Last (L)
#define HDLCTxBD_Dec	    0x00000010 // [4]  Buffer Data Pointer Decrement (D)
// [6:5]Widget Aligment Control (WA) ///////////////////////////////////////////
#define HDLCTxBD_WA0	    0x00000000 // 00 = No invalid bytes
#define HDLCTxBD_WA1	    0x00000020 // 01 = 1 Invalid bytes
#define HDLCTxBD_WA2	    0x00000040 // 10 = 2 invalid bytes
#define HDLCTxBD_WA3	    0x00000060 // 11 = 3 invalid bytes
#define HDLCTxBD_TxComp	    0x04000000 // [26] Transmission Completion (T)

#define HDMARXPTRA    (VPint(Base_Addr+0x703c))	/* HDLC A Rx buffer descriptor pointer */
#define HDMARXPTRB    (VPint(Base_Addr+0x803c))	/* HDLC B Rx buffer descriptor pointer */

#define HDMARXPTR(ch) (VPint(Base_Addr+0x703c + ch*0x1000))


/************************************************************************/
/* HDMARXPTRA DMA Channel A Rx Buffer Descriptor Pointer 		*/
/*            Offset      : 0x703c 					*/
/*            Reset Value : 0xFFFFFFFF  				*/
/* HDMARXPTRB DMA Channel B Rx Buffer Descriptor Pointer 		*/
/*            Offset      : 0x803c 					*/
/*            Reset Value : 0xFFFFFFFF 					*/
/* 									*/
/* [25:0]     DMA Rx Buffer Descriptor Pointer 				*/
/************************************************************************/ 

/************************************************************************/
/* HDLC Receive Frame Descriptor 					*/
/* +---+--------------------------------------------------------------+ */
/* | O |                        Buffer Data Pointer                   | */
/* +---+--------------------------------------------------------------+ */
/* |           Reserved                                               | */
/* +--------+---+-+-+---+---+--+--+--+--+-----------------------------+ */
/* |Reserved|FLV|L|F|ABT|DTM|OV|NO|CE|CD|        Buffer Length        | */
/* +--------+---+-+-+---+---+--+--+--+--+-----------------------------+ */
/* |                  Next Buffer Desriptor Pointer                   | */
/* +------------------------------------------------------------------+ */
/* 									*/
/* [30:0]   Buffer Data Pointer 					*/
/* [31]     Ownership (O) 						*/
/*           0 = CPU 							*/
/*           1 = DMA 							*/
/* [15:0]   Buffer Length 						*/
/*          Received buffer lengths are wrote to this buffer descriptor.*/
/* 									*/
/* Rx Status Bits 							*/
/*          These bits may be regarded as valid when L bit 		*/
/*          (in Rx status bit) is set. 					*/
/* 									*/
/* [16]     CD Lost (CD)						*/
/*           0 = Normal 						*/
/*           1 = CD lost occurs. 					*/
/* 									*/
/* [17]     CRC Error (CE) 						*/
/*           0 = Normal 						*/
/*           1 = CRC error occurs to the frame received. 		*/
/* 									*/
/* [18]     Non-octet Aligned Frame (NO) 				*/
/*           0 = Normal 						*/
/*           1 = Non-octet aligned frame is received. 			*/
/* 									*/
/* [19]     Overrun (OV) 						*/
/*           0 = Normal 						*/
/*           1 = The received frame overruns. 				*/
/* 									*/
/* [20]     DPLL Two Miss (DTM) 					*/
/*           0 = Normal 						*/
/*           1 = DPLL two miss clock occurs. 				*/
/* 									*/
/* [21]     Rx Abort (ABT) 						*/
/*           0 = Normal 						*/
/*           1 = The received frame aborted. 				*/
/* 									*/
/* [22]     First In Frame (F) 						*/
/*           0 = This buffer descriptor status is not the first 	*/
/*	         to the frame. 						*/
/*           1 = This buffer descriptor status is the first 		*/
/*	         to the frame.     					*/
/* 									*/
/* [23]     Last In Frame (L) 						*/
/*           0 = This buffer descriptor status is not the last 		*/
/*               to the frame.  					*/
/*           1 = This buffer descriptor status is the last 		*/
/*	         to the frame.      					*/
/* 									*/
/* [24]     Frame Length Violation (FLV) 				*/
/*           0 = Normal							*/
/*           1 = This received frame length exceeds the value of the 	*/
/*		 maximum frame length register. 			*/
/* 									*/
/* [31:0]   Next Buffer Descriptor Pointer 				*/
/*          The address of the next buffer descriptor 			*/
/************************************************************************/ 

////////////////////////////////////////////////////////////////////////////////
// HDLC Rx Buffer Descriptor (HRxBD) ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define HDLCRxBD_CPU        0x7fffffff 
#define HDLCRxBD_DMA        0x80000000 // DMA owner bit

#define HDLCRxBD_CD	    0x00000001 // [16] CD Lost (CD)
#define HDLCRxBD_CRC	    0x00000002 // [17] CRC Error (CE)
#define HDLCRxBD_NO         0x00000004 // [18] Non-octet Aligned Frame (NO)
#define HDLCRxBD_OV         0x00000008 // [19] Overrun (OV)
#define HDLCRxBD_DTM	    0x00000010 // [20] DPLL Two Miss (DTM)
#define HDLCRxBD_ABT	    0x00000020 // [21] Rx Abort (ABT)
#define HDLCRxBD_F	    0x00000040 // [22] First In Frame (F)
#define HDLCRxBD_L	    0x00000080 // [23] Last In Frame (L)
#define HDLCRxBD_FLV	    0x00000100 // [24] Frame Length Violation (FLV)

#define HMFLRA 	   (VPint(Base_Addr+0x7040))	/* HDLC A Maximum Frame Length */
#define HMFLRB 	   (VPint(Base_Addr+0x8040))	/* HDLC B Maximum Frame Length */

#define HMFLR(ch)  (VPint(Base_Addr+0x7040 + ch*0x1000))

/************************************************************************/
/* HMFLRA     Maximum Frame Length 					*/
/*            Offset      : 0x7040 					*/
/*            Reset Value : 0xXXXX0000 					*/
/* HMFLRB     Maximum Frame Length 					*/
/*            Offset      : 0x8040 					*/
/*            Reset Value : 0xXXXX0000 					*/
/* 									*/
/* [15:0]     Maximum Frame Length	 				*/
/************************************************************************/ 

#define HRBSRA 	   (VPint(Base_Addr+0x7044))	/* HDLC A Maximum buffer size */
#define HRBSRB 	   (VPint(Base_Addr+0x8044))	/* HDLC B Maximum buffer size */

#define HRBSR(ch)  (VPint(Base_Addr+0x7044 + ch*0x1000))

/************************************************************************/
/* HRBSRA     DMA Receive Buffer Size Register  			*/
/*            Offset      : 0x7044 					*/
/*            Reset Value : 0xXXXX0000 					*/
/* HRBSRB     DMAReceive Buffer Size Register 				*/
/*            Offset      : 0x8044 					*/
/*            Reset Value :0xXXXX0000					*/
/************************************************************************/ 

#define HDLCBaseAddr     (Base_Addr+0x7000)
	

#else
/**************************************************/
/* KS32C5000A : HDLC Channel A                    */
/**************************************************/
#define HCON0A 		(VPint(Base_Addr+0x7000))	
#define HCON1A 		(VPint(Base_Addr+0x7004))	
#define HSTATA  	(VPint(Base_Addr+0x7008))	
#define HINTENA 	(VPint(Base_Addr+0x700c))	
#define HTXFIFOCA 	(VPint(Base_Addr+0x7010))	
#define HTXFIFOTA 	(VPint(Base_Addr+0x7014))	
#define HRXFIFOA 	(VPint(Base_Addr+0x7018))	
#define HSADRA 		(VPint(Base_Addr+0x701c))	
#define HBRGTCA 	(VPint(Base_Addr+0x7020))	
#define HPRMBA 		(VPint(Base_Addr+0x7024))	
#define HDMATXMAA 	(VPint(Base_Addr+0x7028))	
#define HDMARXMAA 	(VPint(Base_Addr+0x702c))	
#define HDMATXCNTA 	(VPint(Base_Addr+0x7030))	
#define HDMARXCNTA 	(VPint(Base_Addr+0x7034))	
#define HDMARXBCNTA 	(VPint(Base_Addr+0x7038))	

/**************************************************/
/* KS32C5000A : HDLC Channel B                    */
/**************************************************/
#define HCON0B 		(VPint(Base_Addr+0x8000))	
#define HCON1B 		(VPint(Base_Addr+0x8004))	
#define HSTATB  	(VPint(Base_Addr+0x8008))	
#define HINTENB 	(VPint(Base_Addr+0x800c))	
#define HTXFIFOCB 	(VPint(Base_Addr+0x8010))	
#define HTXFIFOTB 	(VPint(Base_Addr+0x8014))	
#define HRXFIFOB 	(VPint(Base_Addr+0x8018))	
#define HSADRB 		(VPint(Base_Addr+0x801c))	
#define HBRGTCB 	(VPint(Base_Addr+0x8020))	
#define HPRMBB 		(VPint(Base_Addr+0x8024))	
#define HDMATXMAB 	(VPint(Base_Addr+0x8028))	
#define HDMARXMAB 	(VPint(Base_Addr+0x802c))	
#define HDMATXCNTB 	(VPint(Base_Addr+0x8030))	
#define HDMARXCNTB 	(VPint(Base_Addr+0x8034))	
#define HDMARXBCNTB 	(VPint(Base_Addr+0x8038))	
	
#endif

/************************************************************************/ 
/* I2C Bus Register 							*/
/************************************************************************/ 
#define IICCON	 	(VPint(Base_Addr+0xf000))	

/************************************************************************/
/* IICCON 	Control status register 				*/
/*              Offset Address = 0xf000, Reset Value = 0x00000000 	*/
/* 									*/
/* [0]	Buffer flag (BF). 						*/
/*      The BF bit is set when the buffer is empty in transmit mode or  */
/*      when the buffer is full in receive mode. To clear the buffer,	*/
/*      you write a "0" to this bit. The BF bit is cleared automatically*/
/*      whenever the IICBUF register is written or read. 		*/
/*      If you set BF bit to one, the I 2 C -bus is stopped. 		*/
/*      To activate I 2 C-bus, you should clear the BF bit to zero. 	*/
/* 									*/
/* [1]  Interrupt enable (IEN). 					*/
/*      Setting the interrupt enable bit to "1" enables the I 2 C-bus   */
/*      interrupt. 							*/
/* 									*/
/* [2]  Last received bit (LRB). 					*/
/*      The LRB bit is read only. It holds the value of the last 	*/
/*      received bit over the I2C-bus. Normally, this bit will be the 	*/
/*      value of the slave acknowledgement. To check for slave 		*/
/*      acknowledgement, you test the LRB.				*/
/* 									*/
/*       0 = The most recent SDA is Low. (ACK is received) 		*/
/*       1 = The most recent SDA is High. (ACK not received)            */
/* 									*/
/* [3]  Acknowledge enable (ACK). 					*/
/* 									*/
/*      Controls generation of an ACK signal in receive mode. 		*/
/*       0 = Do not generate an ACK at 9th SCL (No more received data 	*/
/*           is required from the slave)				*/
/*       1 = Generate an ACK signal at 9th SCL. 			*/
/* 									*/
/* [5:4]COND1, COND0 These bits control the generation of the Start, 	*/
/* 	Stop, and repeat Start conditions:  				*/
/*       "00" = no effect 						*/
/*       "01" = Start 							*/
/*       "10" = Stop							*/
/*       "11" = Repeat Start. 						*/ 
/* 									*/
/* [6]  Bus busy (BUSY).						*/
/*      This bit is a read-only flag that indicates when the I2C-bus 	*/
/*      is in use. A "1" indicates that the bus is busy. This bit is 	*/
/*      set or cleared by a Start or Stop condition, respectively. 	*/
/* 									*/
/* [7]  Reset 								*/
/*      If "1" is written to the Reset bit, the I 2 C-bus controller 	*/
/*      is reset to its initial state. 					*/
/* 									*/
/* [31:8] Reserved Not applicable. 					*/
/* 									*/
/************************************************************************/ 

////////////////////////////////////////////////////////////////////////////////
// IICCON // Control status register ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define IICCON_BF 	    0x01  // [0] Buffer flag (BF) (1=Tx-empty,Rx-full)
#define IICCON_IEN	    0x02  // [1] Interrupt enable (IEN)
#define IICCON_LRB	    0x04  // [2] Last received bit (LRB)
                                  // ACK not received
/*       0 = The most recent SDA is Low. (ACK is received) 		      */
/*       1 = The most recent SDA is High. (ACK not received)                  */

#define IICCON_NOACK	    0x00  // No more received data is required
                                  // from the slave

#define IICCON_ACK	    0x08  // [3]  Acknowledge enable (ACK)
                                  // Generate an ACK signal at 9th SCL

/* [5:4] These bits control the generation of the Start, Stop,Repeat Start */
#define IICCON_COND	    0x30  // COND1, COND0	   	
#define IICCON_COND_NULL    0x00  // no effect 						      */
#define IICCON_COND_START   0x10  // "01" = Start	   	
#define IICCON_COND_STOP    0x20  // "10" = Stop
#define IICCON_COND_RESTART 0x30  // "11" = Repeat Start

#define IICCON_BUSY	    0x40  // [6]  Bus busy (BUSY)
#define IICCON_RESET	    0x80  // [7]  Reset

#define IICBUF	 	(VPint(Base_Addr+0xf004))

/************************************************************************/
/* IICBUF 	Shift buffer register 					*/
/*              Offset = 0xf004, Reset Value = Undefined 		*/
/* 									*/
/* [7:0]	Data This data field acts as serial shift register and	*/
/*              read buffer for interfacing to the I2C-bus. 		*/
/* 	        All read and write operations to/from the I2C-bus are 	*/
/*              done via this register. The IICBUF register is a 	*/
/*              combination of a shift register and a data buffer. 	*/
/*              8-bit parallel data is always written to the shift 	*/
/*              register, and read from the data buffer. I2C-bus data 	*/
/*              is always shifted in or out of the shift register. 	*/
/* 									*/
/* [31:8]	Reserved Not applicable. 				*/
/************************************************************************/ 

#define IICPS	 	(VPint(Base_Addr+0xf008))	

/************************************************************************/
/* IICPS        Prescaler register 					*/
/*              Offset = 0xf008, Reset Value = 0x00000000 		*/
/* 									*/ 
/* [15:0]       Prescaler value 					*/
/*              This prescaler value is used to generate the serial 	*/
/*              I2C-bus clock. 						*/
/* 									*/
/*              The system clock is divided by : 			*/
/* 									*/
/*                (16 * (prescaler value + 1) + 3) 			*/
/* 									*/
/*              to make the serial I 2 C clock. If the prescaler value 	*/
/*              is zero, the system clock is then divided by 19 to 	*/
/*              make the serial I2C clock. 				*/
/* 								        */
/* [31:16]      Reserved Not applicable. 				*/
/************************************************************************/ 

#define IICCNT 		(VPint(Base_Addr+0xf00c))	

/************************************************************************/
/* IICCNT  	Prescaler counter register 				*/
/*              Offset = 0xf00c, Reset Value = 0x00000000 		*/
/* 									*/
/* [15:0]       Counter value 						*/
/*              This 16-bit value is the value of the prescaler counter.*/
/*              It is read (in test mode only) to check the counter's   */
/*              current value. 						*/
/* 									*/
/* [31:16]      Reserved Not applicable. 				*/
/************************************************************************/ 

/************************************************************************/ 
/* GDMA 0 & 1       							*/
/************************************************************************/ 

#define GDMACON0	(VPint(Base_Addr+0xb000)) /* GDMA controller chanel 0 control register */
#define GDMACON1	(VPint(Base_Addr+0xc000)) /* GDMA controller chanel 1 control register */

/************************************************************************/
/* GDMACON0 GDMA controller channel 0 control register 			*/
/*          Offset      = 0xB000 				        */
/*          Reset Value = 0x00000000					*/
/* GDMACON1 GDMA controller channel 1 control register 			*/
/*          Offset      = 0xC000 					*/
/*          Reset Value = 0x00000000					*/
/* 									*/
/*[0]	  Run enable/disable (RE) 					*/
/*        0 = Disable DMA operation 					*/
/*        1 = Enable DMA operation 					*/
/* 									*/
/* 	  Setting this bit to 1, starts a DMA operation. To stop DMA,	*/
/*        you must clear this bit to 0. You can use the GMA run bit 	*/
/*        control address (GDMACON offset address + 0x20) to manipulate */
/*        this bit. By using the run bit control address, other GDMA    */
/* 	  control register values are not affected. 			*/
/* 									*/
/*[1]     Busy status (BS)						*/
/*        0 = DMA is idle 						*/
/*        1 = DMA is active 						*/
/* 									*/
/*        When DMA starts, this read-only status bit is automatically   */
/*        set to 1. When it is 0, DMA is idle.    			*/
/* 									*/
/*[3:2]   GDMA mode selection (MODE)				        */
/*        Four sources can initiate a DMA operation: 			*/
/*        00 = software (memory-to-memory)				*/
/*        01 = an external DMA request(nXDREQ) 				*/
/*        10 = the UART0 block						*/
/*        11 = the UART1 block					        */
/* 									*/
/*        The mode selection setting determines which source can 	*/
/* 	  initiate a DMA operation at any given time. 			*/
/* 									*/
/* [4]    Destination address direction (DA)				*/
/*        0 = Increase destination address 				*/
/*        1 = Decrease destination address 				*/
/* 									*/
/*        This bit controls whether the destination address will be     */
/*        decremented 1 or incremented 0 during a DMA operation. 	*/
/* 									*/
/* [5]    Source address direction (SA)					*/
/*        0 = Increase source address 					*/
/*        1 = Decrease source address 					*/
/* 									*/
/*        This bit controls whether the source address will be 		*/
/* 	  decremented 1 or incremented 0 during a DMA operation.        */
/* 									*/
/* [6]    Destination address fix (DF) 					*/
/*        0 = Increase/decrease destination address  			*/
/*        1 = Do not ch ange destination address (fix) 			*/
/* 									*/
/*        This bit determines whether or not the destination address 	*/
/* 	  will be changed during a DMA operation. You use this feature  */
/*        when transferring data from multiple sources to a single 	*/
/*        destination.    						*/
/* 									*/
/* [7]    Source address fix (SF) 					*/
/*        0 = Increase/decrease source address 				*/
/*        1 = Do not change source address (fix) 			*/
/* 									*/
/*        This bit determines whether or not the source address will 	*/
/*        be changed during a DMA operation. You use this feature when  */
/*        transferring data from a single source to multiple 		*/
/*	  destinations.    						*/
/* 									*/
/* [8]    Stop interrupt enable (SI) 					*/
/*        0 = Do not generate a Stop interrupt when DMA stops           */
/*        1 = Generate a Stop interrupt when DMA stops  		*/
/* 									*/
/*        To start/stop a DMA operation, you set/clear the run enable 	*/
/* 	  bit. If the Stop interrupt enable bit is 1 when DMA starts, 	*/
/* 	  a Stop interrupt is generated when DMA operation stops. 	*/
/* 	  If this bit is 0, the Stop interrupt is not generated. 	*/
/* 									*/
/* [9]    Four-data burst enable (FB) 					*/
/*        0 = Disable 4-data burst mode 				*/
/*        1 = Enable 4-data burst mode 					*/
/* 									*/
/*        If this bit is set to one, GDMA operates under 4-data burst   */
/*        mode. Under the 4-data burst mode, 4 consecutive source       */
/*        addresses are read and then are written to the consecutive    */
/*        destination addresses. If 4-data burst mode is set to one,    */
/*        "transfer count register" should be set carefully because 	*/
/* 	  the 4-data burst move is executed during decreasing of the 	*/
/* 	  transfer count. The 4-data burst mode can be used only when 	*/
/* 	  GMDA mode is software or external DMA request mode. 		*/
/* 									*/
/*[10]    Transfer direction (for UART0/UART1 only) (TD)		*/
/*        0 = UART0/UART1 to memory 					*/
/*        1 = Memory to UART0/UART1 					*/
/* 									*/
/*        This bit is used to specify the direction of a DMA operation  */
/*        when the mode bits [3:2] are set to '10' (UART0 from/to 	*/
/* 	  memory) or '11' (UART1 from/to memory). 			*/
/*        If this bit is 1, DMA operates in the memory-to-peripheral	*/
/* 	  direction (e.g., to the parallel port or UART).		*/
/*        When it is 0, DMA operates in the peripheral-to-memory 	*/
/*        direction.   							*/
/* 									*/
/*[11]    Single/Block mode (SB)					*/
/*        0 = One nXDREQ initiates a single DMA operation 		*/
/*        1 = One nXDREQ initiates a whole DMA operation 		*/
/* 									*/
/*        This bit determines the number of external DMA requests 	*/
/*        (nXDREQs) that are required for a DMA operation. 		*/
/* 									*/
/*        In Single mode, when [11]=0, the KS32C50100 requires an 	*/
/*        external DMA request for every DMA operation. 		*/
/* 									*/
/*        In Block mode, when [11]=1, the KS32C50100 requires only one 	*/
/*        external DMA request during the entire DMA operation. An 	*/
/*        entire DMA operation is defined as the operation of DMA until */
/*        the counter value is zero. 					*/
/* 									*/
/*        NOTE: You should not use Block mode together with Demand 	*/
/*        mode, or Single mode in conjunction with Continuous mode. 	*/
/* 									*/
/*[13:12] Transfer width (TW)						*/
/*        00 = Byte (8 bits) 						*/
/*        01 = Half-word (16 bits) 					*/
/*        10 = Word (32 bits) 						*/
/*        11 = No use 							*/
/* 									*/
/*        These bits determine the transfer data width to be one byte, 	*/
/*        one half-word, or one word. If you select a byte transfer     */
/*        operation, the source/destination address will be incremented */
/*        or decremented by one with each transfer. Each half-word 	*/
/*        transfer increments or decrements the address by two, and 	*/
/*        each word transfer by four. 					*/
/* 									*/
/* [14]   Continuous mode (CN)					        */
/*        0 = Normal operation 						*/
/*        1 = Hold system bus until the whole DMA operation stops 	*/
/* 									*/
/*        This bit lets the DMA controller hold the system bus until 	*/
/*        the DMA transfer count value is zero. You must therefore 	*/
/*        manipulate this bit carefully so that DMA transfer operations */
/*        do not exceed a acceptable time interval (as, for example, 	*/
/*        in a DRAM refresh operation). 				*/
/* 									*/
/*        NOTE: You can use Continuous mode together with a software    */
/*        request mode. 						*/
/* 									*/
/* [15]   Demand mode (DM)						*/
/*        0 = Normal external DMA mode 					*/
/*        1 = Demand mode 						*/
/* 									*/
/*        Setting this bit speeds up external DMA operations. 		*/
/*        When [15]=1, the DMA transfers data when the external DMA 	*/
/*        request signal (nXDREQ) is active. The amount of data 	*/
/*        transferred depends on how long nXDREQ is active. When nXDREQ */
/*        is active and DMA gets the bus in Demand mode, DMA holds the  */
/*        system bus until the nXDREQ signal becomes non-active. 	*/
/*        Therefore, the period of the active nXDREQ signal should be 	*/
/*        carefully timed so that the entire operation does not exceed	*/
/*        an acceptable interval (as, for example, in a DRAM refresh 	*/
/*	  operation). 				      			*/
/* 									*/
/* NOTE:  In Demand mode, you must clear the Single/Block 		*/
/*        and Continuous mode control bits to 0. 			*/
/************************************************************************/ 


////////////////////////////////////////////////////////////////////////////////
// GDMA Control Register ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

#define GDMACON_RE            0x0001	// [0] Run enable/disable (RE)	
#define GDMACON_BS            0x0002	// [1] Busy status (BS)

// [3:2] GDMA mode selection (MODE) ////////////////////////////////////////////
#define GDMACON_MODE          0x000C 	// GDMA mode mask 
#define GDMACON_MODE_MEMMEM   0x0000 	// 00 = software (memory-to-memory)	
#define GDMACON_MODE_EXTDREQ  0x0004	// 01 = an external DMA request(nXDREQ)
#define GDMACON_MODE_U0       0x0008	// 10 = the UART0 block
#define GDMACON_MODE_U1       0x000C	// 11 = the UART1 block


#define GDMACON_DST_DEC       0x0010	// [4] Destination address direction (DA)     
#define GDMACON_SRC_DEC       0x0020	// [5] Source address direction (SA)     
#define GDMACON_DST_FIX       0x0040	// [6] Destination address fix (DF)
#define GDMACON_SRC_FIX       0x0080	// [7] Source address fix (SF)     
#define GDMACON_SI            0x0100	// [8] Stop interrupt enable (SI)     

#define GDMACON_FB            0x0200	// [9] Four-data burst enable (FB)

#define GDMACON_TD            0x0400	// [10] Transfer direction (for UART0/UART1 only) (TD)
#define GDMACON_TD_UTONEM     0x0000	// UART0/UART1 to memory
#define GDMACON_TD_MEMTOU     0x0400	// memory to UART0/UART1  

#define GDMACON_SB            0x0800	// [11] Single/Block mode (SB)	
#define GDMACON_SB_SINGLE     0x0000	// 0 = One nXDREQ initiates a single DMA operation
#define GDMACON_SB_BLOCK      0x0800	// 1 = One nXDREQ initiates a whole DMA operation

// [13:12] Transfer width (TW) /////////////////////////////////////////////////

#define GDMACON_TW            0x3000	// Transfer width
#define GDMACON_TW_BYTE       0x0000	// 00 = Byte (8 bits)  
#define GDMACON_TW_HALFWORD   0x1000	// 01 = Half-word (16 bits) 
#define GDMACON_TW_WORD       0x2000	// 10 = Word (32 bits)

#define GDMACON_CM            0x4000	// [14] Continuous mode (CN)
#define GDMACON_DM            0x8000	// [15] Demand mode (DM) 

#define GDMASRC0	(VPint(Base_Addr+0xb004)) /* GDMA channel 0 source address rgister */
#define GDMADST0	(VPint(Base_Addr+0xb008)) /* GDMA channel 0 destination address register */
#define GDMASRC1	(VPint(Base_Addr+0xc004)) /* GDMA channel 1 source address register */
#define GDMADST1	(VPint(Base_Addr+0xc008)) /* GDMA channel 1 destination address register */


/************************************************************************/
/* GDMASRC0   GDMA channel 0 source address register 			*/
/*            Offset      = 0xB004 					*/
/*            Reset Value = Undefined  					*/
/* 									*/
/* GDMADST0   GDMA channel 0 destination address register 		*/
/*            Offset      = 0xB008 					*/
/*            Reset Value = Undefined 					*/
/* 									*/
/* GDMASRC1   GDMA channel 1 source address register 			*/
/*            Offset      = 0xC004					*/
/*            Reset Value = Undefined 					*/
/* GDMADST1   GDMA channel 1 destination address register 		*/
/*            Offset      = 0xC008 					*/
/*            Reset Value = Undefined 					*/
/* 									*/
/* [25:0]     Source/destination address 				*/
/************************************************************************/ 

#define GDMACNT0	(VPint(Base_Addr+0xb00c)) /* GDMA 0 transfer count register */
#define GDMACNT1	(VPint(Base_Addr+0xc00c)) /* GDMA 1 transfer count register */

/************************************************************************/
/* GDMACNT0   GDMA channel 0 transfer count register 			*/
/*            Offset      = 0xB00C 					*/
/*            Reset Value = Undefined 					*/
/* 									*/
/* GDMACNT1   GDMA channel 1 transfer count register 			*/
/*            Offset      = 0xC00C					*/
/*            Reset Value = Undefined  					*/
/* 									*/
/* [23:0]     Transfer Count 						*/
/************************************************************************/ 

#define GDMA0_RUN_ENABLE (VPint(Base_Addr+0xb020)) 
#define GDMA1_RUN_ENABLE (VPint(Base_Addr+0xc020)) 


/************************************************************************/ 
/* UART 0 & 1 KS32C50100						*/
/************************************************************************/ 

#ifdef _KS32C50100_
/************************************************************************/ 
/* UARTLCON0  UART0 line control register (KS32C50100)			*/
/*            Offset      = 0xD000 					*/
/*            Reset Value = 0x00					*/
/* UARTLCON1  UART1 line control register (KS32C50100)			*/
/*            Offset      = 0xE000 					*/
/*            Reset Value = 0x00 					*/
/* 									*/
/* [1:0]      Word length (WL) 						*/
/*            This two-bit word length value indicates the number of 	*/
/*            data bits to be transmitted or received per frame: 	*/
/* 									*/
/*             00 = 5 bits						*/
/*             01 = 6 bits						*/
/*             10 = 7 bits						*/
/*             11 = 8 bits						*/
/* 									*/
/* [2]        Number of Stop bits 					*/
/*            This bit specifies how many Stop bits are used to signal  */
/*            end-of-frame (EOF): 					*/
/*             0  = one Stop bit per frame 				*/
/*             1  = two Stop bits per frame 				*/
/* 									*/
/* [5:3]      Parity mode (PMD) 					*/
/*            The 3-bit parity mode value specifies how parity 		*/
/*            generation and checking are to be performed during UART   */
/* 	      transmit and receive operations: 				*/
/* 									*/
/*             0xx = no parity						*/
/*             100 = odd parity						*/
/*             101 = even parity					*/
/*             110 = parity is forced/checked as a "1"			*/	
/*             111 = parity forced/checked as a "0"			*/
/* 									*/
/* [6]        Serial Clock Selection 					*/
/*            This selection bit specifies the clock source 		*/
/*             0 = Internal (MCLK) 					*/
/*             1 = External (UCLK) 					*/
/* 									*/
/* [7]        Infra-red mode 						*/
/*            The KS32C50100 UART block supports infra-red (IR) 	*/
/* 	      transmit and receive operations. In IR mode, the transmit */
/* 	      period is pulsed at a rate of 3/16 that of the normal     */
/*            serial transmit rate (when the transmit data value in 	*/
/*            the UTXBUF register is zero). 				*/
/* 									*/
/*            To enable IR mode operation, you set ULCON[7] to "1". 	*/
/*            Otherwise, the UART operates in normal mode. In IR 	*/
/*            receive mode, the receiver must detect the 		*/
/*            3/16 pulsed period to recognize a zero value in the 	*/
/* 	      receiver buffer register, URXBUF, as the IR receive data. */
/*            When bit [7] is "0", normal UART mode is selected. 	*/
/* 	      When it is "1", infra-red Tx/Rx mode is selected. 	*/
/************************************************************************/ 

#define UARTLCON0 (VPint(Base_Addr+0xd000)) /* UART0 Line control register */
#define UARTLCON1 (VPint(Base_Addr+0xe000)) /* UART1 Line control register */

/* bit 0-1 : Word length */
#define	UARTLCON_WL5            0x00	/* 00 = 5 bits */
#define	UARTLCON_WL6            0x01	/* 01 = 6 bits */
#define	UARTLCON_WL7            0x02	/* 10 = 7 bits */
#define	UARTLCON_WL8            0x03	/* 11 = 8 bits */
#define	UARTLCON_WL     	0x03	/* Word Length mask */

/* bit 2 : Number of stop bits */
#define	UARTLCON_STOP_2         0x04  	/* default one stop bit */

/* bit 3-5 : Parity mode */
/*+===+===+===+---+---+---+						*/
/*| 5 | 4 | 3 | 2 | 1 | 0 | 						*/
/*+===+===+===+---+---+---+						*/
/*  0   x   x   0   0   0  	no parity 				*/
/*  1   0   0   0   0   0	odd parity 				*/
/*  1   0   1   0   0   0       even parity 				*/
/*  1   1   0   0   0   0       parity is forced/checked as 1 		*/
/*  1   1   1   0   0   0       parity forced/checked as 0 		*/

#define	UARTLCON_PMD_NO         0x00	/* no parity */
#define	UARTLCON_PMD_ODD        0x20    /* odd parity */
#define	UARTLCON_PMD_EVEN       0x28    /* even parity */
#define	UARTLCON_PMD_CHK1       0x30    /* parity is forced,checked as a 1 */
#define	UARTLCON_PMD_CHK0       0x38	/* parity is forced,checked as a 0 */
#define	UARTLCON_PMD            0x38	/* parity mode mask */

/* bit 6 : Serial Clock Selection */
#define UARTLCON_UCLK           0x40  	/* for external UART clock */

/* bit 7 : Infra-red mode */
#define	UARTLCON_INFRA_RED      0x80	/* Infra-red mode bit */
#endif 							/* _KS32C50100_ */

#ifdef _KS32C50100_
/************************************************************************/
/* UARTCON0   UART0 control register (KS32C50100)			*/
/*            Offset      = 0xD004 					*/
/*            Reset Value = 0x00 					*/
/* 									*/
/* UARTCON1   UART1 control register (KS32C50100)			*/
/*            Offset      = 0xE004 				        */
/*            Reset Value = 0x00 					*/
/* 									*/
/* [1:0]      Receive mode (RxM) 					*/
/*            This two-bit value determines which function is currently */
/*            able to read data from the UART receive buffer register, 	*/
/* 	      RBR:   							*/
/* 									*/
/*             00 = disable Rx mode					*/
/*             01 = interrupt request					*/
/*             10 = GDMA channel 0 request 				*/
/*             11 = GDMA channel 1 request 				*/
/* 									*/
/* [2]        Rx status interrupt enable (RxSI) 			*/
/*            This bit lets the UART generate an interrupt if an 	*/
/* 	      exception (a break, frame error, parity error, or 	*/
/* 	      overrun error) occurs during a receive operation. 	*/
/* 									*/
/* [4:3]      Transmit mode (TxM) 					*/
/*            This two-bit value determines which function is currently */
/*            able to write Tx data to the UART transmit buffer 	*/
/* 	      register, UTXBUF. 					*/
/* 									*/
/*             00 = disable Tx mode					*/
/*             01 = interrupt request					*/
/*             10 = GDMA channel 0 request				*/
/*             11 = GDMA channel 1 request 				*/
/* 									*/
/* [5]        Data set ready (DSR) 					*/
/*            Setting UCON[5] causes the KS32C50100 to assert its data  */
/*            set ready (DSR) signal output, nUADSR. Clearing this bit  */ 
/* 	      to "0" causes the DSR output to be de-asserted.           */
/* 									*/
/* [6]        Send break (SBK) 						*/
/*            Setting UCON0/1[6] to "1" causes the UART to send a       */
/*            break. If it is "0", a break is not sent. 		*/
/* 									*/
/*            A break is defined as a continuous Low level signal 	*/
/* 	      on the transmit data output with a duration of more than  */
/*            one frame transmission time. 				*/
/* 									*/
/*            By setting this bit when the transmitter is empty 	*/
/* 	      (transmitter empty bit, USTAT[7] = "1"), you can use the  */
/*            transmitter to time the frame. When USTAT[7] is "1", 	*/
/*            write the transmit buffer register, UTXBUF, with the 	*/
/* 	      data to be transmitted. Then, poll the USTAT[7] value. 	*/
/* 	      When USTAT[7] returns to "1", clear (reset) the send 	*/
/*            break bit, UCON0/1[6].              			*/
/* 									*/
/* [7]        Loop-back mode (LPB)					*/
/*            Setting this bit causes the UART to enter Loop-back mode. */
/*            In Loop-back mode, the transmit data output is sent High  */
/*            level and the transmit buffer register, UTXBUF, is 	*/
/* 	      internally connected to the receive buffer register, 	*/
/* 	      URXBUF.               					*/
/* 									*/
/* NOTE :     This mode is provided for test purposes only. For         */
/*            normal operation, this bit should always be "0". 		*/
/************************************************************************/

#define UARTCONT0   (VPint(Base_Addr+0xd004)) /* UART0 control register */
#define UARTCONT1   (VPint(Base_Addr+0xe004)) /* UART1 control register */

/************************************************************************/ 
/* CONTROL REGISTER BIT DEFINITIONS                  			*/
/************************************************************************/ 

/* bit 0-1 : Receive mode */
#define	UARTCON_RXM_OFF            0x00 /* receive disabled */
#define	UARTCON_RXM_INTREQ         0x01 /* status interrupt request */
#define	UARTCON_RXM_GDMA0REQ       0x02 /* GDMA channel 0 req. */
#define	UARTCON_RXM_GDMA1REQ       0x03 /* GDMA channel 1 req. */
#define	UARTCON_RXM                0x03	/* receive mode mask */

/* bit 2 : Rx status interrupt enable */
#define	UARTCON_RXSTAT_INT    /*-*/     0x04  /* generate rx status interrupt */

/* bit 3-4 : Transmit mode selection */
#define	UARTCON_TXM_OFF            0x00 /* transmit disabled */
#define	UARTCON_TXM_INTREQ         0x08 /* interrupt request */
#define	UARTCON_TXM_GDMA0REQ       0x10 /* GDMA channel 0 req. */
#define	UARTCON_TXM_GDMA1REQ       0x18 /* GDMA channel 1 req. */
#define	UARTCON_TXM                0x18 /* transmit modemask */

/* bit 5 : Data Set Ready. Setting UCON[5] causes the KS32C50100 */
/* to assert its data set ready (DSR) signal ouput, nUADSR 	*/
/* Clearing this bit to "0" causes the DSR output to be de-asserted */	
#define	UARTCON_DSR           /*-*/     0x20   /* Assert DSR output(nUADSR)*/

/* bit 6 : Send Break */
#define	UARTCON_SEND_BREAK         0x40   /* Send break */

/* bit 7 : Loop-back enable */
#define	UARTCON_LOOPBACK           0x80   /* Loop-back mode */
#endif 								/* _KS32C50100_ */

#ifdef _KS32C50100_
/************************************************************************/
/* UARTSTAT0  UART0 status register (KS32C50100)			*/
/*            Offset      = 0xD008 					*/
/*            Reset Value = 0xC0  					*/
/* 									*/
/* UARTSTAT1  UART1 status register (KS32C50100)			*/
/*            Offset      = 0xE008 					*/
/*            Reset Value = 0xC0 					*/
/* 									*/
/* [0]        Overrun error USTAT[0] is automatically set to "1" 	*/
/*            whenever an overrun error occurs during a serial data 	*/
/* 	      receive operation. The overrun error indicates that the   */
/* 	      new received data has overwritten old received data 	*/
/* 	      before the old data could be read. 			*/
/* 									*/
/*            If the receive status interrupt enable bit, UCON[2] is 	*/
/* 	      "1", a receive status interrupt is generated if an 	*/
/* 	      overrun error occurs. 					*/
/* 									*/
/*            This bit is automatically cleared to "0" whenever the 	*/
/* 	      UART status register (USTAT) is read. 			*/
/* 									*/
/* [1]        Parity error 						*/
/*            USTAT[1] is automatically set to "1" whenever a parity	*/
/* 	      error occurs during a serial data receive operation. 	*/
/* 	      If the receive status interrupt enable bit, UCON[2], 	*/
/* 	      is "1", a receive status interrupt is generated if a	*/
/* 	      parity error occurs. 		      			*/
/* 									*/
/*            This bit is automatically cleared to "0" whenever 	*/
/* 	      the UART status register (USTAT) is read. 		*/
/* 									*/
/* [2]        Frame error 						*/
/*            USTAT[2] is automatically set to "1" whenever a frame 	*/
/* 	      error occurs during a serial data receive operation. A 	*/
/* 	      frame error occurs when a zero is detected instead of 	*/
/* 	      the Stop bit(s).      					*/
/* 									*/	
/*            If the receive status interrupt enable bit, UCON[2] 	*/
/* 	      is "1", a receive status interrupt is generated if a 	*/
/* 	      frame error occurs. The frame error bit is automatically  */
/* 	      cleared to "0" whenever the UART status register (USTAT)	*/
/* 	      is read.                       				*/
/* 									*/
/* [3]        Break interrupt 						*/
/*            USTAT[3] is automatically set to "1" to indicate that a 	*/
/* 	      break signal has been received. 				*/
/* 									*/
/*            If the receive status interrupt enable bit, UCON[2], is 	*/
/* 	      "1", a receive status interrupt is generated if a break 	*/
/* 	      occurs.        						*/
/* 									*/
/*            The break interrupt bit is automatically cleared to "0"	*/
/* 	      when you read the UART status register. 			*/
/* 									*/
/* [4]        Data terminal ready (DTR) 				*/
/*            The USTAT[4] bit indicates the current signal level at 	*/
/* 	      the data terminal ready (DTR) pins (nUADTR). 		*/
/* 									*/
/*            When this bit is "1", the level at the DTR pin (nUADTR)	*/
/* 	      is Low. When it is "0", the DTR pin is High level. 	*/
/* 									*/
/* [5]        Receive data ready 					*/
/*            USTAT[5] is automatically set to "1" whenever the receive */
/*            data buffer register (URXBUF) contains valid data 	*/
/* 	      received over the serial port. The receive data can then	*/
/* 	      be read from the URXBUF. 					*/
/*            When this bit is "0", the URXBUF does not contain valid 	*/
/* 	      data.   							*/
/* 									*/
/*            Depending on the current setting of the UART receive mode */
/* 	      bits, UCON[1:0], an interrupt or a DMA request is 	*/
/* 	      generated when USTAT[5] is "1". 				*/
/* 									*/
/* [6]        Tx Buffer register empty 					*/
/*            USTAT[6] is automatically set to "1" when the transmit    */
/*            buffer register (UTXBUF) does not contain valid data. In  */
/*            this case, the UTXBUF can be written with the data to be  */
/*            transmitted. 						*/
/* 									*/
/*            When this bit is "0", the UTXBUF contains valid Tx data	*/
/* 	      that has not yet been copied to the transmit shift 	*/
/* 	      register. In this case, the UTXBUF cannot be written with */
/* 	      new Tx data.            					*/
/* 									*/
/*            Depending on the current setting of the SIO transmit mode */
/*            bits, UCON[4:3], an interrupt or a DMA request will be	*/
/*            generated whenever USTAT[6] is "1". 			*/
/* 									*/
/* [7]        Transmit complete (TC) 					*/
/*            USTAT[7] is automatically set to "1" when the transmit 	*/
/*            buffer register has no valid data to transmit and when 	*/
/* 	      the Tx shift register is empty. 				*/
/* 								        */
/*            When the transmitter empty bit is "1", it indicates to 	*/
/*            software that it can now disable the transmitter function */
/* 	      block.							*/
/************************************************************************/ 

#define UARTSTAT0    (VPint(Base_Addr+0xd008)) /* UART0 status register */
#define UARTSTAT1    (VPint(Base_Addr+0xe008)) /* UART1 status register */

#define	UARTSTAT_OVERRUN           0x01   /* overrun error */
#define	UARTSTAT_PARITY            0x02   /* parity error */
#define	UARTSTAT_FRAME             0x04   /* frame error */
#define	UARTSTAT_BREAK             0x08   /* break interrupt */
#define	UARTSTAT_DTR_LOW           0x10   /* data terminal ready */
#define	UARTSTAT_RCV_READY         0x20   /* receive data ready */ 
#define	UARTSTAT_TXB_EMPTY         0x40   /* tx buffer empty */
#define	UARTSTAT_TX_COMPLET        0x80   /* transmit complete */
#define	UARTSTAT_ERROR	    (UARTSTAT_OVERRUN|UARTSTAT_PARITY|UARTSTAT_FRAME)
#endif 							/* _KS32C50100_ */


#define UARTTXH0        (VPint(Base_Addr+0xd00c)) /* UART0 transmit buffer register */
#define UARTTXH1        (VPint(Base_Addr+0xe00c)) /* UART1 transmit buffer register */
// 
#define UARTRXB0        (VPint(Base_Addr+0xd010)) /* UART0 receive buffer register */
#define UARTRXB1        (VPint(Base_Addr+0xe010)) /* UART1 transmit buffer register */

#ifdef _S3C4530_
/************************************************************************/ 
/* UCON0 	UART0 control register S3C4530 				*/
/* 		Offset      = 0xD000 	0				*/
/* 		Reset Value = 0x00					*/
/* UCON1 	UART1 control register S3C4530 				*/ 
/*									*/
/* [1:0] 	Transmit mode (TMODE). This two-bit value determine 	*/
/* 		which function is currently able to write Tx data to 	*/
/* 		the UART transmit buffer register, UTXBUF. 		*/
/* 		  00 = disable Tx mode. 				*/
/* 		  01 = interrupt request				*/
/* 		  10 = GDMA ch 0 request 				*/
/* 		  11 = GDMA ch 1 request 				*/
/* [3:2] 	Receive mode (RMODE). This two-bit value determine 	*/
/* 		which function is currently able to read Rx data.  	*/
/* 		  00 = disable Rx mode. 				*/
/*                01 = interrupt request 				*/
/*                10 = GDMA ch 0 request 				*/
/*                11 = GDMA ch 1 request 				*/
/* [4] 		Send Break (SBR) Set this bit to one to cause the UART  */
/* 		to send a break. If this bit value is zero, a break 	*/
/* 		does not send. A break is defined as a continuous Low   */
/*		level signal on the transmit data output with the 	*/
/* 		duration of more than one frame transmission time. 	*/
/* [5] 		Serial Clock Selection (UCLK)				*/
/* 		This selection bit specifies the clock source. 		*/
/* 		  0 = Internal (MCLK2) 					*/
/* 		  1 = External (UCLK) 					*/
/* [6] 		Auto Baud Rate Detect (ABRD). Setting this bit causes 	*/
/* 		the UART to enter Auto Baud Rate Detect mode. In this 	*/
/* 		mode, UART try to get the baud rate from input data. 	*/
/* [7] 		Look-back mode (LOOPB). 				*/
/* 		Setting this bit causes the UART to enter Loop-back 	*/
/* 		mode. In Loop-back mode, the transmit data output is 	*/
/* 		sent High level and the transmit buffer register, 	*/
/* 		UTXBUF, is internally connected to the receive buffer 	*/
/* 		register, URXBUF. 					*/
/* 									*/
/* 		NOTE:   This mode is provided for test purposes only. 	*/
/* 			For normal operation, this bit should always be */
/* 			"0".						*/
/* [10:8] 	Parity mode (PMD) 					*/
/* 		The 3-bit parity mode value specifies how parity 	*/
/* 		generation and checking are to be performed during UART */
/* 		transmit and receive operations: 			*/
/* 		  0xx = no parity 					*/
/* 		  100 = odd parity 					*/
/* 		  101 = even parity 					*/
/* 		  110 = parity is forced/checked as a "1" 		*/
/*                111 = parity forced/checked as a "0". 		*/
/* [11] 	Number of Stop bits (STB) 				*/
/* 		This bit specifies how many stop bits are used to 	*/
/* 		signal end-of-frame (EOF) :				*/
/* 		  0 = one stop bit per frame 				*/
/* 		  1 = two stop bit per frame 				*/
/* [13:12] 	Word Length (WL) 					*/
/* 		This two bit word length value indicates the number of 	*/
/* 		data bits to be transmitted or received per frame :	*/
/* 		  00 = 5bits 						*/
/* 		  01 = 6bits 						*/
/* 		  10 = 7bits 						*/
/* 		  11 = 8bits 						*/
/* [14] 	Infra-red mode (IR) 					*/
/* 		The S3C4530A UART block supports infra-red (IR) 	*/
/* 		transmit and receive operations. In IR mode, the 	*/
/* 		transmit period is pulsed at a rate of 3/16 that 	*/
/* 		of the normal serial transmit rate (when the transmit 	*/
/* 		data value in the UTXBUF register is zero).		*/
/* 		To enable IR mode operation, you set ULCON[7] to "1". 	*/
/* 		Otherwise, the UART operates in normal mode. In IR 	*/
/* 		receive mode, the receiver must detect the 3/16 pulsed 	*/
/* 		period to recognize a zero value in the receiver buffer */
/* 		register, URXBUF, as the IR receive data.  		*/
/* 		When this bit is "0", normal UART mode is selected. 	*/
/* 		When it is "1", infra-red Tx/Rx mode is selected.	*/
/* [15] 	Reserved This bit should be cleared by zero. 		*/
/* [16] 	Transmit FIFO enable (TFEN) 				*/
/* 		S3C4530A UART block support 32 byte FIFO. If this bit 	*/
/* 		set to one, transmit data moved to Tx FIFO and then 	*/
/* 		sent. 							*/
/* [17] 	Receive FIFO enable (RFEN) 				*/
/* 		S3C4530A UART block support 32 byte FIFO. If this bit 	*/
/* 		set to one, receive data moved to Rx FIFO. 		*/
/* [18] 	Transmit FIFO reset (TFRST) 				*/
/*		If this bit set to one, transmit FIFO will be reset. 	*/
/* 		In this case, if there is data in transmit shift 	*/
/* 		register, it will be sent. 				*/
/* [19] 	Receive FIFO reset (RFRST) 				*/
/* 		If this bit set to one, receive FIFO will be reset. 	*/
/* 		In this case, if there is data in receive shift 	*/
/* 		register, it will be received.  			*/
/* [21:20] 	Transmit FIFO trigger level (TFTL) 			*/
/* 		This two bit trigger level value determines when the 	*/
/* 		transmitter start to transmit data in 32-byte transmit  */
/* 		FIFO : 							*/
/* 		  00 = 30-byte empty/32-byte				*/
/* 		  01 = 24/32 						*/
/* 		  10 = 16/32 						*/
/*                11 = 8/32 						*/
/* [23:22] 	Receive FIFO trigger level (TFTL) 			*/
/* 		This two bit trigger level value determines when the 	*/
/* 		receiver start to move the received data in 32-byte 	*/
/* 		receive FIFO : 						*/
/* 		  00 = 1-byte vaild/32-byte 				*/
/* 		  01 = 8/32 						*/
/* 		  10 = 16/32 						*/
/*		  11 = 24/32						*/
/* [24] 	Data Terminal Ready to pin (DTR) 			*/
/* 		This bit directly controls the nUADTR pin. Setting this */
/* 		bit to one, the nUADTR pin goes to Low level. If you 	*/
/* 		set this bit to zero, it goes High level. 		*/
/* [25] 	Request to Send to pin (RTS) 				*/
/* 		This bit directly controls the nUARTS pin only when 	*/
/* 		the UART is not hardware flow control mode. If this 	*/
/* 		bit set to one, nUARTS pin goes Low level. Otherwise, 	*/
/* 		it remains High level. 					*/
/* [27:26] 	Reserved This bit should be cleared by zero. 		*/
/* [28] 	Hardware Flow Control Enable (HFEN) 			*/
/* 		This bit determines whether UART select hardware flow 	*/
/* 		control or not. If this bit set to one, UART will 	*/
/* 		control all pins concerning to hardware flow control. 	*/
/* 		This pins are nCTS, nDCD and nRTS. 			*/
/* [29] 	Software Flow Control Enable (SFEN) 			*/
/* 		This bit determines whether UART select software flow 	*/
/* 		control or not. If this bit set to one, UART will act 	*/
/* 		in software flow control. In this mode, you have to use */
/* 		Control Character register. 				*/
/* [31:30] 	Reserved This bit should be cleared by zero. 		*/
/************************************************************************/

/* s3c4530A */
#define UCON0        (VPint(Base_Addr+0xd000)) /* UART0 control regisger */
#define UCON1        (VPint(Base_Addr+0xe000)) /* UART1 contgrol register */

/* [1:0] Transmit mode (TMODE)	*/				/* s3c4530A */
#define UCON_TMODE_OFF     	0x00000000 /* 00=transmit disabled */
#define	UCON_TMODE_INTREQ    	0x00000001 /* 01=interrupt request */
#define	UCON_TMODE_GDMA0REQ     0x00000002 /* 10=GDMA channel 0 req. */
#define	UCON_TMODE_GDMA1REQ     0x00000003 /* 11=GDMA channel 1 req. */
#define	UCON_TMODE              0x00000003 /* 11=transmit modemask */

/* [3:2] Receive mode (RMODE) */
#define	UCON_RMODE_OFF          0x00000000 /* 00 = disable Rx mode */
#define	UCON_RMODE_INTREQ       0x00000004 /* 01 = interrupt request */
#define	UCON_RMODE_GDMA0REQ     0x00000008 /* 10 = GDMA ch 0 request */
#define	UCON_RMODE_GDMA1REQ     0x0000000C /* 11 = GDMA ch 1 request */
#define	UCON_RMODE              0x0000000C /* 11 = Rx mode mask */

/* [4]	Send Break (SBR) */
#define	UCON_SBR         	0x00000010 /* [4] Send break */

/* [5]	Serial Clock Selection (UCLK) */
#define UCON_UCLK      		0x00000020 /* [5] external UART clock */

/* [6] 	Auto Baud Rate Detect (ABRD) */
#define UCON_ABRD      		0x00000040 /* [6] Auto Baud Rate Detect */

/* [7] 	Look-back mode (LOOPB) */
#define	UCON_LOOPB              0x00000080 /* [7] Loop-back mode */

/* [10:8] Parity mode (PMD) */
#define	UCON_PMD_NO             0x00000000 /* 0xx = no parity */
#define	UCON_PMD_ODD            0x00000400 /* 100 = odd parity */
#define	UCON_PMD_EVEN           0x00000500 /* 101 = even parity */
#define	UCON_PMD_CHK1         	0x00000600 /* 110 = parity is forced,checked as a 1 */
#define	UCON_PMD_CHK0           0x00000700 /* 111 = parity is forced,checked as a 0 */
#define	UCON_PMD            	0x00000700 /* parity mode mask */

/* [11] Number of Stop bits (STB) */
#define	UCON_STOP_2             0x00000800 /* two stop bit per frame */

/* [13:12] Word Length (WL) */
#define	UCON_WL5                0x00000000 /* 00 = 5 bits */
#define	UCON_WL6                0x00001000 /* 01 = 6 bits */
#define	UCON_WL7                0x00002000 /* 10 = 7 bits */
#define	UCON_WL8                0x00003000 /* 11 = 8 bits */
#define	UCON_WL            	0x00003000 /* [13:12] Word Length mask */

/* [14] Infra-red mode */
#define	UCON_IR                 0x00004000 /* [14] Infra-red mode bit */
/* [15] Reserved */
/* [16] Transmit FIFO enable (TFEN) */
#define UCON_TFEN               0x00010000 /* [16] Transmit FIFO enable */ 
/* [17] Receive FIFO enable (RFEN) */
#define UCON_RFEN               0x00020000 /* [17] Receive FIFO enable */
/* [18] Transmit FIFO reset (TFRST) */
#define UCON_TFRST              0x00040000 /* [18] Transmit FIFO reset */
/* [19] Receive FIFO reset (RFRST) */
#define UCON_RFRST              0x00080000 /* [19] Receive FIFO reset */	
/* [21:20] Transmit FIFO trigger level (TFTL) */
#define UCON_TFTL_30		0x00000000 /* 00 = 30-byte empty/32-byte */		
#define UCON_TFTL_24		0x00100000 /* 01 = 24/32 */
#define UCON_TFTL_16		0x00200000 /* 10 = 16/32 */
#define UCON_TFTL_8		0x00300000 /* 11 = 8/32 */
#define UCON_TFTL		0x00300000 /* transmit fifo trigger level mask */
/* [23:22] Receive FIFO trigger level (RFTL) */
#define UCON_RFTL_1		0x00000000 /* 00 = 1-byte vaild/32-byte */
#define UCON_RFTL_8		0x00400000 /* 01 = 8/32 */	
#define UCON_RFTL_16		0x00800000 /* 10 = 16/32 */
#define UCON_RFTL_24		0x00C00000 /* 11 = 24/32 */
#define UCON_RFTL		0x00C00000 /* receive fifo trigger level mask */

/* [24] Data Terminal Ready to pin (DTR) */
#define UCON_DTR                0x01000000

/* [25] Request to Send to pin (RTS) */
#define UCON_RTS                0x02000000

/* [27:26] Reserved This bit should be cleared by zero.*/
/* [28] Hardware Flow Control Enable (HFEN) */
#define UCON_HFEN               0x10000000 
/* [29] Software Flow Control Enable (SFEN)*/
#define UCON_SFEN               0x20000000 
/* [31:30] 	Reserved This bit should be cleared by zero. */
#endif							// _S3C4530_	

#ifdef _S3C4530_ 
/************************************************************************/
/* USTAT0 	UART0 status register	s3c4530A			*/
/* 		Offset=0xD004						*/
/* 		Reset Value=0xE0240 					*/
/* USTAT1 	UART1 status register 	s3c4530A			*/
/* 		Offset=0xE004 						*/
/* 		Reset value=0xE0240					*/
/* 									*/
/* [0] 		Receive Data Valid (RDV) 				*/
/* 			This bit automatically set to one when Receive 	*/
/* 		FIFO-top or URXBUF contains a valid data received over 	*/
/* 		the serial port. The received data can be read from 	*/
/* 		receive FIFO-top or URXBUF. 				*/
/* 			When this bit is "0", there is no valid data. 	*/
/* 		According to the current setting of the UART receive 	*/
/* 		mode bits, an interrupt or DMA request is generated 	*/
/* 		when USTAT[0] is "1". In case of UCON[3:2]='01' and 	*/
/* 		UINTEN[0]=1,interrupt requested, and UCON[3:2]='10' or 	*/
/* 		'11', DMA request occurred. 				*/
/* 									*/
/* 		You can clear this bit by reading Receive FIFO or URXBUF.*/
/*		NOTE : Whether Receive FIFO top or URXBUF is depends on */
/* 		the UCON[17] RFEN. 					*/
/* 									*/
/* [1] 		Break Signal Detected (BSD) 				*/
/* 		This bit automatically set to one to indicate that a 	*/
/* 		break signal has been received in Receive FIFO-top or 	*/
/* 		URXBUF. If the BSD interrupt enable bit, UINTEN[1], is 	*/
/* 		"1", a interrupt is generated when a break occurs. 	*/
/* 									*/
/* 		You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [2] 		Frame Error (FER) 					*/
/* 		This bit automatically set to "1" whenever a frame 	*/
/* 		error occurs during a serial data receive operation. 	*/
/* 		A frame error occurs when a zero is detected instead 	*/
/* 		of the Stop bit(s). If the FER interrupt enable bit, 	*/
/* 		UINTEN[2], is "1", a interrupt is generated when a 	*/
/* 		frame error occurs. 					*/
/* 									*/
/* 		You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [3] 		Parity Error (PER) 					*/
/* 		This bit automatically set to "1" whenever a parity 	*/
/* 		error occurs during a serial data receive operation. 	*/
/* 		If the PER interrupt enable bit, UINTEN[3], is "1", 	*/
/* 		a interrupt is generated when a parity error occurs. 	*/
/* 									*/
/* 		You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [4] 		Overrun Error (OER) 					*/
/* 		This bit automatically set to "1" whenever an overrun   */
/* 		error occurs during a serial data receive operation. 	*/
/* 		When URXBUF has a previous valid data, but a new 	*/
/* 		received data is going to be written into URXBUF during */
/* 		non-FIFO mode and when a new received data is going to 	*/
/* 		be written into RXFIFO with FIFO full during FIFO mode. */
/* 		USTAT[4] is set to '1'. If the OER interrupt enable bit,*/
/* 		UINTEN[4], is "1", a interrupt is generated when a 	*/
/* 		overrun error occurs. 					*/
/* 									*/
/* 		You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [5] 		Control Character Detect (CCD) 				*/
/* 		USTAT[5] is automatically set to "1" to indicate that 	*/
/* 		a control character has been received.			*/
/* 		If the CCD interrupt enable bit, UINTEN[5], is "1", an 	*/
/* 		interrupt is generated when a control character is 	*/
/* 		detected. 						*/
/* 									*/
/* 		You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [6] 		Data carrier Detect (DCD) 				*/
/* 		This bit set to 1 if nUADCD pin is high at the time 	*/
/*		UART Receiver checks a newly received data whether the  */
/* 		data is good frame or not. 				*/
/* 									*/
/* 		If the DCD interrupt enable bit, UINTEN[6], is "1", 	*/
/* 		a interrupt is generated when a data carrier is detected.*/
/* 		This bit can be used for error check bit in hardware 	*/
/* 		flow control mode. 					*/
/* 									*/
/* [7] 		Receive FIFO Data trigger level reach (RFREA) 		*/
/* 		In Receive FIFO mode, this bit indicate Receive FIFO 	*/
/* 		has valid data and reach Rx trigger level. So UART 	*/
/* 		request DMA to move data in Receive FIFO. In non-FIFO 	*/
/* 		mode, if URXBUF has a received data , this bit is set 	*/
/* 		to '1' also, An interrupt or DMA request is generated 	*/
/* 		when USTAT[7] is "1". In case of UCON[3:2]='01' and 	*/
/* 		UINTEN[7]=1,interrupt requested, and UCON[3:2]='10' 	*/
/* 		or '11', DMA request occurred. 				*/
/* 									*/
/* 		You can clear this bit by reading Receive FIFO or 	*/
/* 		URXBUF with a good data. If any error, this bit is 	*/
/* 		cleared by writing '1' to corresponding error bit in 	*/
/* 		USTAT register. 					*/
/* 									*/
/* [8] 		Receive FIFO empty (RFEMT) 				*/
/*		This bit is only for CPU to monitor UART. When Receive	*/
/* 		FIFO is empty, this bit is set to '1'. After reset, 	*/
/* 		default value is '1' . 					*/
/* 									*/
/* [9] 		Receive FIFO full (RFFUL) 				*/
/* 		This bit is only for CPU to monitor UART. When Receive 	*/
/* 		FIFO is full, this bit is set to '1'. After reset, 	*/
/* 		default value is '0' 					*/
/* 									*/
/* [10] 	Receive FIFO overrun (RFOV) 				*/
/* 		This bit is set to '1' when Receive FIFO overrun occurs */
/* 		during the Receive FIFO mode. 				*/
/* 									*/
/* 		You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [11] 	Receiver in idle (RIDLE)				*/
/* 		This bit is only for CPU to monitor UART. The RxIDLE 	*/
/* 		status bit indicates that the inactive state of RxDATA. */
/* 									*/
/* [12] 	Receive Event time out (E_RxTO) 			*/
/* 		During Receive FIFO mode, if there is a valid data in 	*/
/* 		URXFIFO or Receive FIFO within a promised time internal */
/* 		which is determined according to WL(Word Length), 	*/
/* 		this bit is set to '1' . URXFIFO is for non-FIFO mode	*/
/* 		and Receive FIFO is for FIFO mode. If the E_RxTO 	*/
/* 		interrupt enable bit, UINTEN[12], is "1", an interrupt	*/
/* 		is generated when a receive event time out is detected	*/
/* 		and valid data reside in URXBUF or Receive FIFO. You 	*/
/* 		can clear this bit by writing '1' to this bit. 		*/	
/* 									*/
/* 		NOTE : Event time = WL*4 +12 				*/
/* 		This bit set to one when the Rx data resides in RxFIFO. */
/* 									*/
/* [13] 	Reserved Not applicable. 				*/
/* 									*/
/* [14] 	Data Set ready (DSR) 					*/
/* 		This bit is only for CPU to monitor UART. 		*/
/* 		When nUADSR level is low , this bit is set. 		*/
/* 		And nUADSR high, this bit is cleared. 			*/
/* 									*/
/* [15] 	Clear To Send (CTS) 					*/
/* 		This bit is only for CPU to monitor UART. 		*/
/* 		When nUACTS level is low , this bit is set. And nUACTS 	*/
/* 		high, this bit is cleared. 				*/
/* 									*/
/* [16] 	CTS Event occurred (E_CTS) 				*/
/* 		This bit is set to '1' whenever nUCTS level changed. 	*/
/* 		If the E_CTS interrupt enable bit, UINTEN[16], is "1",	*/
/* 		a interrupt is generated when a CTS event is occurred.  */
/*		You can clear this bit by writing '1' to this bit. 	*/
/* 									*/
/* [17]		Transmit Complete (TC) 					*/
/* 		This bit is only for CPU to monitor UART. 		*/
/* 		USTAT[17] is automatically set to "1" when the transmit	*/
/* 		holding register has no valid data to transmit and when */
/* 		the Tx shift register is empty. After Reset , Default 	*/
/* 		value is '1' 						*/
/* 									*/
/* [18] 	Transmit Holding Register Empty (THE) 			*/
/* 		In Transmit FIFO mode, when Transmit FIFO is empty to	*/
/* 		trigger level, this bit set to '1'. 			*/
/* 		In non-FIFO mode, when UTXBUF is empty without 		*/
/* 		regarding Tx shift register , this bit set to '1'. 	*/
/* 		An interrupt or DMA request is generated when USTAT[18] */
/* 		is "1". In case of UCON[1:0]='01' and UINTEN[18]=1, 	*/
/* 		an interrupt requested, and UCON[1:0]='10' or '11', 	*/
/* 		DMA request occurred. 					*/	
/* 									*/
/* 		You can clear this bit by writing TxDATA into UTXBUF	*/
/* 		or Transmit FIFO.					*/
/* 									*/
/* [19]		Transmit FIFO Empty (TFEMT) 				*/
/* 		This bit is only for CPU to monitor UART. 		*/
/* 		When Transmit FIFO is empty, this bit is set to '1'. 	*/
/*		After reset, default value is '1'  			*/
/* 									*/
/* [20] 	Transmit FIFO full (TFFUL) 				*/
/* 		This bit is only for CPU to monitor UART. 		*/
/* 		When Transmit FIFO is full, this bit is set to '1'. 	*/
/* 		After reset, default value is '0' . 			*/
/* [31:21] 	Reserved Not applicable. 				*/
/************************************************************************/

#define USTAT0        (VPint(Base_Addr+0xd004)) /* UART0 status register */
#define USTAT1        (VPint(Base_Addr+0xe004)) /* UART1 status register */

/* [0] Receive Data Valid (RDV) */
#define USTAT_RDV 		0x00000001	/* rx data */
/* [1] Break Signal Detected (BSD) */
#define USTAT_BSD 		0x00000002	/* break signal detected */
/* [2] Frame Error (FER) */
#define USTAT_FER 		0x00000004	/* rx frame error */
/* [3] Parity Error (PER) */
#define USTAT_PER 		0x00000008	/* rx parity error */
/* [4] Overrun Error (OER) */
#define USTAT_OER 		0x00000010	/* rx overrun error */
/* [5] Control Character Detect (CCD) */
#define USTAT_CCD 		0x00000020	/* control character detected */
/* [6] Data carrier Detect (DCD) */
#define USTAT_DCD 		0x00000040	/* data carrier detect */
/* [7] Receive FIFO Data trigger level reach (RFREA)*/
#define USTAT_RFREA 		0x00000080	/* rx fifo trigger level reached */
/* [8] Receive FIFO empty (RFEMT) */
#define USTAT_RFEMT 		0x00000100	/* receive FIFO empty */
/* [9] Receive FIFO full (RFFUL) */
#define USTAT_RFFUL 		0x00000200	/* receive FIFO full */
/* [10] Receive FIFO overrun (RFOV) */
#define USTAT_RFOV 		0x00000400	/* receive FIFO overrun */
/* [11] Receiver in idle (RIDLE) */
#define USTAT_RIDLE 		0x00000800	/* receiver in idle */
/* [12] Receive Event time out (E_RxTO) */
#define USTAT_E_RxTO 		0x00001000
/* [13] Reserved Not applicable */
/* [14] Data Set ready (DSR) */
#define USTAT_DSR 		0x00004000	/* DSR */
/* [15] Clear To Send (CTS) */
#define USTAT_CTS 		0x00008000	/* STS */
/* [16] CTS Event occurred (E_CTS) */
#define USTAT_E_CTS		0x00010000	/* E_CTS */
/* [17]	Transmit Complete (TC) */
#define USTAT_TC		0x00020000	/* transmit complete */
/* [18] Transmit Holding Register Empty (THE) */
#define USTAT_THE		0x00040000	/* transmit holding register empty */
/* [19] Transmit FIFO Empty (TFEMT) */
#define USTAT_TFEMT		0x00080000	/*  */
/* [20] Transmit FIFO full (TFFUL) */
#define USTAT_TFFUL		0x00100000	
/* [31:21] Reserved Not applicable. */
#endif							/* _S3C4530_ */

#ifdef _S3C4530_
/************************************************************************/
/* UINTEN0 	UART0 Interrupt Enable register 			*/
/* 		Offset      = 0xD008 					*/
/* 		Reset Value = 0x00 					*/
/* UINTEN1 	UART1 Interrupt Enable register 			*/
/* 		Offset      = 0xE008					*/
/* 		Reset Value = 0x00 					*/
/* 									*/
/* [0] 		RDVIE 	Receive Data Valid interrupt enable		*/
/* [1] 		BSDIE 	Break Signal Detected interrupt enable		*/
/* [2] 		FERIE 	Frame Error interrupt enable			*/
/* [3] 		PERIE 	Parity Error interrupt enable			*/
/* [4] 		OERIE 	Overrun Error interrupt enable			*/
/* [5] 		CCDIE 	Control Character Detect interrupt enable	*/
/* [6] 		DCDLIE 	DCD High at receiver checking time interrupt 	*/
/* 			enable						*/
/* [7] 		RFREAIE Receive FIFO Data trigger level reach interrupt */	
/* 			enable						*/
/* [9:8] 		Reserved					*/
/* [10] 	OVFFIE 	Receive FIFO overrun interrupt enable		*/
/* [11] 		Reserved					*/
/* [12] 	E_RxTOIE Receive Event time out interrupt enable	*/	
/* [15:13] 		Reserved					*/
/* [16] 	E_CTSIE CTS Event occurred interrupt enable		*/
/* [17] 		Reserved					*/
/* [18] 	THEIE 	Transmit Holding Register Empty interrupt enable*/
/* [31:19] 		Reserved					*/
/************************************************************************/

#define UINTEN0        (VPint(Base_Addr+0xd008)) /* UART0 interrupt enable register */
#define UINTEN1        (VPint(Base_Addr+0xe008)) /* UART1 interrupt enable register */


/* [0] RDVIE Receive Data Valid interrupt enable */
#define UINTEN_RDVIE 		0x00000001	/* receive data valid */
/* [1] BSDIE Break Signal Detected interrupt enable */
#define UINTEN_BSDIE 		0x00000002	/* break signal detected */
/* [2] FERIE Frame Error interrupt enable */
#define UINTEN_FERIE 		0x00000004
/* [3] PERIE Parity Error interrupt enable */
#define UINTEN_PERIE 		0x00000008

/* [4] OERIE Overrun Error interrupt enable */
#define UINTEN_OERIE 		0x00000010
/* [5] CCDIE Control Character Detect interrupt enable */
#define UINTEN_CCDIE 		0x00000020
/* [6] DCDLIE DCD High at receiver checking time interrupt enable */
#define UINTEN_DCDLIE		0x00000040
/* [7] RFREAIE Receive FIFO Data trigger level reach interrupt enable */
#define UINTEN_RFREAIE		0x00000080
/* [9:8] Reserved */
/* [10] OVFFIE 	Receive FIFO overrun interrupt enable */
#define UINTEN_OVFFIE 		0x00000400
/* [11] Reserved */
/* [12] E_RxTOIE Receive Event time out interrupt enable */
#define UINTEN_E_RxTOIE 	0x00001000
/* [15:13] Reserved */
/* [16]	E_CTSIE CTS Event occurred interrupt enable */
#define UINTEN_E_CTSIE  	0x00010000
/* [17] Reserved */
/* [18] THEIE Transmit Holding Register Empty interrupt enable */
#define UINTEN_THEIE    	0x00040000
/* [31:19] Reserved */
#endif


/************************************************************************/
/* UBRDIV0	UART0 baud rate divisor register 			*/
/*              Offset = 0xd014, Rest Value = 0x00 			*/
/* UBRDIV1      UART1 baud rate divisor register 			*/
/*              Offset = 0xe014, Reset Value = 0x00 			*/
/* 									*/
/* [3:0]        Baud rate divisor value CNT1 				*/
/*               xxx0 = divide by 1 					*/
/*               xxx1 = divide by 16 					*/
/* [15:4] 	Time constant value for CNT0 				*/
/* 									*/
/* BRGOUT = (MCLK2 or UCLK) / (CNT0 + 1) / (16^CNT1) / 16		*/
/* 									*/
/************************************************************************/ 

#define UARTBRD0        (VPint(Base_Addr+0xd014)) /* UART0 baud rate divisor register */
#define UARTBRD1        (VPint(Base_Addr+0xe014)) /* UART1 baud rate divisor register */

									
/************************************************************************/
/* Timer Register  							*/
/************************************************************************/

/************************************************************************/
/* TMOD Timer mode register, Offset = 0x6000 				*/ 
/* [0] Timer 0 enable 							*/
/* 	0 = Disable timer 0 						*/
/* 	1 = Enable Timer 0 						*/
/* [1] Timer 0 mode selection (TMD0) 					*/
/* 	0 = Interval mode 						*/
/* 	1 = Toggle mode 						*/
/* [2] Timer 0 initial TOUT0 value (TCLR0)				*/
/* 	0 = Initial TOUT0 is 0 in toggle mode 				*/   
/* 	1 = initial TOUT0 is 1 in toggle mode 				*/
/* [3] Timer 1 enable 							*/
/* 	0 = Disable timer 0 						*/
/* 	1 = Enable Timer 0 						*/
/* [4] Timer 1 mode selection (TMD1) 					*/
/* 	0 = Interval mode 						*/
/* 	1 = Toggle mode 						*/
/* [5] Timer 1 initial TOUT0 value (TCLR1)				*/
/* 	0 = Initial TOUT0 is 0 in toggle mode 				*/   
/* 	1 = initial TOUT0 is 1 in toggle mode 				*/
/************************************************************************/

#define TMOD  	  	(VPint(Base_Addr+0x6000)) /* Timer 0,1 mode register */	


#define  TMOD0_EN       0x01  /* Timer 0 enable */
#define  TMOD0_TOGGLE   0x02  /* 0, interval mode */
#define  TMOD0_TOUT     0x04  /* Timer 0 Initial TOUT0 value */
#define  TMOD1_EN       0x08  /* Timer 1 enable */
#define  TMOD1_TOGGLE   0x10  /* 0, interval mode */
#define  TMOD1_TOUT     0x20  /* Timer 1 Initial TOUT1 value */

/************************************************************************/
/* TDATA0 	Timer 0 data register 					*/
/*              Offset      = 0x6004					*/
/*              Reset Value = 0x00000000  				*/
/* TDATA1       Timer 1 data register 					*/
/*              Offset      = 0x6008					*/
/*              Reset Value = 0x00000000 				*/
/* 									*/
/* The timer data registers, TDATA0 and TDATA1, contain a value that    */
/* specifies the time-out duration for each timer. The formula for      */
/* calculating the time-out duration is: (Timer data + 1) cycles.       */
/* 									*/
/* [31:0]      Timer 0/1 data value 					*/
/* 									*/
/************************************************************************/

#define TDATA0		(VPint(Base_Addr+0x6004)) /* Timer 0 data register */
#define TDATA1		(VPint(Base_Addr+0x6008)) /* Timer 1 data register */

/* Timer 0/1 count registers */

#define TCNT0		(VPint(Base_Addr+0x600c)) /* Timer 0 count register */
#define TCNT1		(VPint(Base_Addr+0x6010)) /* Timer 1 count register */

/************************************************************************/
/* I/O Port Interface 							*/
/************************************************************************/

#define IOPMOD	  	(VPint(Base_Addr+0x5000)) /* I/O Port Mode Register */

/************************************************************************/
/* I/O  Port Mode Register, Offset 0x5000, Rset Value = 0x00000000 	*/
/* [0]  I/O port mode bit for port 0 					*/
/* 	0 = Input 							*/	
/* 	1 = Output 							*/
/* ..............							*/
/* [17] I/O Port mode bit for port 17 					*/
/* 	0 = Input 							*/
/* 	1 = Output 							*/ 
/************************************************************************/

#define IOPCON  	(VPint(Base_Addr+0x5004)) /* I/O Port Control Register */

/************************************************************************/
/* I/O Prt Control Register (IOPCON)					*/
/* 	 Offset      = 0x5004,						*/
/* 	 Reset Value = 0x00000000 					*/
/* 									*/
/* [4:0] Control external interrupt request 0 input for port 8 (xIRQ0) 	*/ 
/* 									*/
/* 	 [4]   Port 8 for xINTREQ0 					*/
/* 	 	0  = Disable 						*/
/* 	 	1  = Enable 						*/
/* 									*/
/* 	 [3]    0  = Active Low						*/
/* 	 	1  = Active Hight 					*/
/* 									*/
/* 	 [2]    0  = Filtering off					*/
/* 	 	1  = Filtering on 					*/
/* 									*/
/* 	 [1:0]  00 = Level detection 					*/
/*       	01 = Rising edge detection 				*/
/*       	10 = Falling adge detection 				*/
/*       	11 = Both edge detection 				*/ 
/* 									*/
/*[9:5]  Control external interrupt request 1 input for port 9 (xIRQ1)  */ 
/*[14:10]Control external interrupt request 2 input for port 10 (xIRQ2) */ 
/*[19:15]Control external interrupt request 3 input for port 11 (xIRQ3) */ 
/* 									*/
/*[22:20]Control external DMA request 0 input fro port 12 (DRQ0)	*/
/* 									*/
/*	 [22]   Port 12 nXDREQ0 					*/
/*       	0 = Disable						*/
/*       	1 = Disable 						*/
/*	 [21]   0 = Filtering off 					*/ 
/*       	1 = Filtering on 					*/
/*	 [20]   0 = Active Low 						*/ 
/*       	1 = Active Hight 					*/
/* 									*/
/*[25:23]Control external DMA request 1 input fro port 13 (DRQ1)	*/
/* 									*/
/*	 [25]   Port 13 nXDREQ1 					*/
/*       	0 = Disable						*/
/*       	1 = Disable 						*/
/*	 [24]   0 = Filtering off 					*/ 
/*       	1 = Filtering on 					*/
/*	 [23]   0 = Active Low 						*/ 
/*       	1 = Active Hight 					*/
/* 									*/
/*[27:26]Control external DMA acknowledge 0 output for port 14 (DAK0) 	*/
/*	 [27]   Port 14 for nXDACK0 					*/ 
/*       	0 = Disable 						*/
/*       	1 = Enable 						*/
/* 	 [26]   0 = Active low 						*/
/*       	1 = Active Hight 					*/
/* 									*/
/*[29:28]Control external DMA acknowledge 0 output for port 15 (DAK1) 	*/
/*	 [29]   Port 15 for nXDACK1 					*/ 
/*       	0 = Disable 						*/
/*       	1 = Enable 						*/
/*	 [28]   0 = Active low 						*/
/*       	1 = Active Hight 					*/
/* 									*/
/*[30]   Control timeout0 for port 16 (TOEN0) 				*/ 
/*       	0 = Disable 						*/
/*       	1 = Enable 						*/
/*[31]   Control timeout0 for port 17 (TOEN1) 				*/ 
/*       	0 = Disable 						*/
/*       	1 = Enable 						*/
/************************************************************************/

/* [4:0] Control external interrupt request 0 input for port 8 (xIRQ0) */
#define IOPCON_xIRQ0           0x0000001f /* xIRQ0 mask */
#define IOPCON_xIRQ0_LEVEL     0x00000000 /* [0:1] 00 = Level detection */
#define IOPCON_xIRQ0_EDGE_RISE 0x00000001 /* [0:1] 01 = Rising edge detection */
#define IOPCON_xIRQ0_EDGE_FALL 0x00000002 /* [0:1] 10 = Failing edge detection */
#define IOPCON_xIRQ0_EDGE_BOTH 0x00000003 /* [0:1] 11 = Both edge detection */
#define IOPCON_xIRQ0_FILTER_ON 0x00000004 /* [2] Filtering On */
#define IOPCON_xIRQ0_ACT_HIGH  0x00000008 /* [3] Active Hight */
#define IOPCON_xIRQ0_EN        0x00000010 /* [4] xIRQ0 enable bit */


/* [9:5] Control external interrupt request 1 input for port 9 (xIRQ1) */

#define IOPCON_xIRQ1           0x000003e0 /* xIRQ1 mask */ 
#define IOPCON_xIRQ1_OFFSET             5 /* xIRQ1 offset  */ 
//
#define IOPCON_xIRQ1_LEVEL     0x00000000 /* [6:5] 00 = Level detection */
#define IOPCON_xIRQ1_EDGE_RISE 0x00000020 /* [6:5] 01 = Rising edge detection */
#define IOPCON_xIRQ1_EDGE_FALL 0x00000040 /* [6:5] 10 = Failing edge detection */
#define IOPCON_xIRQ1_EDGE_BOTH 0x00000060 /* [6:5] 11 = Both edge detection */
#define IOPCON_xIRQ1_FILTER_ON 0x00000080 /* [7] Filtering On */
#define IOPCON_xIRQ1_ACT_HIGH  0x00000100 /* [8] Active Hight */
#define IOPCON_xIRQ1_EN        0x00000200 /* [9] Enable bit */	

/* [14:10] Control external interrupt request 2 input for port 10 (xIRQ2 */ 

#define IOPCON_xIRQ2           0x00007c00 /* xIRQ2 mask */
#define IOPCON_xIRQ2_OFFSET            10 /* xIRQ1 offset  */ 

//
#define IOPCON_xIRQ2_LEVEL     0x00000000 /* [11:10] 00 = Level detection */
#define IOPCON_xIRQ2_EDGE_RISE 0x00000400 /* [11:10] 01 = Rising edge detection */
#define IOPCON_xIRQ2_EDGE_FALL 0x00000800 /* [11:10] 10 = Failing edge detection */
#define IOPCON_xIRQ2_EDGE_BOTH 0x00000c00 /* [11:10] 11 = Both edge detection */
#define IOPCON_xIRQ2_FILTER_ON 0x00001000 /* [12] Filtering On */
#define IOPCON_xIRQ2_ACT_HIGH  0x00002000 /* [13] Active Hight */
#define IOPCON_xIRQ2_EN        0x00004000 /* [14] Enable bit */


/* [19:15] Control external interrupt request 2 input for port 11 (xIRQ2 */ 

#define IOPCON_xIRQ3           0x000F8000 /* xIRQ3 mask */		
#define IOPCON_xIRQ3_OFFSET            15 /* xIRQ3 offset */	

#define IOPCON_xIRQ3_LEVEL     0x00000000 /* [16:15] 00 = Level detection */
#define IOPCON_xIRQ3_EDGE_RISE 0x00008000 /* [16:15] 01 = Rising edge detection */
#define IOPCON_xIRQ3_EDGE_FALL 0x00010000 /* [16:15] 10 = Failing edge detection */
#define IOPCON_xIRQ3_EDGE_BOTH 0x00018c00 /* [16:15] 11 = Both edge detection */
#define IOPCON_xIRQ3_FILTER_ON 0x00020000 /* [17] Filtering On */
#define IOPCON_xIRQ3_ACT_HIGH  0x00040000 /* [18] Active Hight */
#define IOPCON_xIRQ3_EN        0x00080000 /* [19] Enable bit */

/* [22:20] Control external DMA request 0 input  for port 12 (DRQ0) */

#define IOPCON_DREQ0           0x00700000 /* [22:20] DRQ0 mask */
#define IOPCON_DREQ0_HIGHT     0x00100000 /* [20] Active Hight */
#define IOPCON_DREQ0_FILTER_ON 0x00200000 /* [21] Filtering on */
#define IOPCON_DREQ0_EN        0x00400000 /* [22] Enable bit */

/* [25:23] Control external DMA request 1 input  for port 13 (DRQ1) */

#define IOPCON_DREQ1           0x03800000 /* [25:23] DRQ1 mask */
#define IOPCON_DREQ1_HIGHT     0x00800000 /* [23] Active Hight */
#define IOPCON_DREQ1_FILTER_ON 0x01000000 /* [24] Filtering on */
#define IOPCON_DREQ1_EN        0x02000000 /* [25] DRQ1 Enable bit */

/* [27:26] Control external DMA acknowledge 0 output for port 14 (DACK0) */

#define IOPCON_DACK0           0x0c000000 /* DAK0 mask*/
#define IOPCON_DACK0_HIGHT     0x04000000 /* [26] 1 = Active Hight */
#define IOPCON_DACK0_EN        0x08000000 /* [27] Enable/Disable bit */

/* [29:28] Control external DMA acknowledge 1 output for port 15 (DACK1) */

#define IOPCON_DACK1           0x30000000 /* [29:28] DAK1 mask */
#define IOPCON_DACK1_HIGHT     0x10000000 /* [28] Active Hight */
#define IOPCON_DACK1_EN        0x20000000 /* [29] DACK1 Enable/Disable bit */

/* [30] Control timeout0 for port 16 (TOEN0) */
#define IOPCON_TOEN0_EN        0x40000000 /* Enable/Disable bit */ 

/* [31] Control timeout1 for port 17 (TOEN1) */
#define IOPCON_TOEN1_EN        0x80000000 /* Enable/Disable bit */ 

// 
#define IOPDATA 	(VPint(Base_Addr+0x5008)) 

/************************************************************************/
/* i/O Port Data Register (IOPDATA) Offset = 0x5008, Reset Value = und  */
/************************************************************************/

#ifdef _S3C4530_
/************************************************************************/
/* IOPCON1 	I/O port data register 					*/
/* 		Offset=0x500C, Reset value=0x03fc0000 			*/
/* 									*/
/* [1:0] 	Reserved 						*/
/* [2] 		UART DCD0 or port2 					*/
/* 		  0 = port 2 						*/
/*                1 = DCD0 						*/
/* [3] 		UART CTS0 or port3 					*/
/*                0 = port 3 						*/
/* 		  1 = CTS0 						*/
/* [4] 		UART RTS0 or port4	 				*/
/* 		   0 = port 4 						*/
/* 		   1 = RTS0 						*/
/* [5] 		UART DCD1 or port5 					*/
/*		   0 = port 5						*/
/*		   1 = DCD1						*/
/* [6] 		UART CTS1 or port6					*/
/*		   0 = port 6						*/
/*		   1 = CTS1						*/
/* [7] 		UART RTS1 or port7					*/
/*		   0 = port 7						*/
/*		   1 = RTS1						*/	
/* [17:8] 	Reserved						*/
/* [18] 	port18 or UART RxD0					*/
/*		   0 = port 18						*/
/*		   1 = RxD0						*/	
/* [19] 	port19 or UART DSR0					*/		
/* 		   0 = port 19						*/
/*		   1 = DSR0						*/
/* [20] 	port20 or UART TxD0					*/
/*		   0 = port 20						*/
/*		   1 = TxD0						*/
/* [21] 	port21 or UART DTR0					*/
/*		   0 = port 21						*/
/*		   1 = DTR0						*/
/* [22] 	port22 or UART RxD1					*/
/*		   0 = port 22						*/
/*		   1 = RxD1						*/
/* [23] 	port23 or UART DSR1					*/
/*		   0 = port 23						*/
/* 		   1 = DSR1						*/
/* [24] 	port24 or UART TxD1					*/	
/*		   0 = port 24						*/
/*                 1 = TxD1						*/
/* [25] 	port25 or UART DTR1					*/
/*		   0 = port 25						*/
/*	           1 = DTR1						*/
/************************************************************************/
#define IOPCON1 	(VPint(Base_Addr+0x500C)) 

#define IOPCON1_DCD0 0x00000004  // [2]UART DCD0 or port2
#define IOPCON1_CTS0 0x00000008  // 3 UART CTS0 or port3
#define IOPCON1_RTS0 0x00000010  // 4 UART RTS0 or port4
#define IOPCON1_DCD1 0x00000020  // 5 UART DCD1 or port5
#define IOPCON1_CTS1 0x00000040  // 6 UART CTS1 or port6
#define IOPCON1_RTS1 0x00000080  // 7 UART RTS1 or port7

#define IOPCON1_RXD0 0x00040000  // 18 port18 or UART RxD0
#define IOPCON1_DSR0 0x00080000  // 19 port19 or UART DSR0
#define IOPCON1_TXD0 0x00100000  // 20 port20 or UART TxD0
#define IOPCON1_DTR0 0x00200000  // 21 port21 or UART DTR0
#define IOPCON1_RXD1 0x00400000  // 22 port22 or UART RxD1
#define IOPCON1_DSR1 0x00800000  // 23 port23 or UART DSR1
#define IOPCON1_TXD1 0x01000000  // 24 port24 or UART TxD1
#define IOPCON1_DTR1 0x02000000  // 25 port25 or UART DTR1

#endif 							/* _S3C4530_ */

/************************************************************************/
/* Interrupt Controller Register 					*/
/************************************************************************/

#define	EXT0_INT	  0x000001	/* [0] External interrupt 0 */
#define	EXT1_INT	  0x000002	/* [1] External interrupt 1 */
#define	EXT2_INT	  0x000004	/* [2] External interrupt 2 */
#define	EXT3_INT	  0x000008	/* [3] External interrupt 3 */

#define	UART0_TX_INT	  0x000010	/* [4] UART 0 transmit interrupt */
#define	UART0_RX_ERR_INT  0x000020	/* [5] UART 0 receive and error interrupt */
#define	UART1_TX_INT	  0x000040	/* [6] UART 1 transmit interrupt */
#define	UART1_RX_ERR_INT  0x000080	/* [7] UART 1 receive and error interrupt */

#define	GDMA0_INT	  0x000100	/* [8] GDMA channel 0 interrupt */
#define	GDMA1_INT	  0x000200	/* [9] GDMA channel 1 interrupt */
#define	TIMER0_INT	  0x000400	/* [10] Timer 0 interrupt  */
#define	TIMER1_INT	  0x000800	/* [11] Timer 1 interrupt */

#define	HDLCTxA_INT	  0x001000	/* [12] HDLC channel A Tx interrupt */
#define	HDLCRxA_INT	  0x002000	/* [13] HDLC channel A Rx interrupt */
#define	HDLCTxB_INT	  0x004000	/* [14] HDLC channel B Tx interrupt */
#define	HDLCRxB_INT	  0x008000	/* [15] HDLC channel B Rx interrupt */

#define	BDMA_TX_INT	  0x010000	/* [16] Ethernet controller BDMA Tx interrupt */
#define	BDMA_RX_INT	  0x020000	/* [17] Ethernet controller BDMA Rx interrupt */
#define	MAC_TX_INT	  0x040000	/* [18] Ethernet controller MAC Tx interrupt */
#define	MAC_RX_INT	  0x080000	/* [19] Ethernet controller MAC Rx interrupt */
#define	IIC_INT		  0x100000	/* [20] I2C bus interrupt */

#define	GLOBAL_INT	  0x200000	/* Global interrupt mask bit */
/* scope INTMASK (INTERRUPT MASK REGISTER)*/
/* 0 = Enable interrupt requests */
/* 1 = Disable all interrupt requests */

/* bits numbers */ 

#define	nEXT0_INT		0	/*  [0] external interrupt 0 */
#define	nEXT1_INT		1	/*  [1] external interrupt 1 */ 
#define	nEXT2_INT		2	/*  [2] external interrupt 2 */
#define	nEXT3_INT		3	/*  [3] external interrupt 3 */
#define	nUART0_TX_INT		4	/*  [4] UART 0 transmit interrupt */
#define	nUART0_RX_ERR_INT	5	/*  [5] UART 0 receive & error interrupt */
#define	nUART1_TX_INT		6	/*  [6] UART 1 transmit interrupt */
#define	nUART1_RX_ERR_INT	7	/*  [7] UART 1 receive & error interrupt */
#define	nGDMA0_INT		8	/*  [8] GDMA channel 0 interrupt */
#define	nGDMA1_INT		9	/*  [9] GDMA channel 1 interrupt */
#define	nTIMER0_INT		10	/* [10] Timer 0 interrupt */
#define	nTIMER1_INT		11	/* [11] Timer 1 interrupt */

#define	nHDLCTxA_INT		12	/* [12] HDLC channel A Tx interrupt */
#define	nHDLCRxA_INT		13	/* [13] HDLC channel A Rx interrupt */
#define	nHDLCTxB_INT		14	/* [14] HDLC channel B Tx interrupt */
#define	nHDLCRxB_INT		15	/* [15] HDLC channel B Rx interrupt */

#define	nBDMA_TX_INT		16	/* [16] Ethernet controller BDMA Tx interrupt */
#define	nBDMA_RX_INT		17	/* [17] Ethernet controller BDMA Rx interrupt */
#define	nMAC_TX_INT		18	/* [18] Ethernet controller MAC Tx interrupt */
#define	nMAC_RX_INT		19	/* [19] Ethernet controller MAC Rx interrupt */
#define	nIIC_INT		20	/* [20] I2C bus interrupt  */
#define	nGLOBAL_INT		21	/* */


#define INTMODE         (VPint(Base_Addr+0x4000)) /* Interrupt Mode register  */
/* Eatch of the 21 bits in the interrupt mode enable register correspond to an 
interrupt source. When the source interrupt mode bit is set to 1, the interrupt 
is processed by the ARM7TDMI core in FIQ mode. Otherwise, it is processed in IRQ mode*/
#define INTPEND         (VPint(Base_Addr+0x4004)) /* Interrupt pendig register */
#define INTMASK         (VPint(Base_Addr+0x4008)) /* Interrupt mask register */
/* If bit is 1, the interrupt is not serviced by CPU when the corresponding interrupt */
/* is generated. If global mask bit (bit 21) is 1, no interrupts are serviced */

#define INTPRI0         (VPint(Base_Addr+0x400c)) /* interrupt priority register 0 */
#define INTPRI1		(VPint(Base_Addr+0x4010)) /* interrupt priority register 1 */
#define INTPRI2		(VPint(Base_Addr+0x4014)) /* interrupt priority register 2 */
#define INTPRI3		(VPint(Base_Addr+0x4018)) /* interrupt priority register 3 */
#define INTPRI4		(VPint(Base_Addr+0x401c)) /* interrupt priority register 4 */
#define INTPRI5		(VPint(Base_Addr+0x4020)) /* interrupt priority register 5 */

/*..........3....................2.................1.................0.....*/
/*.........109....87654....321...09876.....543...21098.....765...43210.....*/
/*--------+---+-----------+---+-----------+---+-----------+---+-----------+*/
/*INTPRI0 |000|PRIORITY3  |000|PRIORITY2  |000|PRIORITY1  |000|PRIORITY0  |*/
/*INTPRI1 |000|PRIORITY7  |000|PRIORITY6  |000|PRIORITY5  |000|PRIORITY4  |*/
/*INTPRI2 |000|PRIORITY11 |000|PRIORITY10 |000|PRIORITY9  |000|PRIORITY8  |*/
/*INTPRI3 |000|PRIORITY15 |000|PRIORITY14 |000|PRIORITY13 |000|PRIORITY12 |*/
/*INTPRI4 |000|PRIORITY19 |000|PRIORITY18 |000|PRIORITY17 |000|PRIORITY16 |*/
/*INTPRI5 |000|  00000    |000|  00000    |000|  00000    |000|PRIORITY20 |*/
/*--------+---+-----------+---+-----------+---+-----------+---+-----------+*/
/*PRIORITY0  - Low priority						   */
/*PRIORITY20 - High priority                                               */

/* Reset values */
/* INTPRI0 = 0x03 02 01 00 */
/* INTPRI1 = 0x07 06 05 04 */
/* INTPRI2 = 0x0B 0A 09 08 */	
/* INTPRI3 = 0xF0 0E 0D 0C */
/* INTPRI4 = 0x13 12 11 10 */
/* INTPRI5 = 0x00 00 00 14 */
/* External interrupt 0 - lowest priority */
/* I2C-bus interrupt - highest priority */ 

#define INTOFFSET	(VPint(Base_Addr+0x4024))	/* interrupt offset register */
/* The interrupt offset register contain the interrupt offset address of interrupt */
/* which has highest priority among the pending interrupts (bit position value of */
/* interrupt source << 2) */
#define INTPNDPRI	(VPint(Base_Addr+0x4028))	/* interrupt pending by priority */
#define INTPNDTST	(VPint(Base_Addr+0x402C))	/* interrupt pending test register */ 


/**********************************/
/* Enable HDLC Interrupt for DISI */
/**********************************/

//#define BD_ALLOW_INTERRUPTS     INTMASK &= ~(1<<(0xc000))	/*-*/
//#define BD_DISALLOW_INTERRUPTS  INTMASK |=  (1<<(0xc000))	/*-*/


#endif /* _SNDS_H */
