/*************************************************************************/
/*                                                                       */
/* FILE NAME                                      VERSION                */
/*                                                                       */
/*      mac.c                      KS32C5000, KS32C50100   : version 1.0 */
/*                                                                       */
/* COMPONENT                                                             */
/*                                                                       */
/*                                                                       */
/* DESCRIPTION                                                           */
/*                                                                       */
/*                                                                       */
/* AUTHOR                                                                */
/*                                                                       */
/*                                                                       */
/* DATA STRUCTURES                                                       */
/*                                                                       */
/*                                                                       */
/* FUNCTIONS                                                             */
/*                                                                       */
/*             Main function of MAC Test Program                         */
/*                                                                       */
/* DEPENDENCIES                                                          */
/*                                                                       */
/*                                                                       */
/* HISTORY                                                               */
/*                                                                       */
/*         NAME            DATE                    REMARKS               */
/*                                                                       */
/*      hbahn           09-15-1998      Created initial version 1.0      */
/*                                                                       */
/*************************************************************************/

/* 			*/
/* Modified by 		*/
/* Dmitriy Cherkashin 	*/
/* dch@ucrouter.ru	*/
/* 2002			*/
/*			*/

//#include <string.h>
#include "ks32c50.h"
#include "evm50100.h"
#include "mac.h"

#define MAC_PRINT_ERR_FRAME

// Global Variable for MAC/BDMA and CAM Configuration

extern volatile U32 gMacCon;		/* MAC control register value */
extern volatile U32 gMacTxCon;		/* MAC transmit control register value */
extern volatile U32 gMacRxCon;		/* MAC receive control register value */
extern volatile U32 gBdmaTxCon;		/* BDMA transmit control register value */
extern volatile U32 gBdmaRxCon;		/* BDMA receive control register value */
extern volatile U32 gCamCon;		/* CAM control register value */
extern volatile U32 gStaCon;		/* MDC clock (15:13 STACON)*/	
extern volatile U32 CntlPktTxDone; 	/*-*/// Control Packet Tx Done Flag
extern volatile U32 gCam0_Addr0; 	/* CAM 0 */
extern volatile U32 gCam0_Addr1;	/* CAM 1 */
extern volatile U32 gCTxFDPtr;    	
extern volatile U32 gWTxFDPtr;    	
extern volatile U32 gCRxFDPtr;    	

extern volatile U8 MyMacSrcAddr[6];	/* MAC Address */

// Global variable structure for store status //////////////////////////////////

extern pMACTxStatus  gsMacTxStatus;	/* MAC Transmit Status */  
extern pMACRxStatus  gsMacRxStatus;	/* MAC Receive  Status */
extern pBDMATxStatus gsBdmaTxStatus;	/* BDMA Transmit Status */
extern pBDMARxStatus gsBdmaRxStatus;	/* BDMA Receive Status */

extern TIME tm0 ;

/////////////////////////////////////////////////////////////////////////////////
// Read Error Status and Time ///////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////

void ReadErrReport(void) 
{
// MAC Transmit Status ///////////////////////////////////////////////////////// 

 Print("\nError Report");						    /*0*/
 Print("\nMAC Tx Count...........%8d",gsMacTxStatus.MacTxGood);    /*1*/	

// Transmit collission count  //////////////////////////////////////////////////
 Print("\nTransmit collission ...%8d",gsMacTxStatus.ExCollErr);   /*2*/
// Transmit deferred ///////////////////////////////////////////////////////////
 Print( " Transmit deferred......%8d", gsMacTxStatus.TxDefferedErr);/*2*/

// 
 Print("\nPaused.................%8d", gsMacTxStatus.sPaused);    /*3*/
 Print( " Underrun...............%8d", gsMacTxStatus.UnderErr);   /*3*/

// Deferral ////////////////////////////////////////////////////////////////////
 Print("\nDeferral...............%8d", gsMacTxStatus.DeferErr);   /*4*/
// No carrier //////////////////////////////////////////////////////////////////
 Print( " No carrier.............%8d", gsMacTxStatus.NCarrErr);   /*4*/

// Signal quality error ////////////////////////////////////////////////////////
 Print("\nSignal quality error...%8d", gsMacTxStatus.sSQE);       /*5*/
// Late collision //////////////////////////////////////////////////////////////
 Print( " Late collision.........%8d", gsMacTxStatus.LateCollErr);/*5*/

// Transmit parity error ///////////////////////////////////////////////////////
 Print("\nTransmit parity error..%8d", gsMacTxStatus.TxParErr);   /*6*/
// Transmission halted /////////////////////////////////////////////////////////
 Print( " Transmission halted....%8d", gsMacTxStatus.sTxHalted);  /*6*/

// MAC Receive Status ////////////////////////////////////////////////////////// 
// Control frame received ////////////////////////////////////////////////////// 
 Print("\nControl frame received.%8d", gsMacRxStatus.sCtlRecd);   /*7*/
// Receive 10-Mb/s status //////////////////////////////////////////////////////
 Print( " Receive 10-Mb/s status.%8d", gsMacRxStatus.sRx10Stat);  /*7*/

// Alignment error (Allign) ////////////////////////////////////////////////////
 Print("\nAlignment error........%8d", gsMacRxStatus.AllgnErr);   /*8*/
// CRC error ///////////////////////////////////////////////////////////////////
 Print( " CRC error..............%8d", gsMacRxStatus.sCRCErr);    /*8*/

// Overflow error //////////////////////////////////////////////////////////////
 Print("\nOverflow error.........%8d", gsMacRxStatus.OverflowErr);/*9*/
// Long error //////////////////////////////////////////////////////////////////
 Print( " Long error.............%8d", gsMacRxStatus.sLongErr);   /*9*/

// Receive parity error //////////////////////////////////////////////////////// 
 Print("\nReceive parity error...%8d", gsMacRxStatus.RxParErr);  /*10*/
// Reception halted ////////////////////////////////////////////////////////////
 Print( " Reception halted ......%8d", gsMacRxStatus.sRxHalted); /*10*/

// OvMaxSize;

////////////////////////////////////////////////////////////////////////////////
// BDMA Tx Status //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// BDMA Tx null list (BTxNL) ///////////////////////////////////////////////////
 Print("\nBDMA Tx null list......%8d",gsBdmaTxStatus.BTxNLErr);  /*11*/
// BDMA Tx not owner (BTxNO) ///////////////////////////////////////////////////
 Print( " BDMA Tx not owner......%8d",gsBdmaTxStatus.BTxNOErr);  /*11*/

// BDMA Tx buffer empty (BTxEmpty) /////////////////////////////////////////////
 Print("\nBDMA Tx buffer empty...%8d",gsBdmaTxStatus.BTxEmptyErr);/*12*/

// 
 Print("\nBDMA Received good.....%8d",gsBdmaRxStatus.BdmaRxGood);/*13*/
// BDMA Rx null list (BRxNL) ///////////////////////////////////////////////////
 Print( " BDMA Rx null list......%8d",gsBdmaRxStatus.BRxNLErr);  /*13*/	

// BDMA Rx not owner (BRxNO) ///////////////////////////////////////////////////
 Print("\nBDMA Rx not owner......%8d",gsBdmaRxStatus.BRxNOErr);  /*14*/
// BDMA Rx maximum size over (BRxMSO) //////////////////////////////////////////
 Print( " BDMA Rx max size over..%8d",gsBdmaRxStatus.BRxMSOErr); /*14*/
// 
 Print("\nMissed Error Count.....%8d",EMISSCNT);                 /*15*/
}


////////////////////////////////////////////////////////////////////////////////
// Clear Error Report Area /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void ClearErrReport(void) 
{
// Clear Error Report Variable ///////////////////////////////////////////////// 
 gsMacTxStatus.MacTxGood    = 0;
 gsMacTxStatus.ExCollErr    = 0;	// Transmit collission count 
 gsMacTxStatus.TxDefferedErr= 0;	// Transmit deferred
 gsMacTxStatus.sPaused      = 0;	// Paused
 gsMacTxStatus.UnderErr     = 0; 	// Underrun
 gsMacTxStatus.DeferErr     = 0;	// Deferral
 gsMacTxStatus.NCarrErr     = 0;	// No carrier
 gsMacTxStatus.sSQE         = 0;	// Signal quality error
 gsMacTxStatus.LateCollErr  = 0;	// Late collision
 gsMacTxStatus.TxParErr     = 0;	// Transmit parity error
 gsMacTxStatus.sTxHalted    = 0;	// Transmission halted

 gsMacRxStatus.OvMaxSize    = 0;
 gsMacRxStatus.sCtlRecd     = 0;	// Control frame received
 gsMacRxStatus.sRx10Stat    = 0;	// Receive 10-Mb/s status
 gsMacRxStatus.AllgnErr     = 0;	// Alignment error
 gsMacRxStatus.sCRCErr      = 0;	// CRC error
 gsMacRxStatus.OverflowErr  = 0;	// Overflow error
 gsMacRxStatus.sLongErr     = 0;	// Long error
 gsMacRxStatus.RxParErr     = 0;	// Receive parity error
 gsMacRxStatus.sRxHalted    = 0;	// Reception halted


 gsBdmaTxStatus.BTxNLErr    = 0;	// BDMA Tx null list (BTxNL)
 gsBdmaTxStatus.BTxNOErr    = 0;	// BDMA Tx not owner (BTxNO)
 gsBdmaTxStatus.BTxEmptyErr = 0;	// BDMA Tx buffer empty (BTxEmpty)

 gsBdmaRxStatus.BdmaRxCnt   = 0;
 gsBdmaRxStatus.BdmaRxGood  = 0;
 gsBdmaRxStatus.BRxNLErr    = 0;	// BDMA Rx null list (BRxNL)
 gsBdmaRxStatus.BRxNOErr    = 0;	// BDMA Rx not owner (BRxNO)
 gsBdmaRxStatus.BRxMSOErr   = 0;	// BDMA Rx maximum size over (BRxMSO)
}


////////////////////////////////////////////////////////////////////////////////
// View PHY Control register (address 0) ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationCtrl(void)
{
////////////////////////////////////////////////////////////////////////////////
// Control register (PHY_CNTL_REG) /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 while(1)
  {
   U8 it;

// read and print PHY Control register /////////////////////////////////////////
   U32 phy_ctrl;			// PHY control register value
   phy_ctrl = MiiStationRead		// read control register
    (
     PHY_CNTL_REG	, 		// address off ontrol register (0)
     PHYHWADDR				// PHY address
    );
   Print("\n\n0.Control Register : 0x%04x ",phy_ctrl);

// Reset (R/W,Self Clearing) 0.15 //////////////////////////////////////////////
   Print("\n0.15 [R] Reset (R/W/SC) ...................%s",
    ((phy_ctrl & RESET_PHY) != 0) ?
     "1(Reset chip)" :			// Reset Chip 
     "0(Enable normal operation)"
   ); 

// Loopback mode (R/W) (ENABLE_LOOPBACK) ///////////////////////////////////////
   Print("\n0.14 [L] Loopback mode (R/W) ..............%s",
    ((phy_ctrl & ENABLE_LOOPBACK) != 0) ?
     "1(Enable loopback mode)" :
     "0(Disable loopback mode)"
   );

// Speed selection (R/W) (DR_100MB) ////////////////////////////////////////////
   Print("\n0.13 [S] Speed Selection (R/W) ............%s",
    ((phy_ctrl & DR_100MB) != 0) ?
     "1(100Mbps)" : 			// 100 Mbit 
     "0(10Mbps)"			// 10 Mbit 
   );

// Auto-Negotiatiation Enable (R/W) (ENABLE_AN) ////////////////////////////////
   Print("\n0.12 [A] Auto-Negotiation Enable (R/W) ....%s",
    ((phy_ctrl & ENABLE_AN) != 0) ?
     "1(Enable auto-negotiate process)" :
     "0(Disable auto-negotiate process)"
   );

// Power down (PHY_POWER_DOWN) ///////////////////////////////////////////////// 
   Print("\n0.11 [P] Power Down (R/W) .................%s",
    ((phy_ctrl & PHY_POWER_DOWN) != 0) ?
     "1(Enable power down)" :
     "0(Enable normal operation)"
   );
 
// Isolate (PHY_MAC_ISOLATE) /////////////////////////////////////////////////// 
   Print("\n0.10 [I] Isolate (R/W) ....................%s",
    ((phy_ctrl & PHY_MAC_ISOLATE) != 0) ?
     "1(Electrically isolate from MII)" :
     "0(Normal operation)"
   );

// Restart Auto-negotiation (R/W, Self Cleared) (RESTART_AN) ///////////////////
   Print("\n0.9  [N] Restart Auto-Negotiation (R/W/SC) %s", 
    ((phy_ctrl & RESTART_AN) != 0) ?
     "1(Restart AN)" :
     "0(Normal operation)"
   );

// 0.8 Duplex mode (PHY_FULLDUPLEX) ////////////////////////////////////////////
   Print("\n0.8  [F] Duplex mode (R/W) ................%s", 
    ((phy_ctrl & PHY_FULLDUPLEX) != 0) ?
     "1(Enable full-duplex)" :
     "0(Disable full-duplex)"
   ); 

// 0.7 Colision Test (PHY_COL_TEST) ////////////////////////////////////////////
   Print("\n0.7  [C] Colision Test (R/W) ..............%s",
    ((phy_ctrl & PHY_COL_TEST) != 0) ?
     "1(Enable COL signal test)" :
     "0(Disable COL signal test)"
   );

// 0.6:4 Transceiver Test Mode Not Supported. (RO) /////////////////////////////
   Print("\n0.6:0.4  Transceiver Test Mode (RO) .......%d",
     ((phy_ctrl & PHY_TX_TEST_MODE) >> 4)
    );
   Print("(Not Supported)");
  
// 0.3 Master-Slave Enable. Not Supported. (RO) ////////////////////////////////
   Print("\n0.3      Master-Slave Enable (RO) .........%s", 
    ((phy_ctrl & PHY_MASTER_SL_EN) != 0) ?
     "1(Enable)" :
     "0(Disable)"
   );

// 0.2   Master-SlaveValue. Not Supported. (RO) ////////////////////////////////
   Print("\n0.2      Master-SlaveValue (RO)............%s",
    ((phy_ctrl & PHY_MASTER_SL_VAL) != 0) ?
     "1" :
     "0"
   );
   Print("(Not Supported)");
//
   Print("\n         R/W = Read/Write.");	
   Print("\n         SC  = Self Clearing.");	

// quit from PHY Control register test /////////////////////////////////////////
   Print("\n     [Q] Quit PHY Control Register Test.");	
   Print("\nSelect Test Item : "); 	// 
   it = get_upper();			// get upper character from console
	
   switch(it)
    {
     case 'R' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Reset (R/W) /////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\n\nReset PHY [Y/N/Q]"); 	// reset PHY prompt message
       it = get_upper();		// get upper case character from console
       if(it == 'Q')
        {				// quit from PHY control 
         return;			// register test 
        }
       else
       if(it == 'Y')
        {
////////////////////////////////////////////////////////////////////////////////
// reset PHY ///////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

         Print("\nReseting PHY..."); 
         phy_ctrl |= RESET_PHY;		// set bit 0.15 (Reset bit)
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        }  
      }
      break;

     case 'L' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Enable/Diasble Loopback mode ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nEnable loopback mode [Y/N/Q]");
       it = get_upper();   
       if(it == 'Q')
        {				// quit from PHY control 
         return;			// register test 
        }
       else
       if(
         it == 'Y' ||			// enable loopback 
         it == 'N'			// disable loopback
        )
        {
         if(it == 'Y')
          {
// enable loopback ///////////////////////////////////////////////////////////// 
           Print("\nLoopback mode enabled.");	// 
           phy_ctrl |= ENABLE_LOOPBACK; // set enable loopback mode bit 	
          }
         else
          {
// disable loopback ////////////////////////////////////////////////////////////
           Print("\nLoopback mode disabled.");	
           phy_ctrl &=~ENABLE_LOOPBACK;	// clear enable loopback mode bit 
          }
//            
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        }  
      }
      break;

     case 'S' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Speed Selection /////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nSpeed Selection. Enable 100Mbps [Y/N/Q]"); 
       it = get_upper();		
       if(it == 'Q')   
        {				// quit from PHY control 
         return;			// register test 
        } 
       else
       if(
         it == 'Y' ||			// select speed 100Mbps
         it == 'N'			// select speed 10Mbps		
        )
        {
         if(it == 'Y')
          {
// set speed to 100Mbps ////////////////////////////////////////////////////////
           Print("\nSelected speed 100Mbps."); 
           phy_ctrl |= DR_100MB;	// set 100Mbps bit  
          }
         else
          {
           Print("\nSelected speed 10Mbps."); 
           phy_ctrl &=~DR_100MB;	// clear 100Mbps bit
          }

// 
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        } 
      }
      break;

     case 'A' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Auto-Negotiation Enable (R/W) ///////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\nAuto-Negotiation Enable [Y/N/Q]");
       it = get_upper();
       if(it == 'Q')
        {				// quit from PHY control register 
         return;			// test 
        } 
       else
       if(
         it == 'Y' ||			// enable auto-negotiation  
         it == 'N'			// disable auto-negotiation 
        )
        {
         if(it == 'Y')
          { 
           Print("\nAuto-Negotiation Enabled.");
           phy_ctrl |= ENABLE_AN;	// set auto-negotiation bit
          }  
         else
          {
           Print("\nAuto-Negotiation Disabled.");
           phy_ctrl &=~ENABLE_AN;	// clear auto-negotiation bit
          }

//
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        } 
      }
      break;

     case 'P' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Power Down //////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nPower Down [Y/N/Q]");
       it = get_upper();
       if(it == 'Q')
        {				// return from PHY control 
         return;			// register test 
        } 
       else
       if(
          it == 'Y' ||			// enable power down 
          it == 'N'			// disable power down
        )
        {
         if(it == 'Y')
          {
           Print("\nPower down enabled.");
           phy_ctrl |= PHY_POWER_DOWN;	// set power down bit 
          }
         else
          {
           Print("\nPower down disabled.");
           phy_ctrl &=~PHY_POWER_DOWN;	// clear power down bit 
          }

// 
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        }  
      }
      break;

     case 'I' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Isolate (R/W) ///////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\n\nElectrically Isolate PHY from MII [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        { 				// quit from PHY control register 
         return;			// test 
        }
       else
       if(
          it == 'Y' ||  		// isolate PHY
          it == 'N'			// dont isolate
        )
        {
         if(it == 'Y')
          {
           Print("\nPHY Electrically isolated from MII.");
           phy_ctrl |= PHY_MAC_ISOLATE;	// set isolate bit 
          }
         else
          {
           Print("\nPHY connected to MII.");
           phy_ctrl &=~PHY_MAC_ISOLATE;	// clear isolate bit	
          }
         
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        }  
      }
      break;

     case 'N' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Restart Auto-Negotiation (R/W/SC) ///////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nRestart Auto-Negotiation [Y/N/Q]");
       it = get_upper();
       if(it == 'Q')
        {				// quit from PHY 
         return;			// control register test 
        } 
       else
       if(it == 'Y')
        {
// Restart Auto-negotiation ////////////////////////////////////////////////////
         Print("\nAuto-negotiation restarted.");
         phy_ctrl |= RESTART_AN;	// set restart auto-negotiation bit	
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        }
      }
      break;

     case 'F' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Duplex mode (R/W) ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\nFull-Duplex mode [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {				// quit from PHY control 
         return;			// register test 
        } 
       else
       if( 
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
           Print("\nFull-duplex enabled.");
           phy_ctrl |= PHY_FULLDUPLEX;	// set full-duplex bit 
          }
         else
          {
           Print("\nFull-duplex disabled.");
           phy_ctrl &=~PHY_FULLDUPLEX;	// clear full-duplex bit 
          } 
// 
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        }
      }
      break;

     case 'C' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Colision Test ///////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\n\nColision Test [Y/N/Q]"); 
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from PHY conrol 
         return;			// register test 
        } 
       else
       if(
          it == 'Y' ||			// 
          it == 'N'			// 
        )
        {
         if(it == 'Y')
          {
           Print("\nColision Test enabled."); 
           Print("\nBit 0.14 must be enabled to use this bit."); 
           Print("\nThis bit is used in conjunction with bit 0.14");
           Print("\nto test the COL output.");
//
           phy_ctrl |= PHY_COL_TEST;	// set collision test bit 
          } 
         else
          {
           Print("\nColision Test disabled."); 
           phy_ctrl &=~PHY_COL_TEST;	// clear collision test bit 
          } 
// 
         MiiStationWrite		// write to control register
          (
           PHY_CNTL_REG	, 		// register address
           PHYHWADDR	, 		// PHY address
           phy_ctrl			// register value
          );
        }       
      }
      break;

     case 'Q' : 
      return;

     default  : 
      Print("\nInvalid Test Item Selected") ; 
      break;
    }

   Print("\nPress any key to continue");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    }  
  }
}


////////////////////////////////////////////////////////////////////////////////
// PHY_STATUS_REG // Status register (Address 1) ///////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationStatus(void)
{
 U32 phy_stat_reg;			// PHY status register 
 phy_stat_reg = MiiStationRead
  (
   PHY_STATUS_REG	, 		// status register address
   PHYHWADDR				// PHY address
  );

// 
 Print("\n\n1. Status Register : 0x%04x ", phy_stat_reg);

// 100Base-T4 support (PHYSTAT_100BASET4) //////////////////////////////////////
 Print("\n1.15 100BASE-T4 (RO).....................%s", 
  ((phy_stat_reg & PHYSTAT_100BASET4) != 0) ?	
   "1(Supported)" :			// 100Base-T4
   "0(Not Supported"
 );

// 100Base-X full-duplex (PHYSTAT_100BASEX_FD) /////////////////////////////////
 Print("\n1.14 100BASE-X full-duplex (RO)..........%s", 
  ((phy_stat_reg & PHYSTAT_100BASEX_FD) != 0) ?
   "1(Supported)" :			// 100Base-X full-duplex
   "0(Not Supported)"
 );

// 100BASE-X half-duplex (PHYSTAT_100BASEX_HD) ///////////////////////////////// 
 Print("\n1.13 100BASE-X half-duplex (RO)..........%s", 
  ((phy_stat_reg & PHYSTAT_100BASEX_HD) != 0) ?
   "1(Supported)" : 			// 100BaseX half-duplex
   "0(Not Supported)"
 ); 

// 10 Mb/s full-duplex (PHYSTAT_10BASE_FD) /////////////////////////////////////
 Print("\n1.12 10 Mb/s full-duplex (RO)............%s",
  ((phy_stat_reg & PHYSTAT_10BASE_FD) != 0) ?
   "1(Supported)" :			// 10BaseT full-duplex 
   "0(Not Supported)"
 );

// 10 Mb/s half-duplex (PHYSTAT_10BASE_HD) ///////////////////////////////////// 
 Print("\n1.11 10 Mb/s half-duplex (RO)............%s", 
  ((phy_stat_reg & PHYSTAT_10BASE_HD) != 0) ?
   "1(Supported)" :			// 10BaseT half-duplex
   "0(Not Supported)"
 );
  
// 100BASE-T2 full-duplex (PHYSTAT_100BASET2_FD) ///////////////////////////////
 Print("\n1.10 100BASE-T2 full-duplex (RO).........%s", 
  ((phy_stat_reg & PHYSTAT_100BASET2_FD) != 0) ?	
   "1(Supported)" :			// 100BaseT2 full-duplex
   "0(Not Supported)"
 );

// 100BASE-T2 half-duplex (PHYSTAT_100BASET2_HD) ///////////////////////////////
 Print("\n1.9  100BASE-T2 half-duplex (RO).........%s", 
  ((phy_stat_reg & PHYSTAT_100BASET2_HD) != 0)	?
   "1(Supported)" :			// 100BaseT2 half-duplex
   "0(Not Supported)"
 );

// Master-Slave Configuration Fault (PHYSTAT_MASTERSL_FAULT) ///////////////////
 Print("\n1.7  Master-Slave Configuration Fault....%s", 
  ((phy_stat_reg & PHYSTAT_MASTERSL_FAULT) != 0) ?	 
   "1(Supported)" :			// Master-Slave Fault
   "0(Not Supported)"
 );

// MF Preamble Suppression (PHYSTAT_MFPREAMB_SUPPR) ////////////////////////////
 Print("\n1.6  MF Preamble Suppression (RO)........%s", 
  ((phy_stat_reg & PHYSTAT_MFPREAMB_SUPPR) != 0) ?
   "1(Supported)" :
   "0(Not Supported)"
 );

// Auto-Negotiation Complete (PHYSTAT_AN_COMPLETE) /////////////////////////////
 Print("\n1.5  Auto-Negotiation Complete (RO)......%s", 
  ((phy_stat_reg & PHYSTAT_AN_COMPLETE) != 0) ?
   "1(Auto-negotiation process complete)" :
   "0(Auto-negotiation process not complete)"
 ); 

// Remote Fault (PHYSTAT_REMOTE_FAULT) /////////////////////////////////////////
 Print("\n1.4  Remote Fault (RO/LH)................%s", 
  ((phy_stat_reg & PHYSTAT_REMOTE_FAULT) != 0) ?
   "1(Remote fault condition detected)" :
   "0(No remote fault condition detected)"
 );

// Ato-Neg ability (PHYSTAT_AUTO_NEG_AB) ///////////////////////////////////////
 Print("\n1.3  Auto-Neg. Ability (RO)..............%s", 
  ((phy_stat_reg & PHYSTAT_AUTO_NEG_AB) != 0) ?
   "1(Supported)" :			// Ato-Neg ability
   "0(Not Supported)"
 );
 
// Link Status (PHYSTAT_LINK_UP) ///////////////////////////////////////////////
 Print("\n1.2  Link Status (RO/LL).................%s", 
  ((phy_stat_reg & PHYSTAT_LINK_UP) != 0) ?	
   "1(Link is up)" :			// Link Up Flag
   "0(Link is down)"
 );

// Jabber Detect (10BASE-T Only) (PHYSTAT_JUBBER_DET) //////////////////////////
 Print("\n1.1  Jabber Detect(10BASE-TOnly)(RO/LH)..%s", 
  ((phy_stat_reg & PHYSTAT_JUBBER_DET) != 0) ?
   "1(Jabber condition detected" : 	// Jubber detect
   "0(No jabber condition detected)"
 );

// Extended capability (PHYSTAT_EXT_CAP) ///////////////////////////////////////
 Print("\n1.0  Extended Register Capability (RO)...%s", 
  (phy_stat_reg & PHYSTAT_EXT_CAP) ?	/* Extended capability */	 
   "1(Supported)" :
   "0(No Supported)"
 );

// 
 Print("\n     RO = Read Only");
 Print("\n     LL = Latching Low");
 Print("\n     LH = Latching High");
}


////////////////////////////////////////////////////////////////////////////////
// View PHY Identification register 1, 2 (Address 2, 3) ////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationIden(void)
{
 U32 phy_id_reg1;			// PHY identififcation Register 1
 U32 phy_id_reg2;			// PHY identififcation Register 2
 U32 oui_reg1;				// register 1 OUI bits
 U32 oui_reg2;				// register 2 OUI bits
 U32 val;				// bit value  
 int bit;				// bit number

////////////////////////////////////////////////////////////////////////////////
// PHY_ID_REG1 // PHY identififcation Register 1 (Address 2) ///////////////////
////////////////////////////////////////////////////////////////////////////////

 Print("\n\nPHY.2,3 Identification Registers 1,2.");
 phy_id_reg1 = MiiStationRead
  (
   PHY_ID_REG1	, 			// register address 
   PHYHWADDR				// PHY address
  );

// 
 Print("\n2.PHY identififcation Register 1 ......0x%04x", phy_id_reg1);
 Print("\n2.15:0 bit 3 througt 18 of OUI   ......0x%04x", phy_id_reg1 & 0xFFFF);

// 
 Print("\n  Register bit number .");
 for(bit = 15; bit >= 0; --bit)
  {
   Print("%2d ", bit);			// print bit number
  }

// 
 oui_reg1 = 0;				// register 1 OUI part
 Print("\n  bit value ...........");
 for(bit = 15; bit >= 0; --bit)		// print bit value
  {
   val = ((phy_id_reg1 >> bit) & 1);	// bit value
   if(val == 1)
    {
     oui_reg1 |= (1 << (15 - bit)) << 2;
// Intel LXT970A Dual-Speed Fast Ethernet Transceiver
// Order Number: 249099-001
// January 2001  
// Page 66 : OUI bit numbers from 1 to 24 (not from 0 to 23)
    }

   Print("%2d ", val);
  }

 Print("\n  OUI bit number ......");
 for(bit = 3; bit <= 18; ++bit)
  {
   Print("%2d ", bit);			// print bit number
  }


////////////////////////////////////////////////////////////////////////////////
// PHY_ID_REG2 // PHY identififcation Register 2 (Address 3) ///////////////////
////////////////////////////////////////////////////////////////////////////////

 phy_id_reg2 = MiiStationRead	// read PHY register 
  ( 
   PHY_ID_REG2	, 		// register address
   PHYHWADDR			// PHY address
  );

 Print("\n3.PHY identififcation Register 2 ......0x%04x", phy_id_reg2);

 Print("\n3.15:10 bits 19 throught 24 of the OUI.0x%04x",
        (phy_id_reg2 >> 10) & 0x3F);

// 
 Print("\n  Register bit number .");
 for(bit = 15; bit >= 10; --bit)
  {
   Print("%2d ", bit);			// print bit number
  }

// 
 oui_reg2 = 0;
 Print("\n  bit value ...........");
 for(bit = 15; bit >= 10; --bit)	// print bit value
  {
   val = ((phy_id_reg2 >> bit) & 1);	// bit value
   if(val == 1)
    {
     oui_reg2 |= 1 << (18 + (15 - bit));
    }

   Print("%2d ", val);			// print bit value 
  }

 Print("\n  OUI bit number ......");
 for(bit = 19; bit <= 24; ++bit)
  {
   Print("%2d ", bit);			// print bit number
  }


 Print("\n3.9 : 4 Manufactures Part Number.......0x%x", (phy_id_reg2 >> 4) & 0x3F);
 Print("\n3.3 : 0 Manufactures revision Number...0x%x",  phy_id_reg2 & 0xF);
 Print("\n        OUI............................0x%06x", oui_reg1 | oui_reg2 );
// OUI - 3 bytes
}

////////////////////////////////////////////////////////////////////////////////
// PHY_ANA_REG // Auto Negotiation Advertisement Register (Address 4) //////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationAna(void)
{
 U8 it;
 
 while(1)
  {
   U32 phy_ana_reg;		// Auto Negotiation Advertisement Register
   phy_ana_reg = MiiStationRead	// read register 
    ( 
     PHY_ANA_REG , 		// register address
     PHYHWADDR			// PHY address
    );

   Print("\n\n4.Auto Negotiation Advertisement Register : 0x%04x",phy_ana_reg);

// Next Page (PHYANA_NEXTPAGE) /////////////////////////////////////////////////
   Print("\n4.15     Next Page (RO)...............%s",
    ((phy_ana_reg & PHYANA_NEXTPAGE) != 0) ?
    "1(Supported)" :		// Next Page 
    "0(Not Supported)"
   );

// Remote Fault (PHYANA_REMOTE_FAULT) //////////////////////////////////////////
   Print("\n4.13 [F] Remote Fault (R/W)...........%s",
    ((phy_ana_reg & PHYANA_REMOTE_FAULT) != 0) ?
     "1(Remote fault)" :		// Remote Fault
     "0(No remote fault)"
   ); 

//
   Print("\n4.12:11  Reserved Ignore on read(R/W).%d%d",
     ((phy_ana_reg >> 12) & 1),
     ((phy_ana_reg >> 11) & 1)
    ); 


// Pause (PHYANA_PAUSE) ////////////////////////////////////////////////////////
   Print("\n4.10 [P] Pause (R/W) .................%s",
    ((phy_ana_reg & PHYANA_PAUSE) != 0) ?	/* Pause */
     "1(Pause operation is enabled for full-duplex links)" :
     "0(Pause operation is disabled)"
   ); 

// 100BASE-T4 (PHYANA_100BASE_T4) //////////////////////////////////////////////
   Print("\n4.9  [A] 100BASE-T4 (R/W).............%s", 
    ((phy_ana_reg & PHYANA_100BASE_T4) != 0) ?	/* 100BASE-T4 */
     "1(100BASE-T4 capability is available)" :
     "0(100BASE-T4 capability is not available)"
   );

// 100BASE-TX full-duplex (PHYANA_DR100_TX_FULL) ///////////////////////////////
   Print("\n4.8  [B] 100BASE-TX full-duplex (R/W).%s", 
    ((phy_ana_reg & PHYANA_DR100_TX_FULL) != 0)	?
     "1(DTE is 100BASE-TX full-duplex capable)" :
     "0(DTE is not 100BASE-TX full-duplex capable)"
   );
  
// 100BASE-TX (PHYANA_DR100_TX_HALF) ///////////////////////////////////////////
   Print("\n4.7  [C] 100BASE-TX (R/W).............%s", 
    ((phy_ana_reg & PHYANA_DR100_TX_HALF) != 0) ?
     "1(DTE is 100BASE-TX capable)" :
     "0(DTE is not 100BASE-TX capable)"
   );

// 10BASE-T full-duplex (PHYANA_DR10_TX_FULL) //////////////////////////////////
   Print("\n4.6  [D] 10BASE-T full-duplex (R/W)...%s", 
    ((phy_ana_reg & PHYANA_DR10_TX_FULL) != 0) ?
     "1(DTE is 10BASE-T full-duplex capable)"  :
     "0(DTE is not 10BASE-T full-duplex capable)"
   ); 

// 10BASE-T (PHYANA_DR10_TX_HALF) //////////////////////////////////////////////
   Print("\n4.5  [E] 10BASE-T (R/W)...............%s",
    ((phy_ana_reg & PHYANA_DR10_TX_HALF) != 0) ?
     "1(DTE is 10BASE-T capable)" :
     "0(DTE is not 10BASE-T capable)"
   );

//
   Print("\n4.4:0[S] Selector Filed (R/W).........");
   if((phy_ana_reg & 0x1F) == 0x01)
    {
     Print("<00001> = IEEE 802.3");
    }
   else
   if((phy_ana_reg & 0x1F) == 0x02)
    {
     Print("<00010> = IEEE 802.9 ISLAN-16T");
    }
   else 
   if((phy_ana_reg & 0x1F) == 0x00)
    {
     Print("<00000> = Reserved for future Auto-Negotiation development");
    }
   else 
   if((phy_ana_reg & 0x1F) == 0x1F)
    {
     Print("<11111> = Reserved for future Auto- Negotiation development");
    }
   else
    {
     Print("%x",(phy_ana_reg & 0x1F));
    }
// 
   Print("\n         R/W = Read/Write"); 
   Print("\n         RO  = Read Only"); 
   Print("\n     [Q] Quit");				// Quit from station management 
   Print("\nSelect Test Item : "); 
   it = get_upper();			
	
   switch(it)
    {
     case 'F' : 	
      {
////////////////////////////////////////////////////////////////////////////////
// Remote Fault ////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\nEnable Remote Fault [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {				// return from  Auto-Neg. advertisement
         return;  			// register test 
        }
       else
       if(
          it == 'Y' ||			// enable remote fault 
          it == 'N'			// disable remote fault 
        )
        {
         if(it == 'Y')
          {
// enable remote fault /////////////////////////////////////////////////////////
           Print("\nRemote Fault Enabled.");
           phy_ana_reg |= PHYANA_REMOTE_FAULT;	// set remote fault bit 
          }
         else
          {
// disable remote fault ////////////////////////////////////////////////////////
           Print("\nRemote Fault Disabled.");
           phy_ana_reg &=~PHYANA_REMOTE_FAULT;	// clear remote fault bit 
          }
          
         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
        }  
      }
      break;

     case 'P' : 			// Pause
      {
////////////////////////////////////////////////////////////////////////////////
// Pause ///////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\nEnable Pause operation for full-duplex links [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {				// quit from 
         return;  			// register test 
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable pause operation //////////////////////////////////////////////////////
           Print("\nPause operation is enabled for full-duplex links.");
           phy_ana_reg |= PHYANA_PAUSE;
          }
         else
          {
// disable pause operation /////////////////////////////////////////////////////
           Print("\nPause operation is disabled."); 
           phy_ana_reg &=~PHYANA_PAUSE;
          }
          
         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
        }  
      }
      break;

     case 'A' : 			// 100BASE-T4
      {
////////////////////////////////////////////////////////////////////////////////
// [A] 100BASE-T4 //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\nThe LXT970A does not support 100BASE-T4,");
       Print("\nbut allows this bit to be set to advertise in the");
       Print("\nAuto-Negotiation sequence for 100BASE-T4 operation.");
       Print("\nAn external 100BASE-T4 transceiver could be switched"); 
       Print("\nin if this capability is desired.");
       Print("\nEnable 100BASE-T4 capability [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {
         return;  
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable 100Base-T4 ///////////////////////////////////////////////////////////
           Print("\n100BASE-T4 capability is available.");
           phy_ana_reg |= PHYANA_100BASE_T4;
          }
         else
          {
// disable 100Base-T4 //////////////////////////////////////////////////////////
           Print("\n100BASE-T4 capability is not available.");
           phy_ana_reg &=~PHYANA_100BASE_T4;
          }
          
         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
        }  
      }
      break;

     case 'B' : 			// 100BASE-TX full-duplex
      {
////////////////////////////////////////////////////////////////////////////////
// [B] 100BASE-TX full-duplex //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\nEnable 100Base-TX full-duplex [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {
         return;  
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable 100Base-TX full-duplex ///////////////////////////////////////////////
           Print("\nDTE is 100Base-TX full-duplex capable.");
           phy_ana_reg |= PHYANA_DR100_TX_FULL;
          }
         else
          {
// disable 100Base-TX full-duplex //////////////////////////////////////////////
           Print("\nDTE is not 100BASE-TX full-duplex capable.");
           phy_ana_reg &=~PHYANA_DR100_TX_FULL;
          }
          
         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
        }  
      }
      break;

     case 'C' : 			// 100BASE-TX
      {
////////////////////////////////////////////////////////////////////////////////
// [C] 100BASE-TX //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\nEnable 100Base-TX [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {
         return;  
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
           Print("\nDTE is 100BASE-TX capable.");
           phy_ana_reg |= PHYANA_DR100_TX_HALF;
          }
         else
          {
           Print("\nDTE is not 100BASE-TX capable.");
           phy_ana_reg &=~PHYANA_DR100_TX_HALF;
          }
          
         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
        }  
      }
      break;

     case 'D' : 	// 10BASE-T full-duplex
      {
////////////////////////////////////////////////////////////////////////////////
// [D] 10BASE-T full-duplex ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\nEnable 10Base-T full-duplex [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {
         return;  
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable 10Base-T full-duplex /////////////////////////////////////////////////
           Print("\nDTE is 10BASE-T full-duplex capable.");
           phy_ana_reg |= PHYANA_DR10_TX_FULL;
          }
         else
          {
// disable 10Base-T full-duplex ////////////////////////////////////////////////
           Print("\nDTE is not 10BASE-T full-duplex capable."); 
           phy_ana_reg &=~PHYANA_DR10_TX_FULL;
          }
          
         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
        }  
      }
      break;

     case 'E' : 			// 10BASE-T 
      {
////////////////////////////////////////////////////////////////////////////////
// [E] 10Base-T ////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\nEnable 10Base-T [Y/N/Q]"); 
       it = get_upper();
       if(it == 'Q')
        {
         return;  
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable 10Base-T half duplex /////////////////////////////////////////////////
           Print("\nDTE is 10BASE-T capable.");
           phy_ana_reg |= PHYANA_DR10_TX_HALF;
          }
         else
          {
// disable 10Base-T half duplex ////////////////////////////////////////////////
           Print("\nDTE is not 10BASE-T capable.");
           phy_ana_reg &=~PHYANA_DR10_TX_HALF;
          }
          
         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
        }  
      }
      break;

     case 'S' : 			// Selector Filed 
      {
////////////////////////////////////////////////////////////////////////////////
// [S] Selector Filed //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       while(1)
        {
         Print("\n\nSelector Filed.");
         Print("\n[1] <00001> = IEEE 802.3.");
         Print("\n[2] <00010> = IEEE 802.9 ISLAN-16T.");
         Print("\n[0] <00000> = Reserved for future Auto-Negotiation development.");
         Print("\n[R] <11111> = Reserved for future Auto-Negotiation development.");
         Print("\n[Q] Quit");			// Quit from station management 
         Print("\nSelect Test Item : "); 
         it = get_upper();			
         if(it == 'Q')
          {
           return;
          }
         else
         if(it == '1') 
          {
// <00001> = IEEE 802.3 ////////////////////////////////////////////////////////
           Print("\nSelector Field = <00001>(IEEE 802.3).");
           phy_ana_reg &= ~0x1F;	// clear selector field bits
           phy_ana_reg |=  0x01;

          } 
         else
         if(it == '2') 
          {
// <00010> = IEEE 802.9 ISLAN-16T //////////////////////////////////////////////
           Print("\nSelector Field = <00010>(IEEE 802.9 ISLAN-16T).");
           phy_ana_reg &= ~0x1F;	// clear selector field bits
           phy_ana_reg |=  0x02;
          }
         else 
         if(it == '0') 
          {
// <00000> = Reserved for future Auto-Negotiation development //////////////////
           Print("\nSelector Field = <00000>.");
           phy_ana_reg &= ~0x1F;	// clear selector field bits
           phy_ana_reg |=  0x00;
          }
         else 
         if(it == 'R') 
          {
// <11111> = Reserved for future Auto-Negotiation development //////////////////
           Print("\nSelector Field = <11111>.");
           phy_ana_reg &= ~0x1F;	// clear selector field bits
           phy_ana_reg |=  0x1F;
          }         
         else         
          {
           Print("\nInvalid Test Item Selected"); 
           continue;
          }

         MiiStationWrite		// register write
          ( 
           PHY_ANA_REG  , 		// register address
           PHYHWADDR	,		// PHY address
           phy_ana_reg			// register value
          );
         { break; }
        } 
      }
      break;

     case 'Q' : return;
     default  : 
      Print("\nInvalid Test Item Selected") ; 
      break;
    }

   Print("\nPress any key to continue");
   it = get_upper();
   if(it == 'Q')
    {
     break;
    } 
  }
}  


////////////////////////////////////////////////////////////////////////////////
// PHY_ANLPAR_REG // Auto Negotiation Link Partner Ability Register(Address 5)//
////////////////////////////////////////////////////////////////////////////////

static void PhyStationAnlPar(void)
{
 U32 phy_anlpar_reg;			// Auto Negotiation Link Partner Ability Register
 phy_anlpar_reg = MiiStationRead	// read PHY register  
 (
  PHY_ANLPAR_REG	, 		// register address 	
  PHYHWADDR				// PHY address
 );

 Print("\n5.Auto Negotiation Link Partner Ability : 0x%04x ",phy_anlpar_reg);

// Next Page(PHYANLPAR_NEXTP) //////////////////////////////////////////////////
 Print("\n5.15 Next Page (RO)............%s", 
  ((phy_anlpar_reg & PHYANLPAR_NEXTP) != 0) ?
   "1(LP has ability to send multiple pages)" :
   "0(LP has no ability to send multiple pages)"
 ); 

// Acknowledge(PHYANLPAR_ACK) //////////////////////////////////////////////////
 Print("\n5.14 Acknowledge (RO)..........%s", 
  ((phy_anlpar_reg & PHYANLPAR_ACK) != 0) ?
   "1(LP has received Link Code Word)" :
   "0(LP has not received Link Code Word)"
 );

// Remote Fault(PHYANLPAR_REMFAULT) ////////////////////////////////////////////
 Print("\n5.13 Remote Fault (RO).........%s", 
  ((phy_anlpar_reg & PHYANLPAR_REMFAULT) != 0) ? 
   "1(Remote fault)" :
   "0(No remote fault)"
 ); 

// 5.12:11 Reserved Ignore on read. RO N/A
// Pause(PHYANLPAR_PAUSE) //////////////////////////////////////////////////////
 Print("\n5.10 Pause (RO)................%s", 
  ((phy_anlpar_reg & PHYANLPAR_PAUSE) != 0) ?
   "1(Pause operation is enabled for LP)" :
   "0(Pause operation is disabled for LP)"
 ); 

// 100BASE-T4(PHYANLPAR_100BASET4) /////////////////////////////////////////////
 Print("\n5.9  100BASE-T4 (RO)...........%s", 
  ((phy_anlpar_reg & PHYANLPAR_100BASET4) != 0) ?
   "1(LP is 100BASE-T4 capable)" :
   "0(LP is not 100BASE-T4 capable)"
 ); 

// 100BASE-TX full-duplex(PHYANLPAR_100BASETX_FD) //////////////////////////////
 Print("\n5.8  100BaseTX full-duplex(RO).%s", 
  ((phy_anlpar_reg & PHYANLPAR_100BASETX_FD) != 0) ?
   "1(LP is 100BASE-TX full-duplex capable)" :
   "0(LP is not 100BASE-TX full-duplex capable)"
 ); 

// 100BASE-TX(PHYANLPAR_100BASETX_HD) //////////////////////////////////////////
 Print("\n5.7  100BASE-TX (RO)...........%s", 
  ((phy_anlpar_reg & PHYANLPAR_100BASETX_HD) != 0) ?
   "1(LP is 100BASE-TX capable)" :
   "0(LP is not 100BASE-TX capable)"
 ); 

// 10BASE-T full-duplex(PHYANLPAR_10BASET_FD) //////////////////////////////////
 Print("\n5.6  10BASE-T full-duplex(RO)..%s", 
  ((phy_anlpar_reg & PHYANLPAR_10BASET_FD) != 0) ?
   "1(LP is 10BASE-T full-duplex capable)" :
   "0(LP is not 10BASE-T full-duplex capable)"
 ); 

// 10BASE-T(PHYANLPAR_10BASET_HD) //////////////////////////////////////////////
 Print("\n5.5  10BASE-T (RO).............%s", 
  ((phy_anlpar_reg & PHYANLPAR_10BASET_HD) != 0) ?
   "1(LP is 10BASE-T capable)" :
   "0(LP is not 10BASE-T capable)"
 ); 

// Selector Field S[4:0](PHYANLPAR_SEL_FIELD) //////////////////////////////////
 Print("\n5.4:0 Selector Field (RO)......");
 switch((phy_anlpar_reg & PHYANLPAR_SEL_FIELD) != 0) 
  {
   case 0x01	:
    Print("<00001>(IEEE 802.3)");
    break;

   case 0x02	: 
    Print("<00010>(IEEE 802.9 ISLAN-16T)");
    break;

   case 0x00	:
    Print("<00000>(Reserved for future Auto-Negotiation development)");
    break;

   case 0x1F	:
    Print("<11111>(Reserved ved for future Aut o-Negotiation development)");
    break;
  }
 Print("\n     LP = Link Partner");
}


////////////////////////////////////////////////////////////////////////////////
// PHY_ANE_REG // Auto Negotiation Expansion (Address 6) ///////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationAne(void)
{
 U32 phy_ane_reg;		// Auto Negotiation Expansion register
 phy_ane_reg = MiiStationRead
  ( 
   PHY_ANE_REG	, 
   PHYHWADDR
  );
 Print("\n6. Auto Negotiation Expansion Register : 0x%04x ",phy_ane_reg);

// Parallel Detection Fault(PHYANE_PAR_DET_FAULT) //////////////////////////////
 Print("\n6.4 Parallel Detect.Fault (RO/LH).%s",
  ((phy_ane_reg & PHYANE_PAR_DET_FAULT) != 0) ?	
   "1(Parallel detection fault has occurred)" :
   "0(Parallel detection fault has not occurred)"
 );

// Link Partner Next Page Able(PHYANE_LP_NEXT_PAGE) ////////////////////////////
 Print("\n6.3 LP Next Page Able (RO)........%s", 
  ((phy_ane_reg & PHYANE_LP_NEXT_PAGE) != 0) ?
   "1(Link partner is next page able)" :
   "0(Link partner is not next page able)"
 );

// Next Page Able(PHYANE_NEXT_PAGE) ////////////////////////////////////////////
 Print("\n6.2 Next Page Able (RO)...........%s", 
  ((phy_ane_reg & PHYANE_NEXT_PAGE) != 0) ?	
   "1(Supported)" :
   "0(Not Supported)"
 );

// Page Received(PHYANE_PAGE_REC) //////////////////////////////////////////////
 Print("\n6.1 Page Received (RO/LH).........%s", 
  ((phy_ane_reg & PHYANE_PAGE_REC) != 0) ?
   "1(3 identical and consecutive link code words received)" :
   "0(3 identical and consecutive link code words not received)"
 );

// Link Partner Auto Neg Abl(PHYANE_LP_AUTONEG) ////////////////////////////////
 Print("\n6.0 LP Auto Neg Able (RO).........%s", 
  ((phy_ane_reg & PHYANE_LP_AUTONEG) != 0) ?
   "1(LP is auto-negotiation able)" :
   "0(LP is not auto-negotiation able)"
 ); 

// 
 Print("\n    LP = Link Partner");	
 Print("\n    RO = Read Only");
 Print("\n    LH = Latching High");
}

////////////////////////////////////////////////////////////////////////////////
// PHY_ECNTL_REG1 // Mirror Register (Address 16, Hex 10) //////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationEcntl(void)
{
 U32 phy_ecntl_reg1;			// Mirror Register
 TIME tm;
 int err = 0;				// error flag 
 U8 ch;

//
 Print("\n\nPHY.16. Read/Write test");
 phy_ecntl_reg1 = MiiStationRead
  ( 
   PHY_ECNTL_REG1	, 		// 
   PHYHWADDR				// 
  );
 Print("\n16. Mirror Register : 0x%04x ", phy_ecntl_reg1);
 Print("\nThis register is intended for use in checking the");
 Print("\nMII serial port and has no affect on chip operation.");
 Print("\nPress any key to run read & write test");
 ch = get_upper();

// PHY_ECNTL_REG1 Read Write Test //////////////////////////////////////////////

 TimerReset(1);
 TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));
 TimerStart(1);

 Print("\nPress any key to break read & write test");
 for(phy_ecntl_reg1 = 0; phy_ecntl_reg1 < 0xFFFF; ++phy_ecntl_reg1)
  {
   if(kbd_hit())
    {
     ch = get_upper();
     break; 
    } 

//
   if((phy_ecntl_reg1 & 0x1F) == 0)
    {
     PrintRotSlash((phy_ecntl_reg1 >> 5) & 0x3);
    }
    
// PHY_ECNTL_REG1 Write //////////////////////////////////////////////////////// 
   MiiStationWrite
    (
     PHY_ECNTL_REG1	, 		// Register Address 
     PHYHWADDR		, 		// PHY Device Address
     phy_ecntl_reg1			// writed value
    );

   if(
     phy_ecntl_reg1 != 			// writed value not equal 
     MiiStationRead			// read value
      ( 
       PHY_ECNTL_REG1	, 		// Register Address
       PHYHWADDR			// PHY Device Address 
      )
    )
    {
     err = 1;
     Print("\nPHY_ECNTL_REG1 Write Error");
    }
  }


 GetSysTime(1,& tm); 
 TimerReset(1);
 Print
  (
   "\nTime %d:%d:%d.%d",
    tm.tm_hour	,
    tm.tm_min	,
    tm.tm_sec	,
    tm.tm_ms
  );


//16.15:0  User Defined This register is intended for use in checking 
//         the MII serial port and has no effect on chip operation.

 if(err == 1)
  {
   Print("\nFail.\n");
  }
 else
  {
   Print("\nOk.\n");
  }
}


////////////////////////////////////////////////////////////////////////////////
// PHY_INTEN_REG // Interrupt Enable Register (Address 17, Hex 11) /////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationIntEn(void)
{
 U8 it;
 while(1)
  {
   U32 phy_inten_reg;		// Interrupt Enable Register
   phy_inten_reg = MiiStationRead
    (
     PHY_INTEN_REG	, 
     PHYHWADDR
    );
   Print("\n17.Interrupt Enable Register : 0x%04x ", phy_inten_reg);

// 
   Print("\n17.3 [L] MII driver levels(R/W)...%s",
    ((phy_inten_reg & PHYINTEN_MIIDRVLVL) != 0) ?
     "1(Reduced MII driver levels)" :
     "0(High-strength MII driver levels)"
   );

   Print("\n17.2 [C] Link Loss Criteria(R/W)..%s", 
    ((phy_inten_reg & PHYINTEN_LNK_CRITERIA) != 0) ?	
     "1(Enhanced link loss criteria)" :
     "0(Standard link criteria)"
   );
 
   Print("\n17.1 [E] Enable interrupts(R/W)...%s", 
    ((phy_inten_reg & PHYINTEN_INTEN) != 0) ?	
     "1(Enable interrupts)" :
     "0(Disable interrupts)"
   );


   Print("\n17.0 [T] TINT(R/W)................%s", 
    ((phy_inten_reg & PHYINTEN_TINT) != 0) ?	
     "1(Forces MDINT Low and sets bit 18.15 = 1)" :
     "0(Normal operation)"
   );

   Print("\n         R/W = Read/Write"); 
   Print("\n     [Q] Quit");				// Quit from station management 
   Print("\nSelect Test Item : "); 
   it = get_upper();			
	
   switch(it)
    {
     case 'L' : 
      {
////////////////////////////////////////////////////////////////////////////////
// MII driver levels ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\nReduced MII driver levels [Y/N/Q]");
       it = get_upper();
       if(it == 'Q')
        {				// quit from interrupt 	
         return;			// enable register test  
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
// 
           Print("\nReduced MII driver levels.");
           phy_inten_reg |= PHYINTEN_MIIDRVLVL; 
          } 
         else
          {
           Print("\nHigh-strength MII driver levels.");
           phy_inten_reg &=~PHYINTEN_MIIDRVLVL; 
          } 

// 
         MiiStationWrite
          (
           PHY_INTEN_REG	, 
           PHYHWADDR		,
           phy_inten_reg 
         );
        } 
      }
      break;

     case 'C' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Link Loss Criteria //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\nEnhanced link loss criteria. Link loss criteria"); 
       Print("\nis independent of symbol error rate. Loss of"); 
       Print("\nscrambler lock for more than 1 - 2 msec will"); 
       Print("\nbrings the link down. Link up criteria is based"); 
       Print("\non symbol error rate.");
       Print("\n");
       Print("\nStandard link criteria. Both link up and link"); 
       Print("\nloss are based on symbol error rate.");

       Print("\nEnable Enhanced link loss criteria [Y/N/Q]");
       it = get_upper();
       if(it == 'Q')
        {
         return;
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
           Print("\nEnhanced link loss criteria.");
           phy_inten_reg |= PHYINTEN_LNK_CRITERIA; 
          } 
         else
          {
           Print("\nStandard link criteria");
           phy_inten_reg &=~PHYINTEN_LNK_CRITERIA; 
          } 

// 
         MiiStationWrite
          (
           PHY_INTEN_REG	, 
           PHYHWADDR		,
           phy_inten_reg 
         );
        } 
      }
      break;


     case 'E' : 
      {
////////////////////////////////////////////////////////////////////////////////
// Enable interrupts ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\nEnable interrupts [Y/N/Q]");
       it = get_upper();
       if(it == 'Q')
        {
         return;
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
           Print("\nEnable Interrupts."); 
           phy_inten_reg |= PHYINTEN_INTEN; 
          } 
         else
          {
           Print("\nDisable interrupts.");
           phy_inten_reg &=~PHYINTEN_INTEN; 
          } 

// 
         MiiStationWrite
          (
           PHY_INTEN_REG	, 
           PHYHWADDR		,
           phy_inten_reg 
         );
        } 
      }
      break;


     case 'T' : 
      {
////////////////////////////////////////////////////////////////////////////////
// TINT ////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\nForce MDINT Low and set bit 18.15 = 1 [Y/N/Q]");
       it = get_upper();
       if(it == 'Q')
        {
         return;
        }
       else
       if(
          it == 'Y' ||
          it == 'N'
        )
        {
         if(it == 'Y')
          {
           Print("\nForce MDINT Low and sets bit 18.15 = 1.");
           phy_inten_reg |= PHYINTEN_TINT; 
          } 
         else
          {
           Print("\nNormal operation.");
           phy_inten_reg &=~PHYINTEN_TINT; 
          } 

// 
         MiiStationWrite
          (
           PHY_INTEN_REG	, 
           PHYHWADDR		,
           phy_inten_reg 
         );
        } 
      }
      break;


     case 'Q' : return;
     default  : 
      Print("\nInvalid Test Item Selected") ; 
      break;
    }

   Print("\nPress any key to continue");
   it = get_upper();
   if(it == 'Q')
    {
     break;
    } 
  }
}

////////////////////////////////////////////////////////////////////////////////
// PHY_10BOP_REG // Interrupt Status Register //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationIntStat(void)
{
 U32 phy_intstat_reg;		// Interrupt Status Register
 phy_intstat_reg = MiiStationRead
  (
   PHY_INTSTAT_REG	, 
   PHYHWADDR
  );
 Print("\n18.Interrupt Status Register : 0x%04x ",phy_intstat_reg);

 Print("\n18.15 MINT (RO)....%s", 
  (phy_intstat_reg & PHYINTSTAT_MINT) ?  
   "1(MII interrupt pending)" :
   "0(no MII interrupt pending)"
  );

 Print("\n18.14 XTAL OK (RO).%s", 
  (phy_intstat_reg & PHYINTSTAT_XTALOK) ?
   "1(LXT970A powered up and the on-chip clocks are stable)" :
   "0(XTAL circuit is not stable)"
 );

 Print("\n      RO = Read Only");
}
////////////////////////////////////////////////////////////////////////////////
// PHY_ECNTL_REG2  // Extended control register ////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationIntEcntl(void)
{
 U8 it; 

 while(1) 
  {
   U32 phy_ecntl_reg;			// Extended control register
   phy_ecntl_reg = MiiStationRead	// read control register 
    ( 
     PHY_ECNTL_REG2	, 		// register address
     PHYHWADDR				// PHY address
    );
   Print("\n19. Configuration Register : 0x%04x ", phy_ecntl_reg);

// Txmit Test (100BASE-TX) ///////////////////////////////////////////////////// 
   Print("\n19.14 [T] Txmit Test(100BASE-TX)(R/W)...%s", 
    ((phy_ecntl_reg & PHYECNTL_100BASETX_TXMIT_TEST) != 0) ?
     "1(100BASE-T transmit test enabled)" :
     "0(Normal operation)"
   );

// PHYECNTL_REPEATER_MODE // Repeater Mode /////////////////////////////////////
   Print("\n19.13 [R] Repeater Mode (R/W)...........%s", 
    ((phy_ecntl_reg & PHYECNTL_REPEATER_MODE) != 0) ?
     "1(Enable Repeater Mode)" :
     "0(Enable DTE Mode)"
   );

// MDIO_INT // PHYECNTL_MDIO_INT ///////////////////////////////////////////////
   Print("\n19.12 [M] MDIO_INT (R/W)................%s",  
    ((phy_ecntl_reg & PHYECNTL_MDIO_INT) != 0) ?
     "1(Enable interrupt signaling on MDIO)" :
     "0(Normal operation)"
   );

// TP Loopback (10BASE-T)(PHYECNTL_10BASET_TPLOOPBACK) /////////////////////////
   Print("\n19.11 [P] TP Loopback(10BASE-T)(R/W)....%s", 
    ((phy_ecntl_reg & PHYECNTL_10BASET_TPLOOPBACK) != 0) ?
     "1(Disable 10Base TP Loopback)" :
     "0(Enable 10Base TP Loopback)"
   );

// SQE (10BASE-T)(PHYECNTL_10BASET_SQE) ////////////////////////////////////////
   Print("\n19.10 [E] SQE (10BASE-T) (R/W)..........%s", 
    ((phy_ecntl_reg & PHYECNTL_10BASET_SQE) != 0) ?
     "1(Enable SQE)" :
     "0(Disable SQE)"
   ); 

// Jabber (10BASE-T)(PHYECNTL_10BASET_JABBER) //////////////////////////////////
   Print("\n19.9  [J] Jabber (10BASE-T)(R/W)........%s", 
    ((phy_ecntl_reg & PHYECNTL_10BASET_JABBER) != 0) ?
     "1(Disable jabber)" :
     "0(Normal operation)"
   );

// Link Test (10BASE-T)(PHYECNTL_10BASET_LINK_TEST) ////////////////////////////
   Print("\n19.8  [L] Link Test (10BASE-T)(R/W).....%s", 
    ((phy_ecntl_reg & PHYECNTL_10BASET_LINK_TEST) != 0) ?
     "1(Disable 10BASE-T link integrity test)." :
     "0(Normal operation)"
   ); 

// LEDC Programming bits(PHYECNTL_LEDC_MODE) ///////////////////////////////////
   Print("\n19.7:6[C] LEDC Programming bits(R/W)....");
   switch(phy_ecntl_reg & PHYECNTL_LEDC_MODE)	/* LEDC Mode */
    {
     case PHYECNTL_LEDC_COL	:		/* collision */
      Print("00 LEDC indicates collision");
      break;
  
     case PHYECNTL_LEDC_OFF       :		/* LEDC is off */
      Print("01 LEDC is off");
      break;
 
     case PHYECNTL_LEDC_ACT       :		/* LEDC indicate activity */
      Print("10 LEDC indicates activity");
      break;

     case PHYECNTL_LEDC_ON        :		/* LEDC is on */
      Print("11 LEDC is continuously on");
      break;
    }

// Advance TX Clock(PHYECNTL_ADVTXCLOCK) ///////////////////////////////////////
   Print("\n19.5  [A] Advance TX Clock(R/W).........%s", 
    ((phy_ecntl_reg & PHYECNTL_ADVTXCLOCK) != 0) ? 
     "1(TX clock is advanced relative to TXD<4:0> and TX_ER by 1/2 TX_CLK cycle)" :
     "0(Normal operation)"
   );

// 5B Symbol/4B Nibble (100BASE-X only)(PHYECNTL_5BSYMBOL) /////////////////////
   Print("\n19.4  [5] 5B Symbol/4B Nibble(100BASEX).%s", 
    ((phy_ecntl_reg & PHYECNTL_5BSYMBOL) != 0) ?
     "1(5-bit Symbol Mode)" :
     "0(4-bit Nibble Mode)"
   );

// Scrambler (100BASE-X only)(PHYECNTL_SCRAMBLER) //////////////////////////////
   Print("\n19.3  [S] Scrambler(100BASEX only)(R/W).%s", 
    ((phy_ecntl_reg & PHYECNTL_SCRAMBLER) != 0) ? 
     "1(Bypass Tx scrambler & Rx descrambler)" :
     "0(Scrambler and descrambler enabled)"
   );


// 100BASE-FX (PHYECNTL_100BASEFX) /////////////////////////////////////////////
   Print("\n19.2  [F] 100BASE-FX(R/W)...............%s", 
    ((phy_ecntl_reg & PHYECNTL_100BASEFX) != 0) ?
     "1(Enable 100BASE fiber interface)" :
     "0(Disable fiber interface)"
   ); 
  
// 19.1    Reserved Write as 0; Ignore on read. R/W 0

// Transmit Disconnect(PHYECNTL_TX_DISCONNECT) /////////////////////////////////
   Print("\n19.0  [D] Transmit Disconnect(R/W)......%s", 
    ((phy_ecntl_reg & PHYECNTL_TX_DISCONNECT) != 0) ?
     "1(Disconnect TP transmitter from line)" :
     "0(Normal operation)"
   );

   Print("\n      [Q] Quit");			// Quit from station management 
   Print("\nSelect Test Item : "); 
   it = get_upper();
	
   switch(it)
    {
     case 'T' :
      {
//////////////////////////////////////////////////////////////////////////////// 
// Txmit Test (100BASE-TX) ///////////////////////////////////////////////////// 
//////////////////////////////////////////////////////////////////////////////// 
       Print("\nIf Txmit Test enabled LXT970A transmit data"); 
       Print("\nregardless of link status");
       Print("\nTxmit Test(100BASE-TX) [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable 100Base-Tx Txmit Test ////////////////////////////////////////////////
           Print("\nTxmit 100BASE-TX Test Enabled.");
           phy_ecntl_reg |= PHYECNTL_100BASETX_TXMIT_TEST;
          }
         else
          {
// disable 100Base-Tx Txmit Test ///////////////////////////////////////////////
           Print("\nTxmit 100BASE-TX Test Disabled.");
           phy_ecntl_reg &=~PHYECNTL_100BASETX_TXMIT_TEST;
          }

         MiiStationWrite		// write configuration register
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'R' :
      {
//////////////////////////////////////////////////////////////////////////////// 
// Repeater Mode ///////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

       Print("\n\nEnable Repeater Mode [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable repeater Mode //////////////////////////////////////////////////////// 
           Print("\nRepeater Mode Enabled.");
           phy_ecntl_reg |= PHYECNTL_REPEATER_MODE;
          }
         else
          {
// disable repeater Mode /////////////////////////////////////////////////////// 
           Print("\nRepeater Mode Disabled (Enable DTE Mode).");
           phy_ecntl_reg &=~PHYECNTL_REPEATER_MODE;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg
          );
        }
      }
      break;

     case 'M' :
      {
//////////////////////////////////////////////////////////////////////////////// 
// Enable interrupt signaling on MDIO //////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 

       Print("\n\nEnable interrupt signaling on MDIO [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable interrupt signaling on MDIO ////////////////////////////////////////// 
           Print("\nInterrupt signaling on MDIO enabled.");
           Print("\nBit is ignored unless the interrupt function is enabled (17.1=1).");
           phy_ecntl_reg |= PHYECNTL_MDIO_INT;
          }
         else
          {
// disable interrupt signaling on MDIO ///////////////////////////////////////// 
           Print("\nNormal operation (MDIO interrupt disabled).");
           phy_ecntl_reg &=~PHYECNTL_MDIO_INT;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'P' :
      {
//////////////////////////////////////////////////////////////////////////////// 
// TP Loopback(10BASE-T)////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////// 
       Print("\n\nEnable TP Loopback(10BASE-T) [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// return from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable TP 10Base-T Loopback ///////////////////////////////////////////////// 
// bit cleared. see LXT970A - Dual-Speed Fast Ethernet Transeiver 
// datasheet. Page 71
           Print("\nTP Loopback(10BASE-T) Enabled.");
           phy_ecntl_reg &=~PHYECNTL_10BASET_TPLOOPBACK;
          }
         else
          {
           Print("\nTP Loopback(10BASE-T) Disabled.");
           phy_ecntl_reg |= PHYECNTL_10BASET_TPLOOPBACK;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'E' :
      {
////////////////////////////////////////////////////////////////////////////////
// SQE (10BASE-T) //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nEnable SQE (10BASE-T) [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// enable SQE (10Base-T) ///////////////////////////////////////////////////////
           Print("\nSQE (10Base-T) Enabled.");
           phy_ecntl_reg |= PHYECNTL_10BASET_SQE;
          }
         else
          {
// disable SQE (10Base-T) //////////////////////////////////////////////////////
           Print("\nSQE (10Base-T) Disabled.");
           phy_ecntl_reg &=~PHYECNTL_10BASET_SQE;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register write
          );
        }
      }
      break;

     case 'J' :
      {
////////////////////////////////////////////////////////////////////////////////
// Jabber (10BASE-T) ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\n\nDisable Jabber (10BASE-T) [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {
         return;
        }
       else
       if(
         it == 'Y' || 			// disable jubber
         it == 'N'			// enable jubber
        )
        {
         if(it == 'Y')
          {
// disable 10Base-T Jabber /////////////////////////////////////////////////////
           Print("\nJabber 10Base-T disabled.");
           phy_ecntl_reg |= PHYECNTL_10BASET_JABBER;
          }
         else
          {
// enable 10Base-T Jabber //////////////////////////////////////////////////////
           Print("\nNormal operation. Jabber 10Base-T enabled.");
           phy_ecntl_reg &=~PHYECNTL_10BASET_JABBER;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'L' :
      {
////////////////////////////////////////////////////////////////////////////////
// Disable 10BASE-T link integrity test ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nDisable 10BASE-T link integrity test [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 			// disable link test 
         it == 'N'			// enable link test 
        )
        {
         if(it == 'Y')
          {
// disable 10Base-T link test //////////////////////////////////////////////////
           Print("\n10BASE-T Link Integrity test disabled.");
           phy_ecntl_reg |= PHYECNTL_10BASET_LINK_TEST;
          }
         else
          {
// enable 10Base-T link test ///////////////////////////////////////////////////
           Print("\n10BASE-T Link Integrity test enabled.");
           phy_ecntl_reg &=~PHYECNTL_10BASET_LINK_TEST;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'C' :
      {
////////////////////////////////////////////////////////////////////////////////
// LEDC Programming bits ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       while(1)
        {
         Print("\n\n LEDC Programming bits.");
         Print("\n[C] LEDC indicates collision.");
         Print("\n[F] LEDC is off.");
         Print("\n[A] LEDC indicates activity.");
         Print("\n[N] LEDC is continuously on.");
         Print("\n[Q] Quit");			// Quit from station management 
         Print("\nSelect Test Item : "); 
         it = get_upper();
     
         if(it == 'Q')
          {
           break;
          } 
         else
         if(it == 'C')
          {
// LEDC indicates collision ////////////////////////////////////////////////////
           Print("\nLEDC indicates collision.");
           phy_ecntl_reg &=~PHYECNTL_LEDC_MODE;
           phy_ecntl_reg |= PHYECNTL_LEDC_COL;	/* collision */
          }
         else
         if(it == 'F')
          {
// LEDC is off /////////////////////////////////////////////////////////////////
           Print("\nLEDC is off.");
           phy_ecntl_reg &=~PHYECNTL_LEDC_MODE;
           phy_ecntl_reg |= PHYECNTL_LEDC_OFF;	/* LEDC is off */
          }
         else 
         if(it == 'A')
          {
// LEDC indicates activity /////////////////////////////////////////////////////
           Print("\nLEDC indicates activity.");
           phy_ecntl_reg &=~PHYECNTL_LEDC_MODE;
           phy_ecntl_reg |= PHYECNTL_LEDC_ACT;	/* LEDC indicate activity */
          }
         else 
         if(it == 'N')
          {
// LEDC is continuously on /////////////////////////////////////////////////////
           Print("\nLEDC is continuously on.");
           phy_ecntl_reg &=~PHYECNTL_LEDC_MODE;
           phy_ecntl_reg |= PHYECNTL_LEDC_ON;	/* LEDC is on */
          }
         else
          {
           Print("\nInvalid Item Selected") ; 
           continue;
          }

         MiiStationWrite			// write register 
          ( 
           PHY_ECNTL_REG2	, 		// register address
           PHYHWADDR		,		// PHY address
           phy_ecntl_reg
          );
         { break; }
        }
      }
      break;

     case 'A' :
      {
////////////////////////////////////////////////////////////////////////////////
// Advance TX Clock ////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nEnable Advanced TX Clock [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 			// 
         it == 'N'			// 
        )
        {
         if(it == 'Y')
          {
// enable advanced TX clock ////////////////////////////////////////////////////
           Print("\nTX clock is advanced relative to TXD<4:0>,TX_ER by 1/2 TX_CLK cycle.");
           phy_ecntl_reg |= PHYECNTL_ADVTXCLOCK;
          }
         else
          {
// disable advanced TX clock ///////////////////////////////////////////////////
           Print("\nNormal operation.");
           phy_ecntl_reg &=~PHYECNTL_ADVTXCLOCK;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case '5' :
      {
////////////////////////////////////////////////////////////////////////////////
// Enable 100BASE-X 5B Symbol mode /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

       Print("\n\nEnable 100BASE-X 5B Symbol mode [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// 5-bit Symbol Mode ///////////////////////////////////////////////////////////
           Print("\n5-bit Symbol Mode seted.");
           phy_ecntl_reg |= PHYECNTL_5BSYMBOL;
          }
         else
          {
// 4-bit Nibble Mode ///////////////////////////////////////////////////////////
           Print("\n4-bit Nibble Mode seted.");
           phy_ecntl_reg &=~PHYECNTL_5BSYMBOL;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'S' :
      {
////////////////////////////////////////////////////////////////////////////////
// Scrambler (100BASE-X only)(PHYECNTL_SCRAMBLER) //////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nBypass transmit scrambler and receive descrambler (100BASE-X only) [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// return from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// Disable scrambler ///////////////////////////////////////////////////////////
           Print("\nBypass transmit scrambler and receive descrambler.");
           phy_ecntl_reg |= PHYECNTL_SCRAMBLER;
          }
         else
          {
// enable scrambler ////////////////////////////////////////////////////////////
           Print("\nNormal operation (scrambler and descrambler enabled).");
           phy_ecntl_reg &=~PHYECNTL_SCRAMBLER;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'F' :
      {
////////////////////////////////////////////////////////////////////////////////
// 100BASE-FX //////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nEnable 100Base fiber interface [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// return from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 			// enable fiber interface
         it == 'N'			// disable fiber interface
        )
        {
         if(it == 'Y')
          {
// enable fiber interface ////////////////////////////////////////////////////// 
           Print("\n100Base fiber interface enabled.");
           phy_ecntl_reg |= PHYECNTL_100BASEFX;
          }
         else
          {
// enable TP interface ///////////////////////////////////////////////////////// 
           Print("\nFiber interface disabled. 100Base twisted-pair interface enabled."); 
           phy_ecntl_reg &=~PHYECNTL_100BASEFX;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'D' :
      {
////////////////////////////////////////////////////////////////////////////////
// Transmit Disconnect /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
       Print("\n\nDisconnect TP transmitter from line [Y/N/Q]");
       it = get_upper(); 
       if(it == 'Q')
        {				// quit from configuration 
         return;			// register test 
        }
       else
       if(
         it == 'Y' || 
         it == 'N'
        )
        {
         if(it == 'Y')
          {
// disconnect TP transmitter from line /////////////////////////////////////////
           Print("\nTP transmitter disconnected from line.");
           phy_ecntl_reg |= PHYECNTL_TX_DISCONNECT;
          }
         else
          {
// connnect TP transmitter to line /////////////////////////////////////////////
           Print("\nNormal operation.");
           phy_ecntl_reg &=~PHYECNTL_TX_DISCONNECT;
          }

         MiiStationWrite		// write register 
          ( 
           PHY_ECNTL_REG2	, 	// register address
           PHYHWADDR		,	// PHY address
           phy_ecntl_reg		// register value
          );
        }
      }
      break;

     case 'Q' : return;
     default  : 
      Print("\nInvalid Test Item Selected") ; 
      break;
    }

   Print("\nPress any key to continue");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 
  }
}

////////////////////////////////////////////////////////////////////////////////
// Chip Status Register (PHY_CHIPSTAT_REG) /////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationChipStat(void)
{
 U32 phy_chipstat_reg;		// Chip Status Register 
 phy_chipstat_reg = MiiStationRead
  (
   PHY_CHIPSTAT_REG	, 
   PHYHWADDR
  );
 Print("\n\n20. Chip Status Register : 0x%04x ", phy_chipstat_reg);

// Link Status (PHYCHIPSTAT_LINKUP) ////////////////////////////////////////////

 Print("\n20.13 Link(RO).........................%s", 
  ((phy_chipstat_reg & PHYCHIPSTAT_LINKUP) != 0) ?
   "1(Link is up)" : "0(Link is down)"
 );

// Duplex Mode(PHYCHIPSTAT_FULLDUPLEX) /////////////////////////////////////////

 Print("\n20.12 Duplex Mode(RO)..................%s", 
  ((phy_chipstat_reg & PHYCHIPSTAT_FULLDUPLEX) != 0) ?
   "1(Full-duplex)" : "0(Half-duplex)"
  ); 

// Speed (PHYCHIPSTAT_100Mbps) /////////////////////////////////////////////////  
  
 Print("\n20.11 Speed(RO)........................%s", 
  ((phy_chipstat_reg & PHYCHIPSTAT_100Mbps) != 0) ?
   "1(100 Mbps operation)" : "0(10 Mbps operation)"
 );

// Auto-Negotiation Complete(PHYCHIPSTAT_AUTONEG_COMP) /////////////////////////
 Print("\n20.9  Auto-Negotiation Complete(RO/LH).%s", 
  ((phy_chipstat_reg & PHYCHIPSTAT_AUTONEG_COMP) != 0) ?
   "1(Auto-negotiation process complete)" :
   "0(Auto-negotiation process not complete)"
 );

// Page Received(PHYCHIPSTAT_PAGE_REC) /////////////////////////////////////////

 Print("\n20.8  Page Received(RO/LH).............%s", 
  ((phy_chipstat_reg & PHYCHIPSTAT_PAGE_REC) != 0) ?
   "1(Three identical and consecutive link code words received" :
   "0(Three identical and consecutive link code words not received"
 );
  
// Low-Voltage(PHYCHIPSTAT_LOW_VOLTAGE) ////////////////////////////////////////

 Print("\n20.2  Low-Voltage(RO)..................%s", 
  ((phy_chipstat_reg & PHYCHIPSTAT_LOW_VOLTAGE) != 0) ?
   "1(Low-voltage fault on VCC has occurred)" :
   "0(No fault)"
 ); 
}

////////////////////////////////////////////////////////////////////////////////
// Station Management Clock On /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void MDCOnTest(void)
{
 U32 mdc;
 U8 it;

 while(1)
  { 
   mdc = ((STACON & STACON_MDC) >> STACON_MDC_Off);

   Print("\nSTACON");
   Print("\n15:13 MDC clock rating......%d", mdc);
   Print("\n(%d x 1/fMCLK)", (16 + mdc * 2));
//
   Print("\n[0] 000 16 x (1/fMCLK)");
   Print("\n[1] 001 18 x (1/fMCLK)");
   Print("\n[2] 010 20 x (1/fMCLK)");
   Print("\n[3] 011 22 x (1/fMCLK)");
   Print("\n[4] 100 24 x (1/fMCLK)");
   Print("\n[5] 101 26 x (1/fMCLK)");
   Print("\n[6] 110 28 x (1/fMCLK)");
   Print("\n[7] 111 30 x (1/fMCLK)");
   Print("\n[Q] Quit");

// 
   Print("\nSelect MDC Clock Rate");
   it = get_upper();
   if(it == 'Q')
    {
     return;
    } 

// 
   if(it >= '0' && it <= '7')
    {
     mdc = (it - '0');
     MACCON |= MACCON_MDCOFF;
     gStaCon = (STACON & ~STACON_MDC) | (mdc << STACON_MDC_Off);
     STACON  = gStaCon;
     MACCON &=  ~MACCON_MDCOFF;
     Print("\nMDC Clock Rate is fMCLK / %d = %d", (mdc * 2 + 16), fMCLK / (mdc * 2 + 16));
     return; 
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// Station Management Clock Off ////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MDCOffTest(void)
{
 MACCON |= MACCON_MDCOFF;
 Print("\nMDC Offed\n") ;
}

////////////////////////////////////////////////////////////////////////////////
// Print STACON register ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void PrintSTACON(void)
{
 U32 stacon;
 stacon = STACON;		/* Station management control and address */

 Print("\n4:0   PHY register address (Addr).%d", (stacon & STACON_Addr));
 Print("\n9:5   PHY address (PHY)...........%d",
  ((stacon & STACON_PHY) >> STACON_PHY_Off)  
 );

 Print("\n10    Write (Wr)..................%d",((stacon & STACON_Wr    )? 1:0));
 Print("\n11    Busy bit (Busy).............%d",((stacon & STACON_Busy  )? 1:0)); 
 Print("\n12    Preamble suppress (PreSup)..%d",((stacon & STACON_PreSup)? 1:0));

 stacon = ((stacon & STACON_MDC) >> STACON_MDC_Off);
 Print("\n15:13 MDC clock rating............%d",   stacon );
 Print("\n      fMCLK / ([15:13] * 2 + 16)..%d", 
  (fMCLK / (stacon * 2 + 16))
 );
}


////////////////////////////////////////////////////////////////////////////////
// PHY Station Management Reg Read Write Test //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PhyStationTest(void)
{
 U8 it;					// selected item 

//
 do 
  {
   Print("\n\nPHY Station Management.");
   Print("\n[C] Read & Write PHY.0 Control Register.");	// View PHY Control Register
   Print("\n[S] Read PHY.1 Status Register.");	// View PHY Status Register
   Print("\n[D] Read PHY.2,3 Identification Registers 1,2.");
   Print("\n[A] Read & Write PHY.4 Auto Negotiation Advertisement Register.");
   Print("\n[P] Read PHY.5 Auto Negotiation Link Partner Ability Register.");
   Print("\n[N] Read PHY.6 Auto Negotiation Expansion Register.");
   Print("\n[W] Read & Write PHY.16 test.");
   Print("\n[I] Read & Write PHY.17 Interrupt Enable Register.");
   Print("\n[M] Read PHY.18 Interrupt Status Register.");
   Print("\n[E] Read & Write PHY.19 Extended control register.");
   Print("\n[R] Read PHY.20 Chip Status Register.");
   Print("\n[T] STACON Register.");
   Print("\n[O] MDC On Test");
   Print("\n[F] MDC Off Test");
   Print("\n[Q] Quit");				// Quit from station management 
   Print("\nSelect Test Item : "); 
   it = get_upper();
	
   switch(it)
    {
     case 'C' : PhyStationCtrl(); break;   // PHY Control register
     case 'S' : PhyStationStatus(); break; // PHY Status register
     case 'D' : PhyStationIden(); break;   // PHY Identification register 1,2
     case 'A' : PhyStationAna(); break;	   // Auto-Neg. ability  
     case 'P' : PhyStationAnlPar(); break; // Auto-Neg. Partner ability 	
     case 'N' : PhyStationAne(); break;	   // Auto-Neg. expansion register  
     case 'W' : PhyStationEcntl(); break;  // Read & Write test
     case 'I' : PhyStationIntEn(); break;  // Interrupt enable register
     case 'M' : PhyStationIntStat(); break;// Interrupt status register
     case 'E' : PhyStationIntEcntl();break;// Extended control register
     case 'R' : PhyStationChipStat();break;// Chip Status Register
     case 'T' : PrintSTACON(); break;	   // Print STACON register
     case 'O' : MDCOnTest(); break;	   // MDC On Test
     case 'F' : MDCOffTest(); break;	   // MDC Off Test
     case 'Q' : return;
     default  : 
      Print("\nInvalid Test Item Selected") ; 
      break;
    }

   Print("\nPress any key to continue");
   it = get_upper();
  } 
 while(it != 'Q');
}

////////////////////////////////////////////////////////////////////////////////
// MAC Global Control Register /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


static void MACCONConfig(void)
{
 U32 maccon;				// MACCON register value
 maccon = MACCON;			// read MACCON register value
 Print("\n\nMACCON (MAC Gloabal Control Register) = 0x%08x", maccon);

// 
 Print("\n0  Halt request (HaltReq)...........%s", 
  ((maccon & MACCON_HaltReq) != 0) ?
   "1(Stop Tx & Rx)" : "0"
 );

/* Set this bit to immediately stop all transmission and reception.*/
 Print("\n1  Halt immediate (HaltImm).........%s", 
  ((maccon & MACCON_HaltImm) !=0) ?
   "1(Stop Tx & Rx immediately)" : "0"
 ); 

//
/* Set this bit to reset all MAC control and status register and */
/* MAC state machines. */
 Print("\n2  Software reset (Reset)...........%s", 
  ((maccon & MACCON_SwReset) != 0) ?  "1" : "0"
 );

//
/* Set this bit to start transmission while reception is in progress. */
 Print("\n3  Full-duplex (FullDup)............%s", 
  ((maccon & MACCON_FullDup) != 0) ? 
   "1(Full Duplex Supported)" : "0(Half Duplex Supported)"
 );

//
/* Set this bit to cause transmission signals to be presented */
/* as input to the receive circuit without leaving the*/
/* controller.*/
 Print("\n4  MAC loopback (MACLoop) ..........%s",
  ((maccon & MACCON_MACLoop) != 0) ? "1" : "0"
 );

/* Use this bit to select the connection mode. */
/* If this bit is set to one, 10 M bits/s interface will select */
/* the 10 M bits/s endec. Otherwise, the MII will be selected. */
 Print("\n6  MII-OFF..........................%s", 
  ((maccon & MACCON_MIIOFF) != 0) ? 
   "1(10M bits/s interface selected)" :
   "0(MII selected)"
 );

/* If this bit is set, the Loop_10 external signal is asserted */
/* to the 10-Mb/s endec. */
 Print("\n7  Loop 10 Mb/s (Loop10)............%s", 
  ((maccon & MACCON_Loop10) != 0) ?
   "1" : "0"
 ); 

/* This bit is automatically set when the missed error counter rolls over. */
 Print("\n10 Missed roll (MissRoll)...........%s", 
  ((maccon & MACCON_MissRoll) != 0) ?
   "1(error counter rolls over)" : "0"
 );

//
/* Clear this bit to enable the MDC clock generation */
/* for power management. If it is set to one, the MDC clock */
/* generation is disabled. */	
 Print("\n12 MDC-OFF .........................%s", 
  ((maccon & MACCON_MDCOFF) != 0) ?
   "1(MII Station Management Clock Off)" :
   "0(MDC Clock generation enabled)"
 );

/* Set this bit to generate an interrupt whenever the missed */
/* error counter rolls over.*/
 Print("\n13 Enable missed roll (EnMissRoll)..%s", 
  ((maccon & MACCON_EnMissRoll) != 0) ?
   "1(Enable MissRoll interrupt)" :
   "0(Disable MissRoll interrupt)"
 );

//
/* This bit value is read as a buffered signal on the link 10 pin. */
 Print("\n15 Link status 10 Mb/s (Link10).....%s", 
  ((maccon & MACCON_Link10) != 0) ?
   "1" : "0"
 );
}


////////////////////////////////////////////////////////////////////////////////
// MAC Tx Control Register /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void MACTXCONConfig(void)
{
 U32 mactxcon;
 mactxcon = MACTXCON;

 Print("\nMACTXCON : MAC Tx Control Register = 0x%08x", mactxcon);

// [0] transmit Enable (TxEn) //////////////////////////////////////////////////
 Print("\n0  MAC Tx (TxEn).......................%s", 
  ((mactxcon & MACTXCON_TxEn) != 0) ?
   "1(Enabled)" : "0(Disabled)"
 );

// [1] Transmit Halt Request (TxHalt) //////////////////////////////////////////
 Print("\n1  Transmit Halt Request (TxHalt)......%s", 
  ((mactxcon & MACTXCON_TxHalt) != 0) ?
   "1(Transmit halted))" : "0"
 );	

// [2] Suppress Padding (NoPad) ////////////////////////////////////////////////
 Print("\n2  Suppress Padding (NoPad)............%s", 
  ((mactxcon & MACTXCON_NoPad) != 0) ?
   "1(no generate pad bytes)" : "0"
 );

// [3] Suppress CRC (NoCRC) ////////////////////////////////////////////////////
 Print("\n3  Suppress CRC (NoCRC)................%s", 
  ((mactxcon & MACTXCON_NoCRC) != 0) ?
   "1(No generate CRC)" : "0"
 );

// [4] Fast Back-off (FBack) /////////////////////////////////////////////////// 
 Print("\n4  Fast Back-off (FBack)...............%s", 
  ((mactxcon & MACTXCON_FBack) != 0) ?
   "1" : "0"
 );

// [5] Disable the defer counter (NoDef) ///////////////////////////////////////
 Print("\n5  Disable the defer counter (NoDef)...%s", 
  ((mactxcon & MACTXCON_NoDef) != 0) ?
   "1" : "0"
 );

//
 Print("\n6  Send Pause (SdPause)................%s", 
  ((mactxcon & MACTXCON_SdPause) != 0) ?
   "1" : "0"
 );

/* Set this bit to enable MII 10-Mb/s SQE test mode.*/
 Print("\n7  MII 10-Mb/s SQE test mode en.(SQEn).%s", 
  ((mactxcon & MACTXCON_MII10SQEn) != 0) ?
   "1" : "0"
 );

// Enable Underrun (EnUnder) ///////////////////////////////////////////////////
 Print("\n8  Enable Underrun (EnUnder)...........%s", 
  ((mactxcon & MACTXCON_EnUnder) != 0) ? 
   "1(enabled)" : "0(disabled)"
 );

////////////////////////////////////////////////////////////////////////////////
// Enable Deferral (EnDefer) /////////////////////////////////////////////////// 
////////////////////////////////////////////////////////////////////////////////

 Print("\n9  Enable Deferral (EnDefer)...........%s", 
  ((mactxcon & MACTXCON_EnDefer) != 0) ?
   "1(enabled)" :  "0(disabled)"
 );

// Enable No Carrier (EnNCarr) /////////////////////////////////////////////////  
 Print("\n10 Enable NoCarrier (EnNCarr)..........%s", 
  ((mactxcon & MACTXCON_EnNCarr) != 0)  ?
   "1(enabled)" : "0(disabled)"
 ); 

// EnExColl - interrupt if 16 collision occur in the same packet ///////////////
 Print("\n11 En. Excessive Collision (EnExColl)..%s", 
  ((mactxcon & MACTXCON_EnExColl) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 );

// EnLateColl interrupt if collision occurs after 512 bit times(64 bytes times) 
 Print("\n12 Enable Late Collision (EnLateColl)..%s", 
  ((mactxcon & MACTXCON_EnLateColl) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// EnTxPar -  interrupt if the MAC transmit FIFO has a parity error ////////////
 Print("\n13 FIFO Parity Error (EnTxPar).........%s", 
  ((mactxcon & MACTXCON_EnTxPar) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

////////////////////////////////////////////////////////////////////////////////
// EnComp - interrupt when the MAC transmits or discards one packet ////////////
////////////////////////////////////////////////////////////////////////////////
 Print("\n14 Packet Tx Complete (EnComp).........%s", 
  ((mactxcon & MACTXCON_EnComp) != 0) ?
   "1(enabled)" :  "0(disabled)"
 );
}

////////////////////////////////////////////////////////////////////////////////
// MAC Rx Control Register /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void MACRXCONConfig(void)
{
 U32 macrxcon;
 macrxcon = MACRXCON;
 Print("\nMACRXCON : MAC Receive Control Register = 0x%08x", macrxcon);

// RxEn // Receive enable //////////////////////////////////////////////////////
 Print("\n0  MAC Rx Enabled (RxEn)..............%s", 
  ((macrxcon & MACRXCON_RxEn) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// RxHalt // Receive halt request //////////////////////////////////////////////
 Print("\n1  Receive Halt Requested (RxHalt)....%s", 
  ((macrxcon & MACRXCON_RxHalt) != 0) ?
   "1(Halt Rx after current packet)" :
   "0"
 );

// LongEn // Long enable ///////////////////////////////////////////////////////
 Print("\n2  MAC Rx Long Frame(1518B)(LongEn)...%s", 
  ((macrxcon & MACRXCON_LongEn) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// ShortEn // Short enable /////////////////////////////////////////////////////
 Print("\n3  MAC Rx Short Frame(64B)(ShortEn)...%s", 
  ((macrxcon & MACRXCON_ShortEn) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// Strip CRC value /////////////////////////////////////////////////////////////
 Print("\n4  Strip CRC (StripCRC)...............%s", 
  ((macrxcon & MACRXCON_StripCRC) != 0) ?
   "1(enabled)" : "0(disabled"
 );

// PassCtl // Pass control packet //////////////////////////////////////////////
 Print("\n5  Rx Pass Control Packet (PassCtl)...%s", 
  ((macrxcon & MACRXCON_PassCtl) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// IgnoreCRC // Ignore CRC value ///////////////////////////////////////////////
 Print("\n6  MAC Rx Ignore CRC (IgnoreCRC)......%s", 
  ((macrxcon & MACRXCON_IgnoreCRC) != 0) ?
   "1(disable CRC checking)" :
   "0(enable CRC checking)"
 );

 Print("\nMAC Rx Interrupt :");


// EnAlign // Enable alignment interrupt ///////////////////////////////////////
 Print("\n8  Mis-Allign Interrupt (EnAlign).....%s", 
  ((macrxcon & MACRXCON_EnAlign) != 0) ? 
   "1(enabled)" : "0(disabled)"
 );

// EnCRCErr // Enable CRC error interrupt //////////////////////////////////////
 Print("\n9  CRC Error Interrupt (EnCRCErr).....%s", 
  ((macrxcon & MACRXCON_EnCRCErr) != 0) ?
   "1(enabled)" :  "0(disabled)"
 );

// EnOver // Enable FIFO overflow interrupt ////////////////////////////////////
 Print("\n10 FIFO Overflow Interrupt (EnOver)...%s", 
  ((macrxcon & MACRXCON_EnOver) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// EnLongErr // Enable long error interrupt ////////////////////////////////////
 Print("\n11 Over 1518B Frame Int. (EnLongErr)..%s", 
  ((macrxcon & MACRXCON_EnLongErr) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// EnRxPar // Enable receive parity ////////////////////////////////////////////
 Print("\n13 Parity Error Interrupt (EnRxPar)...%s", 
  ((macrxcon & MACRXCON_EnRxPar) != 0) ?
   "1(enabled)" : "0(disabled)"
 );

// EnGood // Enable Good ///////////////////////////////////////////////////////
 Print("\n14 Rx Good Interrupt(EnGood)..........%s", 
  ((macrxcon & MACRXCON_EnGood) != 0) ?
   "1(enabled)" : "0(disabled)"
 );
}

////////////////////////////////////////////////////////////////////////////////
// Transmit Status Register(MACTXSTAT) /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PrintMACSTAT(char * name, U32 val, U32 mask)
{
 Print("\n%s", name);			// bit name print
 if((val & mask) != 0)
  {
   Print("1");
  } 
 else
  {
   Print("0");
  } 
}

#define PrintMACTXSTAT(name,mask)  PrintMACSTAT(name, mactxstat, mask)
#define PrintMACRXSTAT(name,mask)  PrintMACSTAT(name, macrxstat, mask)

static void MACTXSTATConfig(void)
{
 U32 mactxstat;
 mactxstat = MACTXSTAT;

 Print("\n\nMAC Transmit Status Register = 0x%08x", mactxstat);
//
 Print("\n3:0 Transmit collision count (TxColl)...%d", 
   (mactxstat & MACTXSTAT_TxColl)
  );

/* This bit is set if 16 collisions occur while transmitting the */
/* same packet. In this case, packet transmission is aborted.*/
 PrintMACTXSTAT("4   Excessive collision (ExColl)........", MACTXSTAT_ExColl); 
// TxDeffered transmission of a packet was deferred because of a delay /////////
// during transmission
 PrintMACTXSTAT("5   Transmit deferred (TxDeferred)......", MACTXSTAT_TxDeferred); 
 PrintMACTXSTAT("6   Paused (Paused).....................", MACTXSTAT_Paused); 
 PrintMACTXSTAT("7   Interrupt on transmit (IntTx).......", MACTXSTAT_IntTx); 
 PrintMACTXSTAT("8   Underrun (Under)....................", MACTXSTAT_Under); 
 PrintMACTXSTAT("9   Deferral (Defer)....................", MACTXSTAT_Defer); 
 PrintMACTXSTAT("10  No carrier (NCarr)..................", MACTXSTAT_NCarr); 
 PrintMACTXSTAT("11  Signal quality error (SQE)..........", MACTXSTAT_SQE); 
 PrintMACTXSTAT("12  Late collision (LateColl)...........", MACTXSTAT_LateColl); 
 PrintMACTXSTAT("13  Transmit parity error (TxPar).......", MACTXSTAT_TxPar);
 PrintMACTXSTAT("14  Completion (Comp)...................", MACTXSTAT_Comp);
 PrintMACTXSTAT("15  Transmission halted (TxHalted)......", MACTXSTAT_TxHalted); 
}

///////////////////////////////////////////////////////////////////////////////
// Receive Status Register(MACRXSTAT) /////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

static void MACRXSTATConfig(void)
{
 U32 macrxstat;
 macrxstat = MACRXSTAT;

 Print("\n\nMAC Receive Status Register = 0x%08x", macrxstat);
 PrintMACRXSTAT("5  Control frame received (CtlRecd)..", MACRXSTAT_CtlRecd);
 PrintMACRXSTAT("6  Interrupt on receive (IntRx)......", MACRXSTAT_IntRx);
 PrintMACRXSTAT("7  Receive 10-Mb/s status (Rx10Stat).", MACRXSTAT_Rx10Stat);
 PrintMACRXSTAT("8  Alignment error (AlignErr)........", MACRXSTAT_AllignErr);
 PrintMACRXSTAT("9  CRC error (CRCErr)................", MACRXSTAT_CRCErr);
 PrintMACRXSTAT("10 Overflow error (overflow).........", MACRXSTAT_Overflow);
 PrintMACRXSTAT("11 Long error (LongErr)..............", MACRXSTAT_LongErr);
 PrintMACRXSTAT("13 Receive parity error (RxPar)......", MACRXSTAT_RxPar);
 PrintMACRXSTAT("14 Good received (Good)..............", MACRXSTAT_RxGood);
 PrintMACRXSTAT("15 Reception halted (RxHalted).......", MACRXSTAT_RxHalted);
}


////////////////////////////////////////////////////////////////////////////////
// CAMCON // CAM Control Register //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void CAMCONConfig(void)
{
 U32 camcon;
 camcon = CAMCON;

 Print("\n\nCAMCON. CAM Control Register = 0x%08x", camcon);

 Print("\n0 Accept Unicast station address packet(StationAcc)...%s", 
  ((camcon & CAMCON_StationAcc) != 0) ?
   "1(enabled)" : "0(disabled)"
 ); 

// GroupAcc // Accept any packet with multicast-group station address ////////// 
 
 Print("\n1 Accept Multicast-group address packet(GroupAcc).....%s", 
  ((camcon & CAMCON_GroupAcc) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 ); 

// BroadAcc // Accept any packet with a broadcast station address //////////////

 Print("\n2 Accept Broadcast station address(BroadAcc)..........%s", 
  ((camcon & CAMCON_BroadAcc) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 ); 

// NegCAM // reject packets CAM recognizes,accept others ///////////////////////    

 Print("\n3 Accept Negative CAM recognizes(NegCAM)..............%s", 
  ((camcon & CAMCON_NegCAM) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 );

// CompEn // Compare Enable mode ///////////////////////////////////////////////
  
 Print("\n4 CAM Compare Enable mode(CompEn).....................%s", 
  ((camcon & CAMCON_CompEn) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 );
}

////////////////////////////////////////////////////////////////////////////////
// CAMEN CAM Enable Register ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void CAMENConfig(void)
{
 U32 camen;				// CAMEN value
 int cn;
 camen = CAMEN;

// 
 Print("\n\nCAMEN. CAM Enable Register = 0x%08x", camen);
 for(cn = 0; cn < 21; ++cn)
  {
   Print("\n%2d CAM Entry ......", cn);
   if( (camen & (1 << cn)) != 0)
    {
     Print("1(is valid)");
    }
   else
    {
     Print("0(not valid)");
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// EPZCNT // MAC Received Pause Register ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void EPZCNTConfig(void)
{
 U32 epzcnt;
 epzcnt = EPZCNT;

 Print("\n\nEPZCNT. MAC Received Pause Count Register = 0x%08x", epzcnt);
 Print("\n[15:0] Number of time slots the transmitter was paused due to");
 Print("\n       the receipt of control Pause operation packet from the MAC.");
}

////////////////////////////////////////////////////////////////////////////////
// EPRMZCNT // MAC Remote Pause Register ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void EPRMZCNTConfig(void)
{
 U32 ermpzcnt;
 ermpzcnt = ERMPZCNT;

 Print("\n\nERMPZCNT. MAC Remote Pause Count Register = 0x%08x", ermpzcnt);
 Print("\n[15:0] Number of time slots that a remote MAC was paused as");
 Print("\n       a result of sending control Pause operation packets.");
}


////////////////////////////////////////////////////////////////////////////////
// BDMA Tx Control Register ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void BDMATXCONConfig(void)
{
 U32 bdmatxcon; 
 U32 val;
 bdmatxcon = BDMATXCON;
 Print("\n\nBDMATXCON : BDMA Tx Control Register=0x%08x", bdmatxcon);

////////////////////////////////////////////////////////////////////////////////
// BTxBRST (0x0000F) // BDMA Tx Burst Size Mask ////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 val = bdmatxcon & BDMATXCON_TxBRST;
 Print("\n4:0   BDMA Tx Burst Size........................%d Words",(val + 1));

////////////////////////////////////////////////////////////////////////////////
// BTxSTSKO // BDMA Tx Stop (1)/Skip (0) Frame or Interrupt in case ////////////
// of not Owner the current frame //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 Print("\n5     BDMA Tx Stop/Skip (TxSTSKO)...............%s", 
  ((bdmatxcon & BDMATXCON_TxSTSKO) != 0) ?
   "1(BDMA Tx STOP)" :
   "0(BDMA Tx SKIP)"
 );

////////////////////////////////////////////////////////////////////////////////
// TxCPIE // BDMA Tx Complete to send control /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 Print("\n7     Control Packet Tx Complete int.(TxCPIE)...%s", 
  ((bdmatxcon & BDMATXCON_TxCCPIE) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 );

////////////////////////////////////////////////////////////////////////////////
// TxNLIE // BDMA Tx Null list interrupt enable //////////////////////////////// 
////////////////////////////////////////////////////////////////////////////////
 Print("\n8     BDMA Tx Null list interrupt (TxNLIE)......%s", 
  ((bdmatxcon & BDMATXCON_TxNLIE) != 0) ? 
   "1(enabled)" :
   "0(disabled)"
 );

////////////////////////////////////////////////////////////////////////////////
// BTxNOIE // BDMA Tx Buffer Not Owner /////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 Print("\n9     BDMA Tx Not Owner interrupt (TxNOIE)......%s", 
  ((bdmatxcon & BDMATXCON_TxNOIE) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 ); 

////////////////////////////////////////////////////////////////////////////////
// BTxEmpty // BDMA Tx Buffer Empty Interrupt //////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
 Print("\n10    BDMA Tx Buffer Empty interrupt (TxEmpty)..%s", 
  ((bdmatxcon & BDMATXCON_TxEmpty) != 0) ? 
   "1(enabled)" :
   "0(disabled)"
 );

////////////////////////////////////////////////////////////////////////////////
// BTxMSL111 (0x03800) // BDMA Tx Start Level ////////////////////////////////// 
////////////////////////////////////////////////////////////////////////////////
 val = (bdmatxcon & BDMATXCON_TxMSL111) >> 11;
 Print("\n13:11 BDMA Tx to MAC Tx Start Level.............%d(%d/8 BDMA buffer)", 
  val, val);

// BTxEn // BDMA Tx Enable /////////////////////////////////////////////////////
 Print("\n14    BDMA Tx (TxEn)............................%s", 
  ((bdmatxcon & BDMATXCON_TxEn) != 0) ? 
   "1(enabled)" :
   "0(disabled)"
 );

// BDMATXCON_TxRS // BDMA Tx Reset /////////////////////////////////////////////
 Print("\n15    BDMA Tx Reset (TxRS)......................%s", 
  ((bdmatxcon & BDMATXCON_TxRS) != 0) ?
   "1" :
   "0"
 );
}

////////////////////////////////////////////////////////////////////////////////
// BDMA Rx Control Register ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void BDMARXCONConfig(void)
{
 U32 bdmarxcon;				// BDMARXCON value
// 
 bdmarxcon = BDMARXCON;
 Print("\n\nBDMARXCON : BDMA Receive Control Register = 0x%08x", bdmarxcon);

// BRxBRST // BDMA Rx Burst Size ///////////////////////////////////////////////
 Print("\n4:0 BDMA Rx Burst Size ............................%d Words",
   ((bdmarxcon & BDMARXCON_RxBRST)+1)
  );

// BRxSTSKO // BDMA Rx Stop/Skip Frame or Interrupt(=1) in case of not /////////
// OWNER the current Frame /////////////////////////////////////////////////////
 Print("\n5   BDMA Rx Stop/Skip Frame by owner bit(BRxSTSKO).%s", 
  ((bdmarxcon & BDMARXCON_RxSTSKO) != 0) ?
   "1(STOP if not owner)" :
   "0(SKIP if not owner)"
  );

// BRxMAINC // BDMA Rx Memory Address Inc/Dec //////////////////////////////////

 Print("\n6   BDMA Rx Memory Address Inc/Dec (RxMAINC).......%s", 
  ((bdmarxcon & BDMARXCON_RxMAINC) != 0) ?
   "1(increment)" :
   "0(decrement)"
 );

// BRxDIE // BDMA Rx Every Received Frame Interrupt Enable /////////////////////

 Print("\n7   BDMA Rx Every Received Frame Interrupt(RxDIE)..%s", 
  ((bdmarxcon & BDMARXCON_RxDIE) != 0) ?
   "1(enable)" :
   "0(disable)"
 );

// BRxNLIE // BDMA Rx NULL List Interrupt Enable ///////////////////////////////

 Print("\n8   NULL List Interrupt (RxNLIE)...................%s", 
  ((bdmarxcon & BDMARXCON_RxNLIE) != 0) ?
   "1(enable)" :
   "0(disable)"
 );

// BRxNOIE // BDMA Rx Not Owner Interrupt Enable ///////////////////////////////

 Print("\n9   BDMA Rx Not Owner Interrupt (RxNOIE)...........%s", 
  ((bdmarxcon & BDMARXCON_RxNOIE) != 0) ?
   "1(enable)" :
   "0(disable)"
 );

// BRxMSOIE // BDMA Rx Maximum Size over Interrupr Enable ////////////////////// 

 Print("\n10  BDMA Rx Maximum Size over Interrupt (RxMSOIE)..%s", 
  ((bdmarxcon & BDMARXCON_RxMSOIE) != 0) ?
   "1(enable)" :
   "0(disable)"
 );
   
// BRxLittle // BDMA Rx Big/Little Endian //////////////////////////////////////

 Print("\n11  BDMA Rx Big/Little Endian (RxLittle)...........%s", 
  ((bdmarxcon & BDMARXCON_RxLittle) != 0) ?
   "1(Little)" :
   "0(Big)"
 );

// BDMA Rx Word Allignment (RxWA) //////////////////////////////////////////////

 Print("\n13:12 BDMA Rx Word Allignment (RxWA)...............");
 switch(bdmarxcon & BDMARXCON_RxWA)
  {
   case BDMARXCON_RxWA00 :			// 00 	
// no invalid bytes ////////////////////////////////////////////////////////////
    Print("00(no invalid bytes)");
    break;
    
   case BDMARXCON_RxWA01 :
// BDMA Rx Word Alignment one invalid byte /////////////////////////////////////
    Print("01(one invalid byte)");
    break;

   case BDMARXCON_RxWA10 :
// BDMA Rx Word Alignment two invalid bytes ////////////////////////////////////
    Print("10(two invalid bytes)");
    break;

   case BDMARXCON_RxWA11 :
// BDMA Rx Word Alignment three invalid bytes //////////////////////////////////
    Print("10(three invalid bytes)");
    break;

   default 		 :
    break;
  }


// BRxEn // BDMA Rx Enable /////////////////////////////////////////////////////

 Print("\n14  BDMA Rx Enable (RxEn)..........................%s", 
  ((bdmarxcon & BDMARXCON_RxEn) != 0) ?
   "1(enable)" :
   "0(disable)"
 );

// BDMA Rx Reset // RxRS ///////////////////////////////////////////////////////

 Print("\n15  BDMA Rx Reset (RxRS)...........................%s", 
  ((bdmarxcon & BDMARXCON_RxRS) != 0) ?
   "1" :
   "0"
 );

// RxEmpty // BDMA Rx Buffer empty interrupt /////////////////////////////////// 

 Print("\n16  BDMA Rx Buffer Empty Interrupt (RxEmpty).......%s", 
  ((bdmarxcon & BDMARXCON_RxEmpty) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 ); 
  
// BRxEarly // BDMA Rx Early notify Interrupt //////////////////////////////////
 Print("\n17  BDMA Rx Early notify Interrupt (RxEarly).......%s", 
  ((bdmarxcon & BDMARXCON_RxEarly) != 0) ?
   "1(enabled)" :
   "0(disabled)"
 ); 
}

////////////////////////////////////////////////////////////////////////////////
// BDMASTAT  BDMA Status Register //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void PrintBDMASTAT(char * nam,U32 val, U32 mask)
{
 int pos;
//
 Print("\n");
 pos = 0;
 while(*(nam + pos) != '\0')
  {
   put_byte(*(nam + pos++));   
  }

 while(pos < 60)
  {
   put_byte('.');   
   pos++;
  }

//
 if(mask != 0)
  {
   if((val & mask) != 0)
    {
     Print("1");
    }
   else
    {
     Print("0");
    }
  }
}

static void BDMASTATConfig(void)
{
 U32 bdmastat;
 bdmastat = BDMASTAT;
 Print("\n\nBDMASTAT : BDMA Status Register = 0x%08x", bdmastat);

// BDMA Rx Done Every Received Frame ///////////////////////////////////////////

 PrintBDMASTAT("0    BDMA Rx done every received frame (BRxRDF)",
    bdmastat	,	
    BDMASTAT_BRxRDF	
  );

// BDMA Rx NULL List ///////////////////////////////////////////////////////////
 PrintBDMASTAT("1    BDMA Rx null list (BRxNL)",
    bdmastat 	,	
    BDMASTAT_BRxNL	
  );

// BDMA Rx Not Owner ///////////////////////////////////////////////////////////
 PrintBDMASTAT("2    BDMA Rx not owner (BRxNO)",
    bdmastat 	,	
    BDMASTAT_BRxNO	
  );

// BDMA Rx Maximum Size Over ///////////////////////////////////////////////////
 PrintBDMASTAT("3    BDMA Rx maximum size over (BRxMSO)",
    bdmastat 	,	
    BDMASTAT_BRxMSO	
  );  

// BDMA Rx Buffer Empty ////////////////////////////////////////////////////////
 PrintBDMASTAT("4    BDMA Rx buffer empty (BRxEmpty)",
    bdmastat 	,	
    BDMASTAT_BRxEmpty	
  );  

// Early Notify ////////////////////////////////////////////////////////////////
 PrintBDMASTAT("5    Early notification (BRxSEarly)",
    bdmastat 	,	
    BDMASTAT_BRxSEarly	
  );

// One more frame data in BDMA receive buffer //////////////////////////////////
 PrintBDMASTAT("7    One more frame data in BDMA Rx buffer(BRxFRF)",
    bdmastat 	,	
    BDMASTAT_BRxFRF	
  );

// Number of frames in BDMA receive buffer /////////////////////////////////////
 PrintBDMASTAT("15:8 Number of frames in BDMA receive buffer (BRxNFR)",0,0);
 Print("%d", (bdmastat >> 8) & 0xFF);

// BDMA Tx Complete to send Control Packet /////////////////////////////////////
 PrintBDMASTAT("16   BDMA Tx complete to send control packet (BTxCCP)",
   bdmastat 	,	
   BDMASTAT_BTxCCP	
  ); 

// BDMA Tx Null List ///////////////////////////////////////////////////////////
 PrintBDMASTAT("17   BDMA Tx null list (BTxNL)",
   bdmastat 	,
   BDMASTAT_BTxNL
  );

// BDMA Tx Not Owner////////////////////////////////////////////////////////////
 PrintBDMASTAT("18   BDMA Tx not owner (BTxNO)",
   bdmastat 	,
   BDMASTAT_BTxNO
  );
  
// BDMA Tx Buffer Empty ////////////////////////////////////////////////////////
 PrintBDMASTAT("20   BDMA Tx buffer empty (BTxEmpty)",
   bdmastat	,	
   BDMASTAT_BTxEmpty	
  ); 
}

////////////////////////////////////////////////////////////////////////////////
// BDMAPTR   BDMA Rx & Tx Frame Descriptor Start Address Registers /////////////
////////////////////////////////////////////////////////////////////////////////

static void BDMAPTRConfig(void)
{
 U32 bdmatxptr = BDMATXPTR;
 U32 bdmarxptr = BDMARXPTR;

 Print("\n\nBDMATXPTR BDMA Transmit Frame Descriptor Start Address Register");
 Print("\n25:0 BDMA transmit frame descriptor start address = 0x%08x",
   (bdmatxptr & 0x03FFFFFF)
  );

 Print("\n\nBDMARXPTR BDMA Receive Frame Descriptor Start Address Register");
 Print("\n25:0 BDMA receive frame descriptor start address  = 0x%08x",
   (bdmarxptr & 0x03FFFFFF)
  );
}

////////////////////////////////////////////////////////////////////////////////
// BDMARXLSZ BDMA Receive frame maximum size ///////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void BDMARXLSZConfig(void)
{
 U32 bdmarxlsz;				//				
 bdmarxlsz = BDMARXLSZ;			//
 Print("\n\nBDMARXLSZ Receive Frame Maximum Size Register.");

// 
 Print("\n15:0   BDMA receive frame maximum size (BRxLSZ).%d",
    (bdmarxlsz & 0xFFFF)
  );

// 
 Print("\n31:16  BDMA receive frame length (BRxFSZ).......%d",
   ((bdmarxlsz >> 16) & 0xFFFF)
  );
}

////////////////////////////////////////////////////////////////////////////////
// Test Function for BDMA Rx/Tx Buffer and CAM Read/Write //////////////////////
// Used in Other LoopBack Test Function ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void BdmaBufferTest(void)
{
 unsigned int * base;			// base address of tested buffer
 int loop;				// loop number  
 int pos;				// buffer position
 int err;				// error flag

// 
 Print("\nBDMA Tx/Rx and CAM Buffer Test");

////////////////////////////////////////////////////////////////////////////////
// BDMA transmit (Tx) buffer (64) words BDMATXBUF = Base_Addr+0x9200 ///////////
////////////////////////////////////////////////////////////////////////////////

 err = 0;				// clear error flag 

//
 base = (unsigned int *)		// base address
  (
   Base_Addr 	+ 			// 	
   BDMATXBUF_OFFSET			// 
  );

 Print("\nBDMA Tx Buffer Test. Base Address = 0x%08x. Size=64 words...", base);

// 
 for(loop = 0; loop < 1024; ++loop) 
  {
   for(pos = 0; pos < 64; ++pos)
    {					// fill BDMA Transmit fuffer 
     *(base + pos) = pos;			
    }

   for(pos = 0; pos < 64; ++pos)
    {
     if(*(base + pos) != pos)
      {
       err = 1;				// 
// 
       Print("\nBDMA Tx Buffer Error [0x%08x]=0x%08x, must =0x%08x",
          (base + pos)	,		// address
         *(base + pos)	,		// read value	
           pos				// writed value	
        );
      }
    }
  }

 if(err == 0)
  {
   Print("Ok.");
  }
 else
  {
   Print("Fail.");
  } 

// BDMA Rx Buffer Long Range : 9900 ////////////////////////////////////////////
// BDMA Rx Buffer 1bit Range : 9800 ////////////////////////////////////////////

 err = 0;				// clear error flag 


//
 base = (unsigned int *)		// base address
  (
   Base_Addr 	+ 			
   0x9900
  );

 Print("\nBDMA Rx Buffer Test. Base Address = 0x%08x. Size=64 words....",
   base
  );

// 
 for(loop = 0; loop < 1024; ++loop) 
  {
   for(pos = 0; pos < 64; ++pos)
    {					// fill BDMA Transmit fuffer 
     *(base + pos) = pos;			
    }

   for(pos = 0; pos < 64; ++pos)
    {
     if(*(base + pos) != pos)
      {
       err = 1;				// 
//
       Print("\nBDMA Rx Buffer Error [0x%08x]=0x%08x, must =0x%08x",
          (base + pos)	,		// address
         *(base + pos)	,		// read value
           pos				// writed value 
        );
      }
    }
  }


 if(err == 0)
  {
   Print("Ok.");
  }
 else
  {
   Print("Fail.");
  } 


////////////////////////////////////////////////////////////////////////////////
// CAM // (32 words) ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//
 err = 0;				// clear error flag 
 base = (unsigned int *)		// base address
  (
   Base_Addr 	+ 			
   CAM_BASE_OFFSET
  );

 Print("\nCAM Read/Write Test. Base Address = 0x%08x.Size=32 words....",
   base
  );

// 
 for(loop = 0; loop < 1024; ++loop) 
  {
   for(pos = 0; pos < 32; ++pos)
    {					// fill BDMA Transmit fuffer 
     *(base + pos) = pos;			
    }

   for(pos = 0; pos < 32; ++pos)
    {
     if(*(base + pos) != pos)
      {
       err = 1;				// 
       Print("\nBDMA Tx Buffer Error [0x%08x]=0x%08x, must =0x%08x",
          (base + pos)	,		// address
         *(base + pos)	,		// read value 
           pos				// writed value
        );
      }
    }
  }

 if(err == 0)
  {
   Print("Ok.");
  }
 else
  {
   Print("Fail.");
  } 
}

////////////////////////////////////////////////////////////////////////////////
// Change BDMATXCON (BDMA Transmit Control Register) ///////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void BDMATXCONChange(void)
{
 U8 it;
 Print("\n\nBDMATXCON BDMA Tx Control Register Setup");

// BDMA Tx Enable (BTxEn) //////////////////////////////////////////////////////

 gBdmaTxCon |= BDMATXCON_TxEn;

// BDMA Tx Burst Size Mask (BTxBRST) /////////////////////////////////////////// 
 Print("\nBDMA Tx Burst Size (TxBRST)..................%d",
   (gBdmaTxCon & BDMATXCON_TxBRST)
  );

// BDMA Tx Stop (1)/Skip (0) Frame or Interrupt in case of not Owner the ///////
// current frame (BTxSTSKO) ////////////////////////////////////////////////////
 Print("\nBDMA Tx STOP when Descr.Not Owner (BTxSTSKO).%s",
   ((gBdmaTxCon & BDMATXCON_TxSTSKO) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// [13:11] BDMA transmit to MAC Tx start level (BTxMSL) ////////////////////////
 Print("\nBDMA Tx to MAC Tx Start Level (BTxMSL).......%d",
   ((gBdmaTxCon & BDMATXCON_TxMSL111) >> 11)
  );

 switch(gBdmaTxCon & BDMATXCON_TxMSL111)
  {
   case BDMATXCON_TxMSL000 : Print("No wait to fill the BDMA"); break;
   case BDMATXCON_TxMSL001 : Print("wait to fill 1/8 BDMA"); break; 
   case BDMATXCON_TxMSL010 : Print("wait to fill 2/8 BDMA"); break;
   case BDMATXCON_TxMSL011 : Print("wait to fill 3/8 BDMA"); break;
   case BDMATXCON_TxMSL100 : Print("wait to fill 4/8 BDMA"); break;
   case BDMATXCON_TxMSL101 : Print("wait to fill 5/8 BDMA"); break;
   case BDMATXCON_TxMSL110 : Print("wait to fill 6/8 BDMA"); break;
   case BDMATXCON_TxMSL111 : Print("wait to fill 7/8 BDMA"); break;
  }

// BDMA Transmit interrupts ////////////////////////////////////////////////////

// BDMA Tx Complete to send control packet interrupt Enable (BTxCPIE) //////////
 Print("\nBDMA Tx Interrupt Setting");
 Print("\nControl Packet Tx Complete Int (BTxCPIE).....%s",
   ((gBdmaTxCon & BDMATXCON_TxCPIE) != 0) ?
    "1(enabled)" : "0(disabled)"
  );

// BDMA Tx Buffer Not Owner (BTxNOIE) //////////////////////////////////////////
 Print("\nBDMA Tx Descriptor Not Owner Int.(BTxNOIE)...%s",
   ((gBdmaTxCon & BDMATXCON_TxNOIE) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// BDMA Tx Buffer Empty Interrupt (BTxEmpty) ///////////////////////////////////
 Print("\nBDMA Tx Buffer Empty Interrupt (BTxEmpty)....%s",
   ((gBdmaTxCon & BDMATXCON_TxEmpty) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// BDMA Tx Null list interrupt enable //////////////////////////////////////////
 Print("\nBDMA Tx Null List Interrupt enable (BTxNLIE).%s",
   ((gBdmaTxCon & BDMATXCON_TxNLIE) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// 
 Print("\nChange BDMATXCON [Y/N]");
 it = get_upper();
 if(it != 'Y')
  {
   return;
  } 
 
// BDMA Tx Burst Size Mask (BTxBRST) /////////////////////////////////////////// 

 Print("\nInput BDMA Tx Burst Size (TxBRST)............");
 {
  int brst;					// 
  brst = get_number(10,0);			// 
  gBdmaTxCon &=~BDMATXCON_TxBRST;		// 	
  gBdmaTxCon |= (brst & BDMATXCON_TxBRST);	// 
 }

// BDMA Tx Stop (1)/Skip (0) Frame or Interrupt in case of not Owner the ///////
// current frame (BTxSTSKO) ////////////////////////////////////////////////////

 Print("\nBDMA Tx STOP when Descriptor Not Owner  [Y/N]");
 it = get_upper();
 if(it == 'Y')
  { 
   gBdmaTxCon |= BDMATXCON_TxSTSKO;	// set stop enable bit 	
  } 
 else
 if(it == 'N')
  {
   gBdmaTxCon &=~BDMATXCON_TxSTSKO;	// clear stop enable bit
  } 
 
// [13:11] BDMA transmit to MAC Tx start level (BTxMSL) ////////////////////////

 Print("\nInput BDMA Tx to MAC Tx Start Level (BTxMSL) ");
 {
  int msl;				// 
  msl = get_number(10,0);		// 
  msl = msl << 11;			// 
  gBdmaTxCon &=~BDMATXCON_TxMSL111;
  gBdmaTxCon |= (msl & BDMATXCON_TxMSL111);
 }

// BDMA Transmit interrupts ////////////////////////////////////////////////////

// BDMA Tx Complete to send control packet interrupt Enable (BTxCPIE) //////////

 Print("\nBDMA Tx Interrupt Setting");
 Print("\nControl Packet Tx Complete Int (BTxCPIE)[Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaTxCon |= BDMATXCON_TxCPIE;	// set control packet complete bit 
  }
 else
 if(it == 'N')
  {
   gBdmaTxCon &=~BDMATXCON_TxCPIE;	// clear ctrl packet comp bit
  }

// BDMA Tx Buffer Not Owner (BTxNOIE) //////////////////////////////////////////

 Print("\nBDMA Tx Descriptor Not Owner Interrupt  [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaTxCon |= BDMATXCON_TxNOIE;	// set not owner interrupt bit
  }
 else
 if(it == 'N')
  {
   gBdmaTxCon &=~BDMATXCON_TxNOIE;	// clear not owner interrupt bit
  }

// BDMA Tx Buffer Empty Interrupt (BTxEmpty) ///////////////////////////////////

 Print("\nBDMA Tx Buffer Empty Int (BTxEmpty)     [Y/N]");
 it = get_upper();
 if(it == 'Y')
  { 
   gBdmaTxCon |= BDMATXCON_TxEmpty;	// set Tx Empty bit  
  }
 else
 if(it == 'N')
  { 
   gBdmaTxCon &=~BDMATXCON_TxEmpty;	// clear Tx Empty bit
  }

// BDMA Tx Null list interrupt enable //////////////////////////////////////////

 Print("\nBDMA Tx Null List Interrupt (BTxNLIE)   [Y/N]");
 it = get_upper();
 if(it == 'Y')
  { 
   gBdmaTxCon |= BDMATXCON_TxNLIE;	// set Tx null list interrupt bit 
  }
 else
 if(it == 'N')
  { 
   gBdmaTxCon &=~BDMATXCON_TxNLIE;	// clear Tx null list interrupt bit
  }
}


////////////////////////////////////////////////////////////////////////////////
// BDMA Rx Control Register Config /////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


static void BDMARXCONChange(void)
{
 U8 it;
 Print("\nBDMARXCON BDMA Rx Control Register Setup");

// BDMA Rx Enable (BRxEn) //////////////////////////////////////////////////////
 gBdmaRxCon |= BDMARXCON_RxEn;			

// BDMA Rx Burst Size (BRxBRST)	////////////////////////////////////////////////

 Print("\nBDMA Rx Burst Size (BRxBRST).................%d",
   (gBdmaRxCon & BDMARXCON_RxBRST)
  );

// BDMA Rx Stop/Skip Frame or Interrupt(=1) case of not ////////////////////////
// OWNER the current Frame (BRxSTSKO) //////////////////////////////////////////

 Print("\nBDMA Rx STOP when Descr.Not Owner (BRxSTSKO).%s",
   ((gBdmaRxCon & BDMARXCON_RxSTSKO) != 0) ?
     "1(stop)" : "0(skip)"
  );

// BDMA Rx Memory Address Inc/Dec (BRxMAINC) ///////////////////////////////////
 Print("\nBDMA Rx Memory Increment (BRxMAINC) .........%s",
   ((gBdmaRxCon & BDMARXCON_RxMAINC) != 0) ?
     "1(increment)" : "0(decrement)"
  );

// BDMA Rx Big/Little Endian (BRxLittle) ///////////////////////////////////////
 Print("\nBDMA Rx Memory Little-Endian (RxLittle)......%s",
   ((gBdmaRxCon & BDMARXCON_RxLittle) != 0) ?
     "1(little)" : "0(big)"
  );
////////////////////////////////////////////////////////////////////////////////
 Print("\nBDMA Rx Interrupt Setting");

// BDMA Rx Every Received Frame Interrupt Enable (BRxDIE) //////////////////////

 Print("\nEvery Received Frame Interrupt (BRxDIE)......%s",
   ((gBdmaRxCon & BDMARXCON_RxDIE) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// BDMA Rx NULL List Interrupt Enable (BRxNLIE) ////////////////////////////////
 Print("\nNull List Interrupt .........................%s",
   ((gBdmaRxCon & BDMARXCON_RxNLIE) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// BDMA Rx Not Owner Interrupt Enable (BRxNOIE) ////////////////////////////////
    
 Print("\nDescriptor Not Owner Interrupt (BRxNOIE).....%s",
   ((gBdmaRxCon & BDMARXCON_RxNOIE) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// BDMA Rx Maximum Size over Interrupr Enable (BRxMSOIE) ///////////////////////

 Print("\nMaximum Size Over Interrupt (BRxMSOIE).......%s",
   ((gBdmaRxCon & BDMARXCON_RxMSOIE) != 0) ?
     "1(enabled)" : "0(disabled)" 
  );

// BDMA Rx Buffer empty interrupt (RxEmpty) ////////////////////////////////////

 Print("\nBDMA Rx Buffer Empty Interrupt ..............%s",
   ((gBdmaRxCon & BDMARXCON_RxEmpty) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// BDMA Rx Early notify Interrupt (BRxEarly) ///////////////////////////////////
 
 Print("\nEarly Notify Interrupt ......................%s",
   ((gBdmaRxCon & BDMARXCON_RxEarly) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// BRxRS = BDMA Rx Reset

 Print("\nChange BDMARXCON [Y/N]");
 it = get_upper();
 if(it != 'Y')
  {
   return;
  } 

// BDMA Rx Burst Size (BRxBRST)	////////////////////////////////////////////////

 Print("\nInput BDMA Rx Burst Size (BRxBRST)           ");
 {
  int rxbrst;
  rxbrst = get_number(10,0);			// 
  gBdmaRxCon &= ~BDMARXCON_RxBRST;		// 
  gBdmaRxCon |= (rxbrst & BDMARXCON_RxBRST);	// 
 }

// BDMA Rx Stop/Skip Frame or Interrupt(=1) case of not ////////////////////////
// OWNER the current Frame (BRxSTSKO) //////////////////////////////////////////

 Print("\nBDMA Rx STOP when Descriptor Not Owner  [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaRxCon |= BDMARXCON_RxSTSKO;	// set stop bit 
  }
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxSTSKO;	// clear stop bit 
  }
 

// BDMA Rx Memory Address Inc/Dec (BRxMAINC) ///////////////////////////////////
 Print("\nBDMA Rx Memory Increment (BRxMAINC)     [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaRxCon |= BDMARXCON_RxMAINC;	// set increment bit 
  }
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxMAINC;	// clear increment bit 
  }

// BDMA Rx Big/Little Endian (BRxLittle) ///////////////////////////////////////
 Print("\nBDMA Rx Memory Little-Endian (RxLittle) [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaRxCon |= BDMARXCON_RxLittle;	// set little bit 
  }
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxLittle;	// clear little bit 
  }
 
////////////////////////////////////////////////////////////////////////////////
 Print("\nBDMA Tx Interrupt Setting");

// BDMA Rx Every Received Frame Interrupt Enable (BRxDIE) //////////////////////

 Print("\nEvery Received Frame Interrupt (BRxDIE) [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaRxCon |= BDMARXCON_RxDIE;	// set done interrupt bit 
  }
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxDIE;	// clear done interrupt bit 
  }

// BDMA Rx NULL List Interrupt Enable (BRxNLIE) ////////////////////////////////
 Print("\nNull List Interrupt (BRxNLIE)           [Y/N]");
 it = get_upper();
 if(it == 'Y')
  { 
   gBdmaRxCon |= BDMARXCON_RxNLIE;	// set null list bit 
  }
 else
 if(it == 'N')
  { 	
   gBdmaRxCon &=~BDMARXCON_RxNLIE;	// clear null list bit 
  }

// BDMA Rx Not Owner Interrupt Enable (BRxNOIE) ////////////////////////////////
    
 Print("\nDescriptor Not Owner Interrupt (BRxNOIE)[Y/N]");
 it = get_upper();
 if(it == 'Y')
  {	
   gBdmaRxCon |= BDMARXCON_RxNOIE;	// set not owner bit 
  } 
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxNOIE;	// clear not owner bit 
  } 

// BDMA Rx Maximum Size over Interrupr Enable (BRxMSOIE) ///////////////////////

 Print("\nMaximum Size Over Interrupt (BRxMSOIE)  [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaRxCon |= BDMARXCON_RxMSOIE;	// set max size over bit 
  }
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxMSOIE;	// clear max size over bit 
  }

// BDMA Rx Buffer empty interrupt (RxEmpty) ////////////////////////////////////

 Print("\nBDMA Rx Buffer Empty Interrupt(RxEmpty) [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaRxCon |= BDMARXCON_RxEmpty;	// set rx empty bit 
  }
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxEmpty;	// clear rx empty bit  
  }


// BDMA Rx Early notify Interrupt (BRxEarly) ///////////////////////////////////
 
 Print("\nEarly Notify Interrupt (BRxEarly)       [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gBdmaRxCon |= BDMARXCON_RxEarly;	// set Rx early bit
  }
 else
 if(it == 'N')
  {
   gBdmaRxCon &=~BDMARXCON_RxEarly;	// clear Rx early bit 
  }
}

////////////////////////////////////////////////////////////////////////////////
// Configure BDMA //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void BDMACONFChange(void)
{
 BDMATXCONChange();
 BDMARXCONChange();
 BDMATXCON = gBdmaTxCon;		// BDMA Transmit control register 
 BDMARXCON = gBdmaRxCon;		// BDMA Receive control register
}


////////////////////////////////////////////////////////////////////////////////
// BDMA Configuration //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void BdmaConfig(void)
{
 char it;				// selected item
 while(1) 
  {
   Print("\nBDMA Configuration.");
   Print("\n[R] BDMARXCON BDMA Receive Control Register.");
   Print("\n[T] BDMATXCON BDMA Transmit Control Register.");
   Print("\n[S] BDMASTAT  BDMA Status Register.");
   Print("\n[P] BDMAPTR   BDMA Rx & Tx Frame Descriptor Start Address Registers.");
   Print("\n[M] BDMARXLSZ BDMA Receive frame maximum size.");
   Print("\n[B] BDMA Rx/Tx & CAM Buffers Test.");
   Print("\n[C] Change BDMA Configuration.");
   Print("\n[Q] Quit.");

//
   Print("\nSelect Test Item : ") ; 
   it = get_upper();			// read selected item
	
   switch(it)
    {
     case 'R' : 
// BDMATXCON BDMA Receive Control Register /////////////////////////////////////
      BDMARXCONConfig();  		
      break; 

     case 'T' : 
// BDMATXCON BDMA Transmit Control Register ////////////////////////////////////
      BDMATXCONConfig();		
      break; 

     case 'S' : 
// BDMASTAT BDMA Status Register ///////////////////////////////////////////////
      BDMASTATConfig();   		
      break; 

     case 'P' : 
// BDMAPTR   BDMA Rx & Tx Frame Descriptor Start Address Registers /////////////
      BDMAPTRConfig();			
      break;

     case 'M' : 
// BDMARXLSZ BDMA Receive frame maximum size ///////////////////////////////////
      BDMARXLSZConfig();		
      break;

     case 'B' : 
// BDMA Rx & Tx Buffers, CAM Registers Test ////////////////////////////////////
      BdmaBufferTest();
      break;

     case 'C' :
// Change BDMA Configuration ///////////////////////////////////////////////////
      BDMACONFChange();
      break; 

     case 'Q' : 
      return;

     default : 
      Print("\nInvalid Test Item Selected"); 
      break;
    }

   Print("\nPress any key to Continue"); 
   it = get_upper();
   if(it == 'Q')
    {
     break;
    } 
  } 
}

////////////////////////////////////////////////////////////////////////////////
// configure MAC address ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MACADDRConfigure(void)
{
 U8 addr[6];				// MAC address buffer
 U8 data[12];
 int pos;				// position in MAC address buffer
 U8 it;					// 

//
 Print("\n\nChange MAC Address");
 Print("\nRead MAC Address from serial EEPROM [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   MacAddrReadIIC(addr);		// read MAC address from IIC
  }
 else
  {
   for(pos = 0; pos < 6; ++pos)
    {
     addr[pos] = MyMacSrcAddr[pos];	// read MAC Address from buffer
    }
  }

// 
 Print("\nMAC Adress = ");
 for(pos = 0; pos < 6; ++pos) 
  {
   Print(" %02x", addr[pos]);		// print MAC address
  }

 Print("\nChange MAC Address [Y/N]");
 it = get_upper();
 if(it != 'Y')
  {
   return;
  }

// change MAC address //////////////////////////////////////////////////////////  

 pos = 0; 

 Print("\nInput MAC Address : ");
 while(1)
  {
   U8 ch;
   ch = get_byte();
        if( (ch >= '0') && (ch <= '9') ) { ch -= '0' ; }
   else if( (ch >= 'A') && (ch <= 'F') ) { ch = (ch - 'A') + 10; }
   else if( (ch >= 'a') && (ch <= 'f') ) { ch = (ch - 'a') + 10; }
   else
   if( ch == KEY_CR ) 
    {
     while(pos < 12)
      {
       data[pos++] = 0;
      }
     break;
    } 
   else
    {
     continue; 
    } 
//
   Print("%X", ch);
   if((pos & 0x1) != 0)
    {
     Print(" ");
    }

// 
   data[pos++] = ch;
   if(pos >= 12)
    {
     break;
    } 
  }

 for(pos = 0; pos < 6; ++pos)
  {
   addr[pos]  = data[pos * 2] << 4; 
   addr[pos] |= data[pos * 2 + 1];
  }  

// write MAC address to global buffer //////////////////////////////////////////
 for(pos = 0; pos < 6; ++pos)
  {
   MyMacSrcAddr[pos] = addr[pos];		/* MAC Address */
  }

// 
 Print("\nWrite MAC Address ");
 for(pos = 0; pos < 6; ++pos)
  {
   Print("%02x ", addr[pos]);			/* MAC Address */
  }

 Print("to serial EEPROM [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   MacAddrWriteIIC(addr);
  }
}

////////////////////////////////////////////////////////////////////////////////
// change MACCON (MAC Control Register) ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void MACCONChange(void)
{
 U8 it;
  
 Print("\n\nMACCON Global Control Register");

// FullDup - full duplex support ///////////////////////////////////////////////

 Print("\nSupport Full Duplex (FullDup)...................%s",
   ((gMacCon & MACCON_FullDup) != 0) ? 
    "1(enabled)" : "0(disabled)"
  );

 Print("\nUse 10Mbps 7-Wire Interface (MIIOFF)............%s",
   ((gMacCon & MACCON_MIIOFF) != 0) ?
   "1(enabled)" : "0(disabled)"
  );

 Print("\nMissed Error Count Over Interrupt (EnMissRoll)..%s",
   ((gMacCon & MACCON_EnMissRoll) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// 
 Print("\nChange MACCON [Y/N]");
 it = get_upper();
 if(it != 'Y')
  {
   return;
  } 

// FullDup - full duplex support ///////////////////////////////////////////////

 Print("\nSupport Full Duplex (FullDup)              [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {					// Full Duplex support 
   gMacCon |= MACCON_FullDup;		// set Full duplex bit
  }
 else
 if(it == 'N')
  { 
   gMacCon &=~MACCON_FullDup;		// clear full duplex bit
  } 

// 10Mbps 7-Wire Inreface //////////////////////////////////////////////////////

 Print("\nUse 10Mbps 7-Wire Interface (MIIOFF)       [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
// ConnM01 // Force MACCON_10Mbits endec ///////////////////////////////////////
   gMacCon |= MACCON_MIIOFF;		// set MII OFF interface 
  }
 else 
 if(it == 'N')
  {
// ConnM10 // Force MII (rate determined by MII clock) ///////////////////////// 
   gMacCon &=~MACCON_MIIOFF;		// clear MII OF interface 
  }

 Print("\nMissed Error Count Over Interrupt          [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacCon |= MACCON_EnMissRoll;	// set MissRoll int enable bit 	
  }
 else
 if(it == 'N')
  {
   gMacCon &=~MACCON_EnMissRoll;	// clear MissRoll interrupt bit 
  }  
}


////////////////////////////////////////////////////////////////////////////////
// MAC Tx Control Register Config //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


static void MACTXCONChange(void)
{
 U8 it;
 Print("\n\nMACTXCON MAC Tx Control Register Setup");

// interrupt when the MAC transmits or discards one packet /////////////////////
 gMacTxCon |= MACTXCON_EnComp;

// suppress Padding (NoPad) ////////////////////////////////////////////////////
 Print("\nMAC Tx No Padding (NoPad).......................%s",
   (gMacTxCon & MACTXCON_NoPad) ?
   "1(enabled) " : "0(disabled)"
  );

// Suppress CRC (NoCRC) //////////////////////////////////////////////////////
 Print("\nMAC Tx No CRC (NoCRC)...........................%s",
   (gMacTxCon & MACTXCON_NoCRC) ? 
   "1(enabled) " : "0(disabled)"
  );

// Fast Back-off (FBack) ///////////////////////////////////////////////////////
 Print("\nMAC Tx Use Fast Back-off timers (FBack).........%s", 
   (gMacTxCon & MACTXCON_FBack) ?
    "1(enabled) " : "0(disabled)"
  );

// NoDef // Disable the defer counter //////////////////////////////////////////
 Print("\nMAC Tx Disable the Defer Counter (NoDef)........%s", 
   ((gMacTxCon & MACTXCON_NoDef) != 0) ?
   "(1)" : "(0)"
  );

 Print("\nMAC Tx Interrupt Setting");

// Enable Underrun (EnUnder) ///////////////////////////////////////////////////
 Print("\nUnderrun Interrupt (EnUnder)....................%s",
   (gMacTxCon & MACTXCON_EnUnder) ?
    "1(enabled) " : "0(disabled)"
  );

// Enable Deferral (EnDefer) ///////////////////////////////////////////////////
 Print("\nDeferral Interrupt (EnDefer)....................%s",
   (gMacTxCon & MACTXCON_EnDefer) ?
    "1(enabled)" : "0(disabled)"
  );

// Enable No Carrier (EnNCarr) /////////////////////////////////////////////////
 Print("\nNo Carrier Interrupt (EnNCarr)..................%s", 
   (gMacTxCon & MACTXCON_EnNCarr) ?
    "1(enabled) " : "0(disabled)"
  );

// EnExColl // interrupt if 16 collision occur in the same packet //////////////

 Print("\nExcessive Collision Interrupt (EnExColl)........%s", 
   ((gMacTxCon & MACTXCON_EnExColl) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// EnLateColl	// interrupt if collision occurs after 512 bit times ///////////

 Print("\nLate Collision Interrupt (EnLateColl)...........%s",
   (gMacTxCon & MACTXCON_EnLateColl) ?
    "1(enabled) " : "0(disabled)" 
  );

// interrupt if the MAC transmit FIFO  has a parity error (EnTxPar) ////////////
 Print("\nParity Error Interrupt (EnTxPar)................%s",
   (gMacTxCon & MACTXCON_EnTxPar) != 0 ?
    "1(enabled) " : "0(disabled)"    
  );


// TxEn	   = transmit Enable 
// TxHalt  = Transmit Halt Request 
// SdPause = Send Pause
// MII10En = MII 10Mbps mode enable

 Print("\nChange MACTXCON [Y/N]");
 it = get_upper();
 if(it != 'Y')
  {
   return;
  } 

// suppress Padding (NoPad) ////////////////////////////////////////////////////
 Print("\nMAC Tx No Padding (NoPad)                  [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_NoPad;		// set No Padding bit 
  }
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_NoPad;		// clear No Padding bit 
  }

// Suppress CRC (NoCRC) //////////////////////////////////////////////////////
 Print("\nMAC Tx No CRC (NoCRC)                      [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_NoCRC;		// set No CRC bit 
  }
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_NoCRC;		// clear No CRC bit 
  }

// Fast Back-off (FBack) ///////////////////////////////////////////////////////
 Print("\nMAC Tx Use Fast Back-off timers (FBack)    [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_FBack;		// set FBack bit
  } 
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_FBack;		// clear FBack bit 
  } 
 
// NoDef // Disable the defer counter //////////////////////////////////////////
 Print("\nMAC Tx Disable the Defer Counter (NoDef)   [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_NoDef;		// set No Defer counter
  }
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_NoDef;		// clear No Defer counter
  }
 
 Print("\nMAC Tx Interrupt Setting");
 
// Enable Underrun (EnUnder) ///////////////////////////////////////////////////
 Print("\nEnable Underrun Interrupt (EnUnder)        [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_EnUnder; 	// set enable underrun interrupt bit
  }
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_EnUnder; 	// clear enable underrun bit
  }
 
// Enable Deferral (EnDefer) ///////////////////////////////////////////////////
 Print("\nEnable Deferral Interrupt (EnDefer)        [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_EnDefer; 	// 
  }
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_EnDefer; 	// 
  }
 
// Enable No Carrier (EnNCarr) /////////////////////////////////////////////////
 Print("\nEnable No Carrier Interrupt (EnNCarr)      [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_EnNCarr; 	// set no carrier interrupt bit
  }
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_EnNCarr; 	// clear no carrier interrupt bit 
  }
 
// EnLateColl	// interrupt if collision occurs after 512 bit times ///////////
 Print("\nEnable Late Collision Interrupt(EnLateColl)[Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_EnLateColl; 	// set Late Collision Interrupt bit 
  }
 else
  {
   gMacTxCon &=~MACTXCON_EnLateColl; 	// clear Late Collision Int bit
  }
 
// interrupt if the MAC transmit FIFO  has a parity error (EnTxPar) ////////////
 Print("\nEnable Parity Error Interrupt (EnTxPar)    [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_EnTxPar;
  } 
 else
 if(it == 'Y')
  {
   gMacTxCon &=~MACTXCON_EnTxPar;
  }

// EnExColl // interrupt if 16 collision occur in the same packet //////////////
 Print("\nExcessive Collision Interrupt (EnExColl)   [Y/N]"); 
 it = get_upper();
 if(it == 'Y')
  {
   gMacTxCon |= MACTXCON_EnExColl; 
  }
 else
 if(it == 'N')
  {
   gMacTxCon &=~MACTXCON_EnExColl; 
  }
}


////////////////////////////////////////////////////////////////////////////////
// MAC Rx Control Register Config //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


static void MACRXCONChange(void)
{
 U8 it;
 Print("\n\nMACRXCON MAC Rx Control Register Setup");

 gMacRxCon |=      
  MACRXCON_RxEn    | 			// Receive enable (RxEn)
  MACRXCON_EnGood;			// Enable Good interrupt

// Long enable (LongEn) ////////////////////////////////////////////////////////

 Print("\nMAC Rx Long Frame Enable (LongEn)...............%s",
   ((gMacRxCon & MACRXCON_LongEn) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// ShortEn // Short enable /////////////////////////////////////////////////////
 
 Print("\nMAC Rx Short Frame Enable (ShortEn).............%s",
   ((gMacRxCon & MACRXCON_ShortEn) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// Strip CRC value (StripCRC) //////////////////////////////////////////////////

 Print("\nMAC Rx Strip CRC (StripCRC).....................%s",
   ((gMacRxCon & MACRXCON_StripCRC) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// Pass control packet (PassCtl) ///////////////////////////////////////////////

 Print("\nMAC Rx Pass Control Packet (PassCtl)............%s",
   ((gMacRxCon & MACRXCON_PassCtl) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// Ignore CRC value (IgnoreCRC) ////////////////////////////////////////////////
    
 Print("\nMAC Rx Ignore CRC Check (IgnoreCRC).............%s",
   ((gMacRxCon & MACRXCON_IgnoreCRC) != 0) ?
     "1(enabled) " : "0(disabled)"  
  );

// 
 Print("\nMAC Rx Interrupt Setting");

// Enable alignment interrupt (EnAlign) ////////////////////////////////////////

 Print("\nMis-Allign Interrupt (EnAlign)..................%s",
   ((gMacRxCon & MACRXCON_EnAlign) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// Enable CRC error interrupt (EnCRCErr) ///////////////////////////////////////

 Print("\nCRC Error Interrupt (EnCRCErr)..................%s",
   ((gMacRxCon & MACRXCON_EnCRCErr) != 0) ?
     "1(enabled) " : "0(disabled)" 
  );

// Enable overflow interrupt (EnOver) //////////////////////////////////////////
    
 Print("\nFIFO Overflow Interrupt (EnOver)................%s",
   ((gMacRxCon & MACRXCON_EnOver) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// Enable long error interrupt (EnLongErr) /////////////////////////////////////

 Print("\nLong Frame Interrupt (EnLongErr)................%s",
   ((gMacRxCon & MACRXCON_EnLongErr) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// Enable receive parity (EnRxPar) /////////////////////////////////////////////
 Print("\nParity Error Interrupt (EnRxPar)................%s",
   ((gMacRxCon & MACRXCON_EnRxPar) != 0) ?
     "1(enabled) " : "0(disabled)"
  );

// RxHalt = Receive halt request 


 Print("\nChange MACRXCON [Y/N]");
 it = get_upper();
 if(it != 'Y')
  {
   return;
  } 

// Long enable (LongEn) ////////////////////////////////////////////////////////

 Print("\nMAC Rx Long Frame Enable (LongEn)          [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_LongEn;	// set Enable Long Frame bit 
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_LongEn;	// clear Enable Long Frame Bit
  }

// ShortEn // Short enable /////////////////////////////////////////////////////
 
 Print("\nMAC Rx Short Frame Enable (ShortEn)        [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_ShortEn;	// set Enable Short Frame bit 
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_ShortEn;	// clear Enable Short Frame bit
  }

// Strip CRC value (StripCRC) //////////////////////////////////////////////////

 Print("\nMAC Rx Strip CRC (StripCRC)                [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_StripCRC;	// set Strip CRC bit 
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_StripCRC;	// clear Strip CRC bit 
  }
 
// Pass control packet (PassCtl) ///////////////////////////////////////////////

 Print("\nMAC Rx Pass Control Packet (PassCtl)       [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_PassCtl;	// set Pass Control Packet bit
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_PassCtl;	// clear Pass Control Packet bit
  }

// Ignore CRC value (IgnoreCRC) ////////////////////////////////////////////////
    
 Print("\nMAC Rx Ignore CRC Check (IgnoreCRC)        [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_IgnoreCRC;	// set Ignore CRC bit 
  } 
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_IgnoreCRC;	// 
  } 

// 
 Print("\nMAC Rx Interrupt Setting");

// Enable alignment interrupt (EnAlign) ////////////////////////////////////////

 Print("\nEnable Mis-Allign Interrupt (EnAlign)      [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_EnAlign;
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_EnAlign;
  }

// Enable CRC error interrupt (EnCRCErr) ///////////////////////////////////////

 Print("\nEnable CRC Error Interrupt (EnCRCErr)      [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_EnCRCErr;
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_EnCRCErr;
  }

// Enable overflow interrupt (EnOver) //////////////////////////////////////////

 Print("\nEnable FIFO Overflow Interrupt (EnOver)    [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_EnOver;
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_EnOver;
  }
 
// Enable long error interrupt (EnLongErr) /////////////////////////////////////

 Print("\nEnable Long Frame Interrupt (EnLongErr)    [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {  
   gMacRxCon |= MACRXCON_EnLongErr;
  }
 else
 if(it == 'Y')
  {  
   gMacRxCon &=~MACRXCON_EnLongErr;
  }
 
// Enable receive parity (EnRxPar) /////////////////////////////////////////////

 Print("\nEnable Parity Error Interrupt (EnRxPar)    [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gMacRxCon |= MACRXCON_EnRxPar;
  }
 else
 if(it == 'N')
  {
   gMacRxCon &=~MACRXCON_EnRxPar;
  }
}


////////////////////////////////////////////////////////////////////////////////
// Configure CAM Control Register //////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void CAMCONChange(void)
{
 U8 it;
// Compare Enable mode (CompEn) ////////////////////////////////////////////////
 gCamCon |= CAMCON_CompEn;

// Accept any packet with a unicast station address (StationAcc) ///////////////
 Print("\nAccept Unicast Station Address (StationAcc).....%s",
   ((gCamCon & CAMCON_StationAcc) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// Accept any packet with multicast-group station address (GroupAcc) /////////// 
 Print("\nAccept Multicast-Group Station Address(GroupAcc)%s",
   ((gCamCon & CAMCON_GroupAcc) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// Accept any packet with a broadcast station address (BroadAcc) ///////////////
 Print("\nAccept Broadcast Station Address (BroadAcc).....%s",
   ((gCamCon & CAMCON_BroadAcc) != 0) ?
     "1(enabled)" : "0(disabled)"
  );

// reject packets CAM recognizes, accept others (NegCAM) ///////////////////////

 Print("\nAccept Negative CAM Reconization (NegCAM).......%s",
  ((gCamCon & CAMCON_NegCAM) != 0) ?
    "1(enabled)"  : "0(disabled)"
  );

//
 Print("\n\nConfigure CAM (Y/N) ?");
 it = get_upper();
 if(it != 'Y')
  {
   return;
  }

// Compare Enable mode (CompEn) ////////////////////////////////////////////////

// Accept any packet with a unicast station address (StationAcc) ///////////////
 Print("\nAccept Unicast Station Address (StationAcc)[Y/N]");
 it = get_upper();
 if(it == 'Y')
  { 
   gCamCon |= CAMCON_StationAcc;
  }
 else
 if(it == 'N')
  { 
   gCamCon &=~CAMCON_StationAcc;
  }
 
// Accept any packet with multicast-group station address (GroupAcc) /////////// 
 Print("\nAccept Multicast-Group Station Address     [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gCamCon |= CAMCON_GroupAcc;
  }
 else
 if(it == 'N')
  {
   gCamCon &=~CAMCON_GroupAcc;
  }

// Accept any packet with a broadcast station address (BroadAcc) ///////////////

 Print("\nAccept Broadcast Station Address           [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gCamCon |= CAMCON_BroadAcc;
  }
 else
 if(it == 'N')
  {
   gCamCon &=~CAMCON_BroadAcc;
  }

// reject packets CAM recognizes, accept others (NegCAM) ///////////////////////

 Print("\nAccept Negative CAM Reconization (NegCAM)  [Y/N]");
 it = get_upper();
 if(it == 'Y')
  {
   gCamCon |= CAMCON_NegCAM;
  }
 else
 if(it == 'N')
  {
   gCamCon &=~CAMCON_NegCAM;
  }
}

////////////////////////////////////////////////////////////////////////////////
// change MAC configuration ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void MACCONFChange(void)
{
 MACCONChange();			// Change MACCON	
 MACRXCONChange();			// Change MACRXCON 
 MACTXCONChange();			// Change MACTXCON
 CAMCONChange();	

// 
 MACCON    = gMacCon;
 MACTXCON  = gMacTxCon;			// MAC  Transmit control register
 MACRXCON  = gMacRxCon;			// MAC  Receive control register 
 CAMCON    = gCamCon;			// CAM  control register 
}

////////////////////////////////////////////////////////////////////////////////
// main function to view MAC & BDMA configuration //////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void MacConfig(void)
{
 char it;				// selected item
 while(1) 
  {
   Print("\nMAC Configuration.");
   Print("\n[C] MACCON    MAC Global Control Register.");
   Print("\n[T] MACTXCON  MAC Transmit Control Register.");
   Print("\n[R] MACRXCON  MAC Receive Control Register.");
   Print("\n[S] MACTXSTAT MAC Transmit Status Register.");
   Print("\n[E] MACRXSTAT MAC Receive Status Register.");
   Print("\n[A] CAMCON    CAM Control Register."); 
   Print("\n[N] CAMEN     CAM Enable Register.");
   Print("\n[P] EPZCNT    MAC Received Pause Register.");
   Print("\n[Z] EPRMZCNT  MAC Remote Pause Register.");
   Print("\n[H]           MAC Configuration Change.");
   Print("\n[Q] Quit.");

//
   Print("\nSelect Test Item : ") ; 
   it = get_upper();			// read selected item
	
   switch(it)
    {
     case 'C' : MACCONConfig();   break; // MACCON    MAC Gloabal Control Register)
     case 'T' : MACTXCONConfig(); break; // MACTXCON  MAC Transmit Control Register
     case 'R' : MACRXCONConfig(); break; // MACRXCON  MAC Receive Control Register
     case 'S' : MACTXSTATConfig();break; // MACTXSTAT MAC Transmit Status Register
     case 'E' : MACRXSTATConfig();break; // MACRXSTAT MAC Receive Status Register
     case 'A' : CAMCONConfig();   break; // CAMCON    CAM Control Register
     case 'N' : CAMENConfig();    break; // CAMEN     CAM Enable Register
     case 'P' : EPZCNTConfig();   break; // EPZCNT    MAC Received Pause Register
     case 'Z' : EPRMZCNTConfig(); break; // EPRMZCNT  MAC Remote Pause Register
     case 'H' : MACCONFChange();  break; // change MAC configuration 

     case 'Q' : 
      return;

     default : 
      Print("\nInvalid Test Item Selected"); 
      break;
    }

   Print("\nPress any key to Continue"); 
   it = get_upper();
   if(it == 'Q')
    {
     break;
    } 
  } 
}

////////////////////////////////////////////////////////////////////////////////
// Print MAC Received Data /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int PrintMACFrame(U8 * data, int len, int crcpres)
{	
 int size;
 int pos;

 if(data == 0)
  {
   return(0);
  } 

// Print Destination Address ///////////////////////////////////////////////////
 Print("\nDest Address:");
 for(pos = 0; pos < 6; ++pos) 
  {
   Print("%02x ", *data++);
  }

// print source address ////////////////////////////////////////////////////////
 Print("Source Address :");
 for(pos = 0; pos < 6; ++pos) 
  {
   Print("%02x ", *data++);
  }

 Print("\nLength (%d,0x%x). Type & Length : ", len, len);
 for(pos = 0; pos < 2; ++pos) 
  {
   Print("%02x ", *data++);
  }

 Print("\n");
 size  = len - 14;
 if(crcpres == 1)
  {
   size -= 4;
  }

 while(size > 0)
  {
   for(pos = 0; pos < 16 ; ++pos) 
    {
     Print("%02x ", *data++);
     size--;
     if(size <= 0)
      {
       break;
      }
    }
   Print("\n"); 
  } 

 if(crcpres == 1)
  {
   Print("\nCRC Value:");
   for(pos = 0; pos < 4; ++pos)
    {
     Print("%02x", *data++);
    }
  }
//
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// generate random MAC frame ///////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//
// frame   = MAC frame
// length  = LLC length (packet size - ethhernet header size)
// fixlen  =  0 - random packet length 
//           >0 - fixed packet size 
// randfil =  1 - random packet filling 
// 

int MACFrameRandom(sMACFrame * frame, int *length, int seed, int rand)
{
 U32 * dst;				// destination address 
 int wlen;				// length words
 int pos;
 if(frame == 0 || length == 0)
  {
// null frame pointer or null length pointer   
   return(0);
  } 

// Set destination, source address //////////////////////////////////////////////

 for(pos = 0; pos < 6; ++pos) 
  {
   frame->Header.DestinationAddr[pos] = MyMacSrcAddr[pos];
   frame->Header.SourceAddr[pos]      = MyMacSrcAddr[pos];
  }

// Set length or type fields ///////////////////////////////////////////////////

 if(rand != 0)
  {
   (*length) = PatternGen(seed)&0x7FF;	// random length 
   if((*length) > 1500) 
    {	
     (*length) = 1500;
    }
   if((*length) <   46) 
    {
     (*length) = 46;
    }
  }

// 
 frame->Header.LengthOrType[1] = (U8)(  ((*length) ) & 0xFF );
 frame->Header.LengthOrType[0] = (U8)( (((*length) ) >> 8) & 0xFF );

 frame->LLCData[0] = 0;
 frame->LLCData[1] = 0;
// 
 wlen = ((*length) + 1) >> 2;		// fill words
 dst = (U32 *) (frame->LLCData + 2);	// destination pointer
 if(rand == 0)
  {
// fixed packet data filling ///////////////////////////////////////////////////
   while(wlen-- > 0)  
    {
     *dst++ = seed++;
    }
  }
 else
  {
// random packet data filling //////////////////////////////////////////////////
   while(wlen-- > 0)  
    {
     *dst++ = PatternGen(seed++);
    } 
  } 
// 
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// MAC Loopback Test main function, in Interrupt mode //////////////////////////
////////////////////////////////////////////////////////////////////////////////

int InterruptLoopBackTest(int aut)
{	
 static sMACFrame txframe;		// tx frame 
 static sMACFrame rxframe;		// rx frame 
 int txlength;				// tx frame length 
 int rxlength;				// rx frame length
 U32 rxstatus;				// rx status
 U32 mismatch;				// mismatch count
 int packets;				// send packet count
 int rxcount;
 int txnum;				// tx frame number
 int rxnum;				// rx frame number
 int rand;				// random packets (length & filling)
 int hash;				// hash size
 int cmp;				// compare flag
 U32 rvl;				// return value 
 U8  it = 0;				// console char 

//
 int tm1en;				// timer 1 enable flag 
 TIME tm;				// TIME struct 
 U32 startms;				// start time, ms
 U32 deltams = 0;
 U32 curms;				// end time, ms 
 U32 txbytes = 0;			// tx bytes 
 U32 txbsec = 0;			// bytes/sec

// 
 rxcount  = 0;				// rx frame count 
 mismatch = 0;				// frame mismatch count  
 rvl = 1;				// start return value 

////////////////////////////////////////////////////////////////////////////////
// Tx Packet count /////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

 if(aut == 1)
  {
// auto mode => used default value /////////////////////////////////////////////
   packets = 10000;			// default frames count	
   Print("\nTx Packet Count       %d", 
     packets
    );
  }
 else
  { 
   Print("\nInput Tx Packet Count (default 10000)  ");
   packets = get_number(10,0);		// read packet count from console
   if(packets <= 0)
    {	
     packets = 10000;			// default packet count 
    }
  }  

// hash size ///////////////////////////////////////////////////////////////////
 hash = 100;				// default hash size 

// random or fixed packet length ///////////////////////////////////////////////
 if(aut == 1)
  {
// auto mode => fixed packet length //////////////////////////////////////////// 
   rand = 0;    
   txlength = 1500; 			// frame length 
  }
 else
  { 
   if(get_last() != KEY_CR) { Print("\n"); }
   Print("\rRandom packet length & filling    [Y/N]");
   rand = get_upper();
   if(rand == 'Y' || rand == KEY_CR)
    {
     rand = 1; 				// random packet length & filling 
    }
   else
    {
// fixed packet length /////////////////////////////////////////////////////////
     rand = 0;				//  
     if(get_last() != KEY_CR) { Print("\n"); }
     Print("\rInput packet data length (default 1500)");
     txlength = get_number(10,0);   
     if(txlength == 0)
      {
       txlength = 1500;
      }
     else
     if(txlength < 46)
      {
       txlength = 46;
      }
     else
     if(txlength > 1500)
      {
       txlength = 1500;
      }
    }
  } 

// 
 Enable_Int(nGLOBAL_INT); 
 Print("\nMAC Loopback Test is Running (send %d packets) ...  ", packets);
 if(rand == 0)
  {
   Print("\nPackets data length=%d", txlength);
  }

// 
 if(hash != 0)
  { 
   Print("\n#=%d packets sended.", hash); 
  }
// 
 Print("\nTo break test press [Q]. [S] to stop Rx\n"); 

 while(kbd_hit())
  {					// clear input 
   get_byte();				// from console 
  }

// read start time, convert to ms //////////////////////////////////////////////
 tm1en = TimerEnabled(1);		// read timer 1 enable flag 	
 TimerReset(1);				// reset timer 1 
 TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));	// init timer 1 
 TimerStart(1);				// start timer 1 
 GetSysTime(1,& tm); 			
 startms = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;

 if(rand)
  {
// random packet length & packet filling /////////////////////////////////////// 
   for(txnum = 0; txnum < packets; ++txnum)
    {
     U32 * dst;				// rx frame pointer 
     U32 * src;				// tx frame pointer
     int wlen;				// frame length words

// generate packet & send packet ///////////////////////////////////////////////
     MACFrameRandom(& txframe, & txlength, txnum, rand);
     if(MACSend((U8 *) & txframe, txlength) == 0)  
      {
       rvl = 0;				// test completed & error
       break;
      }

// 
     txbytes += txlength + 14;		// calc tx bytes 
     if(hash != 0)
      {
       if((txnum != 0) && (txnum % hash) == 0)
        {  
         Print("#");
        } 
      }

     if(kbd_hit()) { it = get_byte(); }	// get byte from console 
     if(it == 'S' || it == 's')
      {
       continue;
      } 
//
     rxstatus = 0;
     rxlength = sizeof(rxframe);
     while(MACRecv((U8 *) & rxframe, &rxstatus , &rxlength) == 0)
      {
// wait Rx frame
       if(kbd_hit())
        {
         it = get_byte();		// get byte from console 
         if(it == 'q' || it == 'Q')
          {
           break; 
          }
        }
      }
//
     if(it == 'q' || it == 'Q')
      {
       break; 
      }

    if(kbd_hit())
     {
      it = get_byte();			// get byte from console 
      if(it == 'q' || it == 'Q')
       {				// 
        break; 				
       }
     }

// 
     rxcount += 1; 
//
     if(rxlength != txlength + sizeof(etheader))
      {
       mismatch++;
       rvl = 0;
       Print("\nErr frame length. Tx length=%d, Rx length=%d",  
        txlength, rxlength );
#ifdef MAC_PRINT_ERR_FRAME
       PrintMACFrame((U8 *) & rxframe, rxlength, 0);
#endif 							// MAC_PRINT_ERR_FRAME
       continue;			// tx next frame 	
      }

// 
     dst = (U32 *) & rxframe;		// pointer to rx frame 
     src = (U32 *) & txframe;		// pointer to tx frame 
     wlen = (rxlength >> 2);		// rx length words

//
     cmp = 1;				// 
     while(wlen-- > 0)
      {
       if(*dst != *src)
        {
         cmp = 0;			// clear equal flag 
         break;
        } 
       dst++;				// increment destination 
       src++;				// inc source 
      } 

     if(cmp == 1 && (rxlength & 0x3) != 0)
      {
// compare remain bytes 
       U32 mask;
       switch( (rxlength & 0x3) )
        {
         case  1 : mask = 0xFF;     break;
         case  2 : mask = 0xFFFF;   break;
         case  3 : mask = 0xFFFFFF; break;
         default : mask = 0; break;
        }
       if( (*dst & mask) != (*src & mask) )
        {
         cmp = 0;
        } 
      }

     if(cmp == 0)
      {
// compare error ///////////////////////////////////////////////////////////////
       mismatch++ ;
       Print("\nMismatch Frame (Frame Num=%d)...", txnum);
#ifdef MAC_PRINT_ERR_FRAME
       PrintMACFrame((U8 *) & rxframe, rxlength, 0);
#endif 							// MAC_PRINT_ERR_FRAME
       rvl = 0;
      }
// end of Tx & Rx loop
    }
// end if (rand)
  }  
 else
  {
// fixed length ////////////////////////////////////////////////////////////////
   txnum = 0;				// sended sequense number
   rxnum = 0;				// waited seq number
   it = 0;

   while(txnum < packets)
    {
// generate packet & send packet ///////////////////////////////////////////////
     while(txnum < packets)
      {
       MACFrameRandom(& txframe, & txlength, txnum, rand);
       if(MACSend((U8 *) & txframe, txlength) == 0)  
        {				// frame not sended 
         break;				// => break
        }
       txnum++; 			// tx frames 
       txbytes += txlength + 14;	// tx bytes 
       if(hash != 0)
        {
         if((txnum != 0) && (txnum % hash) == 0)
          {  
           Print("#");
          } 
        }
       if(MACRxBD_count() > 0)
        {
         break;
        } 
      }

     if(kbd_hit()) { it = get_byte(); }
     if(it == 'Q' || it == 'q')
      {
       packets = txnum;
      }

     if(txnum >= packets)
      {				// wait 1 sec
       GetSysTime(1,& tm); 
       deltams = curms = tm.tm_ms + tm.tm_sec * 1000;
       while(deltams < curms + 1000)
        {
         GetSysTime(1,& tm); 
         deltams = tm.tm_ms + tm.tm_sec * 1000;
        } 
       deltams = 1000;
      }


     while(it != 'S' && it != 's')
      {
// rx loop
       int comp;
       U8 * ptr;
       U32 * src;
       int pos;
       U32 num; 
       int wlen;

       if(kbd_hit())
        {
         it = get_byte();
        }  

       rxstatus = 0;			// rx status 
       rxlength = sizeof(rxframe);	// rx length 
       if(MACRecv((U8 *) & rxframe, &rxstatus , &rxlength) == 0)
        {
         break;
        } 

//
       rxcount++; 			// rx frame count 
//
       if(rxlength != txlength + sizeof(etheader))
        {
// frame length error 
         mismatch++;
         rvl = 0;
         Print("\nFrame length error. Wait Rx num=%d, Tx length=%d, Rx length=%d",  
          rxnum, txlength, rxlength );
#ifdef MAC_PRINT_ERR_FRAME
         PrintMACFrame((U8 *) & rxframe, rxlength, 0);
#endif 							// MAC_PRINT_ERR_FRAME
         continue;
        }

//
       comp = 0; 
       ptr = (U8 *) & rxframe;
       for(pos = 0; pos < 6; ++pos)
        {
         if(*(ptr + pos) != MyMacSrcAddr[pos])
          {
           comp |= 1; 
           break;
          }

         if(*(ptr + 6 + pos) != MyMacSrcAddr[pos])
          {
           comp |= 2; 
           break;
          }
        }

       if(comp & 1)
        {
         Print("\nDest Addr Err:");
         for(pos = 0; pos < 6; ++pos)
          {
           Print("%02x ", *(ptr + pos));
          }

         mismatch++;
         rvl = 0;
#ifdef MAC_PRINT_ERR_FRAME
         PrintMACFrame((U8 *) & rxframe, rxlength, 0);
#endif 							// MAC_PRINT_ERR_FRAME
         continue;
        }
       
   
       if(comp & 2)
        {
         Print("\nSrc Addr Err:");
         for(pos = 0; pos < 6; ++pos)
          {
           Print("%02x ", *(ptr + 6 + pos));
          }

         mismatch++;
         rvl = 0;
#ifdef MAC_PRINT_ERR_FRAME
         PrintMACFrame((U8 *) & rxframe, rxlength, 0);
#endif 							// MAC_PRINT_ERR_FRAME
         continue;
        }

       ptr += 12;

       if(
          *(ptr + 0) !=((txlength >> 8) & 0xFF) ||
          *(ptr + 1) != (txlength & 0xFF)
        )
        {
         rvl = 0;
         mismatch++;
         Print("\nFrame type & length filed error 0x%02x 0x%02x", 
          *(ptr + 0), *(ptr + 1)
         );
#ifdef MAC_PRINT_ERR_FRAME
         PrintMACFrame((U8 *) & rxframe, rxlength, 0);
#endif 							// MAC_PRINT_ERR_FRAME
         continue;
        }

// check sequence number 
       src = (U32 *)(rxframe.LLCData+2); // pointer to frame data 
       num = *src++;			// read frame number 
       if(num != rxnum)
        {
// sequense number error 
         Print("\nSequense number error. Wait %d. Recv %d", rxnum, num);
         rvl = 0;			// clear return code
         mismatch++;			// total errors 
         rxnum = num + 1;		// reset wait rx num
#ifdef MAC_PRINT_ERR_FRAME
         PrintMACFrame((U8 *) & rxframe, rxlength, 0);
#endif 							// MAC_PRINT_ERR_FRAME
         continue; 
        }
       else
        {
         rxnum++;			// next wait rx frame number
        }

// check frame filling 
       wlen = ((txlength-2)>> 2) - 1; 	// words count 
       num += 1;			// 
       comp = 1;			// equal flag 
       while(wlen-- > 0)
        {
         if(*src++ != num++)
          {
// filling error 
           rvl = 0;			// clear return value 
           comp = 0;			// clear equal flag 
           mismatch++;			// error count 
           Print("\nFrame filling error");
          }
        }

       if(comp == 1 && ((txlength-2) & 0x3) != 0)
        {
         U32 mask;
         switch((txlength-2) & 0x3)
          {
           case  1 : mask = 0xFF; break;
           case  2 : mask = 0xFFFF; break;
           case  3 : mask = 0xFFFFFF; break;
           default : mask = 0; break;
          } 

         if((*src & mask) != (num & mask))
          {
           rvl = 0;
           comp = 0;
           mismatch++;
           Print("\nFrame filling error");
          }  
        }

#ifdef MAC_PRINT_ERR_FRAME
       if(comp == 0)  
        {
         PrintMACFrame((U8 *) & rxframe, rxlength, 0);
        } 
#endif 							// MAC_PRINT_ERR_FRAME

       if(kbd_hit())
        {
         it = get_byte();
        }  
// end of rx loop
      }
// end of fixed length loop
    }
// end of fixed length if
  }

// 
 GetSysTime(1,& tm); 
 curms = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;
 if(tm1en == 0)
  {
   TimerStop(1);
  }
//
 curms -= startms + deltams;
 txbsec = 0;
 if(curms != 0)
  {					// 
   if(txbytes > 0xFFFFFFFF / 1000)
    {					// U32 overflow 
     curms /= 1000;			// diff time sec
     if(curms != 0)
      {				
       txbsec = txbytes / curms;
      }
    }
   else
    {
     txbsec = txbytes * 1000 / curms;
    }
  }
 Print("\nTotal Tx Frames %d bytes %d, %d bytes/sec, Rx Frames %d",
  txnum, txbytes, txbsec, rxcount);

// 
 ReadErrReport();			// print statistics 
 ClearErrReport();			// clear statistics
// 
 if(txnum != rxcount) {rvl = 0;}
 if(rvl == 1)
  {
   Print("\nOk.");
  }
 else
  {
   Print("\nFail. Total Mismatch Frame is %d.",mismatch);
  } 
// 
 return(rvl);
}

////////////////////////////////////////////////////////////////////////////////
// MAC LoopBack Test ///////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// MAC LoopBack Test ///////////////////////////////////////////////////////////

void MacInternalLoopBack(int aut)
{
 U32 maccon;
 Print("\nMAC Internal Loopback Test");
 maccon = MACCON;
 MACCON |=  MACCON_MACLoop | MACCON_FullDup;	// set Loopback bit
 InterruptLoopBackTest(aut);			// loop back test 
 MACCON = maccon; 				// Disable Loopback bit
}

////////////////////////////////////////////////////////////////////////////////
// Loopback Test by PHY without collision //////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int PhyNoColLoopBack(int aut)
{
 U32 maccon;
 int rvl;
 Print("\n\nMAC - PHY LoopBackTest(Without Collision)");
 Print("\nSet PHY mode : Full-Duplex, 100Mb, Loop Back");

 MiiStationWrite
  ( 
   PHY_CNTL_REG		, 		// Control register 
   PHYHWADDR		, 		// device address
   (
    PHY_FULLDUPLEX 	|		// Full-Duplex Mode 
    DR_100MB 	   	| 		// 100Mb
    ENABLE_LOOPBACK			// Loop Back mode 
   )
  );

// 

 maccon = MACCON;			// save MACCON register
 MACCON |=  MACCON_FullDup;		// set Full Duplex bit
 rvl = InterruptLoopBackTest(aut);	// loop back test 
 MACCON = maccon;			// restore MACCON register
 Print("\nReset PHY");
 ResetPhyChip();
 return(rvl);
}

// Loopback Test by PHY with collision /////////////////////////////////////////

void PhyColLoopBack(int aut)
{
 U32 maccon;
 Print("\n\nMAC - PHY LoopBackTest(With Collision)");
 Print("\nSet PHY mode :  100Mb, Loop Back, Collision Test");
// Full-Duplex,
 MiiStationWrite
  (
   PHY_CNTL_REG		, 		// PHY Control Register 
   PHYHWADDR		, 		// PHY Adress 
   (
//    PHY_FULLDUPLEX 	|		// Full-Duplex Mode 
    DR_100MB 		| 		// 100Mb 
    PHY_COL_TEST 	| 		// Collision Test
    ENABLE_LOOPBACK			// Loop Back Mode 
   ) 
 );

//
 maccon = MACCON;			// save MACCON register
 MACCON |=  MACCON_FullDup;		// set Full Duplex bit
 InterruptLoopBackTest(aut);		// loopback test 
 MACCON = maccon;			// restore MACCON register
 Print("\nReset PHY");
 ResetPhyChip();
}


////////////////////////////////////////////////////////////////////////////////
// MAC All LoopBack Test ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MacAllLoopBack(void)
{
 Print("\nAll LoopBackTest(Without Physical LoopBack)") ;
 MacInternalLoopBack(1) ;		// Internal Loop Back test  
 PhyNoColLoopBack(1) ;			// PHY Loop Back Test without collision  
 PhyColLoopBack(1) ;			// PHY Loop Back Test with collisison
}


////////////////////////////////////////////////////////////////////////////////
// Loop Back Test //////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void MacLoopBackTest(void)
{
 U8 it;

 while(1) 
  {
   Print("\n\nMAC LoopBack Test");
   Print("\n[I] MAC Internal LoopBackTest");
   Print("\n[L] MAC - PHY LoopBackTest(Without Collision)");
   Print("\n[C] MAC - PHY LoopBackTest(WithCollision)");
   Print("\n[E] Physical Ethernet LoopBackTest");
   Print("\n[A] All LoopBackTest(Without Physical LoopBack)");
   Print("\n[Q] Quit MAC LoopBack Test");
   Print("\nSelect Test Item : ") ; 
   it = get_upper();

// 
   switch(it)
    {
     case 'I' : 
// MAC Internal LoopBackTest ///////////////////////////////////////////////////
      MacInternalLoopBack(0); 			
      break;

    case 'L' :  
// MAC PHY LoopBackTest(Without Collision) /////////////////////////////////////
     PhyNoColLoopBack(0); 			
     break;

    case 'C' : 
// MAC PHY LoopBackTest(WithCollision) /////////////////////////////////////////
     PhyColLoopBack(0); 			
     break;

    case 'E' : 
// Physical Ethernet LoopBackTest //////////////////////////////////////////////
     InterruptLoopBackTest(0) ;		
     break ;

    case 'A' : 
// All LoopBackTest(Without Physical LoopBack) /////////////////////////////////
     MacAllLoopBack(); 			
     break;

    case 'Q' :  
     return;

    default : 
     Print("\nInvalid Test Item Selected"); 
     break;
    }
 
   Print("\nPress any key to Continue"); 
   it = get_byte();
   if(it == 'Q')
    {
     return;
    } 
  }
}

////////////////////////////////////////////////////////////////////////////////
// Receive one MAC Frame if Destination address is mine ////////////////////////
////////////////////////////////////////////////////////////////////////////////


void RxMacFrame(void)
{
 sMACFrame rxframe;			// rx frame 
 U32 rxcount;				// rx frame count 
 U32 rxbytes;				// rx bytes count 
 U32 rxsec;				// rx bytes/sec 
 U32 status;				// frame status 
 int length;				// frame length 
 int prn;				// print frame flag
 int tm1en;				// timer 1 enable flag 
 TIME tm;				// TIME struct 
 U32 startms;				// start time, ms
 U32 curms;				// end time, ms 
 int tmval = 0;				// start time ms not valid 
 U32 camreg0;				// saved CAM entry 0 
 U32 camreg1;				// saved CAM entry 1
 U32 camcon;				// saved CAMCON
 U32 camen; 				// saved CAMEN 
 U8 it;
 Print("\nReceive MAC Frame.");

 ClearErrReport();			// Clear Ethernet Statistics
// 
 Print("\nPrint Received Frames     [Y/N]");
 it = get_upper();
 if(it == 'Y') { prn = 1; }
 else { prn = 0;}  

 if(prn == 0)
  {
// no print frames => calc bytes/sec 
   tm1en = TimerEnabled(1);		// read timer 1 enable flag 	
   TimerReset(1);				// reset timer 1 
   TimerInit(1,(ONE_SECOND/TICKS_PER_SECOND));// init timer 1 
   TimerStart(1);
  }  

 rxcount = 0;				// rx frame count 
 rxbytes = 0;				// rx bytes count 

 Print("\nWaiting For Frame Receive [Q/q]\n");

// 
 camcon  = CAMCON;			// save CAMCON
 camen   = CAMEN;			// save CAMEN
 camreg0 = CAM_Reg(0);			// save CAM entry 0 
 camreg1 = CAM_Reg(1);			// save CAM entry 1
 CAM_Reg(0) = 0;			// clear CAM entry 0
 CAM_Reg(1) = 0;			// clear CAM entry 1
 CAMEN = 0;				// clear 
 CAMCON = CAMCON_CompEn|CAMCON_NegCAM;	// accept all frames 
 MACRxFlush();				// flush Rx BD 
 Enable_Int(nGLOBAL_INT);		// enable GLOBAL int
 
 while(1)
  {
   if(kbd_hit())
    {
     U8 ch;
     ch = get_byte();
     if(ch == 'Q' || ch == 'q' )
      {
       break;
      } 
    }  

// 
   status = 0;
   length = sizeof(rxframe);
   if(MACRecv((U8 *) & rxframe, &status , &length) == 0)
    {
     continue;
    } 

// 
   rxcount++;				// rx frame count 
   rxbytes += length;			// rx bytes count 
   if(prn == 0)
    {
// no print frames => calc Rx bytes / sec
     if(tmval == 0)
      {
// received first frame => init start time 
       tmval = 1;
       GetSysTime(1,& tm); 			
       startms = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;
      }
    }
   else 
    {		
// print MAC frame, CRC stripped 
     PrintMACFrame((U8 *) & rxframe, length, 0);	
    }
  }

// 
 CAM_Reg(0) = camreg0;			// restore CAM 0 entry
 CAM_Reg(1) = camreg1;			// restore CAM 1 entry 
 CAMEN =  camen;			// restore CAMEN
 CAMCON = camcon;			// restore CAMCON

 Print("\nRx frames=%d, bytes=%d", rxcount, rxbytes);
 if(prn == 0)
  {
// calc & print bytes sec 
   GetSysTime(1,& tm); 			
   curms = tm.tm_ms + (tm.tm_sec + tm.tm_min * 60) * 1000;
   if(tm1en == 0)
    {
     TimerStop(1);			// stop timer 1
    }

//  
   curms -= startms;			// time diff
   rxsec = 0;				// rx bytes/sec
   if(curms != 0)
    {					// 
     if(rxbytes > 0xFFFFFFFF / 1000)
      {	
       curms /= 1000;	
       if(curms != 0) { rxsec = rxbytes / curms; }
      }
     else
      {
       rxsec = rxbytes * 1000 / curms;
      }
    }
   if(rxsec != 0)
    {					// print rx bytes/sec 
     Print(", %d bytes/sec", rxsec);
    }
  }

// 
 ReadErrReport();			// Print Ethernet Statistics 
 ClearErrReport();			// Clear Ethernet Statistics
}


////////////////////////////////////////////////////////////////////////////////
// Transfer a Frame Data to another Host ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void SingleTransfer(void) 
{
 int seed = 0;

// 
 ClearErrReport();			// clear statistics
 while(1)
  {
   U8 it;
   Print("\nSelect Transmit(T) or Quit(Q)");
   it = get_upper();
   if(it == 'T')
    {
     sMACFrame frame;
     int length = 1500;
     int randfil = 0;
     MACFrameRandom(& frame, & length, seed++, randfil);
     if(MACSend((U8 *)& frame, length) == 0)  
      {
       Print("\nSend MAC Frame error.\n"); 
       break;
      }
    }
   else
   if(it == 'Q')
    {
     break;
    } 
  } 
 
 ReadErrReport();			// print statistics
 ClearErrReport();			// clear statistics
}


////////////////////////////////////////////////////////////////////////////////
// Transmit Multiple packet ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MultiTransfer(void)
{
 sMACFrame frame;			// MAC frame
 int size = 1500; 			// Max size
 int txnum = 0;				// tx frame number

// 
 ClearErrReport();			// clear statistics

// 
 Print("\n\nTransfer Multiple MAC Frame.");
 Print("\n#=100 packets sended.");
 Print("\nPress any key to break test\n");

 while(kbd_hit() != 0)
  {					// clear input 
   get_byte();				// from console
  }

// 
 while(1) 
  {
   size = 1500;
   MACFrameRandom(& frame, & size, txnum, 0);
   if(MACSend((U8 *)& frame, size) == 0) 
    {
     Print("\nSend MAC Packet Error\n");
    }
   else
    {
//
     txnum += 1;
     if((txnum % 100) == 0)
      {
       Print("#");
      }
    }  
 
   if(kbd_hit())
    {
     get_byte();
     break;
    } 
  }

// 
 Print("\nSended %d Packets.\n", txnum);
 ReadErrReport();			// print statistics
 ClearErrReport();			// clear statistics
}


////////////////////////////////////////////////////////////////////////////////
// Transfer MAC Frame //////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void TxMacFrameTest(void)
{
 while(1) 
  {
   U8 it;

   Print("\n\nTransfer MAC Frame") ;
   Print("\n[S] Transfer Single MAC Frame");
   Print("\n[M] Transfer Multiple MAC Frame");
   Print("\n[C] Transfer Control Frame");
   Print("\n[Q] Quit MAC Transfer Test");
   Print("\nSelect Test Item : "); 
   it = get_upper();
	
   switch(it)
    {
     case 'S' : SingleTransfer(); break;	/* Single Packet Transfer */
     case 'M' : MultiTransfer(); break;		/* Multiple Frames Transmit */
     case 'C' : ControlFrameTransfer(); break;	
     case 'Q' : return;
     default  : 
      Print("\nInvalid Test Item Selected"); 
      break ;
    }

   Print("\nPress any key to Continue"); 
   it = get_byte();
   if(it == 'Q')
    {
     return;
    } 
  }
}

////////////////////////////////////////////////////////////////////////////////
// MAC Tx Halt Immediate Test //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MacTxHaltReqTest(void) 
{
 sMACFrame frame;			// MAC frame 
 int length;				// MAC frame data length 
 int randfil = 0;
 U8 it;

 Print("\n\nMAC Tx Halt Request Test");
 while(1) 
  {
   Print("\nSelect (Q)uit or (T)x 2 Frame :");
   it = get_upper();
   if(it == 'Q')
    {
     break;
    }
   else
   if(it != 'T')
    {
     continue;
    } 

   length = 1500; 
   MACFrameRandom(& frame, & length, 100, randfil);
   if(MACSend((U8 *)& frame, length) == 0)  
    {
     Print("\nSend MAC Frame error.\n");
     break;
    }

   if(MACSend((U8 *)& frame, length) == 0)  
    {
     Print("\nSend MAC Frame error.\n");
     break;
    }

   MACCON |=  MACCON_HaltImm;		// Enable Halt Request
   MACCON &= ~MACCON_HaltImm;
  } 

// 
 ReadErrReport();			// Print Ethernet Statistics  
 ClearErrReport();			// Clear Ethernet Statistics
}


/* --------------------------------------------------------------------*/
/* Frame buffer definition for restore received buffer                 */
/* In this diagonostic code, use unused area for received frame buffer */
/* --------------------------------------------------------------------*/

#define captFrameStart 0xCCCCCCCC	// Rx Frame Data Preamble 

U32 * captBase   = 0;			// Rx Frames buffer 
U32 * captLimit  = 0; 			// Rx Frames buffer limit 
U32 * captPtr    = 0;			// Rx Frame pointer
int   captFrames = 0;			// rx frames count 
	
// init capture region 
	
static int InitCaptRegion(void)
{
 U32 rbase [12];			// free mem region base 
 U32 rlimit[12];			// free mem region limit 
 U32 base;				// capture region base address 
 U32 limit;				// capture region limit address
 U32 tbase;				// temp base address
 int size; 				// capture region size  
 int rcnt; 				// count of regions 
 int rn;				// region number 
 
 if(captBase != 0)
  {					// allready init
   return(1);
  }  

// 
 rcnt = 12;				// size of rbase, rlimit 
 if(FreeDramRegions(rbase,rlimit,&rcnt) == 0)
  {					// get free regions error 
   return(0);				// => return
  }

// 
 base  = 0;				// 
 limit = 0;				// 
 size  = 0;				// 
 for(rn = 0; rn < rcnt; ++rn)
  {
   int sz = rlimit[rn] - rbase[rn];	// sizeof rn free region 
   if(sz > size)
    {
     base  = rbase[rn];			// reset region base address 
     limit = rlimit[rn];		// reset region limit address
     size  = sz;			// reset region size 
    }
  }

 if(base == 0)
  {
   base  = 0x10000;
   size -= 0x10000;
  }

// allign base adress to 64K 
 tbase = ((base + 0xFFFF) >> 16) << 16;	// allign base to 64K 
 size -= (tbase - base);		// decrement of region size 	
 base  =  tbase;
 
 if(size <= 0)
  {
// int error
   captBase  = 0;			// clear rx frames buffer base pointer 
   captLimit = 0; 			// clear rx frames buffer limit pointer
   captPtr   = 0;			// clear write pointer
   captFrames = 0;			// clear frames count 	
   return(0);
  }

// 
 captFrames = 0;			// stored frames 
 captBase  = (U32 *) base;		// capture region base address 
 captLimit = (U32 *) limit; 		// capture region limit
 captPtr   = captBase;			// captute region pointer
 return(1);
} 

////////////////////////////////////////////////////////////////////////////////
// write received Frame to capture buffer //////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// data   = data buffer pointer
// status = frame status 
// length = frame length 
// num    = frame sequence number

int PutCaptFrame(U8 * data, U32 status, int length, int num)
{
 U32 time;				// time min, sec
 int wlen;				// frame length, words 
 U32 * src;				// sourse poiter  

 if(captBase == 0)
  {					// capt reg not init
   if(InitCaptRegion() == 0)
    {
     return(0);
    }
  }

// 
 wlen = (length + 3) >> 2; 		// frame length,words
 if(captPtr + wlen + 3 > captLimit)
  {					// out of capture region  
   return(0);
  } 

/* Received Frame Data Store Format */
/*  31     16 15      0  */
/* +----------+----------+ */
/* |     Preamble        | */
/* +----------+----------+ */
/* | Sequence | Time     | */
/* +----------+----------+ */		
/* | Status   | Length   | */
/* +----------+----------+ */
/* |       Rx Data       | */
/* +---------------------+ */

 time = tm0.tm_sec | (tm0.tm_min << 8) ;
 *captPtr++ =  captFrameStart; 		// start of frame
 *captPtr++ = (num << 16) | time;	// sequence number & time 
 *captPtr++ = (status << 16) | length; 	// status & length 

// Move Rx Frame to User Area
 src = (U32 *) data;
 while(wlen-- > 0)
  {
   *captPtr++ = *src++; 
  }
//
 captFrames++;				// total saved frames 
 return(1);
}

////////////////////////////////////////////////////////////////////////////////
// ShowRecevedFrameData ////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void ShowRecevedFrameData(void)
{
 if(captBase == 0) 
  {
   Print("\nBuffer empty");
   return;
  }

 while(1) 
  {
   U32 * ptr;				// user frame buffer pointer
   U8 addr[12];				// source & destination address
   U32 num;				// frame number & frame length 
   U8 it;				// selected from menu item

//
   Print("\n\nShow Received Frame Data(%d Frames)", captFrames);
   Print("\n[A] Show All Received Frame.");	//
   Print("\n[N] Show by Rx Sequence.");		//
   Print("\n[L] Show by Rx Frame Length.");	//
   Print("\n[S] Show by Source Address.");	//
   Print("\n[D] Show by Destination Address.");	//
   Print("\n[C] Show Control Frame.");		//
   Print("\n[Q] Quit");				//
   Print("\nSelect Item");
   it = get_upper();

   if(it == 'Q') 
    {
// Quit from show captured frames //////////////////////////////////////////////
     break;
    }

   if(it == 'N')
    {
////////////////////////////////////////////////////////////////////////////////
// [N] Show by Rx Sequence /////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
     Print("\nEnter Sequence Number :");
     num = get_number(10,0);		// sequence number 
    }
   else
   if(it == 'L')
    {
////////////////////////////////////////////////////////////////////////////////
// [L] Show by Rx Frame Length /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
     Print("\nEnter Rx Frame Length :");
     num = get_number(0,0);		// Rx frame length 
    }
   else
   if(
      it == 'S' ||			// source address
      it == 'D'				// destination address
    )
    {
     int pos;
////////////////////////////////////////////////////////////////////////////////
// Show by Source or Destinaton Address ////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
     if(it == 'S')
      {
// select by source address ////////////////////////////////////////////////////
       Print("\nEnter Source Address      :");
      }
     else
      {
// slect by destination address ////////////////////////////////////////////////
       Print("\nEnter Destination Address :");
      } 

// 
     pos = 0;				// position in source & destination addr	
     while(1) 
      {
       U8 ch;				// 
       ch = get_byte();			// read byte from console
       if(ch == KEY_CR)
        {
// CR => terminate address input ///////////////////////////////////////////////
         while(pos < 12)
          {
           addr[pos++] = 0;
          }
         break;
        }

            if(ch >= '0' && ch <= '9') { ch -= '0';	}	// 0 - 9
       else if(ch >= 'A' && ch <= 'F') { ch = (ch - 'A') + 10;}	// A - F
       else if(ch >= 'a' && ch <= 'f') { ch = (ch - 'a') + 10;}	// a - f
       else
        {
         continue;			// position not changed 
        }

// 
       if(ch <= 9) { put_byte(ch + '0'); }
       else { put_byte('A' + (ch - 10)); }
// 
       addr[pos] = ch;
       pos++;
       if((pos & 1) == 0)
        {
         put_byte(' ');
        }
       if(pos >= 12)
        {
         break;
        }
      }
    }


// start pointer
   ptr = captBase;

// 
   while(ptr < captPtr) 
    {
     U32 sequence;			// rx packet number 
     U32 len;				// rx length 
     U32 stat;				// tx status
     U32 time;				// rx time 
     U32 min;				// rx min
     U32 sec;				// rx sec
     U8  src[6];			// source address
     U8  dst[6];			// destination address
     U8  type[2];			// type or length 
     U8 * data;				// MAC frame data pointer 
     int pos;

     if(*ptr++ != captFrameStart) 
      {
// => clear capture buffer 
       Print("\nIncorrect capture buffer structure.");
       captFrames = 0; 			// frames count 
       captPtr = captBase;		// current write pointer
       break;
      } 


// Received Frame Data Store Format
// +------------------+
// | Preamble         |
// | Sequence| Time   |
// | Status  | Length |
// | Rx Data          |
// +------------------+

     sequence = (*ptr >> 16) & 0xFFFF;	// rx packet number 
     time = *ptr & 0xFFFF ; 		// rx time 
     ptr++;				// 

     stat = (*ptr >> 16) & 0xFFFF;	// rx status 	
     len  = *ptr & 0xFFFF; 		// rx length 
     ptr++;

     min = (time >> 8) & 0xFF;
     sec =  time & 0xFF ;

// 
     data = (U8 *) ptr; 		// MAC frame pointer 
     for(pos = 0; pos < 6; ++pos) 
      {
       dst[pos] = *data++;		// read destination address
      }

     for(pos = 0; pos < 6; ++pos) 
      {
       src[pos] = *data++;		// read source address
      }

     for(pos = 0; pos < 2; ++pos) 
      {
       type[pos] = *data++;		// read type & length 
      }

     if(it == 'A')
      {
// [A] Show All Received Frame /////////////////////////////////////////////////
       Print("\n%d'th Received Frame Data, status = %x", 
         sequence, 			// rx packet number 
         stat				// rx status
        );

       Print("\nReceived Time: %d min %d sec", min, sec );
       PrintMACFrame((U8 *) ptr, len, 0);	// print rx frame
      }
     else 
     if(it == 'N')
      {
// [N] Show by Rx Sequence /////////////////////////////////////////////////////
       if(sequence == num)
	{
	 Print("\n%d'th Received Frame Data(%x)", 
	   sequence	, 		// rx frame number 
	   stat				// rx status
	  );

	 Print("\nReceived Time: %d min %d sec", min, sec );
	 PrintMACFrame((U8 *) ptr, len, 0);
	 break ;
	}
      }
     else 
     if(it == 'L')
      {
// [L] Show by Rx Frame Length /////////////////////////////////////////////////

       if(len <= num)
        {
	 Print("\n+++ %d'th Frame is Less than %d Byte, status %x",
			sequence, num, stat);
	 Print("\nReceived Time: %d min %d sec ", min, sec);
	 PrintMACFrame((U8 *) ptr, len, 0) ;
        }
      }
     else 
     if(it == 'S')
      {
// [S] Show by Source Address //////////////////////////////////////////////////
       int an; 
       int sn;
       for(sn = 0, an = 0; sn < 6; ++sn, an += 2)
        {
	 if( 
	    ((src[sn] >> 4) & 0xF) != addr[an] || 
             (src[sn] & 0xF) != addr[an + 1]
          )
	  {
	   break; 
	  }
	}

       if(sn >= 6)
	{
	 Print("\n%d'th Frame is Source Address Mached(%x)",
	   sequence, stat
	  );
	 Print("\nReceived Time: %d min %d sec ", min, sec );
	 PrintMACFrame((U8 *) ptr, len, 0);
	}	
      }
     else 
     if(it == 'D')
      {
// [D] Show by Destination Address /////////////////////////////////////////////
       int dn;
       int an; 
       for(dn = 0, an = 0; dn < 6; ++dn, an += 2)
        {
	 if( 
	    ((dst[dn] >> 4) & 0xF) != addr[an] ||
             (dst[dn] & 0xF) != addr[an + 1]
          )
	  {
	   break; 
	  }
	}

       if(dn >= 6)
	{
	 Print("\n%d'th Frame is Destination Address Mached(%x)",
  	   sequence, 
  	   stat
  	  );
         Print("\nReceived Time: %d min %d sec ", min, sec);
         PrintMACFrame((U8 *) ptr, len, 0);
	}	
      }
     else 
     if(it == 'C')
      {
       if(
          (type[0] == 0x88) && 
  	  (type[1] == 0x08) 
  	)
	{
// [C] Show Control Frame //////////////////////////////////////////////////////
         Print("\n%d'th Frame is Control Frame(%x)", 
	  sequence, stat
	 );

     	 Print("\nReceived Time: %d min %d sec ", min, sec ) ;
	 PrintMACFrame((U8 *) ptr, len, 0);
	}
      }

    ptr += (len + 3) >> 2; 
    if(kbd_hit())
     {
      U8 ch;
      ch = get_byte();
      if(ch == 'Q' || ch == 'q')
       { break; }
     }
   }
	
   Print("\n Press Any Key to Continue") ;
   it = get_byte();
   if(it == 'Q' || it == 'q')
    {
     break;
    }
  } 
}


////////////////////////////////////////////////////////////////////////////////
// Capture Rx Frame Data ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void CaptureRxFrame(void)
{
 sMACFrame rxframe;			// rx frame 
 U32 rxstatus;				// rx frame status 
 int rxlength;				// rx frame length 

 ClearErrReport();			// Clear Ethernet Statistics
 if(InitCaptRegion() == 0)
  { 					
   return;
  } 

// clear capture region 
 captFrames = 0;			// frames count 
 captPtr = captBase;			// write pointer 
 while(kbd_hit())
  {					// clear input
   get_byte();				// from console 
  }

// 
 MACRxFlush(); 				// flush Rx BD 
 Print("\nPress any key to break test");
 while(1)
  {
   if(kbd_hit())
    {
     U8 ch;
     ch = get_byte();
     break;
    } 

   rxstatus = 0;
   rxlength = sizeof(rxframe);
   if(MACRecv((U8 *) & rxframe, &rxstatus , &rxlength))
    {
// receive frame => write to capture buffer 
     if(PutCaptFrame((U8 *) & rxframe, rxstatus, rxlength, captFrames) == 0)
      {
       Print("\nCapture frame buffer is full.");
       break;
      }
    }
  }

 Print("\nReceived Frame Count is %d\n", captFrames);
}

////////////////////////////////////////////////////////////////////////////////
// Capture All Rx Frame ////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CaptureAllRxFrame(void)
{
 U32 camen   = CAMEN;			// save CAMEN
 U32 camreg0 = CAM_Reg(0);		// save CAM entry 0
 U32 camreg1 = CAM_Reg(1);		// save CAM entry 1
 U32 camcon  = CAMCON;			// save CAMCON
 CAM_Reg(0) = 0;
 CAM_Reg(1) = 0;
 CAMEN = 0;				// disable all CAM antry
 CAMCON = CAMCON_CompEn|CAMCON_NegCAM;	// comp enable & neg compare  
 Print("\n\nCapture All Rx Frame Data");
 CaptureRxFrame() ;
 CAMEN = camen;				// restore CAM registers 
 CAM_Reg(0) = camreg0;			// 
 CAM_Reg(1) = camreg1;
 CAMCON = camcon;
}

////////////////////////////////////////////////////////////////////////////////
// Capture My Rx Frame /////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CaptureMyRxFrame(void)
{
 U32 camen   = CAMEN;			// save CAMEN
 U32 camreg0 = CAM_Reg(0);		// save CAM entry 0
 U32 camreg1 = CAM_Reg(1);		// 
 U32 camcon  = CAMCON;			// save CAMCON
 int pos;				// position in address buffer

// 
 Print("\n\nCapture My Rx Frame Data");
 for(pos = 0; pos < 4; ++pos)
  {
   gCam0_Addr0 <<= 8;
   gCam0_Addr0  |= MyMacSrcAddr[pos];
  }

 for(pos = 4; pos < 6; ++pos)
  {
   gCam0_Addr1 <<= 8;
   gCam0_Addr1  |= MyMacSrcAddr[pos];
  }
 gCam0_Addr1 <<= 16;

// 
 CAM_Reg(0) = gCam0_Addr0;		// set CAM 
 CAM_Reg(1) = gCam0_Addr1;		// entry 0
 CAMEN      = 0x1;			// entry 0 valid 
 CAMCON     = CAMCON_CompEn; 		
// 
 CaptureRxFrame();
// 
 CAMEN      = camen;			// restore CAMEN
 CAM_Reg(0) = camreg0;			// restore CAM reg 0
 CAM_Reg(1) = camreg1;			// restore CAM reg 1 
 CAMCON     = camcon;			// restore CAMCON
}

////////////////////////////////////////////////////////////////////////////////
// Capture Matched Destination Address Rx Frame ////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CaptureMatchedDest(void)
{
 U32 camreg0;				// CAM 
 U32 camreg1;
 U32 camen;
 U32 camcon;
 U8 data[12];				// destination address buffer 
 int pos;				// position in  buffer

 camreg0 = CAM_Reg(0);			// 
 camreg1 = CAM_Reg(1);
 camen = CAMEN;
 camcon = CAMCON;

 Print("\nCapture Matched Destination Rx Frame Data");
 Print("\nEnter Destination Address : ");

 pos = 0;				// start position 
 while(1)
  {
   U8 ch;
   ch = get_byte();			// read byte from console 
   if(ch == KEY_CR) 
    {
// CR => terminate destination address input ///////////////////////////////////
     while(pos < 12)
      {					// zero filling  
       data[pos++] = 0;			// destination address
      }
     break;
    } 

        if(ch >= '0' && ch <= '9') { ch -= '0'; }
   else if(ch >= 'A' && ch <= 'F') { ch = (ch - 'A') + 10; }
   else if(ch >= 'a' && ch <= 'f') { ch = (ch - 'a') + 10; } 
   else
    {
// invalid input ///////////////////////////////////////////////////////////////
     continue;
    }

// 
   if(ch <= 9) {put_byte('0' + ch);}
   else {put_byte('A'+(ch-10)); } 

   data[pos] = ch;			// write console byte to address buffer
   if(pos != 0 && (pos & 1) == 0)
    {
     put_byte(' ');
    }
   pos++;
   if(pos >= 12)
    {
     break;
    } 
  }

// Get First of MAC Address ////////////////////////////////////////////////////

// 
 gCam0_Addr0 = 0x0; 
 for(pos = 0; pos < 8; ++pos) 
  {
   gCam0_Addr0 <<= 4;
   gCam0_Addr0 |= data[pos];
  }
  
// Get Second of MAC Address ///////////////////////////////////////////////////
 
 gCam0_Addr1 = 0x0;
 for(pos = 8; pos < 12; ++pos)
  {
   gCam0_Addr1 <<= 4;
   gCam0_Addr1 |= data[pos];
  }
 gCam0_Addr1 <<= 16;

// 
 CAM_Reg(0) = gCam0_Addr0;
 CAM_Reg(1) = gCam0_Addr1;
 CAMEN = 0x01; 
 CAMCON = CAMCON_CompEn;

// 
 CaptureRxFrame();

// 
 CAM_Reg(0) = camreg0;	
 CAM_Reg(1) = camreg1;
 CAMEN = camen;
 CAMCON = camcon;
}


////////////////////////////////////////////////////////////////////////////////
// Capture Broadcast Rx Frame //////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CaptureBroadFrame(void)
{
 U32 camen; 
 U32 camcon; 				// saved content of  CAMCON
 Print("\n\nCapture Broadcast Rx Frame Data");

 camen  = CAMEN;
 camcon = CAMCON;
 CAMEN  = 0;
 CAMCON = CAMCON_CompEn | CAMCON_BroadAcc;// accept broadcast messages 
 CaptureRxFrame();
 CAMEN  = camen;
 CAMCON = camcon;
}


////////////////////////////////////////////////////////////////////////////////
// Capture Control Rx Frame ////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void CaptureCntlFrame(void)
{
 U32 camcon;				// CAMCON saved content
 U32 macrxcon;				// MACRXCON saved content

 Print("\n\nCapture Control Rx Frame Data") ;

// 
 camcon   = CAMCON;			// 
 macrxcon = MACRXCON;			// 

// 
 CAMCON = 
  CAMCON_CompEn 	| 
  CAMCON_StationAcc 	| 
  CAMCON_GroupAcc 	| 
  CAMCON_BroadAcc 	| 
  CAMCON_NegCAM;

// 
 MACRXCON |= MACRXCON_PassCtl;

 CaptureRxFrame();

 MACRXCON = macrxcon;			// restore MACRXCON
 CAMCON   = camcon;			// restore CAMCON
}

////////////////////////////////////////////////////////////////////////////////
// Capture Selected Framre and Show Ethernet Frame /////////////////////////////
////////////////////////////////////////////////////////////////////////////////

static void CaptureAndShowFrame(void)
{
 U8 it;

 while(1) 
  {
   Print("\n\nCapture & ShowFrame MAC Frame.");
   Print("\n[A] Capture All MAC Frame to Buffer Full.");/* All MAC Frames */
   Print("\n[M] Capture My MAC Address Frame.");	/* My MAC Address Frames */
   Print("\n[B] Capture Broadcast Frame.");		/* BroadCast Frames */ 
   Print("\n[C] Capture Control Frame.");		/* Control Frames */
   Print("\n[D] Capture Matched Destination Address.");	/* Destination */
   Print("\n[S] Show Captured MAC Frame.");		/* Show Captured Frames */
   Print("\n[Q] Quit Show & Capture.");
   Print("\nSelect Test Item : "); 
   it = get_upper();
	
   switch(it)
    {
     case 'A' : CaptureAllRxFrame();   break;
     case 'M' : CaptureMyRxFrame();    break;
     case 'B' : CaptureBroadFrame();   break;
     case 'C' : CaptureCntlFrame();    break;
     case 'D' : CaptureMatchedDest();  break;
     case 'S' : ShowRecevedFrameData();break;
     case 'Q' : return;
     default  : 
      Print("\nInvalid Test Item Selected"); 
      break;
    }

   Print("\nPress any key to Continue") ; 
   it = get_byte();
   if(it == 'Q')
    {
     return;
    }
  }
}

////////////////////////////////////////////////////////////////////////////////
// Main function for MAC Block test ////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void MacTest(void)
{
 U8 it;					// selected test item
 MacInitialize();			// Initialize MAC and BDMA controller

 do 
  {
   Print("\n\nMAC TEST");
   Print("\n[C] Capture & Show Ethernet Frame.");
   Print("\n[L] MAC Loopback Test.");
   Print("\n[T] Transfer Ethernet Frame.");
   Print("\n[R] Receive Ethernet Frame.");
   Print("\n[H] MAC Tx Halt Request Test.");
   Print("\n[M] MAC Configuration.");				
   Print("\n[B] BDMA Configuration.");
   Print("\n[P] PHY MII Station Management.");
   Print("\n[A] View & Change MAC Address.");			// MAC Address
   Print("\n[Q] Quit Ethernet Test");

   Print("\nSelect Test Item : "); 
   it = get_upper();
	
   switch(it)
    {
     case 'A' : 
// View and Change Source MAC Address //////////////////////////////////////////
      MACADDRConfigure();		// 		
      break ;

     case 'C' : CaptureAndShowFrame(); break; /* Capture and show Frame Menu */
     case 'L' : MacLoopBackTest();     break; /* MAC LoopBack Test */
     case 'T' : TxMacFrameTest();      break; /* Transmit MAC frame menu */
     case 'R' : RxMacFrame();      break;      
// BDMA Buffer Test, MAC Rx FIFO, MAC Tx Halt Request, MDC On/Off Test                                 - [M] |\n") ;
// PHY MII Station Management Test
     case 'H' : MacTxHaltReqTest();    break; /* Tx HaltImm Request */
     case 'B' : BdmaConfig();          break; /* BDMA Configuration */
     case 'M' : MacConfig();           break; /* MAC Configuration */
     case 'P' : PhyStationTest();      break; /* PHY Station Read&Write Test */
     case 'Q' : break;
     default : 
      Print("\nInvalid Test Item Selected") ; 
      break ;
    }

   if(it != 'Q')
    {
     Print("\nPress any key to Continue MAC Test") ; 
     it = get_upper();
    }
  } 
 while(it != 'Q');
}


